unit FileSel;

interface

uses
  WinTypes, WinProcs, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, FileCtrl, ExtCtrls;

type
  TFSelForm = class(TForm)
    Panel1: TPanel;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    FileDCB: TDriveComboBox;
    FileDLB: TDirectoryListBox;
    FileFLB: TFileListBox;
    SelLB: TListBox;
    Label1: TLabel;
    FileCnt: TLabel;
    AddBtn: TSpeedButton;
    AddAllBtn: TSpeedButton;
    CloseBtn: TBitBtn;
    Label2: TLabel;
    ComPath: TLabel;
    procedure AddBtnClick(Sender: TObject);
    procedure AddAllBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure SelLBKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FileDCBClick(Sender: TObject);
    procedure FileDLBClick(Sender: TObject);
  private
    { Private declarations }
    procedure InitSelLB(CurDir: String);
    procedure BuildFileList(CDir: String);
    procedure SetBusy( BusyState: Boolean );
  public
    { Public declarations }
    procedure EntryPoint( var TargetPath: String; var SL: TStringList );
    function  RemoveSlash( Src: String ): String;
  end;

var
  FSelForm: TFSelForm;

implementation

{$R *.DFM}

var
    CSL:              TStringList;       { Complete pathnames }
    CommDir:          String;

{======================================================================}
{======================================================================}
procedure TFSelForm.FormCreate(Sender: TObject);
begin
     CSL := TSTringList.Create;
end;

procedure TFSelForm.FormDestroy(Sender: TObject);
begin
     CSL.Free;
end;

{======================================================================}
{======================================================================}
procedure TFSelForm.EntryPoint( var TargetPath: String; var SL: TStringList );
var
   i:         Integer;
   Tmp:       String;
begin
     Compath.Caption := '';
     CommDir := '';
     TargetPath := '';
     SL.Clear;
     CSL.Clear;
     CSL.Sorted := True;
     SelLB.Clear;
     FileCnt.Caption := IntToStr(CSL.Count);
     ShowModal;
     SL.Clear;
     TargetPath := LowerCase(RemoveSlash(CommDir)) + '\';
     if (CSL.Count > 0) then
         begin
         for i := 0 to CSL.Count-1 do
             begin
             {if (Copy(CSL[i],1,Length(TargetPath)) = TargetPath) then
                 SL.Add(Copy(CSL[i],Length(TargetPath)+1,999))
             else}
                 SL.Add(CSL[i]);
             end;
         end;
end;

{==================================================================}
{==================================================================}
function  TFSelForm.RemoveSlash( Src: String ): String;
begin
     if ((Length(Src) > 0) and (Src[Length(Src)] = '\')) then
         Result := Copy(Src,1,Length(Src)-1)
     else
         Result := Src;
end;

{==================================================================}
{==================================================================}
procedure TFSelForm.AddBtnClick(Sender: TObject);
var
   i:         Integer;
   ix:        Integer;
   scnt:      Integer;
   FPath:     String;
begin
     SetBusy( True );
     i := 0;
     if (FileFLB.SelCount > 0) then
         begin
         scnt := 1;
         while( scnt <= FileFLB.SelCount ) do
             begin
             FPath := LowerCase(ExpandFileName(FileFLB.Items[i]));
             if (FileFLB.Selected[i]) then
                 begin
                 if (CSL.Find(FPath,ix) = False) then
                     CSL.Add(FPath);
                 scnt := scnt + 1;
                 end; { if }
             i := i + 1;
             end; { while }
         FileFLB.Update;
         InitSelLB(FileFLB.Directory);
         end; { if }
     FileCnt.Caption := IntToStr(CSL.Count);
     SetBusy( False );
end;

{==================================================================}
{==================================================================}
procedure TFSelForm.AddAllBtnClick(Sender: TObject);
var
    CDir:      String;
begin
     SetBusy( True );
     CDir := LowerCase(RemoveSlash(FileFLB.Directory));
     if (FileFLB.SelCount = 0) then
         begin
         BuildFileList(CDir);
         InitSelLB(ExtractFilePath(FileFLB.Directory));
         end; { if }
     FileCnt.Caption := IntToStr(CSL.Count);
     SetBusy( False );
end;

procedure TFSelForm.CloseBtnClick(Sender: TObject);
begin
     Close;
end;


{==================================================================}
{ 'InitSelLB'  This method processes the pathnames in the CSL      }
{              TStringList and determines the common directory     }
{              level and places it in the DirPath object.  The     }
{              remainder of the path is then stored in SelLB       }
{==================================================================}
procedure TFSelForm.InitSelLB(CurDir: String);
var
   i:            Integer;
   CLen:         Integer;
   Tmp:          String;
begin
     CommDir := '';
     if (CSL.Count > 0) then
         begin
         CommDir := RemoveSlash(CurDir);

{==================================================================}
{ Initialize the SelLB with the remaining part of the file path    }
{==================================================================}
         CLen := Length(CommDir) + 1;
         SelLB.Clear;
         for i := 0 to CSL.Count-1 do
             begin
             Tmp := Copy(CSL[i],Clen+1,(Length(CSL[i])-Length(CommDir)));
             if (Length(Tmp) > 60) then
                 Tmp := '.....' + Copy(Tmp,(Length(Tmp)-59),60);
             SelLB.Items.Add(Tmp);
             end;

         if (Length(CommDir) < 3) then
             CommDir := CommDir + '\';

         ComPath.Caption := RemoveSlash(CommDir) + '\';
         end;
end;

{===================================================================}
{ 'BuildFileList'                                                   }
{===================================================================}
procedure TFSelForm.BuildFileList(CDir: String);
var
   ix:          Integer;
   DirCnt:      Integer;
   FInfo:       TSearchRec;
   DirSL:       TStringList;
   Result:      Integer;
   PName:       String;
begin
    DirSL := TStringList.Create;
    Result := FindFirst(CDir+'\*.*', faAnyFile, FInfo);
    while Result = 0 do
        begin
        PName := LowerCase(CDir + '\' + FInfo.Name);
        if ((FInfo.Attr and faDirectory) = faDirectory) then
            begin
            if (Finfo.Name[1] <> '.') then
                DirSL.Add(PName);
            end
        else
            begin
            if (CSL.Find(PName,ix) = False) then
                CSL.Add(PName);
            end;

        Result := FindNext(FInfo);
        end;

    FindClose(FInfo);

{===================================================================}
{ If the directory we just went through had sub directories, they   }
{ were saved in the DirSL TStringList. Using DirSL, recurse through }
{ the directories and add to the file list.                         }
{===================================================================}
    DirCnt := 0;
    while (DirCnt < DirSL.Count) do
        begin
        BuildFileList(DirSL[DirCnt]);
        DirCnt := DirCnt + 1;
        end;
    DirSL.Free;
end;

{==================================================================}
{ 'SetBusy'  Set the cursor to busy or idle                        }
{==================================================================}
procedure TFSelForm.SetBusy( BusyState: Boolean );
begin
     if ( BusyState = True ) then
         Screen.Cursor := crHourGlass
     else
         Screen.Cursor := crDefault;
end;

{==================================================================}
{ 'SelLBKeyDown'   This method responds to the 'Delete' key being  }
{                  pressed when the SelLB ListBox has the focus.   }
{                  This allows the user to delete entries from the }
{                  ListBox before Compressing or Expanding         }
{==================================================================}
procedure TFSelForm.SelLBKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
    ix:       Integer;
    pos:      Integer;
    scnt:     Integer;
begin
     if ((Key = VK_DELETE) and (SelLB.SelCount > 0)) then
         begin
         if (MessageDlg('Are you sure you want to delete the select files?',
                        mtConfirmation,[mbYes,mbNo],0) = mrYes) then
             begin
             SetBusy( True );
             ix := 0;
             while (SelLB.SelCount > 0) do
                 begin
                 if (SelLB.Selected[ix] = True) then
                     begin
                     SelLB.Items.Delete(ix);
                     CSL.Delete(ix);
                     end
                 else
                     ix := ix + 1;
                 end;
             SetBusy( False );
             end;

         InitSelLB(CommDir);
         FileCnt.Caption := IntToStr(CSL.Count);
         end;
end;

{==================================================================}
{==================================================================}
procedure TFSelForm.FileDCBClick(Sender: TObject);
begin
     CSL.Clear;
     SelLB.Clear;
     FileCnt.Caption := IntToStr(CSL.Count);
     CommDir := '';
     ComPath.Caption := '';
end;

{==================================================================}
{==================================================================}
procedure TFSelForm.FileDLBClick(Sender: TObject);
begin
     CSL.Clear;
     SelLB.Clear;
     FileCnt.Caption := IntToStr(CSL.Count);
     CommDir := '';
     ComPath.Caption := '';
end;

end.

