unit DemoMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, ExtCtrls, Buttons, StdCtrls, FileCtrl, Compress, Segment;

type
  TDemoForm = class(TForm)
    MainMenu1: TMainMenu;
    MFile: TMenuItem;
    MMode: TMenuItem;
    MOptions: TMenuItem;
    MHelp: TMenuItem;
    MHDemoHelp: TMenuItem;
    MHAbout: TMenuItem;
    MMCompress: TMenuItem;
    MMExpand: TMenuItem;
    MMScan: TMenuItem;
    MFExit: TMenuItem;
    Panel1: TPanel;
    Panel2: TPanel;
    MMReset: TMenuItem;
    Panel4: TPanel;
    MsgPanel: TPanel;
    RunPanel: TPanel;
    ArcLB: TListBox;
    SegCompress: TSegCompress;
    Panel5: TPanel;
    Label1: TLabel;
    CurSeg: TLabel;
    Label2: TLabel;
    CurSize: TLabel;
    Label3: TLabel;
    CurDate: TLabel;
    Label4: TLabel;
    CurFiles: TLabel;
    Label5: TLabel;
    CurMode: TLabel;
    Label6: TLabel;
    CurSrc: TLabel;
    Label8: TLabel;
    CurComp: TLabel;
    CompressBtn: TSpeedButton;
    ExpandBtn: TSpeedButton;
    ScanBtn: TSpeedButton;
    Panel6: TPanel;
    OKBtn: TBitBtn;
    ExitBtn: TSpeedButton;
    Label7: TLabel;
    ComPath: TLabel;
    MSelect: TMenuItem;
    ResetBtn: TBitBtn;
    SelectBtn: TBitBtn;
    TSegCompressHelp1: TMenuItem;
    HelpBtn: TBitBtn;
    procedure ExitBtnClick(Sender: TObject);
    procedure ResetBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CompressBtnClick(Sender: TObject);
    procedure ExpandBtnClick(Sender: TObject);
    procedure ScanBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SelectBtnClick(Sender: TObject);
    procedure ArcLBKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure OKBtnClick(Sender: TObject);
    procedure OptionClick(Sender: TObject);
    procedure SegCompressCheckFile(var filepath: string;
      mode: TCProcessMode);
    procedure SegCompressSegBefore(SegMode: TSegMode; SegName: string;
      SegNumber: Word);
    procedure HelpAboutClick(Sender: TObject);
    procedure TSegCompressHelpClick(Sender: TObject);
    procedure MHDemoHelpClick(Sender: TObject);
  private
    { Private declarations }
    procedure SetRunState( RunState: String );
    procedure InitArcLB;
    procedure DemoCompress;
    procedure DemoExpand;
    procedure DemoScan;
    procedure CompressOptions;
    procedure ExpandOptions;
    procedure ScanOptions;
    procedure ResetButtons;
    procedure CompressState( State: Boolean );
    procedure ExpandState( State: Boolean );
    procedure ScanState( State: Boolean );
    procedure SelectState( State: Boolean );
    procedure ResetState( State: Boolean );
    procedure OptionsState( State: Boolean );
  public
    { Public declarations }
  end;

var
  DemoForm: TDemoForm;

implementation

uses Coptions, Eoptions, Soptions, FileSel, SegAbout;

{$R *.DFM}

{==================================================================}
{==================================================================}
const
  TSC_Start = 1000;
  DH_Start = 5000;

type
  TDemoMode = (dmNone,dmCompress,dmExpand,dmScan);

var
  SL:               TStringList;
  AppDir:           String;
  DemoMode:         TDemoMode;
  TargetPath:       String;
  SegHdr:           TSegHeader;

{==================================================================}
{==================================================================}
procedure TDemoForm.FormCreate(Sender: TObject);
begin
     GetDir(0,AppDir);
     AppDir := FSelForm.RemoveSlash(AppDir);
     SL := TStringList.Create;
     SL.Sorted := True;
     ResetBtnClick(Sender);
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.FormDestroy(Sender: TObject);
begin
     SL.Free;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.ResetButtons;
begin
     CompressState( True );
     ExpandState( False );
     ScanState( True );
     SelectState( False );
     ResetState( True );
     OptionsState( False );
     OKBtn.Enabled := False;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.CompressState( State: Boolean );
begin
     CompressBtn.Enabled := State;
     MMCompress.Enabled := State;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.ExpandState( State: Boolean );
begin
     ExpandBtn.Enabled := State;
     MMExpand.Enabled := State;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.ScanState( State: Boolean );
begin
     ScanBtn.Enabled := State;
     MMScan.Enabled := State;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.SelectState( State: Boolean );
begin
     SelectBtn.Enabled := State;
     MSelect.Enabled := State;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.ResetState( State: Boolean );
begin
     ResetBtn.Enabled := State;
     MMReset.Enabled := State;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.OptionsState( State: Boolean );
begin
     MOptions.Enabled := State;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.ExitBtnClick(Sender: TObject);
begin
     if (DemoMode = dmNone) then
         Close;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.ResetBtnClick(Sender: TObject);
begin
     DemoMode := dmNone;
     CompressBtn.Down := False;
     ExpandBtn.Down := False;
     ScanBtn.Down := False;
     ResetButtons;
     SL.Clear;
     TargetPath := '';
     InitArcLB;
     CurMode.Caption := '';
     CurComp.Caption := '';
     CurFiles.Caption := IntToStr(SL.Count);
     CurSrc.Caption := '';
     CurSeg.Caption := '';
     CurSize.Caption := '';
     CurDate.Caption := '';
     ComPath.Caption := TargetPath;
     MsgPanel.Font.Color := clBlack;
     RunPanel.Font.Color := clBlack;
     MsgPanel.Caption := ' Select a mode or Press Help';
     DemoForm.ActiveControl := HelpBtn;
     SetRunState('Ready');
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.CompressBtnClick(Sender: TObject);
begin
     if ( DemoMode <> dmCompress ) then
         begin
         CompressBtn.Down := True;
         ExpandBtn.Down := False;
         ScanBtn.Down := False;
         ExpandState( False );
         ScanState( False );
         SelectState( True );
         ResetState( True );
         OptionsState( True );
         OKBtn.Enabled := False;
         MsgPanel.Font.Color := clBlack;
         RunPanel.Font.Color := clBlack;
         CurMode.Caption := 'Compress';
         DemoMode := dmCompress;
         CompressOptions;
         SelectBtnClick(Sender);
         MsgPanel.Caption := ' Press OK to begin Compressing files';
         OKBtn.Enabled := True;
         DemoForm.ActiveControl := OKBtn;
         end
     else
         CompressBtn.Down := True;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.ExpandBtnClick(Sender: TObject);
begin
     if ( DemoMode <> dmExpand ) then
         begin
         CompressBtn.Down := False;
         ExpandBtn.Down := True;
         CompressState( False );
         ExpandState( True );
         ScanState( False );
         SelectState( False );
         OptionsState( True );
         ResetState( True );
         OKBtn.Enabled := False;
         MsgPanel.Font.Color := clBlack;
         RunPanel.Font.Color := clBlack;
         CurMode.Caption := 'Expand';
         DemoMode := dmExpand;
         ExpandOptions;
         MsgPanel.Caption := ' Press OK to begin Expanding files';
         OKBtn.Enabled := True;
         DemoForm.ActiveControl := OKBtn;
         end
     else
         ExpandBtn.Down := True;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.ScanBtnClick(Sender: TObject);
begin
     if ( DemoMode <> dmScan ) then
         begin
         CompressBtn.Down := False;
         ExpandBtn.Down := False;
         ScanBtn.Down := True;
         CompressState( False );
         ExpandState( False );
         ScanState( True );
         SelectState( False );
         ResetState( True );
         OptionsState( True );
         OKBtn.Enabled := False;
         MsgPanel.Font.Color := clBlack;
         RunPanel.Font.Color := clBlack;
         CurMode.Caption := 'Scan';
         DemoMode := dmScan;
         ScanOptions;
         MsgPanel.Caption := ' Press OK to scan the segmented archive';
         OKBtn.Enabled := True;
         DemoForm.ActiveControl := OKBtn;
         end
     else
         ScanBtn.Down := True;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.HelpAboutClick(Sender: TObject);
begin
     MsgPanel.Font.Color := clBlack;
     RunPanel.Font.Color := clBlack;
     AboutForm.ShowModal;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.SetRunState( RunState: String );
begin
     if ( (RunState = 'Ready') or
          (RunState = 'Aborted') ) then
         Screen.Cursor := crDefault
     else
         Screen.Cursor := crHourGlass;

     RunPanel.Caption := RunState;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.CompressOptions;
begin
     CompOptionForm.EntryPoint(SegCompress);
     CurSrc.Caption := SegCompress.SegDir;
     CurSeg.Caption := SegCompress.SegBaseName;
     CurSize.Caption := IntToStr(SegCompress.SegSize);
     if (CompOptionForm.CompType.ItemIndex = 0) then
         CurComp.Caption := 'LZH'
     else if (CompOptionForm.CompType.ItemIndex = 1) then
         CurComp.Caption := 'RLE'
     else if (CompOptionForm.CompType.ItemIndex = 2) then
         CurComp.Caption := 'None'
     else
         CurComp.Caption := '';
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.ExpandOptions;
begin
     ExpOptionForm.EntryPoint(TargetPath);
     ComPath.Caption := TargetPath;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.ScanOptions;
begin
     ScanOptionForm.EntryPoint(SegCompress);
     CurSrc.Caption := SegCompress.SegDir;
     CurSeg.Caption := SegCompress.SegBaseName;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.InitArcLB;
var
   i:           Integer;
   len:         Integer;
   tmpfile:     String;
begin
     ArcLB.Clear;
     len := Length(SegCompress.TargetPath);
     for i := 0 to SL.Count-1 do
         begin
         tmpfile := SL[i];
         if (Copy(tmpfile,1,len) = SegCompress.TargetPath) then
             ArcLB.Items.Add(Copy(tmpfile,len+1,9999))
         else
             ArcLB.Items.Add(SL[i]);
         end;
     ArcLB.Update;
     CurFiles.Caption := IntToStr(SL.Count);
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.SelectBtnClick(Sender: TObject);
begin
     FSelForm.EntryPoint(TargetPath,SL);
     SegCompress.TargetPath := TargetPath;
     InitArcLB;
     ComPath.Caption := TargetPath;
     MsgPanel.Caption := ' Press OK to compress the selected files';
     OKBtn.Enabled := True;
     DemoForm.ActiveControl := OKBtn;
end;

{==================================================================}
{ 'ArcLBKeyDown'   This method responds to the 'Delete' key being  }
{                  pressed when the ArcLB ListBox has the focus.   }
{                  This allows the user to delete entries from the }
{                  ListBox before Compressing or Expanding         }
{==================================================================}
procedure TDemoForm.ArcLBKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
    ix:       Integer;
    pos:      Integer;
    scnt:     Integer;
begin
     if ((Key = VK_DELETE) and (ArcLB.SelCount > 0)) then
         begin
         if (MessageDlg('Are you sure you want to delete the select files?',
                        mtConfirmation,[mbYes,mbNo],0) = mrYes) then
             begin
             SetRunState('Deleting');
             ix := 0;
             while (ArcLB.SelCount > 0) do
                 begin
                 if (ArcLB.Selected[ix] = True) then
                     begin
                     ArcLB.Items.Delete(ix);
                     SL.Delete(ix);
                     end
                 else
                     ix := ix + 1;
                 end;
             SetRunState('Ready');
             end;
         if (CompressBtn.Down = True) then
             InitArcLB
         else
             {InitArcList;};

         CurFiles.Caption := IntToStr(SL.Count);
         end;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.OKBtnClick(Sender: TObject);
begin
     if (DemoMode = dmCompress) then
         DemoCompress
     else if (DemoMode = dmExpand) then
         DemoExpand
     else if (DemoMode = dmScan) then
         DemoScan ;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.DemoCompress;
var
   compressMode:     TCompressionMethod;
begin
     SetRunState('Busy..');
     ExpandState( False );
     ScanState( False );
     SelectState( False );
     ResetState( False );
     OptionsState( False );
     OKBtn.Enabled := False;
     if (CompOptionForm.CompType.ItemIndex = 0) then
         compressMode := coLZH
     else if (CompOptionForm.CompType.ItemIndex = 1) then
         compressMode := coRLE
     else if (CompOptionForm.CompType.ItemIndex = 2) then
         compressMode := coNone;
     SL.Sorted := False;
     try
       SetRunState('Compressing');
       SegCompress.TargetPath := TargetPath;
       SegCompress.MakeDirectories := False;
       SegCompress.SegCompressFiles(SL,compressMode);
       SetRunState('Ready');
       MsgPanel.Caption := ' Compression Complete, Press Reset Button';
     except on E: Exception do
       begin
       MsgPanel.Font.Color := clRed;
       RunPanel.Font.Color := clRed;
       SetRunState('Aborted');
       MsgPanel.Caption := 'Error: ' + E.Message + ', Press Reset Button';
       end;
     end; { try }
     SL.Sorted := True;
     ResetState( True );
     DemoMode := dmNone;
     DemoForm.ActiveControl := ResetBtn;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.DemoExpand;
var
    SaveCursor:   TCursor;
    Tmp:          String;
begin
     SetRunState('Busy..');
     CompressState( False );
     ScanState( False );
     SelectState( False );
     ResetState( False );
     OptionsState( False );
     OKBtn.Enabled := False;
     if (DirectoryExists(TargetPath) = False) then
         begin
         Tmp := 'Path: ''''' + TargetPath + ''''' does not exist. Create it?';
         SaveCursor := Screen.Cursor;
         Screen.Cursor := crDefault;
         if (MessageDlg(Tmp,mtConfirmation,[mbYes,mbNo],0) = mrNo) then
             begin
             MsgPanel.Font.Color := clRed;
             RunPanel.Font.Color := clRed;
             MsgPanel.Caption := ' Aborted by user.  Press Reset';
             SetRunState('Aborted');
             ResetState( True );
             DemoForm.ActiveControl := ResetBtn;
             Screen.Cursor := SaveCursor;
             exit;
             end;
         Screen.Cursor := SaveCursor;
         ForceDirectories(TargetPath);
         end;

     try
       SetRunState('Expanding');
       SegCompress.MakeDirectories := True;
       SegCompress.TargetPath := TargetPath;
       SegCompress.SegExpandFiles(SL);
       SetRunState('Ready');
       MsgPanel.Caption := ' Expand Complete, Press Reset ';
       SetRunState('Ready');
     except on E: Exception do
       begin
       MsgPanel.Font.Color := clRed;
       RunPanel.Font.Color := clRed;
       SetRunState('Aborted');
       MsgPanel.Caption := 'Error: ' + E.Message + ', Press Reset Button';
       end;
     end; { try }

     ResetState( True );
     DemoMode := dmNone;
     DemoForm.ActiveControl := ResetBtn;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.DemoScan;
begin
     SetRunState('Busy..');
     CompressState( False );
     ExpandState( False );
     SelectState( False );
     ResetState( False );
     OptionsState( False );
     OKBtn.Enabled := False;
     SetRunState('Scanning');
     SL.Clear;
     SL.Sorted := False;
     try
       SetRunState('Scanning');
       SegCompress.SegScanCompressedFile(SL,SegHdr);
       SetRunState('Ready');
       ExpandState( True );
       MsgPanel.Caption := ' Scan Complete, Press Reset or Expand Button';
     except on E: Exception do
       begin
       MsgPanel.Font.Color := clRed;
       RunPanel.Font.Color := clRed;
       SetRunState('Aborted');
       MsgPanel.Caption := 'Error: ' + E.Message + ', Press Reset Button';
       end;
     end; { try }
     InitArcLB;
     if (SegHdr.CompMode = scLZH) then
         CurComp.Caption := 'LZH'
     else if (SegHdr.CompMode = scRLE) then
         CurComp.Caption := 'RLE'
     else if (SegHdr.CompMode = scNone) then
         CurComp.Caption := 'None';
     CurSeg.Caption := SegCompress.SegBaseName;
     CurSize.Caption := IntToStr(SegHdr.SegSize);
     CurDate.Caption := StrPas(SegHdr.SegDate);
     ComPath.Caption := SegCompress.TargetPath;;
     ResetState( True );
     DemoMode := dmNone;
     DemoForm.ActiveControl := ResetBtn;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.OptionClick(Sender: TObject);
begin
     if (DemoMode = dmCompress) then
         CompressOptions
     else if (DemoMode = dmExpand) then
         ExpandOptions
     else if (DemoMode = dmScan) then
         ScanOptions;
     DemoForm.ActiveControl := OKBtn;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.SegCompressCheckFile(var filepath: string;
  mode: TCProcessMode);
var
    Tmp:    String;
    CDir:   String;
begin
     MsgPanel.Caption := ' ' + filepath;
     MsgPanel.Refresh;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.SegCompressSegBefore(SegMode: TSegMode;
  SegName: string; SegNumber: Word);
begin
     MsgPanel.Caption := ' Segment: ' + SegName;
     MsgPanel.Refresh;
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.TSegCompressHelpClick(Sender: TObject);
begin
     Application.HelpFile := AppDir + '\' + 'TSEGHELP.HLP';
     Application.HelpContext(TSC_Start);
end;

{==================================================================}
{==================================================================}
procedure TDemoForm.MHDemoHelpClick(Sender: TObject);
begin
     Application.HelpFile := AppDir + '\' + 'DEMOHELP.HLP';
     Application.HelpContext(DH_Start);
end;

end.
