unit Demobar;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics,
  Controls, Forms, Dialogs, ExtCtrls, StdCtrls, Menus, Buttons;

type
  TSnapon = (sTop,sLeft,sRight,sBottom,
             sTopLH,sTopLV,sTopRH,sTopRV,
             sBotLH,sBotLV,sBotRH,sBotRV,sFloat);
  TTools = class(TForm)
    CapPanel: TPanel;
    MainPanel: TPanel;
    ToolPanel: TPanel;
    OrientBtn: TSpeedButton;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton3: TSpeedButton;
    SpeedButton4: TSpeedButton;
    SpeedButton5: TSpeedButton;
    SpeedButton6: TSpeedButton;
    SpeedButton7: TSpeedButton;
    SpeedButton8: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure CapPanelMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure CapPanelMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure DirsDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure OrientBtnClick(Sender: TObject);
  private
    procedure WMMouseActivate(var Msg : TWMMouseActivate);
                              message WM_MOUSEACTIVATE;
    procedure AppDeactivate(Sender:tobject);
    procedure AppActivate(Sender:tobject);
  public
    Horizontal : boolean;
    DefaultSnap:TSnapOn;
    SnapOnMove:boolean;
    procedure HorizontalBar;
    procedure VerticalBar;
    procedure SnapBar(Caller:TForm; Dock:TSnapOn);
  end;

var
  Tools: TTools;

implementation

{$R *.DFM}

const
  MaxButtons = 39;  {max number of buttons}
var
  LastX,LastY : integer;

Procedure TTools.SnapBar(Caller:TForm; Dock:TSnapOn);
var
  l,t,r,b:integer;
  s : TSnapOn;
begin
  DefaultSnap := Dock;
  if Dock in [sTop,sBottom,sTopLH,sTopRH,sBotLH,sBotRH] then
    HorizontalBar else VerticalBar;

  With Caller do
    begin
      l := left;
      t := top; { caller.top-height-2;}
      r := left+width;
      b := top+height;
    end;
  With Tools do
    Case Dock of
      sTop,sBottom :
        begin
          left := l;
          if dock = sTop then top := t-height-2 else
            top := b + 1;
        end;
      sLeft,sRight:
        begin
          top := t;
          if Dock = sLeft then left := l-width-2 else
            left := r+2;
        end;
      sTopRH,sTopLH,sTopRV,sTopLV :
        begin
          top := 0;
          if Dock in [sTopRH,sTopRV] then
            left := Screen.Width-Width-2 else
              left := 0;
        end;
      sBotRH,sBotLH,sBotRV,sBotLV :
        begin
          top := Screen.Height-height-2;
          if Dock in [sBotRH,sBotRV] then
            left := Screen.Width-Width-2
              else left := 0;
        end;
    End;
end;

procedure TTools.WMMouseActivate(var Msg : TWMMouseActivate);
begin
  Msg.Result := MA_NoActivate;
end;

procedure TTools.AppDeactivate(Sender:tobject);
begin
  Application.NormalizeTopMosts;
  CapPanel.Color := clInactiveCaption;
  CapPanel.Font.Color := clInactiveCaptionText;
  Enabled := false;
end;

procedure TTools.AppActivate(Sender:tobject);
begin
  Application.RestoreTopMosts;
  CapPanel.Color := clActiveCaption;
  CapPanel.Font.Color := clCaptionText;
  Enabled := true;
end;

procedure TTools.VerticalBar;
Var
  i,l,h : integer;
begin
  l := 0;
  h := ToolPanel.controls[0].height;
  width := h+1;
  OrientBtn.Caption := '';
  with ToolPanel do
    begin
      CapPanel.Caption := 'My';
      for i := 0 to ControlCount-1 do
        begin
          Controls[i].left := 1;
          Controls[i].top := controls[i].tag*h;
          inc(l,h);
        end;
    end;
  height := l+CapPanel.height;
  horizontal := false;
end;

procedure TTools.HorizontalBar;
var
  i,l,w : integer;
begin
  l := 0;
  w := ToolPanel.Controls[0].Width;
  CapPanel.Alignment := taCenter;
  CapPanel.Caption := 'My Tool Bar';
  OrientBtn.Caption := '';
  with ToolPanel do
    begin
      for i := 0 to ControlCount-1 do
        begin
          controls[i].top := 0;
          controls[i].left := controls[i].tag*w;
          inc(l,w);
        end;
    end;
  width := l;
  Height := CapPanel.height+w;
  Horizontal := true;
end;

procedure TTools.FormCreate(Sender: TObject);
var
  i : integer;
begin
  Application.OnDeactivate := AppDeactivate;
  Application.OnActivate := AppActivate;
  HorizontalBar;
  SnapOnMove := true;
  Hide;
end;

procedure TTools.CapPanelMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  LastX := x;
  LastY := y;
end;


procedure TTools.CapPanelMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if ssLeft in Shift then
    begin
      Top := Top + (y - LastY);
      left := Left + (x - LastX);
    end;
end;

procedure TTools.DirsDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := true;
end;

procedure TTools.OrientBtnClick(Sender: TObject);
begin
  if OrientBtn.Down then VerticalBar else HorizontalBar;
end;

end.
