{TDirTree Component for Delphi 2  written by Markus Stephany
                                             MirBir.St@T-Online.de
                                             http://home.t-online.de/home/mirbir.st/

 V 1.01    04/01/97

 // a treeview that shows the system's drives and folders (or a part of them ,s.b.)
 // displays explorer-like icons and filenames

 this is free for freeware, public domain and shareware, but not for commercial use
 without my permission.

 if something is going wrong (or even not), contact me (see above), but do not make
 me responsable for anything !!

 added properties :
 Directory      read/write    gets/sets the dirtree's directory
 Drive          read/write    gets/sets the drive (restoring the previously used
                                                   directory for that drive)

 added functions :
 Reload                       reads the items again and tries to set the old dir.
                              (use it e.g. with a shell-notification)

 added event :
 OnAddDir (Filename : tfilename ; var DoAdd : boolean)
                              You can use this event to decide whether to add
                              a drive/path or not (see the sample for more info)}


unit dirtree;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls,shellapi,filectrl;

type taddfileevent = procedure (Filename : tfilename ; var DoAdd : boolean) of object;
type
  TDirTree = class(TcustomTreeView)
  private
    { Private-Deklarationen }
    filist : timagelist;
    fnamen : tstrings;
    factdirs : array['A'..'Z'] of string;
    fonadd : taddfileevent;
    ftdirs : tstringlist;
    fifols : integer ;
    fifoln : integer ;
    fallowchange : boolean;
  protected
    { Protected-Deklarationen }
    function CanExpand(Node: TTreeNode): Boolean ; override;
    procedure createwnd ; override;
    procedure readnew;
    destructor destroy; override;
    procedure addnode (par : ttreenode; var item : ttreenode; a : tfilename);
    procedure getkids (var item : ttreenode);
    procedure addkid (par : ttreenode; a : tfilename);
    procedure getsubkids (par:ttreenode);
    function getdirectory : tfilename;
    function getdrive : char;
    procedure setdrive (val : char);
    procedure setdirectory (val : tfilename);
    function askadd (fnam : tfilename):boolean;
    procedure Change(Node: TTreeNode); override;
    property items stored false;
  public
    { Public-Deklarationen }
    procedure Reload ;
    constructor create (aowner: tcomponent);override;
    procedure FullExpand;
    property Directory : Tfilename read getdirectory write setdirectory;
    property Drive : char read getdrive write setdrive;
  published
    { Published-Deklarationen }
    property OnAddDir : taddfileevent read fonadd write fonadd;
    property BorderStyle;
    property DragCursor;
    property DragMode;
    property HideSelection;
    property OnEditing;
    property OnEdited;
    property OnExpanding;
    property OnExpanded;
    property OnCollapsing;
    property OnCollapsed;
    property OnChanging;
    property OnChange;
    property OnDeletion;
    property Align;
    property Enabled;
    property Font;
    property Color;
    property ParentColor;
    property ParentCtl3D;
    property Ctl3D;
    property TabOrder;
    property TabStop default True;
    property Visible;
    property OnClick;
    property OnEnter;
    property OnExit;
    property OnDragDrop;
    property OnDragOver;
    property OnStartDrag;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnDblClick;
    property OnKeyDown;
    property OnKeyPress;
    property OnKeyUp;
    property PopupMenu;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Beispiele', [TDirTree]);
end;

constructor tdirtree.create;
var sfi : tshfileinfo;
     n : char;
begin
     inherited create(aowner);
     filist := timagelist.create(self);
     fnamen := tstringlist.create;
     ftdirs := tstringlist.create;
     for n := 'A' to 'Z' do
         factdirs[n]:=uppercase(n)+':\';
     factdirs[upcase(getcurrentdir[1])] := uppercase(getcurrentdir);
     try
        filist.handle :=
          shgetfileinfo('',0,sfi,sizeof(tshfileinfo),
            shgfi_sysiconindex or shgfi_smallicon);
        filist.shareimages := true;
        images := filist;
     except;
     end;
     readonly := true;
     sorttype := stnone;
     hideselection := false;
     fallowchange := false;
end;

destructor tdirtree.destroy;
begin
     fallowchange := false;
     fnamen.free;
     ftdirs.free;
     filist.free;
     inherited;
end;

procedure tdirtree.Reload ;
var t : char;
    node : ttreenode;
    tdir : string;
begin
     tdir := getdirectory;
     readnew;
     setdirectory(tdir);
     if items.count > 0 then if selected = nil then setdirectory(copy(tdir,1,3));
     if items.count > 0 then if selected = nil then begin
        node := items.getfirstnode;
        t := upcase(fnamen[integer(node.data)][2]);
            setdirectory(factdirs[t]);
            if getdirectory <> factdirs[t] then begin
               node.selected := true;
               change(node);
            end;
     end;
end;


function tdirtree.askadd (fnam : tfilename):boolean;
begin
     result := true;
     if assigned (fonadd) then fonadd(fnam,result);
end;

procedure tdirtree.setdirectory (val : string);
var az : integer;
    node : ttreenode;
    oldcur : tcursor;
    afg : integer;
    tsr : string;
    label m1;

    function azpos(a:string):integer;
    var ct: integer;
    begin
         result := 0;
         repeat
               ct := pos ('\',a);
               if ct = 0 then exit;
               inc (result,1);
               a:=copy(a,ct+1,maxint);
         until false;
    end;

    function getpart(a:string;b:integer):string;
    var ct,ix : integer;
    begin
         result := a;
         ix := 0;
         for ct := 1 to length(a) do begin
             if a[ct] = '\' then inc (ix,1);
             if ix = b then begin
                result := copy(a,1,ct);
                exit;
             end;
         end;
    end;

    function getroot(a:string):ttreenode;
    var sr : ttreenode;
        ct : integer;
    begin
         result := nil;
         ct:=fnamen.indexof('u'+a);
         if ct > -1 then begin
            sr := items.getfirstnode;
            while sr <> nil do begin
                  if integer(sr.data) = ct then begin
                     getkids(sr);
                     break;
                  end;
                  sr := sr.getnextsibling;
            end;
         end;
         ct := fnamen.indexof('i'+a);
         if ct > -1 then begin
            sr := items.getfirstnode;
            while sr <> nil do begin
                  if integer(sr.data) = ct then begin
                     result := sr;
                     exit;
                  end;
                  sr := sr.getnextsibling;
            end;
         end;
    end;

    function getchi(c:ttreenode;a:string):ttreenode;
    var sr : ttreenode;
        ct : integer;
    begin
         result := nil;
         ct:=fnamen.indexof('u'+a);
         if ct > -1 then begin
            sr := c.getfirstchild;
            while sr <> nil do begin
                  if integer(sr.data) = ct then begin
                     getkids(sr);
                     break;
                  end;
                  sr := c.getnextchild(sr);
            end;
         end;
         ct := fnamen.indexof('i'+a);
         if ct > -1 then begin
            sr := c.getfirstchild;
            while sr <> nil do begin
                  if integer(sr.data) = ct then begin
                     result := sr;
                     exit;
                  end;
                  sr := c.getnextchild(sr);
            end;
         end;
    end;

begin
     if val <> '' then if val[length(val)] <> '\' then val := val+'\';
     if ansicomparetext(val,getdirectory) = 0 then exit;
     if length(val) <> 3 then if not directoryexists(val) then exit;
     // teilstrings holen
     az := azpos(val);
     if az = 0 then exit;
     oldcur := screen.cursor;
     screen.cursor := crhourglass;
     items.beginupdate;
     val := uppercase(val);
     for afg := 1 to az do begin
         // string holen
         tsr := getpart(val,afg);
         if afg = 1 then
         // rootnode herausfinden
            node := getroot(tsr)

         // kindnode herausfinden
         else  node := getchi(node,tsr);
         if node = nil then goto m1;
     end;
     selected := node;
     topitem := node;
m1:     items.endupdate;
     screen.cursor := oldcur;
end;

function tdirtree.getdirectory : string;
begin
     if selected = nil then result := '?' else
     try
        result := copy(fnamen[integer(selected.data)],2,maxint);
     except
           result := '?';
     end;
end;


procedure tdirtree.addkid (par:ttreenode; a : tfilename);
var sfi : tshfileinfo;
    sr : string;
    item : ttreenode;
begin
     sr := '';
     if par = nil then exit;
     //sr := copy(fnamen[integer(par.data)],2,maxint);
     if a[length(a)] <> '\' then a:=a+'\';
     shgetfileinfo(pchar(sr+a),0,sfi,sizeof(sfi),shgfi_sysiconindex or shgfi_smallicon
       or shgfi_displayname);
     fnamen.add('u'+uppercase(sr+a));
     item := par.owner.addchild(par,sfi.szdisplayname);
     item.data := pointer(fnamen.count-1);
     item.imageindex := sfi.iicon;
     shgetfileinfo(pchar(sr+a),0,sfi,sizeof(sfi),shgfi_sysiconindex or shgfi_smallicon
       or shgfi_openicon);
     item.selectedindex := sfi.iicon;
end;

procedure tdirtree.addnode (par:ttreenode; var item : ttreenode; a:tfilename);
var sfi : tshfileinfo;
    sr : string;
begin
     sr := '';
     if a[length(a)] <> '\' then a:=a+'\';
     shgetfileinfo(pchar(sr+a),0,sfi,sizeof(sfi),shgfi_sysiconindex or shgfi_smallicon
       or shgfi_displayname);
     fnamen.add('u'+uppercase(sr+a));
     item := items.add(par,sfi.szdisplayname);
     item.data := pointer(fnamen.count-1);
     item.imageindex := sfi.iicon;
     if sfi.iicon <> fifoln then begin
        shgetfileinfo(pchar(sr+a),0,sfi,sizeof(sfi),shgfi_sysiconindex or shgfi_smallicon
        or shgfi_openicon);
        item.selectedindex := sfi.iicon;
        if length(a) > 3 then begin
           fifoln := item.imageindex;
           fifols := sfi.iicon;
        end;
     end else item.selectedindex := fifols;
end;

procedure tdirtree.getsubkids (par:ttreenode);
var node : ttreenode;
begin
     if csdesigning in componentstate then exit;
     if par = nil then exit;
     node := par.getfirstchild;
     while node <> nil do begin
           getkids(node);
           node := par.getnextchild(node);
     end;
end;

procedure tdirtree.getkids (var item:ttreenode);
var sr : tsearchrec;
    res,ct : integer;
    si : string;
begin
     if csdesigning in componentstate then exit;
     if item = nil then exit;
     ftdirs.clear;
     si:=fnamen[integer(item.data)];
     if si[1]='i' then exit;
     si[1]:='i';
     fnamen[integer(item.data)] := si;
     delete(si,1,1);
     fillchar(sr,sizeof(sr),0);
     res := findfirst(si+'*.*',faanyfile-favolumeid,sr);
     while res = 0 do begin
           if ((sr.attr and fadirectory) > 0) and (sr.name <> '.') and (sr.name <> '..') then
              if (not assigned (fonadd)) or (askadd(si+sr.name+'\')) then
              ftdirs.add(sr.name);
              //addkid(item,si+sr.name);
           res := findnext(sr);
     end;
     findclose(sr);
     ftdirs.sorted := true;
     if ftdirs.count > 0 then for ct := 0 to ftdirs.count -1 do
        addkid(item,si+ftdirs[ct]);
end;

procedure tdirtree.readnew;
var oldcur : tcursor;
    node : ttreenode;
    bits : set of 0..25;
    ct : integer;
    drv : char;
begin
     if handleallocated then if not (csloading in componentstate) then begin
          fallowchange := false;
          oldcur := screen.cursor;
          screen.cursor := crhourglass;
          fnamen.clear;
          items.beginupdate;
          items.clear;
          application.processmessages;
          // Laufwerke einlesen
          integer(bits) := getlogicaldrives;
          for ct := 0 to 25 do
              if (ct in bits) then begin
                  drv := char (ct+ord('A'));
                  if not assigned(fonadd) or (askadd(drv+':\')) then begin
                     addnode(nil,node,drv+':\');
                     getkids(node);
                     getsubkids(node);
                  end;
              end;
          fallowchange := true;
          items.endupdate;
          topitem := selected;
          screen.cursor := oldcur;
     end;
end;

procedure tdirtree.createwnd ;
begin
     inherited;
     readnew;
end;

function tdirtree.CanExpand(Node: TTreeNode): Boolean ;
begin
     getsubkids(node);
     result := inherited CanExpand(Node);
end;

procedure tdirtree.Change(Node: TTreeNode);
var sr : string;
begin
     if not (csdestroying in componentstate) then
     if fallowchange then begin
        if node <> nil then
           if fnamen.count > integer(node.data) then begin
              sr := uppercase( copy(fnamen[integer(node.data)],2,maxint));
              factdirs[sr[1]] := sr;
           end;
        inherited change (node);
     end;
end;

procedure Tdirtree.FullExpand;
var oldcur : tcursor;
var
  Node: TTreeNode;
begin
     oldcur := screen.cursor;
     screen.cursor := crhourglass;
  Node := Items.GetFirstNode;
  while Node <> nil do
  begin
    Node.Expand(True);
    Node := Node.GetNextSibling;
  end;
     screen.cursor := oldcur;
end;

procedure tdirtree.setdrive (val : char);
var a:byte;
    sr : string;
begin
     if upcase(val) = upcase (getdirectory[1]) then exit;
     sr := factdirs[upcase(val)];
     if not directoryexists(sr) then sr := copy(sr,1,3);
     setdirectory(sr);
end;

function tdirtree.getdrive : char;
begin
     result := upcase(getdirectory[1]);
end;

end.
