{*******************************************************
                hgPassWd Unit

   This unit creates the ThgPassword component.
   ThgPassword encapsulates password.dll, the
   functionality of the password protection system.
   Source to the DLL is available for $5.00 Cdn (if I can
   e-mail it).

   After installing the component and setting your
   preferences put a call to AppLogin in your main
   forms OnCreate event. That's it.

   NOTES:
   1) If the Ini file specified as PassWrdFile
   doesn't exist it will be created when you specify
   the Section name.
   2) PASSWORD.DLL must be in your \windows\system
   directory or your App directory.
   3) UserName is a hook for future development.
   Specifically I want to log all users of a given
   App. But thats for a later version.
   4) The "look" is similar to the yrpaswrd component
   since that's where I got the idea but the code is
   all my own.

                Paul Warren
       HomeGrown Software Development
     (c) 1996 Langley British Columbia.
              (604) 530-9097
       e-mail:  hg_soft@uniserve.com
  Home page: http://haven.uniserve.com/~hg_soft

********************************************************}

unit Hgpasswd;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, PassLoad, IniFiles;

type
  EPassWrd = class(Exception);

  ThgPassword = class(TComponent)
  private
    { Private declarations }
    FPassWrdFile: string;
    FSection: string;
    FOkLogin: boolean;
    procedure SetPassWrdFile(NewPFile: string);
    procedure SetSection(NewSection: string);
    procedure SetOkLogin(Ok: boolean);
    function GetOkLogin: boolean;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    function AppLogin: boolean;
  published
    { Published declarations }
    property PassWrdFile: string read FPassWrdFile write SetPassWrdFile;
    property Section: string read FSection write SetSection;
    property OkLogin: boolean read GetOkLogin write SetOkLogin default false;
  end;

procedure Register;

implementation

const
  DefUser: string = 'Sysop';
  DefPassWrd: string = 'rsmampbkfh';

constructor ThgPassword.Create(Aowner: TComponent);
begin
  if not PassWrdLoaded then raise EPassWrd.Create('PASSWORD.DLL not loaded');
  inherited create(AOwner);
  FPassWrdFile := 'PASSWORD.INI';
  FSection := 'Users';
  FOkLogin := false;
end;

function ThgPassword.AppLogin: boolean;
begin
  Result := Login(UserName,FPassWrdFile,FSection);
end;

procedure ThgPassword.SetPassWrdFile(NewPFile: string);
begin
  FPassWrdFile := NewPFile;
end;

procedure ThgPassword.SetSection(NewSection: string);
var Ini: TIniFile;
begin
  FSection := NewSection;
  Ini := TIniFile.Create(FPassWrdFile);
  try
    Ini.WriteString(FSection,DefUser,DefPassWrd);
  finally
    Ini.Free;
  end;
end;

procedure ThgPassword.SetOkLogin(Ok: boolean);
begin
  if Ok then
  begin
    { UserName not implemented in this version - see
      comments in passload.pas }
    if Login(UserName,FPassWrdFile,FSection)then
    FOkLogin := true else FOkLogin := false;
  end else FOkLogin := Ok;
end;

function ThgPassword.GetOkLogin: boolean;
begin
  GetOkLogin := FOkLogin;
end;

procedure Register;
begin
  RegisterComponents('System', [ThgPassword]);
end;

end.
