unit Unit1;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus, Stdctrls;

type

  TSprite = class
  private
    FWidth: integer;
    FHeight: integer;
    FLeft: integer;
    FTop: integer;
    FAndImage, FOrImage: TBitMap;
  public
    property Top: Integer read FTop write FTop;
    property Left: Integer read FLeft write FLeft;
    property Width: Integer read FWidth write FWidth;
    property Height: Integer read FHeight write FHeight;
    constructor Create(AOwner: TComponent);
    destructor Done;
   end;

  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    BackGnd1, BackGnd2: TBitMap;
    Sprite: TSprite;
    GoLeft,GoRight,GoUp,GoDown: boolean;
    procedure MyIdleEvent(Sender: TObject; var Done: Boolean);
    procedure DrawSprite;
  end;

const
  BackGround = 'BACK.BMP'; 

var
  Form1: TForm1;
implementation

{$R *.DFM}

constructor TSprite.Create(AOwner: TComponent);
begin

  inherited Create;

  FAndImage := TBitMap.Create;
  FAndImage.LoadFromFile('AND.BMP');
  FOrImage := TBitMap.Create;
  FOrImage.LoadFromFile('OR.BMP');

  Left := 0;
  Top := 0;
  Height := FAndImage.Height;
  Width := FAndImage.Width;

end;

destructor TSprite.Done;
begin
  FAndImage.Free;
  FOrImage.Free;
end;


procedure TForm1.FormCreate(Sender: TObject);
begin
  BackGnd1 := TBitMap.Create;
  with BackGnd1 do
  begin
    LoadFromFile(BackGround);
    Parent := nil;
  end;

  BackGnd2 := TBitMap.Create;
  with BackGnd2 do
  begin
    LoadFromFile(BackGround);
    Parent := nil;
  end;

  Sprite := TSprite.Create(self);

  GoRight := true;
  GoDown := true;
  GoLeft := false;
  GoUp := false;

  Application.OnIdle := MyIdleEvent;
  ClientWidth := BackGnd1.Width;
  ClientHeight := BackGnd1.Height;
end;

procedure TForm1.MyIdleEvent(Sender: TObject; var Done: Boolean);
begin
  DrawSprite;
  Done := false;
end;

procedure TForm1.DrawSprite;
var
  OldOrigin: TPoint;
  TempRect: TRect;
begin

    With OldOrigin do
    begin
      X := Sprite.Left;
      Y := Sprite.Top;
    end;

    with Sprite do
    begin
      if GoLeft then
        if Left > 0 then
           Left := Left - 1
        else
        begin
           GoLeft := false;
           GoRight := true;
        end;

      if GoDown then
        if (Top + Height) < self.ClientHeight then
           Top := Top + 1
        else
        begin
          GoDown := false;
          GoUp := true;
        end;

      if GoUp then
        if  Top > 0 then
           Top := Top - 1
        else
        begin
          GoUp := false;
          GoDown := true;
        end;

      if GoRight then
      if (Left + Width) < self.ClientWidth then
        Left := Left + 1
      else
      begin
        GoRight := false;
        GoLeft := true;
      end;
  end;
  { Erase the old sprite in BackGnd2 }
  with OldOrigin do
    BitBlt(BackGnd2.Canvas.Handle, X, Y, Sprite.Width, Sprite.Height,
           BackGnd1.Canvas.Handle, X, Y, SrcCopy);

 { Draw the sprite at the new location in BackGnd2 }
  with Sprite do
  begin
    BitBlt(BackGnd2.Canvas.Handle, Left, Top, Width, Height,
           FANDImage.Canvas.Handle, 0, 0, SRCAND);
    BitBlt(BackGnd2.Canvas.Handle, Left, Top, Width, Height,
           FOrImage.Canvas.Handle, 0, 0, SRCPAINT);
  end;

  { Copy a rectangle from BackGnd2 to erase and reposition the sprite to the
    form's canvas }
  with OldOrigin do
    BitBlt(Canvas.Handle, X - 2, Y - 2, Sprite.Width + 2, Sprite.Height + 2,
           BackGnd2.Canvas.Handle, X - 2, Y - 2, SrcCopy);
end;

procedure TForm1.FormPaint(Sender: TObject);
begin
  BitBlt(Canvas.Handle, 0, 0, ClientWidth, ClientHeight,
         BackGnd1.Canvas.Handle, 0, 0, SrcCopy);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin

  BackGnd1.Free;
  BackGnd2.Free;
  Sprite.Free;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
begin
  DrawSprite;
end;


end.
