unit TTT2;
interface
uses SysUtils, Classes, Controls, StdCtrls, Dialogs, Magic;

Type
  TTTTControl2 = class(TWinControl)
                 protected
                   constructor Create(AOwner: TComponent); override;
                   destructor Destroy; override;
                   procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;

                 private { Magic DLL handle }
                   Game: HGame;
                   GameEnded: Boolean;
                   Button: Array[TPlace] of TButton;

                   procedure ButtonClick(Sender: TObject);
                   procedure ComputerMove;
                   procedure UserMove(Move: TPlace);
                 end {TTTTControl2};

  procedure Register;

implementation

  constructor TTTTControl2.Create(AOwner: TComponent);
  var ButtonIndex: TPlace;
  begin
    inherited Create(AOwner);
    Game := 0;
    if MagicLoaded then
    begin
      Game := NewGame;
      GameEnded := False;
      for ButtonIndex := Low(ButtonIndex) to High(ButtonIndex) do
      begin
        Button[ButtonIndex] := TButton.Create(Self);
        Button[ButtonIndex].Parent := Self;
        Button[ButtonIndex].Caption := '';
        Button[ButtonIndex].OnClick := ButtonClick;
      end;
      SetBounds(Left,Top,132,132);
      ComputerMove
    end
    else raise Exception.Create('Error loading MAGIC.DLL...')
  end {Create};

  destructor TTTTControl2.Destroy;
  var ButtonIndex: TPlace;
  begin
    if (Game > 0) then EndGame(Game);
    for ButtonIndex := Low(ButtonIndex) to High(ButtonIndex) do
      Button[ButtonIndex].Destroy;
    inherited Destroy
  end {Destroy};


  procedure TTTTControl2.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
  Const Grid = 3;
        GridX = 2;
        GridY = 2;
  var X,DX,W,Y,DY,H: Word;
  begin
    Inherited SetBounds(ALeft,ATop,AWidth,AHeight);
    X := GridX;
    DX := (Width div (Grid * (GridX+GridX))) * (GridX+GridX);
    W := DX - GridX;
    Y := GridY;
    DY := (Height div (Grid * (GridY+GridY))) * (GridY+GridY);
    H := DY - GridY;
    Button[8].SetBounds(X, Y, W,H);
    Button[1].SetBounds(X, Y+DY, W,H);
    Button[6].SetBounds(X, Y+DY+DY, W,H);
    Inc(X,DX);
    Button[3].SetBounds(X, Y, W,H);
    Button[5].SetBounds(X, Y+DY, W,H);
    Button[7].SetBounds(X, Y+DY+DY, W,H);
    Inc(X,DX);
    Button[4].SetBounds(X, Y, W,H);
    Button[9].SetBounds(X, Y+DY, W,H);
    Button[2].SetBounds(X, Y+DY+DY, W,H)
  end {SetBounds};


  procedure TTTTControl2.ButtonClick(Sender: TObject);
  var ButtonIndex: TPlace;
  begin
    for ButtonIndex := Low(ButtonIndex) to High(ButtonIndex) do
      if Button[ButtonIndex] = Sender as TButton then
        UserMove(ButtonIndex)
  end {ButtonClick};


  procedure TTTTControl2.ComputerMove;
  var Move: TMove;
  begin
    if IsWinner(Game) = NoneID then
    begin
      Move := NextMove(Game,CompID);
      if Move = 0 then
      begin
        GameEnded := True;
        MessageDlg('Neither has won, the game is a draw!', mtInformation, [mbOk], 0)
      end
      else
      begin
        MakeMove(Game,CompID,Move);
        Button[Move].Caption := 'X';
        if IsWinner(Game) = CompID then
          MessageDlg('I have won!', mtInformation, [mbOk], 0)
      end
    end
  end {ComputerMove};

  procedure TTTTControl2.UserMove(Move: TPlace);
  begin
    if IsWinner(Game) <> NoneID then
    begin
      if IsWinner(Game) = UserID then
        MessageDlg('You have already won!', mtInformation, [mbOk], 0)
      else
        MessageDlg('I have already won!', mtInformation, [mbOk], 0)
    end
    else
    begin
      if GameEnded then
        MessageDlg('The game already has ended!', mtInformation, [mbOk], 0)
      else
      begin
        if GetValue(Game, Move) <> NoneID then
          MessageDlg('This place is occupied!', mtWarning, [mbOk], 0)
        else
        begin
          Button[Move].Caption := '0';
          MakeMove(Game,UserID,Move);
          if IsWinner(Game) = UserID then
            MessageDlg('Congratulations, you have won!', mtInformation, [mbOk], 0)
          else
            ComputerMove
        end
      end
    end
  end {UserMove};


  procedure Register;
  begin
    RegisterComponents('Dr.Bob', [TTTTControl2])
  end {Register};
end.
