unit TTT3;
interface
uses SysUtils, Classes, Controls, StdCtrls, Dialogs, Magic;

Const
  UserChar = '0';
  CompChar = 'X';

Type
  TTTTControl3 = class(TWinControl)
                 protected
                   constructor Create(AOwner: TComponent); override;
                   destructor Destroy; override;
                   procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;

                 private { Magic DLL handle }
                   Game: HGame;
                   GameEnded: Boolean;

                 private { 9 game buttons }
                   Button: Array[TPlace] of TButton;
                   procedure ButtonClick(Sender: TObject);
                   procedure ComputerMove;
                   procedure UserMove(Move: TPlace);

                 private { start button }
                   StartButton: TButton;
                   procedure StartButtonClick(Sender: TObject);
                 end {TTTTControl3};

  procedure Register;

implementation

  constructor TTTTControl3.Create(AOwner: TComponent);
  var ButtonIndex: TPlace;
  begin
    inherited Create(AOwner);
    Game := 0;
    GameEnded := True;

    StartButton := TButton.Create(Self);
    StartButton.Parent := Self;
    StartButton.Visible := True;
    StartButton.Caption := 'Humor me...';
    StartButton.OnClick := StartButtonClick;

    for ButtonIndex := Low(ButtonIndex) to High(ButtonIndex) do
    begin
      Button[ButtonIndex] := TButton.Create(Self);
      Button[ButtonIndex].Parent := Self;
      Button[ButtonIndex].Caption := '';
      Button[ButtonIndex].Visible := False;
      Button[ButtonIndex].OnClick := ButtonClick;
    end;
    SetBounds(Left,Top,132,132)
  end {Create};

  destructor TTTTControl3.Destroy;
  var ButtonIndex: TPlace;
  begin
    if (Game > 0) then EndGame(Game);
    StartButton.Destroy;
    for ButtonIndex := Low(ButtonIndex) to High(ButtonIndex) do
      Button[ButtonIndex].Destroy;
    inherited Destroy
  end {Destroy};


  procedure TTTTControl3.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
  Const Grid = 3;
        GridX = 2;
        GridY = 2;
  var X,DX,W,Y,DY,H: Word;
  begin
    Inherited SetBounds(ALeft,ATop,AWidth,AHeight);
    StartButton.SetBounds(0,0,Width,Height);
    X := GridX;
    DX := (Width div (Grid * (GridX+GridX))) * (GridX+GridX);
    W := DX - GridX;
    Y := GridY;
    DY := (Height div (Grid * (GridY+GridY))) * (GridY+GridY);
    H := DY - GridY;
    Button[8].SetBounds(X, Y, W,H);
    Button[1].SetBounds(X, Y+DY, W,H);
    Button[6].SetBounds(X, Y+DY+DY, W,H);
    Inc(X,DX);
    Button[3].SetBounds(X, Y, W,H);
    Button[5].SetBounds(X, Y+DY, W,H);
    Button[7].SetBounds(X, Y+DY+DY, W,H);
    Inc(X,DX);
    Button[4].SetBounds(X, Y, W,H);
    Button[9].SetBounds(X, Y+DY, W,H);
    Button[2].SetBounds(X, Y+DY+DY, W,H)
  end {SetBounds};


  procedure TTTTControl3.StartButtonClick(Sender: TObject);
  var ButtonIndex: TPlace;
  begin
    if MagicLoaded then
    begin
      Game := NewGame;
      GameEnded := False;
      StartButton.Visible := False;
      for ButtonIndex := Low(ButtonIndex) to High(ButtonIndex) do
        Button[ButtonIndex].Visible := True;
      ComputerMove
    end
    else
      MessageDlg('Error loading MAGIC.DLL...', mtInformation, [mbOk], 0)
  end {ButtonClick};


  procedure TTTTControl3.ButtonClick(Sender: TObject);
  var ButtonIndex: TPlace;
  begin
    for ButtonIndex := Low(ButtonIndex) to High(ButtonIndex) do
      if Button[ButtonIndex] = Sender as TButton then
        UserMove(ButtonIndex)
  end {ButtonClick};


  procedure TTTTControl3.ComputerMove;
  var Move: TMove;
  begin
    if IsWinner(Game) = NoneID then
    begin
      Move := NextMove(Game,CompID);
      if Move = 0 then
      begin
        GameEnded := True;
        MessageDlg('Neither has won, the game is a draw!', mtInformation, [mbOk], 0)
      end
      else
      begin
        MakeMove(Game,CompID,Move);
        Button[Move].Caption := CompChar;
        if IsWinner(Game) = CompID then
          MessageDlg('I have won!', mtInformation, [mbOk], 0)
      end
    end
  end {ComputerMove};

  procedure TTTTControl3.UserMove(Move: TPlace);
  begin
    if IsWinner(Game) <> NoneID then
    begin
      if IsWinner(Game) = UserID then
        MessageDlg('You have already won!', mtInformation, [mbOk], 0)
      else
        MessageDlg('I have already won!', mtInformation, [mbOk], 0)
    end
    else
    begin
      if GameEnded then
        MessageDlg('The game already has ended!', mtInformation, [mbOk], 0)
      else
      begin
        if GetValue(Game, Move) <> NoneID then
          MessageDlg('This place is occupied!', mtWarning, [mbOk], 0)
        else
        begin
          Button[Move].Caption := UserChar;
          MakeMove(Game,UserID,Move);
          if IsWinner(Game) = UserID then
            MessageDlg('Congratulations, you have won!', mtInformation, [mbOk], 0)
          else
            ComputerMove
        end
      end
    end
  end {UserMove};


  procedure Register;
  begin
    RegisterComponents('Dr.Bob', [TTTTControl3])
  end {Register};
end.
