unit Unit1;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ShellAPI, StdCtrls, ExtCtrls, Spin, Gauges, Buttons;

const
     MaxFilenameLength = 255; {Used by wm_Dropfiles to set buffer length}

type
  TForm1 = class(TForm)
    Label3: TLabel;
    Label4: TLabel;
    OpenDialog1: TOpenDialog;
    Button3: TButton;
    ColorDialog1: TColorDialog;
    GroupBox1: TGroupBox;
    CheckBox3: TCheckBox;
    Gauge1: TGauge;
    GroupBox2: TGroupBox;
    Label7: TLabel;
    Label6: TLabel;
    Button2: TButton;
    Label5: TLabel;
    ListBox2: TListBox;
    GroupBox3: TGroupBox;
    Label1: TLabel;
    ListBox1: TListBox;
    GroupBox4: TGroupBox;
    Button5: TButton;
    CheckBox2: TCheckBox;
    GroupBox5: TGroupBox;
    CheckBox1: TCheckBox;
    Button4: TButton;
    SpinEdit1: TSpinEdit;
    Label8: TLabel;
    GroupBox6: TGroupBox;
    Label2: TLabel;
    Edit1: TEdit;
    Button1: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Edit1Change(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure SpinEdit1Change(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure CheckBox2Click(Sender: TObject);
    procedure CheckBox3Click(Sender: TObject);
  private
    { Private declarations }
     WastingTime: Boolean;
     procedure AppOnMessage(var Msg : TMsg; var Handled : Boolean);
     procedure WMDropFiles (var Msg: TWMDropFiles); message wm_dropfiles;
  public
    { Public declarations }
    FileList: TStringList;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormCreate(Sender: TObject);
begin
{Ensure Window opens itself in state set by CmdShow}
case CmdShow of
          sw_ShowMinimized,
          sw_ShowMinNoActive     : WindowState := wsMinimized;

          sw_ShowMaximized       : WindowState := wsMaximized

else      WindowState := wsNormal
end;

    {Enable both the main form _and_ the TApplication Window to
     accept dragged files}
    DragAcceptFiles(Handle, True);
    DragAcceptFiles(Application.Handle, True);

    {Assign special event handler to OnMessage event}
    Application.OnMessage := AppOnMessage;

    {Create Filelist here so it is available throughout the
     application}
    FileList := TStringList.Create;

    {Set caption with executable's command line}
    Label4.Caption := Application.Exename;

    {Set Icon Caption}
    Application.Title := Edit1.Text;

    {Set intial value}
    Application.ShowHint := False;
end;

procedure TForm1.WMDropFiles(var Msg: TWMDropFiles);
       var
         Counter, NumFiles : LongInt;
         Temp : array[0..MaxFilenameLength] of Char;
       begin
         {With $FFFF (-1) set as second parameter, DragQueryFiles returns the total
          number of files dragged to the form.  Note that Delphi has placed
          the message information into a more understandable structure, TWMDropFiles.
          See 'Message-record Types' in the online help for a complete list
          of these event specific message structures.}
         NumFiles := DragQueryFile(Msg.Drop, $FFFF, Temp, MaxFilenameLength);
         for Counter := 0 to NumFiles - 1 do
           begin
             {Now DragQueryFile will return a null-terminated string
              that has each filename}
             DragQueryFile(Msg.Drop, Counter, Temp, MaxFilenameLength);
             FileList.Add(StrPas(Temp));
           end;
         DragFinish(Msg.Drop);
         Listbox1.Items := FileList;
       end;

procedure TForm1.AppOnMessage(var Msg : TMsg; var Handled : Boolean);
var
         Counter, NumFiles : LongInt;
         Temp : array[0..MaxFilenameLength] of Char;
begin
     {Perform the same function as above for the Application Window, but
      only when it is iconized}
     if (Msg.Message = WM_DropFiles) and IsIconic(Application.Handle) then
     begin
         NumFiles := DragQueryFile(Msg.WParam, $FFFF, Temp, MaxFilenameLength);
         for Counter := 0 to NumFiles - 1 do
           begin
             DragQueryFile(Msg.WParam, Counter, Temp, MaxFilenameLength);
             FileList.Add(StrPas(Temp));
           end;
         DragFinish(Msg.WParam);
         Listbox1.Items := FileList;
         Handled := True;
     end;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
     {Always destroy what you create (unless is has an owner)!}
     FileList.Free;
end;

procedure TForm1.Edit1Change(Sender: TObject);
begin
     {Set the Icon Caption to the users selected text}
     Application.Title := Edit1.Text;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
   Icon: TIcon;
begin
     {Get an icon and load it into the Application.  The new icon
     will now show up when the application is minimized}
     Icon := TIcon.Create;
     if OpenDialog1.Execute then
     begin
        Icon.LoadFromFile(OpenDialog1.Filename);
        Application.Icon := Icon;
     end;
     Icon.Free;
end;

procedure TForm1.Button2Click(Sender: TObject);
var
  I: Integer;
begin
  ListBox2.Clear;
  for I := 0 to ComponentCount - 1 do
  begin
    Listbox2.Items.Add(Components[I].Name);
    if Components[I] is TLabel then {Use Run-time typing to check type}
    begin
         {Toggle the text of only TLabels between Red and Black.
         {Note that each component must be typecast first.}
         if TLabel(Components[I]).Font.Color = clBlack then
            TLabel(Components[I]).Font.Color := clRed
         else TLabel(Components[I]).Font.Color := clBlack;
    end
 end;
  Label7.Caption := IntToStr(ComponentCount);
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
     Close;
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
     if ColorDialog1.Execute then Application.HintColor := ColorDialog1.Color;
end;

procedure TForm1.Button5Click(Sender: TObject);
begin
     Application.NormalizeTopMosts;{Allow dialogs on top}
     MessageBox(Form1.Handle, 'This should be on top.', 'Message Box', MB_OK);
     Application.RestoreTopMosts; {Return to normal}
end;

procedure TForm1.CheckBox1Click(Sender: TObject);
begin
     {Toggle Hint state}
     Application.ShowHint := not Application.ShowHint;
end;

procedure TForm1.CheckBox2Click(Sender: TObject);
begin
     {Toggle fsStayOnTop state}
     if FormStyle = fsStayOnTop then FormStyle := fsNormal
        else FormStyle := fsStayOnTop;
end;

procedure TForm1.CheckBox3Click(Sender: TObject);
var
   Increment: Integer;
begin
     Increment := 1;
     repeat {Waste time, but allow processing of Windows messages}
           Gauge1.Progress := Gauge1.Progress + Increment;
           if Gauge1.Progress = Gauge1.MaxValue then Increment := -1;
           if Gauge1.Progress = Gauge1.MinValue then Increment := 1;
           Application.ProcessMessages;
     until (not CheckBox3.Checked) or (Application.Terminated);
     Gauge1.Progress := 0;
end;

procedure TForm1.SpinEdit1Change(Sender: TObject);
begin
     Application.HintPause := SpinEdit1.Value;
end;

end.
