unit NfyUtil;

interface

uses
  WinTypes;

var
  ProblemTask: THandle;

function DoString(Str: Longint): String;

function DoLogError(Block: Longint): String;

function DoLogParamError(Block: Longint): String;

function DebuggerRunning: Boolean;

implementation

uses
  ToolHelp, SysUtils, WinProcs;

var
  TaskEntry: TTaskEntry;
  ModuleEntry: TModuleEntry;

function DoString(Str: Longint): String;
var
  Loop: Byte;
begin
  Result := StrPas(PChar(Str));
  if Length(Result) > 0 then
    for Loop := Length(Result) downto 1 do
      if Result[Loop] in [#10, #13] then
        Delete(Result, Loop, 1);
end;

function DoLogError(Block: Longint): String;
begin
  with PNfyLogError(Block)^ do
  begin
    case wErrCode of
    { KERNEL errors }
      err_GAlloc: Result := 'GlobalAlloc failed';
      err_GRealloc: Result := 'GlobalRealloc failed';
      err_GLock: Result := 'GlobalLock failed';
      err_LAlloc: Result := 'LocalAlloc failed';
      err_LRealloc: Result := 'LocalRealloc failed';
      err_LLock: Result := 'LocalLock failed';
      err_Allocres: Result := 'AllocResource failed';
      err_LockRes: Result := 'LockResource failed';
      err_LoadModule: Result := 'LoadModule failed';

    { USER errors }
      err_CreateDlg: Result := 'Couldn''t create dialog as LoadMenu failed';
      err_CreateDlg2: Result := 'Couldn''t create dialog as CreateWindow failed';
      err_RegisterClass: Result := 'RegisterClass failed as class is already registered';
      err_DCBusy: Result := 'Device context cache is full';
      err_CreateWnd: Result := 'Couldn''t create window as class was not found';
      err_StrucExtra: Result := 'Program is using unallocated space';
      err_LoadStr: Result := 'LoadString failed';
      err_LoadMenu: Result := 'LoadMenu failed';
      err_NestedBeginPaint: Result := 'Program has nested BeginPaints';
      err_BadIndex: Result := 'Bad index to G/SetClassLong, G/SetClassWord, G/SetWindowLong or G/SetWindowWord';
      err_CreateMenu: Result := 'Couldn''t create menu';

    { GDI errors }
      err_CreateDC: Result := 'CreateDC or CreateCompatibleDC or CreateIC] failed';
      err_CreateMeta: Result := 'CreateMetafile failed';
      err_DelObjSelected: Result := 'Program is trying to delete a bitmap selected in a DC';
      err_SelBitmap: Result := 'Program is trying to select an already selected bitmap';
    else
      Result := 'Unknown error';
    end;
    TaskEntry.dwSize := SizeOf(TaskEntry);
    TaskFindHandle(@TaskEntry, ProblemTask);
    ModuleEntry.dwSize := SizeOf(ModuleEntry);
    ModuleFindHandle(@ModuleEntry, TaskEntry.hModule);
    Result := Format('Error: %s in %s', [Result,
       ExtractFileName(StrPas(ModuleEntry.szExePath))]);
  end;
end;

function DoLogParamError(Block: Longint): String;
const
  MsgType: array[False..True] of String[7] = ('error', 'warning');
var
  BadParam: Longint;
begin
  with PNfyLogParamError(Block)^ do
  begin
    case wErrCode of
    { Generic parameter values }
      err_Bad_Value: Result := 'invalid 16-bit paramater';
      err_Bad_Flags: Result := 'invalid bit flags';
      err_Bad_Index: Result := 'invalid or out of range index';
      err_Bad_DValue: Result := 'invalid 32-bit value';
      err_Bad_DFlags: Result := 'invalid 32-bit flags';
      err_Bad_DIndex: Result := 'invalid or out of range 32-bit index';
      err_Bad_Ptr: Result := 'invalid pointer';
      err_Bad_Func_Ptr: Result := 'invalid function pointer';
      err_Bad_Selector: Result := 'invalid selector';
      err_Bad_String_ptr: Result := 'invalid PChar';
      err_Bad_Handle: Result := 'invalid generic handle';

    { KERNEL parameter errors }
      err_Bad_HInstance: Result := 'invalid instance handle';
      err_Bad_HModule: Result := 'invalid module handle';
      err_Bad_Global_Handle: Result := 'invalid global handle';
      err_Bad_Local_Handle: Result := 'invalid local handle';
      err_Bad_Atom: Result := 'invalid atom';
      err_Bad_Hfile: Result := 'invalid file handle';

    { USER parameter errors }
      err_Bad_HWnd: Result := 'invalid window handle';
      err_Bad_HMenu: Result := 'invalid menu handle';
      err_Bad_HCursor: Result := 'invalid cursor handle';
      err_Bad_HIcon: Result := 'invalid icon handle';
      err_Bad_HDWP: Result := 'invalid window-position structure handle';
      err_Bad_CID: Result := 'invalid comms id';
      err_Bad_HDrvr: Result := 'invalid driver handle';

    { GDI parameter errors }
      err_Bad_COORDS: Result := 'invalid x, y coordinates';
      err_Bad_GDI_OBJECT: Result := 'invalid GDI object';
      err_Bad_HDC: Result := 'invalid device context handle';
      err_Bad_HPEN: Result := 'invalid pen handle';
      err_Bad_HFONT: Result := 'invalid font handle';
      err_Bad_HBRUSH: Result := 'invalid brush handle';
      err_Bad_HBITMAP: Result := 'invalid bitmap handle';
      err_Bad_HRGN: Result := 'invalid region handle';
      err_Bad_HPALETTE: Result := 'invalid palette handle';
      err_Bad_HMETAFILE: Result := 'invalid metafile handle';
    else
      Result := 'Unknown';
    end;
    case (wErrCode and err_Size_Mask) of
      err_Byte: BadParam := Byte(lpBadParam);
      err_Word: BadParam := Word(lpBadParam);
      err_DWord: BadParam := Longint(lpBadParam);
    end;
    TaskEntry.dwSize := SizeOf(TaskEntry);
    TaskFindHandle(@TaskEntry, ProblemTask);
    ModuleEntry.dwSize := SizeOf(ModuleEntry);
    ModuleFindHandle(@ModuleEntry, TaskEntry.hModule);
    Result := Format('Parameter %s: %s $%x in %s',
      [MsgType[wErrCode and err_Warning <> 0], Result, BadParam,
       ExtractFileName(StrPas(ModuleEntry.szExePath))]);
  end;
end;

type
  PDebugRec = ^TDebugRec;
  TDebugRec = record
    dhMagic1,
    dhZero,
    dhMagic2,
    dhHookProc,
    dhDebugHooked: Longint;
    dhKind: Word; { Use TExceptionKind enumerated type above }
    dhAddr,
    dhCookie,
    dhNameLen,
    dhName,
    dhMsgLen,
    dhMsg,
    dhWantException,
    dhDoneExcept: Longint;
  end;

const
  DebuggerHook  = $24; { Offset in DS of pointer to debugger data }

function DebuggerRunning: Boolean;
begin
  Result := (PrefixSeg <> 0) and
    (LoWord(PDebugRec(Ptr(DSeg, DebuggerHook)^)^.dhDebugHooked) <> 0);
end;

end.
