unit Errhand;

interface

procedure ErrorHandler(ErrorCode: Integer; ErrorAddr: Pointer);

implementation

uses SysUtils;

type
  { Custom exception class }
  ERuntimeError = class(Exception);

const
  { Internal values used to represent run-time errors }
  { Values identified from \DELPHI\SOURCE\RTL\SYS\SE.ASM }
  reInOutError = 0;
  reOutOfMemory = 1;
  reInvalidPtr = 2;
  reDivByZero = 3;
  reRangeError = 4;
  reIntOverflow = 5;
  reInvalidOp = 6;
  reZeroDivide = 7;
  reOverflow = 8;
  reUnderflow = 9;
  reInvalidCast = 10;

  { References into your EXEs string }
  { table for ready-made error strings }
  { Values taken from \DELPHI\SOURCE\RTL\SYS\SYSUTILS.INC }
  SInOutError = 65416;
  SFileNotFound = 65417;
  SInvalidFilename = 65418;
  STooManyOpenFiles = 65419;
  SAccessDenied = 65420;
  SInvalidDrive = 65421;
  SEndOfFile = 65422;
  SDiskFull = 65423;
  SInvalidInput = 65424;
  SDivByZero = 65424;
  SRangeError = 65425;
  SIntOverflow = 65426;
  SInvalidOp = 65427;
  SZeroDivide = 65428;
  SOverflow = 65429;
  SUnderflow = 65430;
  SInvalidPointer = 65431;
  SInvalidCast = 65432;

{ Build an appropriate exception by looking at InOutRes }
function CreateInOutError: ERuntimeError;
type
  TErrorRec = record
    Code: Integer;
    Ident: Word;
  end;
const
  { Lookup table for I.O error codes }
  ErrorMap: array[0..7] of TErrorRec = (
    (Code: 2;   Ident: SFileNotFound),
    (Code: 3;   Ident: SInvalidFilename),
    (Code: 4;   Ident: STooManyOpenFiles),
    (Code: 5;   Ident: SAccessDenied),
    (Code: 15;  Ident: SInvalidDrive),
    (Code: 100; Ident: SEndOfFile),
    (Code: 101; Ident: SDiskFull),
    (Code: 106; Ident: SInvalidInput));
var
  I: Integer;
begin
  I := Low(ErrorMap);
  while (I <= High(ErrorMap)) and (ErrorMap[I].Code <> InOutRes) do Inc(I);
  if I <= High(ErrorMap) then
    Result := ERuntimeError.CreateRes(ErrorMap[I].Ident)
  else
    Result := ERuntimeError.CreateResFmt(SInOutError, [InOutRes]);
  InOutRes := 0;
end;

{ This is the replacement error handler }
procedure ErrorHandler(ErrorCode: Integer; ErrorAddr: Pointer);
const
  { These values are string table entries as made by SysUtils }
  ExceptMap: array[reInvalidPtr..reInvalidCast] of Word = (
    SInvalidPointer, SDivByZero,  SRangeError, SIntOverflow,
    SInvalidOp,      SZeroDivide, SOverflow,   SUnderflow,
    SInvalidCast);
var
  E: ERuntimeError;
begin
  { Find out what type of software runtime error occurred }
  case ErrorCode of
    { Special case for I/O errors - there are lots of them }
    reInOutError: E := CreateInOutError;
    { Special case for low memory - there is an already allocated }
    { exception waiting to be raised by this procedure }
    reOutOfMemory: OutOfMemoryError;
  else
    { Create an exception, with a string based on the error }
    E := ERuntimeError.CreateRes(ExceptMap[ErrorCode]);
  end;
  raise E at ErrorAddr;
end;

end.
