unit Joystick;
interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, MMSystem, IniFiles;

type
  TReturnLevel = (Level1,Level2,Level3);

  {This is an undocumented structure so we must declare it.}
  TJoyCalibrate = record
    XBase      : Word;
    XDelta     : Word;
    YBase      : Word;
    YDelta     : Word;
    ZBase      : Word;
    ZDelta     : Word;
  end; {TJoyCalibrate}

  TJoyStick = class(TTimer)
  private { Private declarations }
    FJoyInfo1       : TJoyInfo;
    FJoyCaps1       : TJoyCaps;
    FJoyCalibrate   : TJoyCalibrate;
    FJoy1B1         : Word;
    FJoy1B2         : Word;
    FNumOfJoySticks : Word;
    FJoystick1Ready : Boolean;
    FReturnLevel    : TReturnLevel;

  protected { Protected declarations }

  public { Public declarations }
    constructor Create(AOwner : TComponent); override;
    property    NumOfJoySticks : Word     read FNumOfJoySticks; {read only}
    property    Joy1X          : Word     read FJoyInfo1.wXPos;
    property    Joy1Y          : Word     read FJoyInfo1.wYPos;
    property    Joy1Z          : Word     read FJoyInfo1.wZPos;
    property    Joy1B1         : Word     read FJoy1B1;
    property    Joy1B2         : Word     read FJoy1B2;
    function    InitJoy        : Boolean;
    procedure   ProcessJoystickInfo;
    procedure   CalibrateJoyStick;

  published { Published declarations }
    property    ReturnLevel : TReturnLevel read FReturnLevel write FReturnLevel;

  end; {TJoyStick}

{undocumented use}
function joySetCalibration(JoyID : Word; XBase : PWord; XDelta : PWord;
         YBase : PWord; YDelta : PWord; ZBase : PWord; ZDelta : PWord): Word;

procedure Register;

implementation

{undocumented use}
function joySetCalibration(JoyID : Word; XBase : PWord; XDelta : PWord;
         YBase : PWord; YDelta : PWord; ZBase : PWord; ZDelta : PWord): Word;
         external 'MMSYSTEM';

procedure TJoystick.ProcessJoystickInfo;
begin {process info for 1 joystick}
  case FReturnLevel of
    Level1 : begin {raw data from Joystick}
      joyGetPos(JOYSTICKID1, @FJoyInfo1);
      FJoy1B1 :=  FJoyInfo1.wButtons and 1;
      FJoy1B2 := (FJoyInfo1.wButtons and 2) shr 1;
    end; {Level1}

    Level2 : begin {data SHR 11, values 0-31}
      joyGetPos(JOYSTICKID1, @FJoyInfo1);
      FJoyInfo1.wXPos := FJoyInfo1.wXPos SHR 11;
      FJoyInfo1.wYPos := FJoyInfo1.wYPos SHR 11;
      FJoyInfo1.wZPos := FJoyInfo1.wZPos SHR 11;
      FJoy1B1 :=  FJoyInfo1.wButtons and 1;
      FJoy1B2 := (FJoyInfo1.wButtons and 2) shr 1;
    end; {Level2}

    Level3 : begin {data SHR 13, values 0-7}
      joyGetPos(JOYSTICKID1, @FJoyInfo1);
      FJoyInfo1.wXPos := FJoyInfo1.wXPos SHR 13;
      FJoyInfo1.wYPos := FJoyInfo1.wYPos SHR 13;
      FJoyInfo1.wZPos := FJoyInfo1.wZPos SHR 13;
      FJoy1B1 :=  FJoyInfo1.wButtons and 1;
      FJoy1B2 := (FJoyInfo1.wButtons and 2) shr 1;
    end; {Level3}
  end; {case}

end; {TJoystick.ProcessJoystickInfo}

function TJoystick.InitJoy : Boolean;
begin
  FNumOfJoysticks := joyGetNumDevs;
  if FNumOfJoysticks in [1,2] then {system can support 1 or 2 joysticks}
    begin
      Result := True;
      joyGetDevCaps(JOYSTICKID1, @FJoyCaps1, SizeOf(FJoyCaps1));
      if joyGetPos(JOYSTICKID1, @FJoyInfo1) = JOYERR_NOERROR then
        FJoystick1Ready := True {joystick is plugged in and ready}
      else
        FJoystick1Ready := False; {driver present, no joystick plugged in}
    end {if}
  else
    Result := False; {no joystick driver is present}
end; {TJoystick.InitJoy}

constructor TJoyStick.Create(AOwner : TComponent);
begin
  inherited Create(AOwner);
  Enabled := False; {override default setting of True}
  FReturnLevel := Level1; {raw values from Joystick}
end; {TJoyStick.Create}

{No error checking is performed, since the values are coming from SYSTEM.INI
 and not a user. The string passed in will have a length of 4. It would be easy
 to add necessary error checking if needed}
function HexStrToInt(var HexStr : String) : Word;
var
  X       : ShortInt;
  TempInt : Word;
  Total   : Word;

begin
  HexStr := AnsiLowerCase(HexStr); {convert to lower case}
  Total := 0;
  for X :=  1 to Length(HexStr) do
    begin
      if (HexStr[X] >= 'a') then { value is between a-f }
        TempInt := Ord(HexStr[X]) - Ord('a') + 10
      else { value is between 0-9 }
        TempInt := Ord(HexStr[X]) - Ord('0');
      Total := (Total shl 4) + TempInt;
    end; {for}
    Result := Total;
end; {HexStrToInt}

procedure TJoyStick.CalibrateJoystick;
var
  Joy1ConfigStr : String[29];
  Joy2ConfigStr : String[29];
  SysIni : TIniFile;
  HexStr : String[4];

begin
  SysIni := TIniFile.Create('SYSTEM.INI');
  Joy1ConfigStr := SysIni.ReadString('joystick.drv', 'JoyCal0', '');
  Joy2ConfigStr := SysIni.ReadString('joystick.drv', 'JoyCal1', '');
  Move(Joy1ConfigStr[1], HexStr[1], 4);
  HexStr[0] := Chr(4);
  FJoyCalibrate.XBase := HexStrToInt(HexStr);
  Move(Joy1ConfigStr[6], HexStr[1], 4);
  HexStr[0] := Chr(4);
  FJoyCalibrate.XDelta := HexStrToInt(HexStr);
  Move(Joy1ConfigStr[11], HexStr[1], 4);
  HexStr[0] := Chr(4);
  FJoyCalibrate.YBase := HexStrToInt(HexStr);
  Move(Joy1ConfigStr[16], HexStr[1], 4);
  HexStr[0] := Chr(4);
  FJoyCalibrate.YDelta := HexStrToInt(HexStr);
  Move(Joy1ConfigStr[21], HexStr[1], 4);
  HexStr[0] := Chr(4);
  FJoyCalibrate.ZBase := HexStrToInt(HexStr);
  Move(Joy1ConfigStr[26], HexStr[1], 4);
  HexStr[0] := Chr(4);
  FJoyCalibrate.ZDelta := HexStrToInt(HexStr);
  joySetCalibration(0, @FJoyCalibrate.XBase, @FJoyCalibrate.XDelta,
                       @FJoyCalibrate.YBase, @FJoyCalibrate.YDelta,
                       @FJoyCalibrate.ZBase, @FJoyCalibrate.ZDelta);
  Move(Joy2ConfigStr[1], HexStr[1], 4);
  HexStr[0] := Chr(4);
  FJoyCalibrate.XBase := HexStrToInt(HexStr);
  Move(Joy2ConfigStr[6], HexStr[1], 4);
  HexStr[0] := Chr(4);
  FJoyCalibrate.XDelta := HexStrToInt(HexStr);
  Move(Joy2ConfigStr[11], HexStr[1], 4);
  HexStr[0] := Chr(4);
  FJoyCalibrate.YBase := HexStrToInt(HexStr);
  Move(Joy2ConfigStr[16], HexStr[1], 4);
  HexStr[0] := Chr(4);
  FJoyCalibrate.YDelta := HexStrToInt(HexStr);
  Move(Joy2ConfigStr[21], HexStr[1], 4);
  HexStr[0] := Chr(4);
  FJoyCalibrate.ZBase := HexStrToInt(HexStr);
  Move(Joy2ConfigStr[26], HexStr[1], 4);
  HexStr[0] := Chr(4);
  FJoyCalibrate.ZDelta := HexStrToInt(HexStr);
  joySetCalibration(1, @FJoyCalibrate.XBase, @FJoyCalibrate.XDelta,
                       @FJoyCalibrate.YBase, @FJoyCalibrate.YDelta,
                       @FJoyCalibrate.ZBase, @FJoyCalibrate.ZDelta);
end; {TJoyStick.CalibrateJoystick}

procedure Register;
begin
  RegisterComponents('Develop', [TJoyStick]);
end;

end. {Joystick}
