unit Main;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, Joystick, MMSystem;

type
  TfrmMain = class(TForm)
    JoyStick1: TJoyStick;
    procedure JoyStick1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    FPoint     : TPoint; {stores the cursor position}
    FWavFile   : THandle; {handle to a memory location for the wave file}
    FWavPtr    : Pointer; {pointer to the wave file in memory}
  public
    { Public declarations }
  end;

var
  frmMain: TfrmMain;

implementation
{$R *.DFM}

procedure TfrmMain.JoyStick1Timer(Sender: TObject);
begin
  Joystick1.ProcessJoystickInfo; {poll the joystick}
  GetCursorPos(FPoint); {get the cursors current position}

  if Joystick1.Joy1X < 2 then {test the directional info from the joystick}
    FPoint.X := FPoint.X - 10 {and set the new cursor coordinates}
  else if Joystick1.Joy1X > 5 then
    FPoint.X := FPoint.X + 10;

  if Joystick1.Joy1Y < 2 then
    FPoint.Y := FPoint.Y - 10
  else if Joystick1.Joy1Y > 5 then
    FPoint.Y := FPoint.Y + 10;

  SetCursorPos(FPoint.X, FPoint.Y); {place the cursor at the new coordinates}

  if Joystick1.Joy1B1 = 1 then {if button is pressed, change the cursor}
    begin
      if Screen.Cursor = crMultiDrag then
        Screen.Cursor := crDefault
      else
        Screen.Cursor :=  Pred(Screen.Cursor)
    end;

  if Joystick1.Joy1B2 = 1 then {if button is pressed, play the wave file}
    sndPlaySound(FWavPtr, SND_ASYNC or SND_MEMORY);
end;

procedure TfrmMain.FormCreate(Sender: TObject);
var
  DingWav    : PChar;
  WinDirSize : Integer;
  WavSize    : LongInt;
  WavPath    : array[0..144] of Char;
  WavHandle  : THandle;

begin
  DingWav := '\ding.wav'; {the wave file we want to play}
  FillChar(WavPath, SizeOf(WavPath), #0); {init the buffer}
  if Joystick1.InitJoy then {attemp to initialize the joystick}
    begin
      Joystick1.CalibrateJoystick; {calibrate the joystick}
      JoyStick1.Enabled := True; {enable the joystick component}
    end;
  WinDirSize := GetWindowsDirectory(WavPath, SizeOf(WavPath)); {find Win dir}
  Move(DingWav[0], WavPath[WinDirSize], StrLen(DingWav)); {append the wave file}
  WavHandle := _lopen(WavPath, OF_READ); {open the file}
  WavSize := _llseek(WavHandle, 0, 2); {determine the file size}
  _llseek(WavHandle, 0, 0); {reset the file to the begining}
  FWavFile := GlobalAlloc( GMEM_MOVEABLE + GMEM_SHARE, WavSize); {alloc memory}
  FWavPtr :=  GlobalLock(FWavFile); {get a pointer to the allocated memory}
  _lread(WavHandle, FWavPtr, WavSize); {copy wave file to the memory location}
  _lclose(WavHandle); {close the file}
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  GlobalUnlock(FWavFile); {remove the lock on the memory}
  GlobalFree(FWavFile); {free the memory}
end;

end.
