unit XLAutom;

interface

uses
  SysUtils, OleAuto;

type
  TExcelFont=class(TOleObject)
  private
    function GetBold: Integer;
    procedure SetBold(iBold: Integer);
    function GetItalic: Integer;
    procedure SetItalic(iItalic: Integer);
    function GetSize: Integer;
    procedure SetSize(iSize: Integer);
  public
    property Bold: Integer read GetBold write SetBold;
    property Italic: Integer read GetItalic write SetItalic;
    property Size: Integer read GetSize write SetSize;
  end;

  TExcelRange=class(TOleObject)
  private
    FFont: TExcelFont;
    function GetFont: TExcelFont;
    function GetValue: String;
    procedure SetValue(sName: String);
  public
    property Font: TExcelFont read GetFont write FFont;
    property Value: String read GetValue write SetValue;
  end;

  TExcelWorkSheet=class(TOleObject)
  public
    function Cells(rowIndex, columnIndex: Integer): TExcelRange;
  end;

  TExcelWorkBook=class(TOleObject)
  public
    procedure Save;
  end;

  TExcelWorkBooks=class(TOleObject)
    procedure FileOpen(fileName: String);
  end;

  TExcelObj=class(TOleObject)
  private
    FWorkBooks: TExcelWorkBooks;
    FActiveWorkBook: TExcelWorkBook;
    FActiveWorkSheet: TExcelWorkSheet;
    function GetVisible: Integer;
    procedure SetVisible(iVisible: Integer);
    function GetWorkBooks: TExcelWorkBooks;
    function GetActiveWorkBook: TExcelWorkBook;
    function GetActiveWorkSheet: TExcelWorkSheet;
  public
    property Visible: Integer read GetVisible write SetVisible;
    property WorkBooks: TExcelWorkBooks read GetWorkBooks write FWorkBooks;
    property ActiveWorkBook: TExcelWorkBook read GetActiveWorkBook write FActiveWorkBook;
    property ActiveWorkSheet: TExcelWorkSheet read GetActiveWorkSheet write   FActiveWorkSheet;
    procedure Quit;
  end;

implementation

function TExcelFont.GetBold: Integer;
var
  iBold: Integer;
begin
  GetOleProperty('Bold', 'Integer', iBold);
  Result := iBold;
end;

procedure TExcelFont.SetBold(iBold: Integer);
begin
  SetOleProperty('Bold', 'Integer', iBold);
end;

function TExcelFont.GetItalic: Integer;
var
  iItalic: Integer;
begin
  GetOleProperty('Italic', 'Integer', iItalic);
  Result := iItalic;
end;

procedure TExcelFont.SetItalic(iItalic: Integer);
begin
  SetOleProperty('Italic', 'Integer', iItalic);
end;

function TExcelFont.GetSize: Integer;
var
  iSize: Integer;
begin
  GetOleProperty('Size', 'Integer', iSize);
  Result := iSize;
end;

procedure TExcelFont.SetSize(iSize: Integer);
begin
  SetOleProperty('Size', 'Integer', iSize);
end;

function TExcelRange.GetFont: TExcelFont;
var
  pIntFont: PInterface;
begin
  GetOleProperty('Font', 'pInterface', pIntFont);
  FFont := TExcelFont.ConnectInterface(pIntFont);
  Result := FFont;
end;

function TExcelRange.GetValue: String;
const
  pszName: PChar = '123456789012345678901234567890123456789012345678901234567890';
begin
  GetOleProperty('Value', 'PChar', pszName);
  Result := StrPas(pszName);
end;

procedure TExcelRange.SetValue(sName: String);
const
  pszName: PChar = '123456789012345678901234567890123456789012345678901234567890';
begin
  StrPCopy(pszname, sName);
  SetOleProperty('Value', 'PChar', pszName);
end;

function TExcelWorkSheet.Cells(rowIndex, columnIndex: Integer): TExcelRange;
const
  pszName: PChar = '123456789012345678901234567890123456789012345678901234567890';
var
  pCell: pInterface;
begin
  SetOleMethodArg('Integer', columnIndex);
  SetOleMethodArg('Integer', rowIndex);
  CallOleFunction('Cells', 'pInterface', pCell);
  Result := TExcelRange.ConnectInterface(pCell);
end;

procedure TExcelWorkBook.Save;
begin
  CallOleProc('Save');
end;

procedure TExcelWorkBooks.FileOpen(FileName: String);
const
  pszName: PChar = '123456789012345678901234567890123456789012345678901234567890';
begin
  StrPCopy(pszName, FileName);
  SetOleMethodArg('PChar', pszName);
  CallOleProc('Open');
end;

function TExcelObj.GetVisible: Integer;
var
  iVisible: Integer;
begin
  GetOleProperty('Visible', 'Integer', iVisible);
  Result := iVisible;
end;

procedure TExcelObj.SetVisible(iVisible: Integer);
begin
  SetOleProperty('Visible', 'Integer', iVisible);
end;

function TExcelObj.GetWorkBooks: TExcelWorkBooks;
var
 pIntWorkBooks: PInterface;
begin
  GetOleProperty('WorkBooks', 'pInterface', pIntWorkBooks);
  FWorkBooks := TExcelWorkBooks.ConnectInterface(pIntWorkBooks);
  Result := FWorkBooks;
end;

function TExcelObj.GetActiveWorkBook: TExcelWorkBook;
var
 pIntWorkBook: PInterface;
begin
  GetOleProperty('ActiveWorkBook', 'pInterface', pIntWorkBook);
  FActiveWorkBook := TExcelWorkBook.ConnectInterface(pIntWorkBook);
  Result := FActiveWorkBook;
end;

function TExcelObj.GetActiveWorkSheet: TExcelWorkSheet;
var
 pIntWorkSheet: PInterface;
begin
  GetOleProperty('ActiveSheet', 'pInterface', pIntWorkSheet);
  FActiveWorkSheet := TExcelWorkSheet.ConnectInterface(pIntWorkSheet);
  Result := FActiveWorkSheet;
end;

procedure TExcelObj.Quit;
begin
  CallOleProc('Quit');
end;

end.
