unit FLabel;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls;

type
  TFixedLabel = class(TLabel)
  private
    { Private declarations }
    FEnabled: boolean;
    FFocusEnable: boolean;
    FOrigColor: TColor;
    procedure UpdateFocusedEnabled;
    procedure SetEnabled(Value: boolean);
    function GetFont: TFont;
    procedure SetFont(Value: TFont);
    function IsFontStored: boolean;
    procedure SetFocusEnable(Value: boolean);
    procedure CMDialogChar(var Message: TCMDialogChar); message CM_DIALOGCHAR;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property Enabled: boolean read FEnabled write SetEnabled default True;
    property Font: TFont read GetFont write SetFont stored IsFontStored;
    property FocusEnable: boolean read FFocusEnable write SetFocusEnable default True;
  end;

procedure Register;

implementation
type
  TPublicWinControl = class(TWinControl)
  public
    property Color;
  end;

constructor TFixedLabel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FEnabled := true;
  FOrigColor := Font.Color;
  FFocusEnable := true;
  inherited Enabled := true; { This is the default, just to make sure... }
end;

procedure TFixedLabel.SetEnabled(Value: boolean);
var
  LabelColor : TColor;
begin
  if Value <> FEnabled then
  begin
    inherited Enabled := true;
    FEnabled := Value;
    if FEnabled then
      LabelColor := FOrigColor
    else
    begin
      FOrigColor := inherited Font.Color;
      LabelColor := clBtnFace;
    end;
    if (Parent <> nil) and
       (TPublicWinControl(Parent).Color = LabelColor) then
      LabelColor := clGray;
    inherited Font.Color := LabelColor;
    UpdateFocusedEnabled;
  end;
end;

function TFixedLabel.GetFont: TFont;
begin
  Result := inherited Font;
end;

procedure TFixedLabel.SetFont(Value: TFont);
begin
  inherited Font := Value;
  FOrigColor := inherited Font.Color;
end;

function TFixedLabel.IsFontStored: boolean;
begin
  Result := not ParentFont;
end;

procedure TFixedLabel.UpdateFocusedEnabled;
begin
  if FFocusEnable and (FocusControl <> nil) then
    FocusControl.Enabled := Enabled;
end;

procedure TFixedLabel.SetFocusEnable(Value: boolean);
begin
  if FFocusEnable <> Value then
  begin
    FFocusEnable := Value;
    UpdateFocusedEnabled;
  end;
end;

procedure TFixedLabel.CMDialogChar(var Message: TCMDialogChar);
begin
  if Enabled then
    inherited;
end;

procedure Register;
begin
  RegisterComponents('BugFixes', [TFixedLabel]);
end;

end.
