unit YAST;

{ Yet-Another-Stack-Tracer

  Description: A general call-back based stack-trace utility.

  Compiles with Delphi 1.0 and BP 7.0 in real-mode, DPMI and Windows mode.
  Requires the ValidPtr unit if compiled for DOS.

  Written by Hallvard Vassbotn, January 1996 (hallvard@falcon.no)    }

interface

type
  PBytes = ^TBytes;
  TBytes = array[0..(High(Word)-$f) div sizeof(byte)] of byte;
  PWords = ^TWords;
  TWords = array[0..(High(Word)-$f) div sizeof(word)] of word;
  TStackInfo = record
    CallersBP : word;
    DumpSize  : word;
    ParamSize : word;
    IsFar     : boolean;
    ReturnLog : word;
    case integer of
    1: (CallerAdr : pointer;
        ReturnAdr : pointer;
        DumpPtr   : PBytes;
        ParamPtr  : PWords);
    2: (CallerOfs : word;
        CallerSeg : word;
        ReturnOfs : word;
        ReturnSeg : word;
        DumpOfs   : word;
        DumpSeg   : word;
        ParamOfs  : word;
        ParamSeg  : word)
  end;
  TReportStackFrame = function(var StackInfo: TStackInfo; PrivateData: Pointer): boolean;

procedure TraceStack(ReportStackFrame: TReportStackFrame; PrivateData: Pointer);

implementation

uses
{$IFDEF WINDOWS}
  {$IFDEF VER80}
    WinProcs;
  {$ELSE}
    Win31;
  {$ENDIF}
{$ELSE}
  ValidPtr;
{$ENDIF}

type
  PtrRec = record
    Ofs, Seg : Word;
  end;
  TFarStackFrame = record
    CallersBP : word;
    case integer of
    1: (CallerAdr : pointer);
    2: (CallerOfs : word;
        CallerSeg : word)
  end;
  TNearStackFrame = record
    CallersBP : word;
    CallerOfs : word;
  end;
  PStackFrame = ^TStackFrame;
  TStackFrame = TFarStackFrame;

function GetSSBPPtr: pointer; inline
($8C/$D2    { MOV DX, SS }
/$89/$E8);  { MOV AX, BP }

function LogSeg(Seg: word): word;
begin
{$IFDEF MSDOS}
  LogSeg := Seg;
{$ELSE}
  if Seg <> 0 then
    LogSeg := Word(Ptr(Seg, 0)^)
  else
    LogSeg := Seg;
{$ENDIF}
end;

procedure CorrectBP(var BP: word);
{ Handle Windows stack frames (i.e. Inc BP in far prolog code) }
begin
  if Odd(BP) then Dec(BP);
end;

function IsFarCode(Addr: pointer): boolean;
begin
{$IFDEF WINDOWS}
  IsFarCode := not IsBadCodePtr(Addr);
{$ELSE}
  IsFarCode := ValidCodePointer(Addr, 1);
{$ENDIF}
end;

function NextStackFrame(var StackFrame: PStackFrame;
                        var StackInfo : TStackInfo): boolean;
var
  More: boolean;
begin
  More := (StackFrame^.CallersBP <> 0) and (StackFrame^.CallerAdr <> nil);
  if More then
    with StackInfo do
    begin
      CallersBP  := StackFrame^.CallersBP;
      CorrectBP(CallersBP);
      CallerAdr  := StackFrame^.CallerAdr;
      DumpPtr    := Pointer(StackFrame);
      DumpSize   := (CallersBP - PtrRec(StackFrame).Ofs);
      ParamPtr   := Pointer(DumpPtr);
      ParamSize  := DumpSize div 2;
      IsFar      := IsFarCode(CallerAdr);
      if IsFar then
      begin
        ReturnAdr := CallerAdr;
        Dec(ParamSize, SizeOf(TFarStackFrame) div 2);
        Inc(ParamOfs , SizeOf(TFarStackFrame));
      end
      else
      begin
        ReturnOfs := CallerOfs;
        Dec(ParamSize, SizeOf(TNearStackFrame) div 2);
        Inc(ParamOfs , SizeOf(TNearStackFrame));
      end;
      ReturnLog := LogSeg(ReturnSeg);

      PtrRec(StackFrame).Ofs := StackFrame^.CallersBP;
      CorrectBP(PtrRec(StackFrame).Ofs);
    end;
  NextStackFrame := More;
end;

procedure TraceStack(ReportStackFrame: TReportStackFrame; PrivateData: Pointer);
var
  StackFrame : PStackFrame;
  StackInfo  : TStackInfo;
begin
  FillChar(StackInfo, SizeOf(StackInfo), 0);
  StackInfo.ReturnSeg := CSeg;
  StackFrame := GetSSBPPtr;
  while NextStackFrame(StackFrame, StackInfo) and
        ReportStackFrame(StackInfo, PrivateData) do
    {Loop};
end;

end.
