unit Instmain;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Gauges,
  Buttons, StdCtrls, IniFiles, DdeMan;

type
  TInstallWin = class(TForm)
    Gauge1: TGauge;
    Install: TBitBtn;
    Done: TBitBtn;
    Panel1: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    DdeClientConv1: TDdeClientConv;
    procedure DoneClick(Sender: TObject);
    procedure InstallClick(Sender: TObject);
  private
    TSInstDir: String;
    function IDAPI_Installed: Boolean;
    procedure InstallIDAPI(DLLPath, CfgFile: String);
    procedure CopyFiles(srcdir, destdir: String; prog: word);
    function CheckDir(dir: String): Boolean;
    procedure CreateGroup;
  end;

var
  InstallWin: TInstallWin;

implementation

{$R *.FRM}

uses TSDir, OpStr2, MsgDlg, Excepts, FileCtrl, Strings, Messages;

type
  EShellDDEError = class(EMessage);

const
  { Program string literals }
  SamePathMsg = 'Path already exists. Overwrite existing files?';
  TSString    = 'Into what directory would you like to install TimeSlice?';
  IDString    = 'Into what directory would you like to install IDAPI?';
  FilePathBad = 'You already have a file of that name.  Please rename the file ' +
                'or choose a new directory name.';
  DDEOpenError   = 'Error in establishing DDE link with Windows shell.';
  DDECommError   = 'Error in DDE communications link with Windows shell.';
  DDECreateGroup = '[CreateGroup(TimeSlice)]';
  DDEShowGroup   = '[ShowGroup(TimeSlice,1)]';
  DDEAddItem     = '[AddItem(%s,"TimeSlice",%s)]';

  { constant values }
  deNoMoreFiles = 18;
  TSFiles = 2;
  IDFiles = 1;

function TInstallWin.IDAPI_Installed: Boolean;
var
  Str: String;
  WinIniFile: TIniFile;
begin
  Result := True;
  WinIniFile := TIniFile.Create('Win.ini');
  Str := WinIniFile.ReadString('IDAPI', 'DLLPath', '');
  if Str = '' then
    Result := False
  else begin
    Str := WinIniFile.ReadString('IDAPI', 'ConfigFile01', '');
    if Str = '' then Result := False;
  end;
  WinIniFile.Free;
end;

procedure TInstallWin.DoneClick(Sender: TObject);
begin
  Close;
end;

procedure TInstallWin.CopyFiles(srcdir, destdir: String; prog: word);
var
  SR: TSearchRec;
  OldFile: String[13];
  BailOut: Boolean;
begin
  BailOut := False;
  FindFirst(srcdir + '\*.*', faArchive, SR);
  OldFile := '********.***';
  if SR.Name <> '' then begin
    try
      repeat
        if SR.Name <> JustFilename(ParamStr(0)) then begin
          Label2.Caption := SR.Name;
          Application.ProcessMessages;
          FileCopy(AddBackSlash(srcdir) + SR.Name, destdir + SR.Name);
          with Gauge1 do begin
            Progress := Progress + prog;
            if Progress > 100 then Progress := 100;
          end;
        end;
        FindNext(SR);
        If SR.Name = OldFile then
          BailOut := True
        else
          OldFile := SR.Name;
      until BailOut;
    except
      on EInOutError(E) do MessageDlg('I/O Error: ' + E.Message, mtError, [mbOk], 0);
    end;
    Label2.Caption := '(Done)';
  end;
end;

procedure TInstallWin.InstallIDAPI(DLLPath, CfgFile: String);
var
  WinIniFile: TIniFile;
begin
  WinIniFile := TIniFile.Create('Win.ini');
  WinIniFile.WriteString('IDAPI', 'DLLPath', DLLPath);
  WinIniFile.WriteString('IDAPI', 'ConfigFile01', CfgFile);
  if WinIniFile.ReadString('Borland Language Drivers', 'LDPath', '') = '' then
    WinIniFile.WriteSTring('Borland Language Drivers', 'LDPath',
                           AddBackSlash(DLLPath) + 'langdrv');
  WinIniFile.Free;
end;

function TInstallWin.CheckDir(dir: String): Boolean;
var
  SR: TSearchRec;
begin
  Result := True;
  FillChar(SR, SizeOf(SR), 0);
  FindFirst(dir, faAnyFile, SR);     { Is there a file or dir of this name? }
  if SR.Name <> '' then begin        { yes? }
    FindFirst(dir, faDirectory, SR); { Is there a dir of this name? }
    if SR.Name <> '' then begin      { yes? }
      if not MessageDlg(SamePathMsg, mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        Result := False;
    end
    else begin
      MessageDlg(FilePathBad, mtError, [mbOk], 0);
      Result := False;
    end;
  end
  else
    MkDir(dir);
end;

procedure TInstallWin.InstallClick(Sender: TObject);
var
  SR: TSearchRec;
  ErrCode: Integer;
  ItLives: Boolean;
begin
  with TSDirWin do begin
    Label1.Caption := TSString;
    InstDir.Text := 'c:\tslice';
  end;
  if TSDirWin.ShowModal = mrOk then begin
    if CheckDir(TSDirWin.InstDir.Text) then
      TSInstDir := AddBackSlash(TSDirWin.InstDir.Text);
      CopyFiles(AddBackSlash(JustPathname(ParamStr(0))), TSInstDir, TSFiles);
    if not IDAPI_Installed then begin
      with TSDirWin do begin
        Label1.Caption := IDString;
        InstDir.Text := 'c:\idapi';
      end;
      if TSDirWin.ShowModal = mrOk then begin
        if CheckDir(TSDirWin.InstDir.Text) then begin
          CopyFiles(AddBackSlash(JustPathname(ParamStr(0))) + 'idapi\',
                    AddBackSlash(TSDirWin.InstDir.Text), IDFiles);
          if CheckDir(AddBackSlash(TSDirWin.InstDir.Text) + 'langdrv') then begin
            CopyFiles(AddBackSlash(JustPathname(ParamStr(0))) + 'idapi\langdrv\',
                      AddBackSlash(TSDirWin.InstDir.Text) + 'langdrv\', IDFiles);
            InstallIDAPI(AddBackSlash(TSDirWin.InstDir.Text),
                         AddBackSlash(TSDirWin.InstDir.Text) + 'idapi.cfg');
            Gauge1.Progress := 100;
          end;
        end;
      end;
    end
    else
      Gauge1.Progress := 100;
    CreateGroup;
    Install.Enabled := False;
  end;
end;

procedure TInstallWin.CreateGroup;
var
  Temp, Temp2 : String;
  PMHand: hWnd;
begin
  PMHand := FindWindow('Progman', 'Program Manager');
  with DdeClientConv1 do begin
    if not SetLink('PROGMAN', 'PROGMAN') then
      MessageDlg(DDEOpenError, mtError, [mbOk], 0)
    else try
      { Create TimeSlice group }
      if not ExecuteMacro(DDECreateGroup, False) then
        raise EShellDDEError.Create('Could not create group.');
      { Show group }
      ExecuteMacro(DDEShowGroup, False);
      Temp := TSInstDir + 'tslice.exe';
      Temp2 := Format(DDEAddItem, [@Temp, @Temp]);
      { Add TimeSlice to group }
      if not ExecuteMacro(Temp2, False) then
        raise EShellDDEError.Create('Could not add item.');
    except
      on EShellDDEError(E) do
        MessageDlg(DDECommError + #13#10#13#10 + E.Message, mtError, [mbOk], 0);
    end;
    CloseLink;
    if PMHand = 0 then begin     { close progman if I opened it }
      PMHand := FindWindow('Progman', 'Program Manager');
      PostMessage(PMHand, wm_Close, 0, 0);
    end;
  end;
end;

begin
  RegisterClasses([TInstallWin, TGauge, TBitBtn, TPanel, TLabel, TBevel,
    TDdeClientConv, TDdeClientItem]);
  InstallWin := TInstallWin.Create(Application);
end.
