{*******************************************************
                Enhanced Edits [EnhEdits unit]

This unit contains the code for 4 enhanced edits. TRightEdit,
TLongEdit, TRealEdit and TFloatEdit. TRightEdit is the base
class for the other three and is written by Dr. Bob and
released to the public domain in Issue three of the Delphi
Magazine. Thanks Dr Bob!! (Note: I have not registered the
TRightEdit because I don't have a use for it right now. If
you want to register it just include a line in the register
procedure and include a bit map in EnhEdits.dcr)

These edits, especially TRealEdit, were written to improve the
already rich VCL. When Dr. Bob published his right aligned edit
I re-wrote these edits as decendents of the TRightEdit which I
feel completed the suite.

TLongEdit allows Setting the unpublished Text property via a
LongInt Value property. In addition there are AsString, AsInteger,
AsWord and AsByte methods for runtime convenience.

TRealEdit Gets and Sets the unpublished Text property via a
TRealStr Value property. The input is checked for validity as
a real before acceptance. Why do this?? The object inspector will
not allow the use of reals so machines, like one of mine, without
the 80x87 coprocessor can't Set or Get floating point values at
design time. Therefore, the TRealProperty property editor should
be useful in many situations.

TFloatEdit is an editor for machines with the80x87 coprocessor.
With TFloatEdit you can use anything up to type extended. For
convenience, there are AsString, AsSingle, AsDouble and AsReal
methods.

I hope you find this code useful. If you make improvements please
e-mail me a copy. Enjoy.

                Paul Warren
       HomeGrown Software Development
     (c) 1995 Langley British Columbia.
              (604) 530-9097
       e-mail:  hg_soft@uniserve.com
  Home page: http://haven.uniserve.com/~hg_soft

********************************************************}

unit EnhEdits;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Menus, Dialogs, StdCtrls, DsgnIntf, StrLib;

type
  TRealStr = string[20];

  TRealStrProperty = class(TStringProperty)
    function GetValue: string; override;
    procedure SetValue(const Value: string); override;
  end;

  TRightEdit = class(TCustomMemo)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
  published
    { Published declarations }
  { property AutoSelect; }
  { property AutoSize; }
    property BorderStyle;
    property CharCase;
    property Color;
    property Ctl3D;
    property Cursor;
    property DragCursor;
    property DragMode;
    property Enabled;
    property Font;
    property Height;
    property HelpContext;
    property HideSelection;
    property Hint;
    property Left;
    property MaxLength;
    property Name;
    property OEMConvert;
    property OnChange;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnEnter;
    property OnExit;
    property ParentColor;
    property ParentCtl3D;
    property ParentFont;
    property ParentShowHint;
  { property PasswordChar; }
    property PopupMenu;
    property ReadOnly;
    property ShowHint;
    property TabOrder;
    property Tag;
    {property Text;}
    property Top;
    property Visible;
    property Width;
  end;

  TLongEdit = class(TRightEdit)
  private
    { Private declarations }
    FValue: LongInt;
    FMinValue: LongInt;
    FMaxValue: LongInt;
    procedure SetValue(Value: LongInt);
    function GetValue: Longint;
    function CheckValue(NewValue: LongInt): LongInt;
    procedure SetMaxValue(NewValue: LongInt);
    procedure SetMinValue(NewValue: LongInt);
    procedure SetAsString(NewValue: string);
    function GetAsString: string;
    procedure SetAsInteger(NewValue: integer);
    function GetAsInteger: integer;
    procedure SetAsWord(NewValue: word);
    function GetAsWord: word;
    procedure SetAsByte(NewValue: byte);
    function GetAsByte: byte;
    procedure FormatText;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
  protected
    { Protected declarations }
    procedure KeyPress(var Key: Char); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    property AsString: string read GetAsString write SetAsString;
    property AsInteger: integer read GetAsInteger write SetAsInteger;
    property AsWord: word read GetAsWord write SetAsWord;
    property AsByte: byte read GetAsByte write SetAsByte;
  published
    { Published declarations }
    property Value: LongInt read GetValue write SetValue;
    property MinValue: LongInt read FMinValue write SetMinValue default 0;
    property MaxValue: LongInt read FMaxValue write SetMaxValue default 0;
  end;

  TRealEdit = class(TRightEdit)
  private
    { Private declarations }
    FDecimals: word;
    FMinValue: TRealStr;
    FMaxValue: TRealStr;
    procedure CMExit(var Message: TCMExit); message CM_EXIT;
    procedure SetAsReal(NewValue: real);
    function GetAsReal: real;
    procedure SetValue(NewValue: TRealStr);
    function GetValue: TRealStr;
    procedure SetDecimals(NewValue: word);
    function CheckValue(NewValue: real): real;
  protected
    { Protected declarations }
    procedure KeyPress(var Key: Char); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    property AsReal: real read GetAsReal write SetAsReal;
  published
    { Published declarations }
    property Value: TRealStr read GetValue write SetValue;
    property Decimals: word read FDecimals write SetDecimals default 1;
    property MinValue: TRealStr read FMinValue write FMinValue;
    property MaxValue: TRealStr read FMaxValue write FMaxValue;
  end;

  TNumericType = (ntGeneral, ntExponent, ntFixed);

  TFloatEdit = class(TRightEdit)
  private
    { Private declarations }
    FValue: Extended;
    FDecimals: word;
    FMinValue: Extended;
    FMaxValue: Extended;
    FDigits: word;
    FNumericType: TNumericType;
    procedure SetValue(Value: Extended);
    function GetValue: Extended;
    procedure SetMaxValue(NewValue: Extended);
    procedure SetMinValue(NewValue: Extended);
    function CheckValue(NewValue: Extended): Extended;
    procedure SetDecimals(NewValue: word);
    procedure SetDigits(NewValue: word);
    procedure SetNumericType(Value: TNumericType);
    procedure SetAsString(NewValue: string);
    function GetAsString: string;
    procedure SetAsDouble(NewValue: Double);
    function GetAsDouble: Double;
    procedure SetAsSingle(NewValue: Single);
    function GetAsSingle: Single;
    procedure SetAsReal(NewValue: real);
    function GetAsReal: real;
    procedure FormatText;
  protected
    { Protected declarations }
    procedure KeyPress(var Key: Char); override;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    property AsString: string read GetAsString write SetAsString;
    property AsDouble: Double read GetAsDouble write SetAsDouble;
    property AsSingle: Single read GetAsSingle write SetAsSingle;
    property AsReal: real read GetAsReal write SetAsReal;
  published
    { Published declarations }
    property Value: Extended read GetValue write SetValue;
    property Decimals: word read FDecimals write SetDecimals default 1;
    property MinValue: Extended read FMinValue write SetMinValue;
    property MaxValue: Extended read FMaxValue write SetMaxValue;
    property Digits: word read FDigits write SetDigits default 12;
    property NumericType: TNumericType read FNumericType write SetNumericType default ntGeneral;
  end;

procedure Register;

implementation

{ TREALSTRPROPERTY - Property editor to emulate real
  types for machines without 80x87 coprocessors }

{ Trim and chop the value returned to the
   Object Inspector }
function TRealStrProperty.GetValue: string;
begin
  Result := LRTrim(Chop(GetStrValue, 20));
end;

{ Attempt to convert the string returned from the
  Object Inspector to a real. If successful write
  the value to the object. }
procedure TRealStrProperty.SetValue(const Value: string);
var
  Temp: string[20];
  code: integer;
begin
  Temp := LRTrim(Value);
  StrToReal(code, Value);
  if code <> 0 then MessageDlg('Value must be a real number', mtError, [mbOk], 0)
  else SetStrValue(Value);
end;

constructor TRightEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Align := alNone;
  Alignment := taRightJustify;
  ScrollBars := ssNone;
  WantReturns := False;
  WantTabs := False;
  WordWrap := False;
end;

{ Set the object bounds so it can't be changed at design
  time. }
procedure TRightEdit.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  if AHeight > (2 * abs(Font.Height)) then AHeight := 2 * abs(Font.Height);
  inherited SetBounds(ALeft, ATop, AWidth, AHeight);
end;

{ Construct the TLongEdit }
constructor TLongEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FMinValue := 0;
  FMaxValue := 0;
  FValue := 0;
  Text := '0';
end;

{ Set the unpublished Text property to its string
  representation. Check the value is in range. }
procedure TLongEdit.SetValue(Value: LongInt);
begin
  FValue := Value;
  FormatText;
end;

{ Get the Text property as a LongInt by converting
  from its string representation. }
function TLongEdit.GetValue: LongInt;
begin
  FValue := StrToInt(Text);
  Result := CheckValue(FValue);
end;

{ Method to set the FMinValue property, check that
  Value is still in range and update the Text }
procedure TLongEdit.SetMinValue(NewValue: LongInt);
begin
  if FMinValue <> NewValue then
  begin
    FMinValue := NewValue;
    if FMinValue > FMaxValue then FMinValue := FMaxValue;
    if FValue < FMinValue then
      FValue := FMinValue;
    FormatText;
  end;
end;

{ Method to set the FMaxValue property, check that
  Value is still in range and update the Text }
procedure TLongEdit.SetMaxValue(NewValue: LongInt);
begin
  if FMaxValue <> NewValue then
  begin
    FMaxValue := NewValue;
    if FMaxValue < FMinValue then FMaxValue := FMinValue;
    if FValue > FMaxValue then
      FValue := FMaxValue;
    FormatText;
  end;
end;

{ Function to check the Value property is in range
  Min < Value < Max. }
function TLongEdit.CheckValue(NewValue: LongInt): LongInt;
begin
  Result := NewValue;
  if (FMaxValue <> FMinValue) then
  begin
    if NewValue < FMinValue
      then Result := FMinValue
    else 
      if NewValue > FMaxValue then Result := FMaxValue;
  end;
end;

{ Method to update the Text property }
procedure TLongEdit.FormatText;
var
  L: LongInt;
begin
  L := FValue;
  Text := IntToStr(L);
end;

{ Method to get Value as a string }
function TLongEdit.GetAsString: string;
begin
  Result := Text;
end;

{ Method to Set Value as a string }
procedure TLongEdit.SetAsString(NewValue: string);
begin
  FValue := CheckValue(StrToInt(NewValue));
  FormatText;
end;

{ Method to get Value as an integer }
function TLongEdit.GetAsInteger: integer;
const
  MaxInteger: integer = 32767;
  MinInteger: integer = -32768;
begin
  Result := 0;
  if (Text <> '') and not (Text = '-') then
    FValue := StrToInt(Text);
  if (FValue <= MaxInteger) and (FValue >= MinInteger) then
     Result := FValue;
end;

{ Method to Set Value as an integer }
procedure TLongEdit.SetAsInteger(NewValue: integer);
begin
  FValue := NewValue;
  FormatText;
end;

{ Method to get Value as a word }
function TLongEdit.GetAsWord: word;
const
  MaxWord: Word = 65535;
  MinWord: Word = 0;
begin
  Result := 0;
  if (Text <> '') and not (Text = '-') then
    FValue := StrToInt(Text);
  if (FValue <= MaxWord) and (FValue >= MinWord) then
     Result := FValue;
end;

{ Method to Set Value as a word }
procedure TLongEdit.SetAsWord(NewValue: word);
begin
  FValue := NewValue;
  FormatText;
end;

{ Method to get Value as a byte }
function TLongEdit.GetAsByte: byte;
const
  MaxByte: byte = 255;
  MinByte: byte = 0;
begin
  Result := 0;
  if (Text <> '') and not (Text = '-') then
    FValue := StrToInt(Text);
  if (FValue <= MaxByte) and (FValue >= MinByte) then
     Result := FValue;
end;

{ Method to Set Value as a byte }
procedure TLongEdit.SetAsByte(NewValue: byte);
begin
  FValue := NewValue;
  FormatText;
end;

{ Check the Value property is in range before allowing
  user to exit the edit.  }
procedure TLongEdit.CMExit(var Message: TCMExit);
var
  L: LongInt;
begin
  L := StrToInt(Text);
  if (L > MaxValue) or (L < MinValue) then
  begin
    SelectAll;
    SetFocus;
  end else
  begin
    FValue := L;
    inherited;
  end;
end;

{ Don't accept invalid characters }
procedure TLongEdit.KeyPress(var Key: Char);
begin
  if Key in ['0'..'9', '-', #8] then
    inherited KeyPress(Key)
  else begin
    Key := #0;
    MessageBeep(0);
  end;
end;

{ Construct the TRealEdit }
constructor TRealEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  Decimals := 1;
  MinValue := '0.0';
  MaxValue := '0.0';
  Text := '0.0';
end;

{ Check the Value property is in range before allowing
  user to exit the edit. This is done by performing a conversion
  to type real and checking the value against Min and Max.
  If the conversion was successful a value WILL be returned. }
procedure TRealEdit.CMExit(var Message: TCMExit);
var
  code: integer;
  R: real;
begin
  R := StrToReal(code, Text);
  Text := RealToStr(CheckValue(R), Decimals);
  if code = 0 then
  inherited else
  begin
    SelectAll;
    SetFocus;
  end;
end;

{ Don't accept invalid characters }
procedure TRealEdit.KeyPress(var Key: Char);
begin
  if Key in ['0'..'9', '-', '+', 'e', 'E', DecimalSeparator, #8] then
    inherited KeyPress(Key)
  else begin
    Key := #0;
    MessageBeep(0);
  end;
end;

{ Set the unpublished Text property to its string
  representation. Check the value is in range by performing
  a conversion. }
procedure TRealEdit.SetValue(NewValue: TRealStr);
var
  R: real;
  code: integer;
begin
  try
    R := StrToReal(code, NewValue);
    Text := RealToStr(CheckValue(R), Decimals);
  except
  end;
end;

{ Get the Text property as a real by converting
  from its string representation. }
function TRealEdit.GetValue: TRealStr;
var
  R: real;
  code: integer;
begin
  try
    R := StrToReal(code, Text);
    Result := RealToStr(CheckValue(R), Decimals);
  except
  end;
end;

{ Method to get Text as a real }
function TRealEdit.GetAsReal: real;
var
  code: integer;
begin
  try
    if Text <> '' then
      Result := CheckValue(StrToReal(code, Text));
  except
  end;
end;

{ Method to Set Text as a real }
procedure TRealEdit.SetAsReal(NewValue: real);
begin
  try
    Value := RealToStr(CheckValue(NewValue), Decimals);
  except
  end;
end;

{ Method to set FDecimal property. Note the conversion. This
  is to force a chnge when Decimals is changed at design time }
procedure TRealEdit.SetDecimals(NewValue: word);
var
  R: real;
  code: integer;
begin
  FDecimals := NewValue;
  try
    R := StrToReal(code, Text);
    Text := RealToStr(R, Decimals);
  except
  end;
end;

{ Function to check the Value property is in range
  Min < Value < Max. }
function TRealEdit.CheckValue(NewValue: real): real;
var
  Max, Min: real;
  code: integer;
begin
  Result := NewValue;
  Max := StrToReal(code, MaxValue);
  Min := StrToReal(code, MinValue);
  if (Max <> Min) then
  begin
    if NewValue < Min then Result := Min
    else if NewValue > Max then Result := Max;
  end;
end;

{ Construct the TFloatEdit }
constructor TFloatEdit.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FDecimals := 1;
  FDigits := 12;
  FValue := 0.0;
  FMinValue := 0;
  FMaxValue := 0;
  Text := '0.0';
end;

{ Set the unpublished Text property to its string
  representation. Check the value is in range. }
procedure TFloatEdit.SetValue(Value: Extended);
begin
  FValue := Value;
  FormatText;
end;

{ Get the Text property as a Floating type by
  converting from its string representation. }
function TFloatEdit.GetValue: Extended;
begin
  FValue := StrToFloat(Text);
  Result := CheckValue(FValue);
end;

{ Method to set the FMinValue property, check that
  Value is still in range and update the Text }
procedure TFloatEdit.SetMinValue(NewValue: Extended);
begin
  if FMinValue <> NewValue then
  begin
    FMinValue := NewValue;
    if FMinValue > FMaxValue then FMinValue := FMaxValue;
    if FValue < FMinValue then
      FValue := FMinValue;
    FormatText;
  end;
end;

{ Method to set the FMaxValue property, check that
  Value is still in range and update the Text }
procedure TFloatEdit.SetMaxValue(NewValue: Extended);
begin
  if FMaxValue <> NewValue then
  begin
    FMaxValue := NewValue;
    if FMaxValue < FMinValue then FMaxValue := FMinValue;
    if FValue > FMaxValue then
      FValue := FMaxValue;
    FormatText;
  end;
end;

{ Function to check the Value property is in range
  Min < Value < Max. }
function TFloatEdit.CheckValue(NewValue: Extended): Extended;
begin
  Result := NewValue;
  if (FMaxValue <> FMinValue) then
  begin
    if NewValue < FMinValue
      then Result := FMinValue
    else
      if NewValue > FMaxValue then Result := FMaxValue;
  end;
end;

{ Method to set FDecimal property. }
procedure TFloatEdit.SetDecimals(NewValue: word);
begin
  if FDecimals <> NewValue then
  begin
    FDecimals := NewValue;
    FormatText;
  end;
end;

procedure TFloatEdit.SetDigits(NewValue: word);
begin
  if FDigits <> NewValue then
  begin
    FDigits := NewValue;
    FormatText;
  end;
end;

procedure TFloatEdit.SetNumericType(Value: TNumericType);
begin
  if FNumericType <> Value then
  begin
    FNumericType := Value;
    FormatText;
  end;
end;

{ Method to get Value as a string }
function TFloatEdit.GetAsString: string;
begin
  Result := Text;
end;

{ Method to Set Value as a string }
procedure TFloatEdit.SetAsString(NewValue: string);
begin
  FValue := CheckValue(StrToFloat(NewValue));
  FormatText;
end;

{ Method to get Value as a double }
function TFloatEdit.GetAsDouble: Double;
const
  MaxDouble: Double = 1.7E308;
  MinDouble: Double = -1.7E308;
begin
  Result := 0;
  if (Text <> '') and not (Text = '-') then
    FValue := StrToFloat(Text);
  if (FValue <= MaxDouble) and (FValue >= MinDouble) then
     Result := FValue;
end;

{ Method to Set Value as a double }
procedure TFloatEdit.SetAsDouble(NewValue: Double);
begin
  FValue := NewValue;
  FormatText;
end;

{ Method to get Value as a single }
function TFloatEdit.GetAsSingle: Single;
const
  MaxSingle: Single = 3.4E38;
  MinSingle: Single = -3.4E38;
begin
  Result := 0;
  if (Text <> '') and not (Text = '-') then
    FValue := StrToFloat(Text);
  if (FValue <= MaxSingle) and (FValue >= MinSingle) then
     Result := FValue;
end;

{ Method to Set Value as a single }
procedure TFloatEdit.SetAsSingle(NewValue: Single);
begin
  FValue := NewValue;
  FormatText;
end;

{ Method to get Value as a real }
function TFloatEdit.GetAsReal: real;
const
  MaxReal: real = 1.7E38;
  MinReal: real = -1.7E38;
begin
  Result := 0;
  if (Text <> '') and not (Text = '-') then
    FValue := StrToFloat(Text);
  if (FValue <= MaxReal) and (FValue >= MinReal) then
     Result := FValue;
end;

{ Method to Set Value as a real }
procedure TFloatEdit.SetAsReal(NewValue: real);
begin
  FValue := NewValue;
  FormatText;
end;

procedure TFloatEdit.FormatText;
var
  X: extended;
begin
  X := FValue;
  case FNumericType of
    ntGeneral: Text := FloatToStrF(X, ffGeneral, FDigits, FDecimals);
    ntExponent: Text := FloatToStrF(X, ffExponent, FDigits, FDecimals);
    ntFixed: Text := FloatToStrF(X, ffFixed, FDigits, FDecimals);
  end;
end;

{ Don't accept invalid characters }
procedure TFloatEdit.KeyPress(var Key: Char);
begin
  if Key in ['0'..'9', '-', '+', 'e', 'E', DecimalSeparator, #8] then
    inherited KeyPress(Key)
  else begin
    Key := #0;
    MessageBeep(0);
  end;
end;

{ Register the property editor to apply to all types
  TRealStr in all components regardless of the
  property name }

{ Register all the edits }
procedure Register;
begin
  RegisterPropertyEditor(TypeInfo(TRealStr), nil, '', TRealStrProperty);
  RegisterComponents('Additional', [TLongEdit]);
  RegisterComponents('Additional', [TRealEdit]);
  RegisterComponents('Additional', [TFloatEdit]);
end;

end.
