unit ChrConst ;
(*****) interface (*******************************)


const
(*  ASCII control characters *)
(* ------------------------------------------------ *)
(*    name  byte  ctrl   description          notes *)
(* ------------------------------------------------ *)
  NUL = #00 ; (*  ^@     null                    Spacing character, used to terminate strings in C and Windows.           *)
  SOH = #01 ; (*  ^A     Start Of Heading        Begins transmission of data block or new file.                           *)
  STX = #02 ; (*  ^B     Start TeXt              Marks beginning of text following header data.                           *)
  ETX = #03 ; (*  ^C     End TeXt                May mark beginning of error checking data.                               *)
  EOT = #04 ; (*  ^D     End Of Transmission     Sign-off code, sometimes marks end of file.                              *)
  ENQ = #05 ; (*  ^E     ENQuiry                 Requests Status info from remote location.                               *)
  ACK = #06 ; (*  ^F     ACKnowledge             Verifies the success of communications between locations.                *)
  BEL = #07 ; (*  ^G     BELl                    Beeps speaker (or rings Teletype Bell), signalling need of attention.    *)
  BS  = #08 ; (*  ^H     BackSpace.                                                                                       *)
  HT  = #09 ; (*  ^I     Horizontal Tab.                                                                                  *)
  LF  = #10 ; (*  ^J     Line Feed.                                                                                       *)
  VT  = #11 ; (*  ^K     Vertical Tab.                                                                                    *)
  FF  = #12 ; (*  ^L     Form Feed.              Skip to next page.                                                       *)
  CR  = #13 ; (*  ^M     Carriage Return.                                                                                 *)
  SO  = #14 ; (*  ^N     Shift Out               Changes character set.                                                   *)
  SI  = #15 ; (*  ^O     Shift In                Changes character set.                                                   *)
  DLE = #16 ; (*  ^P     Data Link Escape        Modifies meaning of subsequent characters (like ESC).                    *)
  DC1 = #17 ; (*  ^Q     Device Control 1        Used as XON to signal remote location to transmit.                       *)
  DC2 = #18 ; (*  ^R     Device Control 2        General purpose toggle signal.                                           *)
  DC3 = #19 ; (*  ^S     Device Control 3        Used as XOFF to signal remote location to not transmit.                  *)
  DC4 = #20 ; (*  ^T     Device Control 4        General purpose toggle signal.                                           *)
  NAK = #21 ; (*  ^U     Negative AcKnowledge                                                                             *)
  SYN = #22 ; (*  ^V     SYNchronous Idle        Used between data blocks in synchronous communications.                  *)
  ETB = #23 ; (*  ^W     End Transmission Block  Variant of ETX                                                           *)
  CAN = #24 ; (*  ^X     CANcel                  Usually signals transmission error.                                      *)
  EM  = #25 ; (*  ^Y     End Medium              Signals physical end of data source.                                     *)
  SUB = #26 ; (*  ^Z     SUBstitute              Replaces characters that are invalid or undisplayable; EOF marker in DOS *)
  ESC = #27 ; (*  ^[     ESCape                  Marks following characters as a control sequence                         *)
  FS  = #28 ; (*  ^/     File Separator          Marks logical boundary between files.                                    *)
  GS  = #29 ; (*  ^]     Group Separator         Marks logical boundary between data groups.                              *)
  RS  = #30 ; (*  ^^     Record Separator        Marks logical boudnary between data records.                             *)
  US  = #31 ; (*  ^_     Unit Separator          Marks logical boundary between data units.                               *)
  SPC = #32 ; (*  <none> Space                                                                                            *)
  DEL = #127; (*  <none> DELete                  Eliminates other characters.                                             *)

const
  CRLF = CR + LF ; (*  DOS Text file line termination. *)
  RETURN = CRLF ;
  ENTER = CRLF ;
  SPACE = SPC ;



(*****) implementation (**************************)
{$ifdef ver80 }
initialization
{$else}
begin
{$endif}

    (* unit ChrConst -- initialization code *)
  (* NONE *)
end (* unit ChrConst -- initialization code *) .

