unit Names4u2;

interface

uses
  SysUtils, Classes;

type
  TDataRec = packed record
    { The form's edit box has its MaxLength property set to 30 }
    Name: String[30];
    { Only interested in the date portion of this date/time value }
    DOB: TDateTime;
  end;

  TDataFile = class
  private
    FDataFile: TFileStream;
  protected
    function GetCount: Longint;
    function GetCurrent: Longint;
    function GetRecord(Index: Longint): TDataRec;
    procedure SetCurrent(RecNo: Longint);
    procedure SetRecord(Index: Longint; const DataRec: TDataRec);
  public
    constructor Create;
    destructor Destroy; override;
    property Count: Longint read GetCount;
    property Current: Longint
      read GetCurrent write SetCurrent;
    property Records[Index: Longint]: TDataRec
      read GetRecord write SetRecord; default;
  end;

implementation

uses
  WinProcs, Forms, NetLock, Consts;

const
  FileName = 'DataFile.Dat';

{$ifdef Ver80}
function ExtractFileDir(const FileName: String): String;
var
  I: Integer;
begin
  Result := ExtractFilePath(FileName);
  I := Length(Result);
  if (I > 1) and (FileName[I] = '\') and (FileName[I - 1] <> ':') then
    { This is compiled in Delphi 1 only, so this is fine }
    Dec(Result[0]);
end;
{$endif}

constructor TDataFile.Create;
begin
  { Make current directory where EXE file is, just in case }
  ChDir(ExtractFileDir(Application.ExeName));
  { Make file if it ain't there }
  if not FileExists(FileName) then
  begin
    { We don't need a try..finally block here cos if the }
    { file creation fails, the constructor raises an }
    { exception which causes the object to be freed }
    FDataFile := TFileStream.Create(FileName, fmCreate);
    FDataFile.Free;
  end;
  FDataFile := TFileStream.Create(FileName, fmOpenReadWrite or fmShareDenyNone);
end;

destructor TDataFile.Destroy;
begin
  if Assigned(FDataFile) then
    FDataFile.Free;
  inherited Destroy;
end;

function TDataFile.GetCount: Longint;
begin
  Result := FDataFile.Size div SizeOf(TDataRec);
end;

function TDataFile.GetCurrent: Longint;
begin
  Result := FDataFile.Seek(0, soFromCurrent);
  if Result > -1 then
    Result := Result div SizeOf(TDataRec);
end;

function TDataFile.GetRecord(Index: Longint): TDataRec;
begin
  Current := Index;
  if FDataFile.Read(Result, SizeOf(TDataRec)) < SizeOf(TDataRec) then
    raise EListError.CreateRes(SListIndexError);
  { Go back to the beginning of the read record }
  Current := Index;
end;

procedure TDataFile.SetCurrent(RecNo: Longint);
begin
  { Anything past EOF is considered EOF }
  if RecNo > Count then
    RecNo := Count;
  FDataFile.Seek(RecNo * SizeOf(TDataRec), soFromBeginning);
end;

procedure TDataFile.SetRecord(Index: Longint; const DataRec: TDataRec);
var
  X: EInOutError;
begin
  Current := Index;
  if not LockFileArea(FDataFile.Handle, Current * SizeOf(TDataRec),
    SizeOf(TDataRec), False) then
  begin
    X := EInOutError.Create('Cannot lock file');
    { Set up a file access denied type exception }
    X.ErrorCode := 5;
    raise X;
  end;
  try
    if FDataFile.Write(DataRec, SizeOf(TDataRec)) < SizeOf(TDataRec) then
      raise EInOutError.Create('Cannot write to file');
    { Go back to the beginning of the written record }
    Current := Index;
  finally
    LockFileArea(FDataFile.Handle, Current * SizeOf(TDataRec),
      SizeOf(TDataRec), True);
  end;
end;

end.
