unit Shapes;

interface

uses Graphics, Classes;

type
  {Base class of the shapes to draw}
  TMyShape = class(TObject)
  private
    fSize   : Integer;
    fTop    : Integer;
    fLeft   : Integer;
    fColour : TColor;
  public
    property Size   : Integer read fSize   write fSize;
    property Top    : Integer read fTop    write fTop;
    property Left   : Integer read fLeft   write fLeft;
    property Colour : TColor  read fColour write fColour;

    procedure Draw(Dest : TCanvas); virtual;
  end;


  TCircle = class(TMyShape)
  public
    procedure Draw(Dest : TCanvas); override;
  end;


  TSquare = class(TMyShape)
  public
    procedure Draw(Dest : TCanvas); override;
  end;


  PMyPointerList = ^TMyPointerList;
  TMyPointerList = array[0..MaxListSize - 1] of TMyShape;

  TShapeList = class(TList)
  protected
    function  Get(Index : Integer) : TMyShape;
    procedure Put(Index : Integer;Item : TMyShape);
    function  GetList : PMyPointerList;
  public
    function  Add(Item : TMyShape) : Integer;
    function  First : TMyShape;
    function  Expand : TShapeList;
    function  IndexOf(Item : TMyShape): Integer;
    procedure Insert(Index : Integer;Item : TMyShape);
    function  Last : TMyShape;
    function  Remove(Item : TMyShape) : Integer;
    property  Items[Index : Integer] : TMyShape read Get write Put;
    property  List : PMyPointerList read GetList;
    {New method to draw all the shapes}
    procedure DrawAllShapes(Dest : TCanvas);
    {Override the destructor to free all the shapes}
    destructor Destroy; override;
  end;


implementation


{------------------ Implementation of TMyShape ------------------}

procedure TMyShape.Draw(Dest : TCanvas);
begin {Draw}
  Dest.Pen.Color   := Colour;
  Dest.Brush.Color := Colour;
end;  {Draw}


{------------------ Implementation of TCircle -------------------}

procedure TCircle.Draw(Dest : TCanvas);
begin {Draw}
  inherited Draw(Dest);
  Dest.Ellipse(Left,Top,Left + Size,Top + Size);
end;  {Draw}


{------------------ Implementation of TSquare -------------------}

procedure TSquare.Draw(Dest : TCanvas);
begin {Draw}
  inherited Draw(Dest);
  Dest.Rectangle(Left,Top,Left + Size,Top + Size);
end;  {Draw}


{------------------ Implementation of TShapeList ------------------}

function TShapeList.GetList : PMyPointerList;
begin
  Result := PMyPointerList(inherited List);
end;

function TShapeList.Get(Index : Integer): TMyShape;
begin
  Result := TMyShape(inherited Get(Index));
end;

procedure TShapeList.Put(Index : Integer;Item : TMyShape);
begin
  inherited Put(Index,Item);
end;

function TShapeList.Add(Item: TMyShape) : Integer;
begin
  inherited Add(Item);
end;

function TShapeList.Expand : TShapeList;
begin
  Result := TShapeList(inherited Expand);
end;

function TShapeList.First : TMyShape;
begin
  Result := TMyShape(inherited First);
end;

function TShapeList.IndexOf(Item : TMyShape) : Integer;
begin
  Result := inherited IndexOf(Item);
end;

procedure TShapeList.Insert(Index : Integer;Item : TMyShape);
begin
  inherited Insert(Index,Item);
end;

function TShapeList.Last : TMyShape;
begin
  Result := TMyShape(inherited Last);
end;

function TShapeList.Remove(Item: TMyShape): Integer;
begin
  Result := inherited Remove(Item);
end;


procedure TShapeList.DrawAllShapes(Dest : TCanvas);
  var
    i : Integer;
begin {DrawAllShapes}
  {Step through the list drawing each shape}
  for i := 0 to Count - 1 do
    Items[i].Draw(Dest);
end;  {DrawAllShapes}


destructor TShapeList.Destroy;
  var
    i : Integer;
begin {Destroy}
  {Free all the shapes memory}
  for i := 0 to Count - 1 do
    Items[i].Free;
end;  {Destroy}


end.
