unit Poweru;

interface

function Power(Number, Exponent: Extended): Extended;

implementation

uses SysUtils;

function Power(Number, Exponent: Extended): Extended;
const
  { from SysUtils.Inc }
  SInvalidOp = 65428;
begin
  Result := 0;
  if Exponent = 0 then
    { Any number to power zero: x^0 = 1 }
    Result := 1
  else if Number > 0 then
    { +ve number, any power: x^y = exp(ln(x)*y) }
    Result := Exp(Ln(Number) * Exponent)
  else if Number = 0 then
  begin
    { 0, any power > 0 : x^y = 0 - the default result }
    if Exponent < 0 then
      { 0, any power < 0 : x^y = undefined }
      raise EInvalidOp.CreateRes(SInvalidOp);
  end
  else
    { -ve number, any power - only valid for whole powers}
    if Frac(Exponent) = 0 then
    begin
      Result := Exp(Ln(-Number) * Exponent);
      { If the power is odd, result is -ve }
      if Odd(Trunc(Exponent)) then
        Result := -Result;
    end
    else
      { Fractional power of negative number not allowed so give }
      { exception. It's manually raised, rather than a real one }
      { being generated. This allows DLL code to trap the problem }
      { Real math (plus other hardware) exceptions evade DLL }
      { exception handling and go straight to the EXE }
      raise EInvalidOp.CreateRes(SInvalidOp);
end;

end.
