{$A+,B-,D-,F+,G+,I-,K+,L-,N+,P+,Q-,R-,S+,T-,V+,W-,X+,Y-}
{.$M 32768,4096}
library Vicious;
uses
{$IFDEF WIN32}
  Windows, ShareMem,
{$ELSE}
  WinTypes, WinProcs,
{$ENDIF}
  SysUtils, Dialogs, ToolIntf, ExptIntf, VirtIntf, DBTables, Controls, Forms,
  VcsIntf, Options, {.Archive, Put,.} InfoForm, About;

Type
  TVicious = class (TIVCSCLient)
  public
    constructor Create;

    function GetIDString: string; override;
    { Called at initialization. Client should return a unique
      identification string.
    }
    function GetMenuName: string; override;
    { Called to retrieve the name of the main menu item
      to be added to the application's menu bar.
    }
    function GetVerbCount: Integer; override;
    { Called to determine the number of available verbs. This
      function will not be called if the GetMenuName function
      returns a blank string (indicating no menu).
    }
    function GetVerb(Index: Integer): string; override;
    { Called to retrieve the menu text for each verb (return a
      blank string to create a seperator bar).
    }
    function GetVerbState(Index: Integer): Word; override;
    { Called to determine the state of a particular verb (either
      0, vsEnabled or vsChecked).
    }
    procedure ExecuteVerb(Index: Integer); override;
    { Called when the user selects a verb from a menu.
    }
    procedure ProjectChange; override;
    { Called when there is any state change of the current
      project, i.e. when a project is destroyed or created.
    }

  private
    ProjectName: String; { local }
    ArchiveName: String; { remote }
  end;


procedure HandleException;
begin
  if Assigned(ToolServices) then
    ToolServices.RaiseException(ReleaseException)
end {HandleException};


  constructor TVicious.Create;
  begin
    try
      ProjectName := '';
      ArchiveName := '';
    except
      HandleException
    end
  end {Create};

  function TVicious.GetIDString: string;
  begin
    try
      Result := 'DrBob.Vicious'
    except
      HandleException
    end
  end {GetIDString};

  function TVicious.GetMenuName: string;
  begin
    try
      Result := '&ViCiouS'
    except
      HandleException
    end
  end {GetMenuName};

  function TVicious.GetVerbCount: Integer;
  begin
    try
      Result := 7
    except
      HandleException
    end
  end {GetVerbCount};

  function TVicious.GetVerb(Index: Integer): string;
  begin
    try
      case index of
        0: Result := '&Options...';
        1: Result := '&Archive...';
        2: Result := '&Get (check out)...';
        3: Result := '&Put (check in)...';
        4: if ProjectName = '' then Result := '&Information...'
           else
             Result := '&Information about ' + ProjectName + '...';
        5: Result := ''; { seperator }
        6: Result := '&About...'
      end
    except
      HandleException
    end
  end {GetVerb};

  procedure TVicious.ExecuteVerb(Index: Integer);
  begin
    try
      case index of
        0: begin
             OptionsFrm := TOptionsFrm.Create(nil);
             if OptionsFrm.ShowModal = idOk then
               ArchiveName := OptionsFrm.FileListBox1.FileName;
             OptionsFrm.Free;
             OptionsFrm := nil
           end {Options};
        1: { begin
             ArchiveFrm := TArchiveFrm.Create(nil);
             ArchiveFrm.Table1.DatabaseName := ExtractFilePath(ArchiveName);
             ArchiveFrm.Table1.TableName := ExtractFileName(ArchiveName);
             ArchiveFrm.Table1.Open;
             ArchiveFrm.ShowModal;
             ArchiveFrm.Table1.Close;
             ArchiveFrm.Free;
             ArchiveFrm := nil
           end {Archive};
        2: ;
        3: ;
        4: begin
             InformationFrm := TInformationFrm.Create(nil);
             InformationFrm.ShowModal;
             InformationFrm.Free;
             InformationFrm := nil
           end {Information};
        6: begin
             AboutBox := TAboutBox.Create(nil);
             AboutBox.ShowModal;
             AboutBox.Free;
             AboutBox := nil
           end {About};
      end
    except
      HandleException
    end
  end {ExecuteVerb};

  function TVicious.GetVerbState(Index: Integer): Word;
  begin
    try
      Result := vsEnabled;
      if (index in [1,2,3]) and (ArchiveName = '') then Result := 0
    except
      HandleException
    end
  end {GetVerbState};

  procedure TVicious.ProjectChange;
  begin
    try
      if ToolServices <> nil then
        ProjectName := ExtractFileName(ToolServices.GetProjectName);
      if (ProjectName <> '') and (ArchiveName <> '') then
        MessageDlg('The project just changed!', mtWarning, [mbOk], 0)
    except
      HandleException
    end
  end {ProjectChange};


  function InitVCS(Delphi: TIToolServices): TIVCSClient;
    {$IFDEF WIN32} stdcall; {$ELSE} export; {$ENDIF}
  begin
    ExptIntf.ToolServices := Delphi;
    if Assigned(ToolServices) then
      Application.Handle := ToolServices.GetParentHandle
    else MessageDlg('ToolServices not available!', mtError, [mbOk], 0);
    Result := TViCious.Create
  end {InitVCS};

exports
  InitVCS name VCSManagerEntryPoint;


begin
  if Date >= EncodeDate(1996,8,1) then
    MessageDlg('** This beta version of ViCiouS is outdated **'#13#13+
               'Please obtain a new version from LIB 22 of the'#13+
               'DELPHI forum or on http://www.pi.net/~drbob/',
                mtWarning, [mbOK], 0)
end.
