unit Snglins2;

{ This unit controls the TDataView form. This form is created (via a  }
{ call from the SnglIns9 unit) when the first instance of a TSLSingle }
{ or TSLSingleNum is created. Its purpose is to simply allow you to   }
{ view and amend the data in the class instance. Whenever the data is }
{ changed, all the registered OnChanged methods are called for the    }
{ class via the overridden Changed method. As well as showing the data}
{ in the list, the form also shows the object address - this allows   }
{ you to compare this with the client forms to ensure that they       }
{ reference the same object.                                          }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, SnglIns9;

type
  TDataView = class(TForm)
    ListBox1: TListBox;
    BtnAdd: TButton;
    BtnDelete: TButton;
    EditNewData: TEdit;
    Label1: TLabel;
    PnlObjAddr: TPanel;
    Panel1: TPanel;
    Label2: TLabel;
    procedure ListBox1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure BtnDeleteClick(Sender: TObject);
    procedure EditNewDataChange(Sender: TObject);
    procedure BtnAddClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    FObjectAddr: TSLSingle;
    procedure SetObjectAddr(Value: TSLSingle);
    procedure SetButtons;
  public
    { Public declarations }
    { This property is simply to allow the object address to be viewed}
    { on the form. It is set in the Create class function.            }
    property ObjectAddr: TSLSingle
      read FObjectAddr
      write SetObjectAddr;
  end;

implementation

{$R *.DFM}

{ Set the state of the buttons depending on the data in the edit box  }
{ and the item selected in the listbox.                               }

procedure TDataView.SetButtons;
begin
  BtnDelete.Enabled := (ListBox1.ItemIndex <> -1);
  BtnAdd.Enabled := (EditNewData.Text <> '');
end;

procedure TDataView.ListBox1Click(Sender: TObject);
begin
  SetButtons;
end;

procedure TDataView.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TDataView.BtnDeleteClick(Sender: TObject);
var
  S: string;

begin
  S := ListBox1.Items[ListBox1.ItemIndex];
  ListBox1.Items.Delete(ListBox1.ItemIndex);
  { By deleting the entry, the registered OnChanged event will be     }
  { called by the class.                                              }
  FObjectAddr.Delete(FObjectAddr.IndexOf(s));
  SetButtons;
end;

procedure TDataView.EditNewDataChange(Sender: TObject);
begin
  BtnAdd.Enabled := (EditNewData.Text <> '');
end;

procedure TDataView.BtnAddClick(Sender: TObject);
begin
  { The following code performs a test on the result from the Add     }
  { method to see if the Add request was ignored (This will only      }
  { happen if the class is TSLSingleNum and the first character is    }
  { not a digit). If this happens a beep is produced. If the Add is   }
  { ignored the Changed method will not be called.                    }
  if FObjectAddr.Add(EditNewData.Text) <> -1 then
    ListBox1.Items.Add(EditNewData.Text)
  else
    MessageBeep($FFFF);
end;

procedure TDataView.FormCreate(Sender: TObject);
begin
  SetButtons;
end;

{ This property access method has simply been implemented to display  }
{ the object address on the form.                                     }

procedure TDataView.SetObjectAddr(Value: TSLSingle);
begin
  FObjectAddr := Value;
  ListBox1.Items.Assign(FObjectAddr);
  PnlObjAddr.Caption := Format('Object address[%p]',[Pointer(FObjectAddr)]);
end;

end.
