unit Snglins3;

{ This is the unit for the ClientForm. The Create constructor is      }
{ called from unit SnglIns1. After the form has been created the      }
{ DataView property is set (from unit SnglIns1). The property access  }
{ routine SetDataView creates the required class.                     }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, SnglIns2, SnglIns9;

type
  TDataView = (dvAll,dvNumeric);

type
  TClientForm = class(TForm)
    Memo1: TMemo;
    BtnClose: TButton;
    Panel1: TPanel;
    PnlDataPrefix: TPanel;
    Label1: TLabel;
    procedure FormDestroy(Sender: TObject);
    procedure BtnCloseClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    { Pointer the the TSLSingle or TSLSingleNum 'instance' }
    FSLSingle: TSLSingle;
    procedure RefreshView;
    procedure SetDataView(Value: TDataView);
  public
    { Public declarations }
    property DataView: TDataView write SetDataView;
  end;

var
  ClientForm: TClientForm;

implementation

uses SnglIns1;

{$R *.DFM}

{ This method is called whenever the central data is changed. It is   }
{ registered via a call to RegisterOnChangedEvent.                    }

procedure TClientForm.RefreshView;
begin
  Memo1.Lines.BeginUpdate;
  Memo1.Lines.Assign(FSLSingle);
  Memo1.Lines.EndUpdate;
end;

{ This property access method is called from module SnglIns1.Pas after}
{ the client form has been created. It indicates which class type     }
{ needs to be associated with this form.                              }

procedure TClientForm.SetDataView(Value: TDataView);
begin

  case Value of
    dvAll:
      begin
        FSLSingle := TSLSingle.Create;
        PnlDataPrefix.Caption := 'All';
      end;
    dvNumeric:
      begin
        FSLSingle := TSLSingleNum.Create;
        PnlDataPrefix.Caption := 'Numeric';
      end;
  end;

  Label1.Caption := Format('Object Addr: %p',[Pointer(FSLSingle)]);

  RefreshView;

  FSLSingle.RegisterOnChangedEvent(RefreshView);

end;

procedure TClientForm.FormDestroy(Sender: TObject);
begin
  FSLSingle.UnRegisterOnChangedEvent(RefreshView);

  { Due to polymorphism, the correct Free method will be called as    }
  { Free has been defined as virtual.                                 }

  FSLSingle.Free;

  { This is for demonstration purposes only - to allow us to see how  }
  { many copies are active. }

  MainWindow.DisplayInstanceCounts;
end;

procedure TClientForm.BtnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TClientForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Action := caFree;
end;

end.
