 {$A+,B-,D-,F-,G+,I+,K+,L-,N+,P+,Q-,R-,S+,T+,V-,W-,X+,Y-}
 unit FileCopy;
 (*
   FILECOPY 1.5 (Public Domain)
   Borland Delphi 1.0
   Author: Robert E. Swart (100434.2072@compuserve.com)
     Date: 1995-08-27
   -----------------------------------------------------------------
   This unit implements a FastFileCopy procedure that is usable from
   Borland Pascal (real mode, DPMI or Windows) and Borland Delphi. A
   callback routine (or nil) can be given as extra argument.

   Example of usage:

   {$IFDEF WINDOWS}
    uses FileCopy, WinCrt;
   {$ELSE}
    uses FileCopy, Crt;
   {$ENDIF}

      procedure CallBack(Position, Size: LongInt); far;
      var i: Integer;
      begin
        { do you stuff here... }
        GotoXY(1,1);
        for i:=1 to (80 * Position) div Size do write('X')
      end {CallBack};

    begin
      FastFileCopy('C:\AUTOEXEC.BAT', 'C:\AUTOEXEC.BAK', nil);
      FastFileCopy('C:\CONFIG.SYS', 'C:\CONFIG.BAK', CallBack)
    end.
 *)
 interface

 Type
   TCallBack = procedure (Position, Size: LongInt); { export; }

   procedure FastFileCopy(Const InFileName, OutFileName: String;
                          CallBack: TCallBack);

 implementation
 {$IFDEF VER80}
 uses SysUtils;
 {$ELSE}
   {$IFDEF WINDOWS}
   uses WinDos;
   {$ELSE}
   uses Dos;
   {$ENDIF}
 {$ENDIF}

   procedure FastFileCopy(Const InFileName, OutFileName: String;
                          CallBack: TCallBack);
   Const BufSize = 8*4096; { 32Kbytes gives me the best results }
   Type
     PBuffer = ^TBuffer;
     TBuffer = Array[1..BufSize] of Byte;
   var Size: Word;
       Buffer: PBuffer;
       infile,outfile: File;
       SizeDone,SizeFile,TimeDateFile: LongInt;
   begin
     if (InFileName <> OutFileName) then
     begin
       Buffer := nil;
       Assign(infile,InFileName);
       System.Reset(infile,1);
       {$IFDEF VER80}
       try
       {$ELSE}
       begin
       {$ENDIF}
         SizeFile := FileSize(infile);
         Assign(outfile,OutFileName);
         System.Rewrite(outfile,1);
         {$IFDEF VER80}
         try
         {$ELSE}
         begin
         {$ENDIF}
           SizeDone := 0;
           New(Buffer);
           repeat
             BlockRead(infile,Buffer^,BufSize,Size);
             Inc(SizeDone,Size);
             if (@CallBack <> nil) then
               CallBack(SizeDone,SizeFile);
             BlockWrite(outfile,Buffer^,Size)
           until Size < BufSize;
           {$IFDEF VER80}
           FileSetDate(TFileRec(outfile).Handle,
             FileGetDate(TFileRec(infile).Handle));
           {$ELSE}
           GetFTime(infile, TimeDateFile);
           SetFTime(outfile, TimeDateFile);
           {$ENDIF}
         {$IFDEF VER80}
         finally
         {$ENDIF}
           if Buffer <> nil then Dispose(Buffer);
           System.close(outfile)
         end;
       {$IFDEF VER80}
       finally
       {$ENDIF}
         System.close(infile)
       end
     end
     {$IFDEF VER80}
     else
       Raise EInOutError.Create('File cannot be copied onto itself')
     {$ENDIF}
   end {FastFileCopy};
 end.
