unit Whatsnew;
{ To register, call: RegisterLibraryExpert(TWhatsNewExpert.Create)
}
interface
uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, ExptIntf;

Type
  TWhatsNewForm = class(TForm)
    Label1: TLabel; { 'BOLIB.DCL - What's New' }
    Memo1: TMemo;   { contents of What's New file }
    CloseBtn: TBitBtn;
    procedure FormActivate(Sender: TObject);
  end;


Type
  TWhatsNewExpert = class(TIExpert)
  public
    constructor Create; virtual;
    destructor Destroy; override;

    { Expert Style }
    function GetStyle: TExpertStyle; override;

    { Expert Strings }
    function GetName: string; override;
    function GetIDString: string; override;

    function GetMenuText: string; override;
    function GetState: TExpertState; override;

    function GetGlyph: HBITMAP; override;
    function GetComment: string; override;

    { Launch the Expert }
    procedure Execute; override;

  private
    Timer: TTimer;
    procedure OnTimer(Sender: TObject);
  end;

Var
  WhatsNewForm: TWhatsNewForm;


implementation
uses IniFiles;
{$R *.DFM}

Const
  WhatsNewFileName = 'L:\LIBRARY\DELPHI\WHATS.NEW';
  WhatsNewFileDate: LongInt = 0;


procedure TWhatsNewForm.FormActivate(Sender: TObject);
begin
  if FileExists(WhatsNewFileName) then
    Memo1.Lines.LoadFromFile(WhatsNewFileName)
end;


  constructor TWhatsNewExpert.Create;
  begin
    TIExpert.Create;
    with TIniFile.Create('DELPHI.INI') do
    begin
      WhatsNewFileDate := ReadInteger('BOLIB', 'WhatsNew', 0);
      Free
    end;
    Timer := TTimer.Create(nil);
    Timer.Interval := 65535;
    Timer.OnTimer := OnTimer;
    Timer.Enabled := True
  end {Create};

  destructor TWhatsNewExpert.Destroy;
  begin
    with TIniFile.Create('DELPHI.INI') do
    begin
      WriteInteger('BOLIB', 'WhatsNew', WhatsNewFileDate);
      Free
    end;
    Timer.Free;
    Inherited Destroy
  end {Destroy};


  function TWhatsNewExpert.GetStyle: TExpertStyle;
  begin
    Result := esStandard
  end {GetStyle};

  function TWhatsNewExpert.GetName: String;
  begin
    Result := 'What''s New Expert'
  end {GetName};

  function TWhatsNewExpert.GetIDString: String;
  begin
    Result := 'DrBob.WhatsNewExpert'
  end {GetIDString};


  function TWhatsNewExpert.GetMenuText: String;
  begin
    Result := 'BOLIB.DCL &What''s New...'
  end {GetMenuText};

  function TWhatsNewExpert.GetState: TExpertState;
  begin
    Result := [esEnabled]
  end {GetState};


  function TWhatsNewExpert.GetGlyph: HBITMAP;
  begin
    Result := 0 { not needed for esStandard }
  end {GetGlyph};

  function TWhatsNewExpert.GetComment: String;
  begin
    Result := '' { not needed for esStandard }
  end {GetComment};


  procedure TWhatsNewExpert.Execute;
  { Execute: do your 'expert' stuff here... }
  Const Already: Boolean = False;
  begin
    if not Already then
    begin
      Already := True;
      with TWhatsNewForm.Create(nil) do
      begin
        ShowModal;
        Free
      end;
      Already := False
    end
  end {Execute};


  procedure TWhatsNewExpert.OnTimer(Sender: TObject);
  begin
    if (FileAge(WhatsNewFileName) > WhatsNewFileDate) then
    begin
      (Sender AS TTimer).Enabled := False;
       Execute;
      (Sender AS TTimer).Enabled := True
    end;
    WhatsNewFileDate := FileAge(WhatsNewFileName)
  end {OnTimer};
end.
