unit Tesths1;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, HeapStat;

type
  THeapStatForm = class(TForm)
    AllocMemEdit: TEdit;
    AllocBtn: TButton;
    MemBlockList: TListBox;
    DeallocBtn: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    FreeAllBtn: TButton;
    Label13: TLabel;
    Label14: TLabel;
    ShowStat: TCheckBox;
    procedure AllocBtnClick(Sender: TObject);
    procedure DeallocBtnClick(Sender: TObject);
    procedure MemBlockListClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FreeAllBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    OutF : Text;
    procedure FreeListItem(Idx: integer);
    procedure SetDeallocEnable;
    procedure KillList;
    procedure HeapStatEvent(const HS: THeapStat);
  public
    { Public declarations }
  end;

var
  HeapStatForm: THeapStatForm;

implementation

{$R *.DFM}

procedure THeapStatForm.SetDeallocEnable;
begin
  DeallocBtn.Enabled := (MemBlockList.Items.Count > 0) and
                        (MemBlockList.ItemIndex >= 0);
end;

procedure THeapStatForm.FreeListItem(Idx: integer);
var
  p : pointer;
  Size : word;
begin
  if Idx >= 0 then
  begin
    TObject(P) := MemBlockList.Items.Objects[Idx];
    Size := StrToInt(MemBlockList.Items[Idx]);
    FreeMem(P, Size);
    MemBlockList.Items.Delete(Idx);
    if Idx > 0 then
      MemBlockList.ItemIndex := Idx-1
    else
    if MemBlockList.Items.Count > 0 then
      MemBlockList.ItemIndex := 0;
    SetDeallocEnable;
  end;
end;

procedure THeapStatForm.AllocBtnClick(Sender: TObject);
var
  p : pointer;
  Size : word;
begin
  Size := StrToInt(AllocMemEdit.Text);
  GetMem(P, Size);
  MemBlockList.Items.AddObject(AllocMemEdit.Text, TObject(P));
  SetDeallocEnable;
end;

procedure THeapStatForm.DeallocBtnClick(Sender: TObject);
begin
  FreeListItem(MemBlockList.ItemIndex);
end;

procedure THeapStatForm.MemBlockListClick(Sender: TObject);
begin
  SetDeallocEnable;
end;

procedure THeapStatForm.HeapStatEvent(const HS: THeapStat);
var
  Op : Char;
begin
  with HS do
  begin
    if ShowStat.Checked then
    { Show statistics on the form.
      Note that setting the capton properties will call allocation routines.
      That explains the "leak" that is displayed. }
    begin
      Label5.Caption := IntToStr(AllocBytes);
      Label6.Caption := IntToStr(DeallocBytes);
      Label7.Caption := IntToStr(DiffBytes);
      Label8.Caption := IntToStr(BigBytes);
      Label9.Caption := IntToStr(AllocCount);
      Label10.Caption := IntToStr(DeallocCount);
      Label11.Caption := IntToStr(DiffCount);
      Label12.Caption := IntToStr(BigCount);
    end;
    case ThisOp of
      hoAlloc   : Op := '>';
      hoDealloc : Op := '<';
      hoBigAlloc: Op := '*';
    else          Op := '?';
    end;
    { Log statistics to file }
    Writeln(OutF, Format('%s %10d%10d%10d%10d%10d',
                         [Op, ThisSize, DiffBytes, DiffCount, BigBytes,  BigCount]));
  end;
end;

procedure THeapStatForm.KillList;
var
  i : integer;
begin
  for i := MemBlockList.Items.Count-1 downto 0 do
    FreeListItem(i);
end;

procedure THeapStatForm.FormShow(Sender: TObject);
begin
  HeapStatInit(HeapStatEvent, nil);
end;

procedure THeapStatForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  KillList;
  if not HeapStatDone then
    ShowMessage('HeapStat callback was recursive');
  System.Close(OutF);
end;

procedure THeapStatForm.FreeAllBtnClick(Sender: TObject);
begin
  KillList;
end;

procedure THeapStatForm.FormCreate(Sender: TObject);
begin
  System.Assign(OutF, 'TestHS.Log');
  System.Rewrite(OutF);
  Writeln(OutF, Format('Op%10s%10s%10s%10s%10s',
                       ['ThisSize', 'DiffBytes', 'DiffCount', 'BigBytes', 'BigCount']));
end;

end.
