unit TBHEXED;
{$I+}
interface
uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls, Forms,
  Dialogs, Grids, TBHexVu;

Type
  TBHexEditor = class(TBHexViewer)
    private
      { Private declarations }
      FCopyBlock: TBlock;
      FChanged: Boolean;
    protected
      { Protected declarations }
      procedure SetOffset(AnOffset: LongInt); override;
      procedure SetFileName(AFileName: TFileName); override;
      procedure SetEditText(ACol, ARow: Longint; const Value: string); override;
      function GetEditLimit: Integer; override;
      function GetBlock: TBlock;
      procedure SetBlock(Const ABlock: TBlock);
    public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
    published
      { Published declarations }
      property Changed: Boolean read FChanged;
    end {TBHexEditor};

implementation

  constructor TBHexEditor.Create(AOwner: TComponent);
  begin
    inherited Create(AOwner);
    FAbout := 'TBHexEditor (c) 1996 by Dr.Bob [100434,2072]-> see http://www.pi.net/~drbob/';
    FChanged := False;
    Options := Options + [goEditing]
  end {Create};

  destructor TBHexEditor.Destroy;
  begin
    if (FileName <> '') and Changed {FCopyBlock <> GetBlock} and { save before exit }
      (WinProcs.MessageBox(GetFocus,
               'Save Block to File?', 'Close', MB_ICONQUESTION or MB_YESNO) = IDYES) then
       SetBlock(FCopyBlock);
    inherited Destroy
  end {Destroy};

  procedure TBHexEditor.SetOffset(AnOffset: LongInt);
  begin
    if (FileName <> '') and Changed {FCopyBlock <> GetBlock} and { save before exit }
      (MessageDlg('Save Block to File?', mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
       SetBlock(FCopyBlock);
    inherited SetOffset(AnOffset);
    FCopyBlock := GetBlock;
    FChanged := False
  end {SetOffset};

  procedure TBHexEditor.SetFileName(AFileName: TFileName);
  begin
    if (FileName <> '') and Changed {FCopyBlock <> GetBlock} and { save before exit }
      (MessageDlg('Save Block to File?', mtConfirmation, [mbYes, mbNo], 0) = mrYes) then
       SetBlock(FCopyBlock);
    FChanged := False;
    inherited SetFileName(AFileName);
    { which calls SetOffset }
  end {SetFileName};

  function TBHexEditor.GetEditLimit: Integer;
  begin
    GetEditLimit := 2
  end {GetEditLimit};

  procedure TBHexEditor.SetEditText(ACol, ARow: Longint; const Value: string);
  var i,j,k: Integer;
      Line: String;
  begin
    if (ACol in [1..16]) and (ARow in [1..16]) and (Value <> '') then
    begin
      FChanged := True;
      k := Pred(ARow) * BlockChar;
      i := FCopyBlock[k + ACol];
      try
        if FileName = '' then raise Exception.Create(FileName);
        FCopyBlock[k + ACol] := StrToInt('$'+Value);
        inherited SetEditText(ACol, ARow, Value);
        Line := '';
        for j:=1 to BlockChar do
        begin
          Inc(k);
          if k <= Size then if FCopyBlock[k] < 32 then Line := Line + ' '
                                                  else Line := Line + Chr(FCopyBlock[k])
        end;
        Cells[17,ARow] := Line
      except
        FCopyBlock[k + ACol] := i;
        MessageBeep($FFFF)
      end
    end
    else
      inherited SetEditText(ACol, ARow, Value)
  end {SetEditText};

  function TBHexEditor.GetBlock: TBlock;
  begin
    GetBlock := FBlock
  end {GetBlock};

  procedure TBHexEditor.SetBlock(Const ABlock: TBlock);
  begin
    FBlock := ABlock;
    Seek(FFile,FOffset);
    BlockWrite(FFile,FBlock,FSize)
  end {SetBlock};
end.
