unit TlHelp16;

interface

//Copied relevant stuff from Delphi 1's ToolHelp.Pas

//THandle changed to THandle16 (it's 32-bit in Win32)

//Bool (LongBool in Delphi 2) changed to WordBool (as it is in Delphi 1)

//Records turned to packed records with $Align Off

//EBP stack frames generated with $StackFrames On

//Initialisation and finalisation secion used to pre-allocated
//and tidy away memory buffers for one of each types of structure
//(the routines may be called lots. Doing it once off is much quicker)

//DLL loaded in initialisation section

//Thunktions written

uses
  QTThunkU;

{$Align Off}
{$StackFrames On}

{ Generic stuff for 16-bit }
type
  THandle = THandle16;
  Bool = WordBool;

{ General symbols }
const
  max_Path = 255;
  max_Module_Name = 8 + 1;

{ Module list walking }
type
  PModuleEntry = ^TModuleEntry;
  TModuleEntry = record
    dwSize: Longint;
    szModule: array[0..max_Module_Name] of Char;
    hModule: THandle;
    wUsageFlags: Word;
    szExePath: array[0..max_Path] of Char;
    wNext: Word;
  end;

function ModuleFirst(lpModule: PModuleEntry): Bool;
function ModuleNext(lpModule: PModuleEntry): Bool;
function ModuleFindName(lpModule: PModuleEntry; lpstrName: PChar): THandle;
function ModuleFindHandle(lpModule: PModuleEntry; hModule: THandle): THandle;

{ Task list walking }
type
  PTaskEntry = ^TTaskEntry;
  TTaskEntry = record
    dwSize: Longint;
    hTask: THandle;
    hTaskParent: THandle;
    hInst: THandle;
    hModule: THandle;
    wSS: Word;
    wSP: Word;
    wStackTop: Word;
    wStackMinimum: Word;
    wStackBottom: Word;
    wcEvents: Word;
    hQueue: THandle;
    szModule: array[0..max_Module_Name] of Char;
    wPSPOffset: Word;
    hNext: THandle;
  end;

function TaskFirst(lpTask: PTaskEntry): Bool;
function TaskNext(lpTask: PTaskEntry): Bool;
function TaskFindHandle(lpTask: PTaskEntry; hTask: THandle): Bool;

{ Local heap walking }
type
  PLocalEntry = ^TLocalEntry;
  TLocalEntry = record
    dwSize: Longint;
    hHandle: THandle;
    wAddress: Word;
    wSize: Word;
    wFlags: Word;
    wcLock: Word;
    wType: Word;
    hHeap: Word;
    wHeapType: Word;
    wNext: Word;
  end;

function LocalFirst(lpLocal: PLocalEntry; hHeap: THandle): Bool;
function LocalNext(lpLocal: PLocalEntry): Boolean;

{ Information function }
type
  PSysHeapInfo = ^TSysHeapInfo;
  TSysHeapInfo = record
    dwSize: Longint;
    wUserFreePercent: Word;
    wGDIFreePercent: Word;
    hUserSegment: THandle;
    hGDISegment: THandle;
  end;

function SystemHeapInfo(lpSysHeap: PSysHeapInfo): Bool;

implementation

uses
  Windows, SysUtils;

var
  ToolHelp16Handle: Windows.THandle;
  ProcAddress,
  NamePtr32Bit, NamePtr16Bit: Pointer;
  MEPtr32Bit, MEPtr16Bit: PModuleEntry;
  TEPtr32Bit, TEPtr16Bit: PTaskEntry;
  LEPtr32Bit, LEPtr16Bit: PLocalEntry;
  SHIPtr32Bit, SHIPtr16Bit: PSysHeapInfo;

function ModuleFirst(lpModule: PModuleEntry): Bool;
var
  EatStackSpace: String[$3C];
begin
  // Ensure buffer isn't optimised away
  EatStackSpace := '';
  ProcAddress := GetProcAddress16(ToolHelp16Handle, 'ModuleFirst');
  if Assigned(ProcAddress) then
  begin
    MEPtr32Bit^ := lpModule^;
    asm
      push  MEPtr16Bit
      mov   edx, ProcAddress
      call  QT_Thunk
      mov   Result, ax
    end;
    lpModule^ := MEPtr32Bit^;
  end;
end;

function ModuleNext(lpModule: PModuleEntry): Bool;
var
  EatStackSpace: String[$3C];
begin
  // Ensure buffer isn't optimised away
  EatStackSpace := '';
  ProcAddress := GetProcAddress16(ToolHelp16Handle, 'ModuleNext');
  if Assigned(ProcAddress) then
  begin
    MEPtr32Bit^ := lpModule^;
    asm
      push  MEPtr16Bit
      mov   edx, ProcAddress
      call  QT_Thunk
      mov   Result, ax
    end;
    lpModule^ := MEPtr32Bit^;
  end;
end;

function ModuleFindName(lpModule: PModuleEntry; lpstrName: PChar): THandle;
var
  EatStackSpace: String[$3C];
begin
  // Ensure buffer isn't optimised away
  EatStackSpace := '';
  ProcAddress := GetProcAddress16(ToolHelp16Handle, 'ModuleFindName');
  if Assigned(ProcAddress) then
  begin
    MEPtr32Bit^ := lpModule^;
    //Get and fill 16-bit buffer with source data
    NamePtr16Bit := GlobalAllocPointer16(GPTR, max_Module_Name,
      NamePtr32Bit, lpstrName^, StrLen(lpstrName));
    asm
      push  MEPtr16Bit
      push  NamePtr16Bit
      mov   edx, ProcAddress
      call  QT_Thunk
      mov   Result, ax
    end;
    lpModule^ := MEPtr32Bit^;
    GlobalFreePtr16(NamePtr16Bit);
  end;
end;

function ModuleFindHandle(lpModule: PModuleEntry; hModule: THandle): THandle;
var
  EatStackSpace: String[$3C];
begin
  // Ensure buffer isn't optimised away
  EatStackSpace := '';
  ProcAddress := GetProcAddress16(ToolHelp16Handle, 'ModuleFindHandle');
  if Assigned(ProcAddress) then
  begin
    MEPtr32Bit^ := lpModule^;
    asm
      push  MEPtr16Bit
      push  hModule
      mov   edx, ProcAddress
      call  QT_Thunk
      mov   Result, ax
    end;
    lpModule^ := MEPtr32Bit^;
  end;
end;

function TaskFirst(lpTask: PTaskEntry): Bool;
var
  EatStackSpace: String[$3C];
begin
  // Ensure buffer isn't optimised away
  EatStackSpace := '';
  ProcAddress := GetProcAddress16(ToolHelp16Handle, 'TaskFirst');
  if Assigned(ProcAddress) then
  begin
    TEPtr32Bit^ := lpTask^;
    asm
      push  TEPtr16Bit
      mov   edx, ProcAddress
      call  QT_Thunk
      mov   Result, ax
    end;
    lpTask^ := TEPtr32Bit^;
  end;
end;

function TaskNext(lpTask: PTaskEntry): Bool;
var
  EatStackSpace: String[$3C];
begin
  // Ensure buffer isn't optimised away
  EatStackSpace := '';
  ProcAddress := GetProcAddress16(ToolHelp16Handle, 'TaskNext');
  if Assigned(ProcAddress) then
  begin
    TEPtr32Bit^ := lpTask^;
    asm
      push  TEPtr16Bit
      mov   edx, ProcAddress
      call  QT_Thunk
      mov   Result, ax
    end;
    lpTask^ := TEPtr32Bit^;
  end;
end;

function TaskFindHandle(lpTask: PTaskEntry; hTask: THandle): Bool;
var
  EatStackSpace: String[$3C];
begin
  // Ensure buffer isn't optimised away
  EatStackSpace := '';
  ProcAddress := GetProcAddress16(ToolHelp16Handle, 'TaskFindHandle');
  if Assigned(ProcAddress) then
  begin
    TEPtr32Bit^ := lpTask^;
    asm
      push  TEPtr16Bit
      push  hTask
      mov   edx, ProcAddress
      call  QT_Thunk
      mov   Result, ax
    end;
    lpTask^ := TEPtr32Bit^;
  end;
end;

function LocalFirst(lpLocal: PLocalEntry; hHeap: THandle): Bool;
var
  EatStackSpace: String[$3C];
begin
  // Ensure buffer isn't optimised away
  EatStackSpace := '';
  ProcAddress := GetProcAddress16(ToolHelp16Handle, 'LocalFirst');
  if Assigned(ProcAddress) then
  begin
    LEPtr32Bit^ := lpLocal^;
    asm
      push  LEPtr16Bit
      push  hHeap
      mov   edx, ProcAddress
      call  QT_Thunk
      mov   Result, ax
    end;
    lpLocal^ := LEPtr32Bit^;
  end;
end;

function LocalNext(lpLocal: PLocalEntry): Boolean;
var
  EatStackSpace: String[$3C];
begin
  // Ensure buffer isn't optimised away
  EatStackSpace := '';
  ProcAddress := GetProcAddress16(ToolHelp16Handle, 'LocalNext');
  if Assigned(ProcAddress) then
  begin
    LEPtr32Bit^ := lpLocal^;
    asm
      push  LEPtr16Bit
      mov   edx, ProcAddress
      call  QT_Thunk
      mov   Result, al
    end;
    lpLocal^ := LEPtr32Bit^;
  end;
end;

function SystemHeapInfo(lpSysHeap: PSysHeapInfo): Bool;
var
  EatStackSpace: String[$3C];
begin
  // Ensure buffer isn't optimised away
  EatStackSpace := '';
  ProcAddress := GetProcAddress16(ToolHelp16Handle, 'SystemHeapInfo');
  if Assigned(ProcAddress) then
  begin
    SHIPtr32Bit^ := lpSysHeap^;
    asm
      push  SHIPtr16Bit
      mov   edx, ProcAddress
      call  QT_Thunk
      mov   Result, ax
    end;
    lpSysHeap^ := SHIPtr32Bit^;
  end;
end;

initialization
  MEPtr16Bit := GlobalAllocPtr16(GPTR, SizeOf(TModuleEntry));
  MEPtr32Bit := Ptr16To32(MEPtr16Bit);
  TEPtr16Bit := GlobalAllocPtr16(GPTR, SizeOf(TTaskEntry));
  TEPtr32Bit := Ptr16To32(TEPtr16Bit);
  LEPtr16Bit := GlobalAllocPtr16(GPTR, SizeOf(TLocalEntry));
  LEPtr32Bit := Ptr16To32(LEPtr16Bit);
  SHIPtr16Bit := GlobalAllocPtr16(GPTR, SizeOf(TSysHeapInfo));
  SHIPtr32Bit := Ptr16To32(SHIPtr16Bit);
  ToolHelp16Handle := LoadLibrary16('TOOLHELP.DLL');
finalization
  GlobalFreePtr16(MEPtr16Bit);
  GlobalFreePtr16(TEPtr16Bit);
  GlobalFreePtr16(LEPtr16Bit);
  GlobalFreePtr16(SHIPtr16Bit);
  FreeLibrary16(ToolHelp16Handle);
end.
