unit fmMain;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls, Dialogs,
  StdCtrls, Forms, DBCtrls, DB, DBGrids, DBTables, Grids, ExtCtrls, Menus;

type
  TfrmMain = class(TForm)
    qryGetEmployeesEMP_NO: TSmallintField;
    qryGetEmployeesFIRST_NAME: TStringField;
    qryGetEmployeesLAST_NAME: TStringField;
    qryGetEmployeesPHONE_EXT: TStringField;
    qryGetEmployeesHIRE_DATE: TDateTimeField;
    qryGetEmployeesDEPT_NO: TStringField;
    qryGetEmployeesJOB_CODE: TStringField;
    qryGetEmployeesJOB_GRADE: TSmallintField;
    qryGetEmployeesJOB_COUNTRY: TStringField;
    qryGetEmployeesSALARY: TFloatField;
    qryGetEmployeesFULL_NAME: TStringField;
    qryGetEmployeesStatus: TStringField;
    dsGetEmployees: TDataSource;
    qryGetEmployees: TQuery;
    dbEmployee: TDatabase;
    grpEditView: TGroupBox;
    Panel2: TPanel;
    grdEmployees: TDBGrid;
    DBNavigator: TDBNavigator;
    btnPost: TButton;
    grpFilters: TGroupBox;
    chkModified: TCheckBox;
    chkInserted: TCheckBox;
    chkDeleted: TCheckBox;
    btnUndo: TButton;
    btnUndoAll: TButton;
    chkUnmodified: TCheckBox;
    btnExit: TButton;
    procedure FormCreate(Sender: TObject);
    procedure btnPostClick(Sender: TObject);
    procedure btnUndoAllClick(Sender: TObject);
    procedure chkFilterClick(Sender: TObject);
    procedure dsGetEmployeesDataChange(Sender: TObject; Field: TField);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure qryGetEmployeesCalcFields(DataSet: TDataSet);
    procedure btnUndoClick(Sender: TObject);
    procedure qryGetEmployeesStartChanges(DataSet: TDataSet);
    procedure qryGetEmployeesEndChanges(DataSet: TDataSet);
    procedure btnExitClick(Sender: TObject);
    procedure qryGetEmployeesUpdateRecord(DataSet: TDataSet;
      UpdateKind: TUpdateKind; var UpdateAction: TUpdateAction);
  private
  public
    ChangingData: Boolean;   { True when user is changing a record and
                               hasn't posted yet }
  end;

var
  frmMain: TfrmMain;

implementation

{$R *.DFM}

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  qryGetEmployees.Open;
  chkFilterClick(nil);
  if not qryGetEmployees.CanModify then
    ShowMessage('dataset is read-only');
end;

procedure TfrmMain.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  with qryGetEmployees do
    if Active and UpdatesPending then
      CanClose := MessageDlg('Do you want to discard your changes?', mtWarning, [mbYes, mbNo], 0) = mrYes;
end;

procedure TfrmMain.chkFilterClick(Sender: TObject);
var
  UpdateFilters: TUpdateRecordTypes;
begin
  UpdateFilters := [];
  if chkUnmodified.Checked then Include(UpdateFilters, rtUnmodified);
  if chkModified.Checked then Include(UpdateFilters, rtModified);
  if chkInserted.Checked then Include(UpdateFilters, rtInserted);
  if chkDeleted.Checked then Include(UpdateFilters, rtDeleted);
  qryGetEmployees.UpdateRecordTypes := UpdateFilters;
  if UpdateFilters = [] then
  begin
    ShowMessage('At least one display filter must be checked');
    UpdateFilters := qryGetEmployees.UpdateRecordTypes;
    chkUnmodified.Checked := (rtUnmodified in UpdateFilters);
    chkModified.Checked := (rtModified in UpdateFilters);
    chkInserted.Checked := (rtInserted in UpdateFilters);
    chkDeleted.Checked := (rtDeleted in UpdateFilters);
  end;
end;

procedure TfrmMain.qryGetEmployeesCalcFields(DataSet: TDataSet);
begin
  { UpdateStatus is for the current record, contrary to what the
    online help says. }
  with qryGetEmployeesStatus do
  begin
    case DataSet.UpdateStatus of
      usUnmodified: AsString := 'Unmodified';
      usModified: AsString := 'Modified';
      usInserted: AsString := 'Inserted';
      usDeleted: AsString := 'Deleted';
    end;
  end;
end;

procedure TfrmMain.dsGetEmployeesDataChange(Sender: TObject;
  Field: TField);
begin
  { Enable/disable the buttons depending on whether there are changes. }
  if ChangingData then btnPost.Enabled := True
  else btnPost.Enabled := qryGetEmployees.UpdatesPending;
  btnUndoAll.Enabled := btnPost.Enabled;
  btnUndo.Enabled := (qryGetEmployees.UpdateStatus <> usUnmodified) or
                     ChangingData;
end;

procedure TfrmMain.qryGetEmployeesStartChanges(DataSet: TDataSet);
begin
  ChangingData := True;
  dsGetEmployeesDataChange(DataSet, nil);
end;

procedure TfrmMain.qryGetEmployeesEndChanges(DataSet: TDataSet);
begin
  ChangingData := False;
  dsGetEmployeesDataChange(DataSet, nil);
end;

procedure TfrmMain.btnPostClick(Sender: TObject);
begin
  dbEmployee.ApplyUpdates([qryGetEmployees]);
end;

procedure TfrmMain.btnUndoClick(Sender: TObject);
begin
  qryGetEmployees.RevertRecord;
end;

procedure TfrmMain.btnUndoAllClick(Sender: TObject);
begin
  qryGetEmployees.CancelUpdates;
end;

procedure TfrmMain.btnExitClick(Sender: TObject);
begin
  Close
end;

procedure TfrmMain.qryGetEmployeesUpdateRecord(DataSet: TDataSet;
  UpdateKind: TUpdateKind; var UpdateAction: TUpdateAction);
begin
  // *
end;

end.
