unit Resform;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics,
  Controls, Forms, Dialogs, ExtCtrls, Gauges,
  StdCtrls, Menus, ShellAPI;

const
  wm_IconMessage = wm_User;

type
  TMemForm = class(TForm)
    RamGauge: TGauge;
    VirtualGauge: TGauge;
    Label1: TLabel;
    Label2: TLabel;
    Timer1: TTimer;
    PopupMenu1: TPopupMenu;
    Details1: TMenuItem;
    About1: TMenuItem;
    N1: TMenuItem;
    Close1: TMenuItem;
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Details1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  public
    procedure IconTray (var Msg: TMessage);
      message wm_IconMessage;
  private
    nid: TNotifyIconData;
    Closing: Boolean;
  end;

var
  MemForm: TMemForm;

implementation

{$R *.DFM}
{$R RYG.RES}

function FmtMem (N: Integer): string;
begin
  if N > 1024*1024 then
    FmtMem := Format ('%.1f MB', [n / (1024*1024)])
  else
    FmtMem := Format ('%.1f KB', [n / 1024])
end;

procedure TMemForm.Timer1Timer(Sender: TObject);
var
  MemInfo : TMemoryStatus;
begin
  MemInfo.dwLength := Sizeof (MemInfo);
  GlobalMemoryStatus (MemInfo);
  RamGauge.Progress := MemInfo.dwAvailPhys div
    (MemInfo.dwTotalPhys div 100);
  VirtualGauge.Progress := MemInfo.dwAvailPageFile div
    (MemInfo.dwTotalPageFile div 100);

  Caption := Format ('Memory: = %d - %d',
    [RamGauge.Progress, VirtualGauge.Progress]);

  Label1.Caption := Format ('RAM:'#13'%s'#13'(%s)',
    [FmtMem (MemInfo.dwAvailPhys),
    FmtMem (MemInfo.dwTotalPhys)]);
  Label2.Caption := Format ('Swap:'#13'%s'#13'(%s)',
    [FmtMem (MemInfo.dwAvailPageFile),
    FmtMem (MemInfo.dwTotalPageFile)]);

  // set icon color
  if RamGauge.Progress > 5 then
    Icon.Handle :=
      LoadIcon (HInstance, 'GREEN')
  else if VirtualGauge.Progress > 20 then
    Icon.Handle :=
      LoadIcon (HInstance, 'YELLOW')
  else
    Icon.Handle :=
      LoadIcon (HInstance, 'RED');

  // update the NotifyIcon structure
  nid.hIcon := Icon.Handle;
  strcopy (nid.szTip, PChar(Caption));
  nid.uFlags := nif_Icon or nif_Tip;
  Shell_NotifyIcon (NIM_MODIFY, @nid);
end;

procedure TMemForm.IconTray (var Msg: TMessage);
var
  Pt: TPoint;
begin
  if Msg.lParam = wm_rbuttondown then
  begin
    GetCursorPos (Pt);
//    SetForegroundWindow (Handle);
    PopupMenu1.Popup (Pt.x, Pt.y);
  end;
  if Msg.lParam = wm_lbuttondblclk then
    Details1Click (self);
end;

procedure TMemForm.FormCreate(Sender: TObject);
begin
  // do *not* allow closing
  Closing := False;

  // load the initial icon
  Icon.Handle :=
    LoadIcon (HInstance, 'GREEN');

  // fill the NotifyIcon data structure
  nid.cbSize := sizeof (nid);
  nid.wnd := Handle;
  nid.uID := 1; // icon ID
  nid.uCallBAckMessage := wm_IconMessage;
  nid.hIcon := Icon.Handle;
  nid.szTip := 'Free memory';
  nid.uFlags := nif_Message or
    nif_Icon or nif_Tip;
  Shell_NotifyIcon (NIM_ADD, @nid);

  // update the output ASAP
  Timer1Timer (self);
end;

procedure TMemForm.FormDestroy(Sender: TObject);
begin
  nid.uFlags := 0;
  Shell_NotifyIcon (NIM_DELETE, @nid);
end;

procedure TMemForm.Details1Click(Sender: TObject);
begin
  ShowWindow (Handle, sw_ShowNormal);
  SetForegroundWindow (Handle);
end;

procedure TMemForm.About1Click(Sender: TObject);
begin
  MessageDlg ('Windows 95 Memory Meter'#13 +
    'Copyright Marco Cant 1996'#13 +
    '(100273.2610@compuserve.com)',
    mtInformation, [mbOk], 0);
end;

procedure TMemForm.Close1Click(Sender: TObject);
begin
  Closing := True;
  Close;
end;

procedure TMemForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if not Closing then
  begin
    Action := caNone;
    ShowWindow (Handle, sw_Hide);
  end;
end;

end.

(*********************************
// (From SHELLAPI.H)
typedef struct _NOTIFYICONDATA {
  DWORD cbSize;           // size of struct, you must set
  HWND hWnd;              // HWND sending notification
  UINT uID;               // ID of icon (callback WPARAM)
  UINT uFlags;            // see below
  UINT uCallbackMessage;  // sent to your wndproc
  HICON hIcon;            // handle of icon
  CHAR szTip[64];         // tip text
} NOTIFYICONDATA;

// uFlags
#define NIF_MESSAGE 0x1   // uCallbackMessage is valid
#define NIF_ICON    0x2   // hIcon is valid
#define NIF_TIP     0x4   // szTip is valid

hWnd is a handle to your window that owns the icon.
uID can be any ID you like that identifies your tray icon (
in case you have more than one). Typically, youll use its
resource ID. hIcon can be a handle to any icon, including
predefined system icons like IDI_HAND, IDI_QUESTION,
IDI_EXCLAMATION, or IDI_WINLOGO, the Windows logo.

Displaying icons is nice, but whats really fun are events.
To receive notification when the user moves the mouse over
or clicks on your tray icon, you can set uCallbackMessage to
your very own message ID, and set the NIF_MESSAGE flag. When
the user moves or clicks the mouse over the icon, Windows
will call your window proc with hWnd equal to your window
handle, specified in hWnd; messageID is the value you
specified in uCallbackMessage; wParam is the value you
specified as uID; and lParam is a mouse event (such as
WM_LBUTTONDOWN).

Shell_NotifyIcon is short, simple, and sweet. But like
most Windows API functions, its a little clunky and
assemblerish. So I encapsulated it in a C++ class,
CTrayIcon (see Figure 1). CTrayIcon hides NOTIFYICONDATA,
message codes, flags, and all that rot. It presents a more
programmer-friendly interface to tray icons. But CTrayIcon
is more than just a wrapper for Shell_NotifyIconits a
miniframework. It enforces the correct user interface
behavior for tray icons, as per the Windows Interface
Guidelines for Software (available on MSDN). Heres the
executive sumary:
	Tray icons should have tooltips.
	Right-click should display a popup menu with
commands that bring up property sheets or other windows
related to the icon.
	Left-double-click should execute the default
command in this menu.
	Left-click should display further information
or controls for the object represented by the tray icon.
(For example, the volume control when you left-click the
sound icon.) If theres no information or control that
applies, dont do anything.
*********************************)
