library ReportEX;
uses
  ShareMem, ExptIntf, ToolIntf, Menus, Forms, Dialogs, Classes,
  ReportFM;

{ EXPERT TYPE DEFINITION }

Type
  TBReportExpert = class(TIExpert)
  public
    constructor Create; virtual;
    destructor Destroy; override;

    function GetStyle: TExpertStyle; override;
    function GetIDString: String; override;
    function GetName: String; override;
    function GetAuthor: String; override;

  protected
    procedure OnClick(Sender: TIMenuItemIntf); virtual;

  private
    MenuItem: TIMenuItemIntf;
  end {TBReportExpert};

{ EXPERT SUPPORT FUNCTION }

procedure HandleException;
begin
  if Assigned(ToolServices) then
    ToolServices.RaiseException(ReleaseException)
  { Aplication.Handle := ToolServices.GetParentHandle; }
end {HandleException};

{ EXPERT IMPLEMENTATION }

constructor TBReportExpert.Create;
var Main: TIMainMenuIntf;
    ToolsTools: TIMenuItemIntf;
    Tools: TIMenuItemIntf;
begin
  inherited Create;
  MenuItem := nil;
  if ToolServices <> nil then
  try
    Main := ToolServices.GetMainMenu;
    if Main <> nil then { we've got the main menu }
    try
      ToolsTools := Main.FindMenuItem({'ViewPrjMgrItem}'ToolsGalleryItem');
      if ToolsTools <> nil then { we've got the " Tools | Tools" item }
      try
        Tools := ToolsTools.GetParent;
        if Tools <> nil then { we've got the Tools menu }
        try
          MenuItem := Tools.InsertItem(ToolsTools.GetIndex+1,
                                      '&Error Reports',
                                       'BERT','',
                                        ShortCut(Ord('E'),[ssCtrl]),0,0,
                                       [mfEnabled, mfVisible], OnClick)
        finally
          Tools.DestroyMenuItem
        end
      finally
        ToolsTools.DestroyMenuItem
      end
    finally
      Main.Free
    end
  except
    HandleException
  end
end {Create};

destructor TBReportExpert.Destroy;
begin
  try
    if MenuItem <> nil then MenuItem.DestroyMenuItem;
    inherited Destroy
  except
    HandleException
  end
end {Destroy};

function TBReportExpert.GetStyle: TExpertStyle;
begin
  try
    Result := esAddIn
  except
    HandleException
  end
end {GetStyle};

function TBReportExpert.GetIDString: String;
begin
  try
    Result := 'Bolesian.Error.Report.Tool'
  except
    HandleException
  end
end {GetIDString};

function TBReportExpert.GetName: String;
begin
  try
    Result := 'Bolesian.Error.Report.Tool'
  except
    HandleException
  end
end {GetName};

function TBReportExpert.GetAuthor: String;
begin
  try
    Result := 'Bob.Swart'
  except
    HandleException
  end
end {GetAuthor};


procedure TBReportExpert.OnClick(Sender: TIMenuItemIntf);
begin
  try
    with TReportForm.Create(Application) do
    try
      ShowModal
    finally
      Free
    end
  except
    HandleException
  end
end {OnClick};

{ DLL EXPERT INTERFACE }

procedure DoneExpert;
begin
{ ShowMessage(ParamStr(0)+' unloaded!') }
end {DoneExpert};

function InitExpert(ToolServices: TIToolServices;
                    RegisterProc: TExpertRegisterProc;
                var Terminate: TExpertTerminateProc): Boolean; stdcall;
begin
  try
    Result := True;
    ExptIntf.ToolServices := ToolServices; { Save! }
    if ToolServices <> nil then
      Application.Handle := ToolServices.GetParentHandle;
    Terminate := DoneExpert;
    Result := RegisterProc(TBReportExpert.Create);
  except
    HandleException
  end
end {InitExpert};

exports
  InitExpert name ExpertEntryPoint;

begin
{ ShowMessage(ParamStr(0)+' loaded!') }
end.
