{ Use TStringGrid }

unit Nwworld1;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, Grids;

type
  TFrmNewWorld1 = class(TForm)
    Grid: TStringGrid;
    BtnClose: TBitBtn;
    BtnMerge: TBitBtn;
    BtnSplit: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure GridMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
  protected
    FSortCol: Longint;
    FSortDesc: Boolean;
  public
    procedure SetHeading;
    procedure AddRecords;
    procedure AddRecord(Country: String; Area, Population: Longint);
    procedure ColumnClicked(Col: Longint);
    procedure SortGrid(ACol: Longint; Desc: Boolean);
    procedure GridExchange(Grid: TStringGrid; row0, row1: Longint);
  end;


var
  FrmNewWorld1: TFrmNewWorld1;

implementation

{$R *.DFM}

function FormatNum(n: Longint): String;
var l: Integer;
begin
  Result := IntToStr(n);
  l := Length(Result);
  if l > 3 then
    Result := Copy(Result,1,l-3) + ThousandSeparator + Copy(Result,l-2,3);
  if l > 6 then
    Result := Copy(Result,1,l-6) + ThousandSeparator + Copy(Result,l-5,7);
  if l > 9 then
    Result := Copy(Result,1,l-9) + ThousandSeparator + Copy(Result,l-8,11);
end;

procedure TFrmNewWorld1.SetHeading;
begin
  with Grid do
  begin
    Cells[1,0] := 'Country';
    Cells[2,0] := 'Area (1000 sq km)';
    Cells[3,0] := 'Population';
  end;
end;

procedure TFrmNewWorld1.AddRecord(Country: String; Area, Population: Longint);
var r: integer;
begin
  with Grid do
  begin
    r := RowCount;
    Cells[1,r] := Country;
    Cells[2,r] := FormatNum(Area);
    Cells[3,r] := FormatNum(Population);
    RowCount := r+1;
  end;
end;

procedure TFrmNewWorld1.AddRecords;
begin
  AddRecord('Canada',           9976,   25963000);
  AddRecord('China',            9597, 1072200000);
  AddRecord('Cuba',              110,   10462000);
  AddRecord('Eqypt',            1002,   50000000);
  AddRecord('Germany',           357,   77864000);
  AddRecord('India',            3268,  783940000);
  AddRecord('Iraq',              438,   17093000);
  AddRecord('Ireland',            70,    3700000);
  AddRecord('Israel',             21,    4399000);
  AddRecord('Japan',             370,  122053000);
  AddRecord('Russia',          17075,  145311000);
  AddRecord('Switzerland',        41,    6586000);
  AddRecord('Syria',             185,   10969000);
  AddRecord('United Kingdom',    244,   56878000);
  AddRecord('United States',    9363,  243773000);
end;

procedure TFrmNewWorld1.FormCreate(Sender: TObject);
begin
  Grid.RowCount := 1;   { Clear the grid. VCL reset FixedRows to 0 }
  Grid.ColWidths[0] := 8;
  SetHeading;
  AddRecords;
  Grid.FixedRows := 1;
end;

procedure TFrmNewWorld1.GridMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var ACol, ARow: Longint;
begin
  Grid.MouseToCell(X, Y, ACol, ARow);
  if ARow < Grid.FixedRows then            { clicked on header row? }
    ColumnClicked(ACol);
end;

procedure TFrmNewWorld1.ColumnClicked(Col: Longint);
begin
  { toggle between descending and ascending order }
  if (Col = FSortCol) and FSortDesc then
    FSortDesc := False
  else begin
    FSortCol := Col;
    FSortDesc := True;
  end;
  SortGrid(FSortCol, FSortDesc);
end;

{ Exchange two rows of a TStringGrid }
procedure TFrmNewWorld1.GridExchange(Grid: TStringGrid; row0, row1: Longint);
var
  i: Longint;
  s: String;
begin
  with Grid do
    for i := 1 to ColCount do
    begin
      s := Cells[i,row0];
      Cells[i,row0] := Cells[i,row1];
      Cells[i,row1] := s;
    end;
end;

procedure TFrmNewWorld1.SortGrid(ACol: Longint; Desc: Boolean);
var i,j: Longint;
begin
  with Grid do
    { bubble sort }
    for i := 1 to RowCount-1-FixedRows do      { Remember to minus FixedRows! }
    begin
      for j := RowCount-FixedRows downto i+1 do
      begin
        if Desc and (Cells[ACol,j] > Cells[ACol,j-1]) then
          GridExchange(Grid,j,j-1);
        if (not Desc) and (Cells[ACol,j] < Cells[ACol,j-1]) then
          GridExchange(Grid,j,j-1);
      end;
    end;
end;

end.
