{ Use TDrawGrid }

unit Nwworld2;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, Grids;

type
  TCountry = Class
    Name: String;
    Area, Population: Longint;
    Selected: Boolean;
    constructor Create(AName: String; AArea, APopulation: Longint);
  end;

  TCountryList = Class(TList)
  private
    function Get(index: Integer): TCountry;
    procedure Put(Index: Integer; Item: TCountry);
  public
    procedure Sort(ACol: Longint; Desc: Boolean);
    property Items[Index: Integer]: TCountry read Get write Put;
  end;

  TFrmNewWorld2 = class(TForm)
    BtnClose: TBitBtn;
    BtnMerge: TBitBtn;
    BtnSplit: TBitBtn;
    Grid: TDrawGrid;
    procedure FormCreate(Sender: TObject);
    procedure GridMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure GridDrawCell(Sender: TObject; Col, Row: Longint; Rect: TRect;
      State: TGridDrawState);
    procedure GridMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure GridKeyPress(Sender: TObject; var Key: Char);
    procedure BtnMergeClick(Sender: TObject);
    procedure BtnSplitClick(Sender: TObject);
  private
    { Private declarations }
  protected
    FSortCol: Longint;
    FSortDesc: Boolean;
    Countries: TCountryList;
  public
    procedure AddRecords;
    procedure AddRecord(Name: String; Area, Population: Longint);
    procedure ColumnClicked(Col: Longint);
  end;

var
  FrmNewWorld2: TFrmNewWorld2;

implementation

{$R *.DFM}

uses MergeFrm, SplitFrm;

constructor TCountry.Create(AName: String; AArea, APopulation: Longint);
begin
  Name      := AName;
  Area      := AArea;
  Population:= APopulation;
end;

function TCountryList.Get(index: Integer): TCountry;
begin
  Result := TCountry(inherited Get(Index));
end;

procedure TCountryList.Put(Index: Integer; Item: TCountry);
begin
  inherited Put(Index,Item);
end;

procedure TCountryList.Sort(ACol: Longint; Desc: Boolean);
var
  i,j: Longint;
  a,b: TCountry;
  greater: Boolean;
begin
  { bubble sort }
  for i := 1 to Count-1 do
  begin
    for j := Count downto i+1 do
    begin
      a := Items[j-2];
      b := Items[j-1];
      case ACol of
        1: greater := a.Name > b.Name;
        2: greater := a.Area > b.Area;
        3: greater := a.Population > b.Population;
      end;
      if (Desc and not greater) or (not Desc and greater) then
        Exchange(j-1,j-2);
    end;
  end;
end;

function FormatNum(n: Longint): String;
var l: Integer;
begin
  Result := IntToStr(n);
  l := Length(Result);
  if l > 3 then
    Result := Copy(Result,1,l-3) + ThousandSeparator + Copy(Result,l-2,3);
  if l > 6 then
    Result := Copy(Result,1,l-6) + ThousandSeparator + Copy(Result,l-5,7);
  if l > 9 then
    Result := Copy(Result,1,l-9) + ThousandSeparator + Copy(Result,l-8,11);
end;

procedure TFrmNewWorld2.AddRecord(Name: String; Area, Population: Longint);
begin
  Countries.Add(TCountry.Create(Name,Area,Population));
end;

procedure TFrmNewWorld2.AddRecords;
begin
  AddRecord('Canada',           9976,   25963000);
  AddRecord('China',            9597, 1072200000);
  AddRecord('Cuba',              110,   10462000);
  AddRecord('Eqypt',            1002,   50000000);
  AddRecord('Germany',           357,   77864000);
  AddRecord('India',            3268,  783940000);
  AddRecord('Iraq',              438,   17093000);
  AddRecord('Ireland',            70,    3700000);
  AddRecord('Israel',             21,    4399000);
  AddRecord('Japan',             370,  122053000);
  AddRecord('Russia',          17075,  145311000);
  AddRecord('Switzerland',        41,    6586000);
  AddRecord('Syria',             185,   10969000);
  AddRecord('United Kingdom',    244,   56878000);
  AddRecord('United States',    9363,  243773000);
end;

procedure TFrmNewWorld2.FormCreate(Sender: TObject);
begin
  Countries := TCountryList.Create;
  Grid.ColWidths[0] := 8;
  AddRecords;
  Grid.RowCount := Countries.Count+1;
end;

procedure TFrmNewWorld2.FormDestroy(Sender: TObject);
var i: Integer;
begin
  for i:= Countries.Count-1 downto 0 do
    Countries.Items[i].Free;
  Countries.Free;
end;

procedure TFrmNewWorld2.ColumnClicked(Col: Longint);
begin
  { toggle between descending and ascending order }
  if (Col = FSortCol) and FSortDesc then
    FSortDesc := False
  else begin
    FSortCol := Col;
    FSortDesc := True;
  end;
  Countries.Sort(FSortCol, FSortDesc);
  Grid.Invalidate;
end;

procedure DrawCellText(Sender: TDrawGrid; const Text: String;
  ACol, ARow: Longint; ARect: TRect; State: TGridDrawState;
  Alignment: TAlignment);
var
  x,w: Integer;
begin
  with Sender do
  begin
    if (ACol >= FixedCols) and (ARow >= FixedRows) then
      if gdSelected in State then
      begin
        Canvas.Brush.Color := clHighlight;
        Canvas.Font.Color := clHighlightText;
      end else
      begin
        Canvas.Brush.Color := Color;
        Canvas.Font.Color := clWindowText;
      end;

    w := Canvas.TextWidth(Text);
    case Alignment of
      taLeftJustify:  x := ARect.Left + 2;
      taCenter:       x := (ARect.Left + ARect.Right - w) div 2;
      taRightJustify: x := ARect.Right - w - 2;
    end;
    ExtTextOut(Canvas.Handle,
      x, ARect.Top + 2, ETO_CLIPPED or
      ETO_OPAQUE, @ARect, @(Text[1]), Length(Text), nil);
  end;
end;

procedure TFrmNewWorld2.GridDrawCell(Sender: TObject; Col, Row: Longint;
  Rect: TRect; State: TGridDrawState);
var
  Text: String;
  Alignment: TAlignment;
begin
  { Assign text to display }
  Text := '';
  if Row = 0 then                       { header row }
    case Col of
      1: Text := 'Country';
      2: Text := 'Area (1000 sq km)';
      3: Text := 'Population';
    end
  else                                  { data rows }
    with Countries.Items[Row-1] do
    begin
      if Selected then
        Include(State,gdSelected)
      else
        Exclude(State,gdSelected);
      case Col of
        1: Text := Name;
        2: Text := FormatNum(Area);
        3: Text := FormatNum(Population);
      end;
    end;

  { Set text alignment }
  Alignment := taLeftJustify;
  if Col in [2,3] then
    Alignment := taRightJustify;

  DrawCellText(Sender as TDrawGrid, Text, Col, Row, Rect, State, Alignment);
end;

procedure TFrmNewWorld2.GridMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var ACol, ARow: Longint;
begin
  Grid.MouseToCell(X, Y, ACol, ARow);
  if ARow < Grid.FixedRows then            { clicked on header row? }
    ColumnClicked(ACol);
end;

procedure TFrmNewWorld2.GridMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  ACol, ARow: Longint;
  i: Longint;
begin
  with Grid do
  begin
    MouseToCell(X, Y, ACol, ARow);
    if (ARow >= FixedRows) and (ARow >= FixedRows) then
    begin
      if ssCtrl in Shift then
        with Countries.Items[Grid.Row-1] do
          selected := not selected
      else begin
        for i := 0 to Countries.Count-1 do
          with Countries.Items[i] do
            selected := i = ARow-1;
      end;
      Invalidate;
    end;
  end;
end;

procedure TFrmNewWorld2.GridKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = ' ' then
  begin
    with Countries.Items[Grid.Row-1] do
      selected := not selected;
    Grid.Invalidate;
  end;
end;

procedure TFrmNewWorld2.BtnMergeClick(Sender: TObject);
var
  i: Longint;
  NewCountry: TCountry;
begin
  NewCountry := TCountry.Create('',0,0);

  for i := 0 to Countries.Count-1 do
    with Countries.Items[i] do
      if Selected then
      begin
        NewCountry.Name       := NewCountry.Name + Name;
        NewCountry.Area       := NewCountry.Area + Area;
        NewCountry.Population := NewCountry.Population + Population;
      end;

  FrmMerge.FCountry := NewCountry;
  if FrmMerge.ShowModal = mrOk then
  begin
    Countries.Add(NewCountry);
    for i := Countries.Count-1 downto 0 do
    begin
      if Countries.Items[i].Selected then
      begin
        Countries.Items[i].Free;
        Countries.Delete(i);
      end;
    end;
  end;

  Grid.RowCount := Countries.Count+1;
  Grid.Invalidate;
end;

procedure TFrmNewWorld2.BtnSplitClick(Sender: TObject);
var
  i: Longint;
  Country, Country1, Country2: TCountry;
begin
  for i := 0 to Countries.Count-1 do
    with Countries.Items[i] do
      if Selected then
      begin
        Country1 := TCountry.Create(Name+'1', Area div 2, Population div 2);
        Country2 := TCountry.Create(Name+'2', Area div 2, Population div 2);
        break;
      end;

  FrmSplit.FCountry1 := Country1;
  FrmSplit.FCountry2 := Country2;
  if FrmSplit.ShowModal = mrOk then
  begin
    Countries.Items[i].Free;
    Countries.Delete(i);
    Countries.Add(Country1);
    Countries.Add(Country2);
  end;

  Grid.RowCount := Countries.Count+1;
  Grid.Invalidate;
end;

end.
