unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    PageControl1: TPageControl;
    PaperInfo: TTabSheet;
    ListBox1: TListBox;
    Gen: TTabSheet;
    ListBox2: TListBox;
    Label4: TLabel;
    Label5: TLabel;
    PrinterCombo: TComboBox;
    Characteristics: TTabSheet;
    DevCapsLB: TListBox;
    TabSheet1: TTabSheet;
    RasterCapLB: TListBox;
    TabSheet2: TTabSheet;
    CurveCapLB: TListBox;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    TabSheet5: TTabSheet;
    LineCapLB: TListBox;
    PolyCapLB: TListBox;
    TextCapLB: TListBox;
    GenDataLB: TListBox;
    procedure FormCreate(Sender: TObject);
    procedure PrinterComboChange(Sender: TObject);
  private
    Device, Driver, Port: array[0..255] of char;
    ADevMode: THandle;
   public
    procedure GetBinNames;
    procedure GetDuplexSupport;
    procedure GetCopies;
    procedure GetEMFStatus;
    procedure GetResolutions;
    procedure GetTrueTypeInfo;
    procedure GetCasePaperNames;
    procedure GetDevCapsPaperNames;
    procedure GetDevCaps;
    procedure GetRasterCaps;
    procedure GetCurveCaps;
    procedure GetLineCaps;
    procedure GetPolyCaps;
    procedure GetTextCaps;
  end;

var
  Form1: TForm1;

{ DeviceCapabilitiesA exists in WINSPOOL.DRV although you'll find that most
  documentation says that it exists in GDI32.DLL.  }
function DeviceCapabilitiesA(pDevice, pPort: Pchar; fwCapability: Word;
         pOutput: Pchar; DevMode: PdeviceMode):
         Integer; stdcall; external 'winspool.drv';

implementation
uses
  Printers;

type

  { Types for holding bin names }
  TBinName = array[0..23] of char;
  TBinNames = array[0..0] of TBinName;  // Where used set $R- to prevent error

  { Types for holding paper names }
  TPName = array[0..63] of char;
  TPNames = array[0..0] of TPName;  // Where used set $R- to prevent error

  { Types for holding resolutions }
  TResolution = array[0..1] of integer;
  TResolutions = array[0..0] of TResolution;  // Where used set $R- to prevent error

  { Type for holding array of pages sizes (word types) }
  TPageSizeArray = Array[0..0] of word;   // Where used set $R- to prevent error

var
 Rslt: Integer;

{$R *.DFM}

procedure TForm1.GetBinNames;
var
  BinNames: Pointer;
  i: integer;
begin
{$R-} // Range checking must be turned off here.
  { First determine how many bin names are available. }
  Rslt := DeviceCapabilitiesA(Device, Port, DC_BINNAMES, nil, nil);
  if Rslt > 0 then begin
    { Each bin name is 24 bytes long. Therefore, allocate Rslt*24 bytes to hold
      the bin names. }
    GetMem(BinNames, Rslt*24);
    try
      { Now retrieve the bin names in the allocated block of memory. }
      if DeviceCapabilitiesA(Device, Port, DC_BINNAMES, BinNames, nil) = -1 then
        raise Exception.Create('DevCap Error');
      { Add the information to the appropriate list box. }
      GenDataLB.Items.Add('BIN NAMES:');
      for i := 0 to Rslt - 1 do
        GenDataLB.Items.Add('    '+StrPas(TBinNames(BinNames^)[i]));
    finally
      FreeMem(BinNames, Rslt*24); // Don't forget to free the allocated memory.
    end;
  end;
{$R+} // Turn range checking back on. 
end;

procedure TForm1.GetDuplexSupport;
begin
  { This function uses DeviceCapabilitiesA to determine whether or not the
    printer device supports duplex printing. }
  Rslt := DeviceCapabilitiesA(Device, Port, DC_DUPLEX, nil, nil);
  if Rslt = 1 then
    GenDataLB.Items.Add('Duplex Printing: Yes')
  else // Rslt should be zero.
    GenDataLB.Items.Add('Duplex Printing: No')
end;

procedure TForm1.GetCopies;
begin
  { This function determines how many copies the device can be set to print.
    if the result is not greater then 1 then the print logic must be
    executed multiple times }
  Rslt := DeviceCapabilitiesA(Device, Port, DC_COPIES, nil, nil);
  if Rslt > 0 then // Some devices report a 0 value.
    GenDataLB.Items.Add('Copies that printer can print: '+IntToStr(Rslt))
end;

procedure TForm1.GetEMFStatus;
begin
  { This function determines if the device supports the enhanced metafiles. }
  Rslt := DeviceCapabilitiesA(Device, Port, DC_EMF_COMPLIANT, nil, nil);
  if Rslt = 1 then
    GenDataLB.Items.Add('EMF Compliant: Yes')
  else if Rslt = -1 then
    GenDataLB.Items.Add('EMF Compliant: No')
end;

procedure TForm1.GetResolutions;
var
  Resolutions: Pointer;
  i: integer;
begin
{$R-}  // Range checking must be turned off.
{ Determine how many resolutions are available. }
  Rslt := DeviceCapabilitiesA(Device, Port, DC_ENUMRESOLUTIONS, nil, nil);
  if Rslt > 0 then begin
    { Allocate the memory to hold the different resolutions which are
      represented by integer pairs, ie: 300, 300 }
    GetMem(Resolutions, (SizeOf(Integer)*2)*Rslt);
    try
      { Retrieve the different resolutions. }
      if DeviceCapabilitiesA(Device, Port, DC_ENUMRESOLUTIONS,
      Resolutions, nil) = -1 then
        Raise Exception.Create('DevCaps Error');
      { Add the resolution information to the appropriate list box. }
      GenDataLB.Items.Add('RESOLUTION CONFIGURATIONS: '+IntToStr(Rslt));
      for i := 0 to Rslt - 1 do
        GenDataLB.Items.Add('    '+IntToStr(TResolutions(Resolutions^)[i][0])+
          ' '+IntToStr(TResolutions(Resolutions^)[i][1]));
    finally
      FreeMem(Resolutions, SizeOf(Integer)*Rslt*2); // Don't forget to free the
    end;                                            //   memory.
  end;
{$R+}  // Turn range checking back on.
end;

procedure TForm1.GetTrueTypeInfo;
begin
  { Get the True-Type font capabilities of the device represented as bitmasks }
  Rslt := DeviceCapabilitiesA(Device, Port, DC_TRUETYPE, nil, nil);
  if Rslt <> 0 then
    { Now mask out the individual true type capabilities and indicate the
      result in the appropriate list box. }
    with GenDataLB.Items do begin
      Add('TRUE TYPE FONTS:');
      if (Rslt and DCTT_BITMAP) = DCTT_BITMAP then
        Add('   Prints true type fonts as graphics: Yes')
      else Add('   Prints true type fonts as graphics: No');

      if (Rslt and DCTT_DOWNLOAD) = DCTT_DOWNLOAD then
        Add('   Downloads true type fonts: Yes')
      else Add('   Downloads true type fonts: No');

      if (Rslt and DCTT_DOWNLOAD_OUTLINE) = DCTT_DOWNLOAD_OUTLINE then
        Add('   Downloads outline true type fonts: Yes')
      else Add('   Downloads outline true type fonts: No');

      if (Rslt and DCTT_SUBDEV) = DCTT_SUBDEV then
        Add('   Substitutes device for True Type fonts: Yes')
      else Add('   Substitutes device for True Type fonts: No')
    end;
end;

procedure TForm1.GetCasePaperNames;
{ This method shows the paper types available for a selected printer by
  getting an array of word value that map to DMPAPER_XXXX constants defined
  in WINDOWS.PAS. These constants refer to specific paper types. See the
  method GetDevCapsPaperNames which gets the paper names from the
  DeviceCapabilitiesA function. }
var
  P: Pointer;
  pSize: Integer;
  i: integer;
begin
{$R-} // Range checking must be turned off. 
  ListBox1.Items.Clear;
  { First, determine how many paper types are available. }
  Rslt := DeviceCapabilitiesA(Device, Port, DC_PAPERS, nil, nil);
  if Rslt > 0 then begin
    { Allocate the array of words }
    GetMem(P, Rslt*SizeOf(Word));
    try
      { Retrieve the array of words that refer to different paper types. Then
        show that type in the appropriate list box. }
      DeviceCapabilitiesA(Device, Port, DC_PAPERS, P, nil);
      for i := 0 to Rslt - 1 do begin
        pSize := TPageSizeArray(P^)[i];
        with ListBox1.Items do
          case pSize of
            DMPAPER_LETTER: Add('DMPAPER_LETTER');
  //          DMPAPER_FIRST: Add('DMPAPER_LETTER') -- Same as above
            DMPAPER_LETTERSMALL: Add('DMPAPER_LETTERSMALL');
            DMPAPER_TABLOID: Add('DMPAPER_TABLOID');
            DMPAPER_LEDGER: Add('DMPAPER_LEDGER');
            DMPAPER_LEGAL: Add('DMPAPER_LEGAL');
            DMPAPER_STATEMENT: Add('DMPAPER_STATEMENT');
            DMPAPER_EXECUTIVE: Add('DMPAPER_EXECUTIVE');
            DMPAPER_A3: Add('DMPAPER_A3');
            DMPAPER_A4: Add('DMPAPER_A4');
            DMPAPER_A4SMALL: Add('DMPAPER_A4SMALL');
            DMPAPER_A5: Add('DMPAPER_A5');
            DMPAPER_B4: Add('DMPAPER_B4');
            DMPAPER_B5: Add('DMPAPER_B5');
            DMPAPER_FOLIO: Add('DMPAPER_FOLIO');
            DMPAPER_QUARTO: Add('DMPAPER_QUARTO');
            DMPAPER_10X14: Add('DMPAPER_10X14');
            DMPAPER_11X17: Add('DMPAPER_11X17');
            DMPAPER_NOTE: Add('DMPAPER_NOTE');
            DMPAPER_ENV_9: Add('DMPAPER_ENV_9');
            DMPAPER_ENV_10: Add('DMPAPER_ENV_10');
            DMPAPER_ENV_11: Add('DMPAPER_ENV_11');
            DMPAPER_ENV_12: Add('DMPAPER_ENV_12');
            DMPAPER_ENV_14: Add('DMPAPER_ENV_14');
            DMPAPER_CSHEET: Add('DMPAPER_CSHEET');
            DMPAPER_DSHEET: Add('DMPAPER_DSHEET');
            DMPAPER_ESHEET: Add('DMPAPER_ESHEET');
            DMPAPER_ENV_DL: Add('DMPAPER_ENV_DL');
            DMPAPER_ENV_C5: Add('DMPAPER_ENV_C5');
            DMPAPER_ENV_C3: Add('DMPAPER_ENV_C3');
            DMPAPER_ENV_C4: Add('DMPAPER_ENV_C4');
            DMPAPER_ENV_C6: Add('DMPAPE$_ENV_C6');
            DMPAPER_ENV_C65: Add('DMPAPER_ENV_C65');
            DMPAPER_ENV_B4: Add('DMPAPER_ENV_B4');
            DMPAPER_ENV_B5: Add('DMPAPER_ENV_B5');
            DMPAPER_ENV_B6: Add('DMPAPER_ENV_B6');
            DMPAPER_ENV_ITALY: Add('DMPAPER_ENV_ITALY');
            DMPAPER_ENV_MONARCH: Add('DMPAPER_ENV_MONARCH');
            DMPAPER_ENV_PERSONAL: Add('DMPAPER_ENV_PERSONAL');
            DMPAPER_FANFOLD_US: Add('DMPAPER_FANFOLD_US');
            DMPAPER_FANFOLD_STD_GERMAN: Add('DMPAPER_FANFOLD_STD_GERMAN');
            DMPAPER_FANFOLD_LGL_GERMAN: Add('DMPAPER_FANFOLD_LGL_GERMAN');
            DMPAPER_ISO_B4: Add('DMPAPER_ISO_B4');
            DMPAPER_JAPANESE_POSTCARD: Add('DMPAPER_JAPANESE_POSTCARD');
            DMPAPER_9X11: Add('DMPAPER_9X11');
            DMPAPER_10X11: Add('DMPAPER_10X11');
            DMPAPER_15X11: Add('DMPAPER_15X11');
            DMPAPER_ENV_INVITE: Add('DMPAPER_ENV_INVITE');
            DMPAPER_RESERVED_48: Add('DMPAPER_RESERVED_48');
            DMPAPER_RESERVED_49: Add('DMPAPER_RESERVED_49');
            DMPAPER_LETTER_EXTRA: Add('DMPAPER_LETTER_EXTRA');
            DMPAPER_LEGAL_EXTRA: Add('DMPAPER_LEGAL_EXTRA');
            DMPAPER_TABLOID_EXTRA: Add('DMPAPER_TABLOID_EXTRA');
            DMPAPER_A4_EXTRA: Add('DMPAPER_A4_EXTRA');
            DMPAPER_LETTER_TRANSVERSE: Add('DMPAPER_LETTER_TRANSVERSE');
            DMPAPER_A4_TRANSVERSE: Add('DMPAPER_A4_TRANSVERSE');
            DMPAPER_LETTER_EXTRA_TRANSVERSE: Add('DMPAPER_LETTER_EXTRA_TRANSVERSE');
            DMPAPER_A_PLUS: Add('DMPAPER_A_PLUS');
            DMPAPER_B_PLUS: Add('DMPAPER_B_PLUS');
            DMPAPER_LETTER_PLUS: Add('DMPAPER_LETTER_PLUS');
            DMPAPER_A4_PLUS: Add('DMPAPER_A4_PLUS');
            DMPAPER_A5_TRANSVERSE: Add('DMPAPER_A5_TRANSVERSE');
            DMPAPER_B5_TRANSVERSE: Add('DMPAPER_B5_TRANSVERSE');
            DMPAPER_A3_EXTRA: Add('DMPAPER_A3_EXTRA');
            DMPAPER_A5_EXTRA: Add('DMPAPER_A5_EXTRA');
            DMPAPER_B5_EXTRA: Add('DMPAPER_B5_EXTRA');
            DMPAPER_A2: Add('DMPAPER_A2');
            DMPAPER_A3_TRANSVERSE: Add('DMPAPER_A3_TRANSVERSE');
            DMPAPER_A3_EXTRA_TRANSVERSE: Add('DMPAPER_A3_EXTRA_TRANSVERSE');
//            DMPAPER_LAST: Add('DMPAPER_LAST'); SAME AS ABOVE
            DMPAPER_USER: Add('DMPAPER_USER');
        else
          ListBox1.Items.Add(IntToStr(pSize));
        end;
      end;
    finally
      FreeMem(P, Rslt*SizeOf(Word)); // Don't forget to free memory.
    end;
  end;
{$R+}  // Turn range checking back on. 
end;

procedure TForm1.GetDevCapsPaperNames;
{ This method gets the paper types available on a selected printer from the
  DeviceCapabilitiesA function. }
var
  PaperNames: Pointer;
  i: integer;
begin
{$R-}  // Range checking off.
  ListBox2.Items.Clear;
  { First get the number of paper names available. }
  Rslt := DeviceCapabilitiesA(Device, Port, DC_PAPERNAMES, nil, nil);
  if Rslt > 0 then begin
    { Now allocate the array of paper names. Each paper name is 64 bytes.
      Therefore, allocate Rslt*64 of memory. }
    GetMem(PaperNames, Rslt*64);
    try
      { Retrieve the list of names into the allocated memory block. }
      if DeviceCapabilitiesA(Device, Port, DC_PAPERNAMES,
      PaperNames, nil) = - 1 then
        raise Exception.Create('DevCap Error');
      { Add the paper names to the appropriate list box. }
      for i := 0 to Rslt - 1 do
       ListBox2.Items.Add(StrPas(TPNames(PaperNames^)[i]));
    finally
      FreeMem(PaperNames, Rslt*64); // Don't forget to free the memory. }
    end;
  end;
{$R+} // Range checking back on. 
end;

procedure TForm1.GetDevCaps;
{ This method retrives various capabilities of the selected printer device by
  using the GetDeviceCaps function. Refer to the Online API help for the
  meaning of each of these items. }
begin
  with DevCapsLB.Items do begin
    Clear;
    Add('Width in millimeters: '+
      IntToStr(GetDeviceCaps(Printer.Handle, HORZSIZE)));
    Add('Height in millimeter:'+
      IntToStr(GetDeviceCaps(Printer.Handle, VERTSIZE)));
    Add('Width in pixels: '+
      IntToStr(GetDeviceCaps(Printer.Handle, HORZRES)));
    Add('Height in pixels: '+
      IntToStr(GetDeviceCaps(Printer.Handle, VERTRES)));
    Add('Pixels per horizontal inch: '+
      IntToStr(GetDeviceCaps(Printer.Handle, LOGPIXELSX)));
    Add('Pixels per vertical inch: '+
      IntToStr(GetDeviceCaps(Printer.Handle, LOGPIXELSY)));
    Add('Color bits per pixel: '+
      IntToStr(GetDeviceCaps(Printer.Handle, BITSPIXEL)));
    Add('Number of color planes: '+
      IntToStr(GetDeviceCaps(Printer.Handle, PLANES)));
    Add('Number of brushes: '+
      IntToStr(GetDeviceCaps(Printer.Handle, NUMBRUSHES)));
    Add('Number of pens: '+
      IntToStr(GetDeviceCaps(Printer.Handle, NUMPENS)));
    Add('Number of fonts: '+
      IntToStr(GetDeviceCaps(Printer.Handle, NUMFONTS)));
    Rslt := GetDeviceCaps(Printer.Handle, NUMCOLORS);
    if Rslt = -1 then
      Add('Number of entries in color table: > 8')
    else Add('Number of entries in color table: '+
      IntToStr(Rslt));
    Add('Relative pixel drawing width: '+
      IntToStr(GetDeviceCaps(Printer.Handle, ASPECTX)));
    Add('Relative pixel drawing height: '+
      IntToStr(GetDeviceCaps(Printer.Handle, ASPECTY)));
    Add('Diagnal pixel drawing width: '+
      IntToStr(GetDeviceCaps(Printer.Handle, ASPECTXY)));
    if GetDeviceCaps(Printer.Handle, CLIPCAPS) = 1 then
      Add('Clip to rectangle: Yes')
    else
      Add('Clip to rectangle: No');
  end;
end;

procedure TForm1.GetRasterCaps;
{ This method gets the various raster capabilities of the selected printer
  device by using the GetDeviceCaps funtion with the RASTERCAPS index. Refer
  to the online help for information on each capability. }
var
  RCaps: Integer;
begin
  RasterCapLB.Clear;
  with RasterCapLB  .Items do begin
    RCaps := GetDeviceCaps(Printer.Handle, RASTERCAPS);
    if (RCaps and RC_BANDING) = RC_BANDING then Add('Banding: Yes')
    else Add('Banding: No');
    if (RCaps and RC_BITBLT) = RC_BITBLT then Add('Bitblt capable: Yes')
    else Add('BitBlt capable: No');
    if (RCaps and RC_BITMAP64) = RC_BITMAP64 then
      Add('Supports bitmaps > 64K: Yes')
    else Add('Supports bitmaps > 64K: No');
    if (RCaps and RC_DI_BITMAP) = RC_DI_BITMAP then Add('DIB support: Yes')
    else Add('DIB support: No');
    if (RCaps and RC_FLOODFILL) = RC_FLOODFILL then
      Add('Floodfill support: Yes')
    else Add('Floodfill support: No');
    if (RCaps and RC_GDI20_OUTPUT) = RC_GDI20_OUTPUT then
      Add('Windows 2.0 support: Yes')
    else Add('Windows 2.0 support: No');
    if (RCaps and RC_PALETTE) = RC_PALETTE then Add('Palette based device: Yes')
    else Add('Palette based device: No');
    if (RCaps and RC_SCALING) = RC_SCALING then Add('Scaling support: Yes')
    else Add('Scaling support: No');
    if (RCaps and RC_STRETCHBLT) = RC_STRETCHBLT then
      Add('StretchBlt support: Yes')
    else Add('StretchBlt support: No');
    if (RCaps and RC_STRETCHDIB) = RC_STRETCHDIB then
      Add('StretchDIBits support: Yes')
    else Add('StretchDIBits support: No');
  end;
end;

procedure TForm1.GetCurveCaps;
{ This method gets the various curve capabilities of the selected printer
  device by using the GetDeviceCaps funtion with the CURVECAPS index. Refer
  to the online help for information on each capability. }
var
  CCaps: Integer;
begin
  CurveCapLB.Clear;
  with CurveCapLB.Items do begin
    CCaps := GetDeviceCaps(Printer.Handle, CURVECAPS);
    if (CCaps and CC_NONE) = CC_NONE then Add('Curve support: No')
    else Add('Curve support: Yes');
    if (CCaps and CC_CIRCLES) = CC_CIRCLES then Add('Circle support: Yes')
    else Add('Circle support: No');
    if (CCaps and CC_PIE) = CC_PIE then Add('Pie support: Yes')
    else Add('Pie support: No');
    if (CCaps and CC_CHORD) = CC_CHORD then Add('Chord arc support: Yes')
    else Add('Chord arc support: No');
    if (CCaps and CC_ELLIPSES) = CC_ELLIPSES then Add('Ellipse support: Yes')
    else Add('Ellipse support: No');
    if (CCaps and CC_WIDE) = CC_WIDE then Add('Wide border support: Yes')
    else Add('Wide border support: No');
    if (CCaps and CC_STYLED) = CC_STYLED then Add('Styled border support: Yes')
    else Add('Styled border support: No');
    if (CCaps and CC_WIDESTYLED) = CC_WIDESTYLED then
      Add('Wide and styled border support: Yes')
    else Add('Wide and styled border support: No');
    if (CCaps and CC_INTERIORS) = CC_INTERIORS then Add('Interior support: Yes')
    else Add('Interior support: No');
    if (CCaps and CC_ROUNDRECT) = CC_ROUNDRECT then
      Add('Round rectangle support: Yes')
    else Add('Round rectangle support: No');
  end;
end;

procedure TForm1.GetLineCaps;
{ This method gets the various line drawing capabilities of the selected printer
  device by using the GetDeviceCaps funtion with the LINECAPS index. Refer
  to the online help for information on each capability. }
var
  LCaps: Integer;
begin
  LineCapLB.Clear;
  with LineCapLB.Items do begin
    LCaps := GetDeviceCaps(Printer.Handle, LINECAPS);
    if (LCaps and LC_NONE) = LC_NONE then Add('Line support: No')
    else Add('Line support: Yes');
    if (LCaps and LC_POLYLINE) = LC_POLYLINE then Add('Polyline support: Yes')
    else Add('Polyline support: No');
    if (LCaps and LC_MARKER) = LC_MARKER then Add('Marker support: Yes')
    else Add('Marker support: No');
    if (LCaps and LC_POLYMARKER) = LC_POLYMARKER then
      Add('Multiple marker support: Yes')
    else Add('Multiple marker support: No');
    if (LCaps and LC_WIDE) = LC_WIDE then Add('Wide line support: Yes')
    else Add('Wide line support: No');
    if (LCaps and LC_STYLED) = LC_STYLED then Add('Styled line support: Yes')
    else Add('Styled line support: No');
    if (LCaps and LC_WIDESTYLED) = LC_WIDESTYLED then
      Add('Wide and styled line support: Yes')
    else Add('Wide and styled line support: No');
    if (LCaps and LC_INTERIORS) = LC_INTERIORS then
      Add('Interior support: Yes')
    else Add('Interior support: No');
  end;
end;

procedure TForm1.GetPolyCaps;
{ This method gets the various polygonal capabilities of the selected printer
  device by using the GetDeviceCaps funtion with the POLYGONALCAPS index. Refer
  to the online help for information on each capability. }
var
  PCaps: Integer;
begin
  PolyCapLB.Clear;
  with PolyCapLB.Items do begin
    PCaps := GetDeviceCaps(Printer.Handle, POLYGONALCAPS);
    if (PCaps and PC_NONE) = PC_NONE then Add('Polygon support: No')
    else Add('Polygon support: Yes');

    if (PCaps and PC_POLYGON) = PC_POLYGON then
      Add('Alternate fill polygon support: Yes')
    else Add('Alternate fill polygon support: No');

    if (PCaps and PC_RECTANGLE) = PC_RECTANGLE then
      Add('Rectangle support: Yes')
    else Add('Rectangle support: No');

    if (PCaps and PC_WINDPOLYGON) = PC_WINDPOLYGON then
      Add('Winding-fill polygon support: Yes')
    else Add('Window-fill polygon support: No');

    if (PCaps and PC_SCANLINE) = PC_SCANLINE then
      Add('Single scanline support: Yes')
    else Add('Single scanline support: No');

    if (PCaps and PC_WIDE) = PC_WIDE then
      Add('Wide border support: Yes')
    else Add('Wide border support: No');

    if (PCaps and PC_STYLED) = PC_STYLED then
      Add('Styled border support: Yes')
    else Add('Styled border support: No');

    if (PCaps and PC_WIDESTYLED) = PC_WIDESTYLED then
      Add('Wide and styled border support: Yes')
    else Add('Wide and styled border support: No');

    if (PCaps and PC_INTERIORS) = PC_INTERIORS then
      Add('Interior support: Yes')
    else Add('Interior support: No');
  end;
end;

procedure TForm1.GetTextCaps;
{ This method gets the various text drawing capabilities of the selected printer
  device by using the GetDeviceCaps funtion with the TEXTCAPS index. Refer
  to the online help for information on each capability. }
var
  TCaps: Integer;
begin
  TextCapLB.Clear;
  with TextCapLB.Items do begin
    TCaps := GetDeviceCaps(Printer.Handle, TEXTCAPS);
    if (TCaps and TC_OP_CHARACTER) = TC_OP_CHARACTER then
      Add('Character output precision: Yes')
    else
      Add('Character output precision: No');

    if (TCaps and TC_OP_STROKE) = TC_OP_STROKE then
      Add('Stroke output precision: Yes')
    else
      Add('Stroke output precision: No');

    if (TCaps and TC_CP_STROKE) = TC_CP_STROKE then
      Add('Stroke clip precision: Yes')
    else
      Add('Stroke clip precision: No');

    if (TCaps and TC_CR_90) = TC_CR_90 then
      Add('90 degree character rotation: Yes')
    else
      Add('90 degree character rotation: No');

    if (TCaps and TC_CR_ANY) = TC_CR_ANY then
      Add('Any degree character rotation: Yes')
    else
      Add('Any degree character rotation: No');

    if (TCaps and TC_SF_X_YINDEP) = TC_SF_X_YINDEP then
      Add('Independent scale in X and Y direction: Yes')
    else
      Add('Independent scale in X and Y direction: No');

    if (TCaps and TC_SA_DOUBLE) = TC_SA_DOUBLE then
      Add('Doubled character for scaling: Yes')
    else
      Add('Doubled character for scaling: No');

    if (TCaps and TC_SA_INTEGER) = TC_SA_INTEGER then
      Add('Integer multiples only for character scaling: Yes')
    else
      Add('Integer multiples only for character scaling: No');

    if (TCaps and TC_SA_CONTIN) = TC_SA_CONTIN then
      Add('Any multiples for exact character scaling: Yes')
    else
      Add('Any multiples for exact character scaling: No');

    if (TCaps and TC_EA_DOUBLE) = TC_EA_DOUBLE then
      Add('Double weight characters: Yes')
    else
      Add('Double weight characters: No');

    if (TCaps and TC_IA_ABLE) = TC_IA_ABLE then
      Add('Italicized characters: Yes')
    else
      Add('Italicized characters: No');

    if (TCaps and TC_UA_ABLE) = TC_UA_ABLE then
      Add('Underlined characters: Yes')
    else
      Add('Underlined characters: No');

    if (TCaps and TC_SO_ABLE) = TC_SO_ABLE then
      Add('Strikeout characters: Yes')
    else
      Add('Strikeout characters: No');

    if (TCaps and TC_RA_ABLE) = TC_RA_ABLE then
      Add('Raster fonts: Yes')
    else
      Add('Raster fonts: No');

    if (TCaps and TC_VA_ABLE) = TC_VA_ABLE then
      Add('Vector fonts: Yes')
    else
      Add('Vector fonts: No');

    if (TCaps and TC_SCROLLBLT) = TC_SCROLLBLT then
      Add('Scrolling using bit-block transfer: No')
    else
      Add('Scrolling using bit-block transfer: Yes');
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  { Store the printer names in the combo box. }
  PrinterCombo.Items.Assign(Printer.Printers);
  { Display the default printer in the combo box. }
  PrinterCombo.ItemIndex := Printer.PrinterIndex;
  PrinterComboChange(nil);  // Invoke the combo's OnChange event 
end;

procedure TForm1.PrinterComboChange(Sender: TObject);
begin
  Screen.Cursor := crHourGlass;
  try
    { Populate combo with available printers }
    Printer.PrinterIndex := PrinterCombo.ItemIndex;
    with Printer do
      GetPrinter(Device, Driver, Port, ADevMode);
    { Fill the general page with printer information }
    with GenDataLB.Items do begin
      Clear;
      Add('Port: '+Port);
      Add('Device: '+Device);
      Rslt := DeviceCapabilitiesA(Device, Port, DC_DRIVER, nil, nil);
      Add('Driver Version: '+IntToStr(Rslt));
    end;
    { The functions below make use of the GetDeviceCapabilitiesA function.}
    GetBinNames;
    GetDuplexSupport;
    GetCopies;
    GetEMFStatus;
    GetResolutions;
    GetTrueTypeInfo;

    { The functions below make use of the GetDeviceCaps function. }
    GetCasePaperNames;    // Fill the Paper Types page.
    GetDevCapsPaperNames;
    GetDevCaps;    // Fill Device Caps page.
    GetRasterCaps; // Fill Raster Caps page.
    GetCurveCaps;  // Fill Curve Caps page.
    GetLineCaps;   // Fill Line Caps page.
    GetPolyCaps;   // Fill Polygonal Caps page.
    GetTextCaps;   // Fill Text Caps page.
  finally
    Screen.Cursor := crDefault;
  end;
end;

end.
