unit mbNav;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, DBTables, DBCtrls, DBConsts;

procedure Register;

{*** TmbDBNavigator class definition ******************************************}

type
  TmbDBNavigator = class (TDBNavigator)
  private
    FOnClickFirst: TDataSetNotifyEvent;
    FOnClickPrior: TDataSetNotifyEvent;
    FOnClickNext: TDataSetNotifyEvent;
    FOnClickLast: TDataSetNotifyEvent;
    FOnClickInsert: TDataSetNotifyEvent;
    FOnClickDelete: TDataSetNotifyEvent;
    FOnClickEdit: TDataSetNotifyEvent;
    FOnClickPost: TDataSetNotifyEvent;
    FOnClickCancel: TDataSetNotifyEvent;
    FOnClickRefresh: TDataSetNotifyEvent;
    procedure Click(Sender: TObject);
  protected
    procedure BtnClick(Index: TNavigateBtn); virtual;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property OnClickFirst: TDataSetNotifyEvent
      read FOnClickFirst write FOnClickFirst;
    property OnClickPrior: TDataSetNotifyEvent
      read FOnClickPrior write FOnClickPrior;
    property OnClickNext: TDataSetNotifyEvent
      read FOnClickNext write FOnClickNext;
    property OnClickLast: TDataSetNotifyEvent
      read FOnClickLast write FOnClickLast;
    property OnClickInsert: TDataSetNotifyEvent
      read FOnClickInsert write FOnClickInsert;
    property OnClickDelete: TDataSetNotifyEvent
      read FOnClickDelete write FOnClickDelete;
    property OnClickEdit: TDataSetNotifyEvent
      read FOnClickEdit write FOnClickEdit;
    property OnClickPost: TDataSetNotifyEvent
      read FOnClickPost write FOnClickPost;
    property OnClickCancel: TDataSetNotifyEvent
      read FOnClickCancel write FOnClickCancel;
    property OnClickRefresh: TDataSetNotifyEvent
      read FOnClickRefresh write FOnClickRefresh;
  end;

implementation

{*** TmbDBNavigator methods ***************************************************}

constructor TmbDBNavigator.Create(AOwner: TComponent);
var
  I: TNavigateBtn;
begin
  inherited Create(AOwner);
  for I := Low(Buttons) to High(Buttons)
    do Buttons[I].OnClick := Click;
end;

procedure TmbDBNavigator.Click(Sender: TObject);
begin
  BtnClick (TNavButton (Sender).Index);
end;

procedure TmbDBNavigator.BtnClick(Index: TNavigateBtn);
var
  FOnNavClick: ENavClick;
  ds: TDataSet;
begin
  if (DataSource <> nil) and (DataSource.State <> dsInactive) then
  begin
    ds := DataSource.DataSet;
    with ds do
    begin
      case Index of
        nbPrior:  if Assigned(FOnClickPrior)
                    then FOnClickPrior(ds)
                    else Prior;
        nbNext:   if Assigned(FOnClickNext)
                    then FOnClickNext(ds)
                    else Next;
        nbFirst:  if Assigned(FOnClickFirst)
                    then FOnClickFirst(ds)
                    else First;
        nbLast:   if Assigned(FOnClickLast)
                    then FOnClickLast(ds)
                    else Last;
        nbInsert: if Assigned(FOnClickInsert)
                    then FOnClickInsert(ds)
                    else Insert;
        nbEdit:   if Assigned(FOnClickEdit)
                    then FOnClickEdit(ds)
                    else Edit;
        nbCancel: if Assigned(FOnClickCancel)
                    then FOnClickCancel(ds)
                    else Cancel;
        nbPost:   if Assigned(FOnClickPost)
                    then FOnClickPost(ds)
                    else Post;
        nbRefresh:if Assigned(FOnClickRefresh)
                    then FOnClickRefresh(ds)
                    else Refresh;
        nbDelete: if Assigned(FOnClickDelete)
                    then FOnClickDelete(ds)
                  else
                    if not ConfirmDelete or
                        (MessageDlg (LoadStr(SDeleteRecordQuestion),
                         mtConfirmation, mbOKCancel, 0) <> idCancel)
                    then Delete;
      end;
    end;
  end;
  FOnNavClick := OnClick;
  if not (csDesigning in ComponentState) and Assigned(FOnNavClick) then
    FOnNavClick(Self, Index);
end;

{*** Register component *******************************************************}

procedure Register;
begin
  RegisterComponents('Data Controls', [TmbDBNavigator]);
end;

end.
