unit Subclexu;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Grids, Outline, ExtCtrls, Spin;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    BtnEnum: TButton;
    BtnTrack: TButton;
    BtnSubClass: TButton;
    BtnUnSubClass: TButton;
    Panel2: TPanel;
    Outline1: TOutline;
    Panel3: TPanel;
    Label1: TLabel;
    MessageLog: TListBox;
    CBBeep: TCheckBox;
    SEMinX: TSpinEdit;
    SEMinY: TSpinEdit;
    SEMaxX: TSpinEdit;
    SEMaxY: TSpinEdit;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    procedure BtnEnumClick(Sender: TObject);
    procedure BtnSubClassClick(Sender: TObject);
    procedure BtnUnSubClassClick(Sender: TObject);
    procedure Outline1Click(Sender: TObject);
    procedure BtnTrackMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure BtnTrackMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BtnTrackMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormResize(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    FTracking: Boolean;
    FSCClassName: string;
    FSCText: string;
    FSCIndex: Integer;
    function MyWndProc(Handle: hWnd; Msg: Word; wParam: word; lParam: LongInt): LongInt;
    procedure WMGetMinMaxInfo(var Msg: TWMGetMinMaxInfo); message WM_GETMINMAXINFO;
  public
    { Public declarations }
    property SCClassName: string read FSCClassName write FSCClassName;
    property SCText: string read FSCText write FSCText;
    property SCIndex: Integer read FSCIndex write FSCIndex;
  end;

var
  Form1: TForm1;

implementation

var
  ParentCount: Integer;
  ChildCount: Integer;
  CurrentParent: hWnd;

{$R *.DFM}

type
  TMyWndProc = Function(Handle: hWnd; Msg: Word; wParam: word; lParam: LongInt): LongInt of object;

var
  OldWndProc: TFarProc;
  SubClassedHandle: hWnd;
  AsmWndProcInstance: TFarProc;
  MyWndProcAddr: TMyWndProc;

function IsWindow16Bit(Handle: hWnd): Boolean;
begin
  { This call returns zero for a 32 bit app, from 16bit app }
  Result := SetWindowLong(Handle,gwl_WndProc,GetWindowLong(Handle,gwl_WndProc)) <> 0;
end;

function RetrieveClassName(Handle: hWnd): string;
var
  Work: array[0..255] of char;

begin
  GetClassName(Handle,Work,sizeof(Work));
  Result := StrPas(Work);
end;

function RetrieveWindowText(Handle: hWnd): string;
var
  Work: array[0..255] of char;

begin
  GetWindowText(Handle,Work,sizeof(Work));
  Result := StrPas(Work);
end;

function OutlineIndexOfWindow(Handle: hWnd): Integer;
var
  I: Integer;

begin
  for I := 1 to Form1.OutLine1.ItemCount do
    if Form1.OutLine1.Items[I].Data = Pointer(Handle) then
      begin
        Result := I;
        Exit;
      end;
  Result := 1; { 'DeskTop' }
end;


function EnumChildProc(Handle: hWnd; OutLine: TOutLine): Bool; export;
var
  ClassName: string;
  S: string;

begin
  Result := True;
  Inc(ChildCount);
  S := Format('[%s] "%s"',[RetrieveClassName(Handle),RetrieveWindowText(Handle)]);
  OutLine.AddChildObject(OutlineIndexOfWindow(CurrentParent),S,Pointer(Handle));
end;

function EnumWindowsProc(Handle: hWnd; OutLine: TOutline): Bool; export;
var
  S: string;

begin
  Result := True;
  if IsWindow16Bit(Handle) then
    begin
      Inc(ParentCount);
      CurrentParent := Handle;
      S := Format('[%s] "%s"',[RetrieveClassName(Handle),RetrieveWindowText(Handle)]);
      OutLine.AddChildObject(1,S,Pointer(Handle));
      ChildCount := 0;
      EnumChildWindows(Handle,@EnumChildProc,longInt(OutLine));
    end;
end;

procedure TForm1.BtnEnumClick(Sender: TObject);
var
  I: Integer;
  StringSorted: TStrings;

begin
  Screen.Cursor := crHourGlass;
  try
    OutLine1.BeginUpdate;
    OutLine1.Clear;
    Outline1.AddChild(0,'DeskTop');
    ParentCount := 0;
    EnumWindows(@EnumWindowsProc,LongInt(OutLine1));
    OutLine1.Items[1].Expand;
    OutLine1.EndUpdate;
    BtnTrack.Enabled := True;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TForm1.BtnSubClassClick(Sender: TObject);
begin
  MyWndProcAddr := MyWndProc;
  SubClassedHandle := hWnd(OutLine1.Items[Outline1.SelectedItem].Data);
  if SubClassedHandle = MessageLog.Handle then
    ShowMessage('No way!!!! This is the subclass log window.');
  OldWndProc := TFarProc(SetWindowLong(SubClassedHandle,gwl_WndProc,LongInt(AsmWndProcInstance)));
  if OldWndProc <> nil then
    begin
      BtnUnSubClass.Enabled := True;
      BtnSubClass.Enabled := False;
    end;
end;

procedure TForm1.BtnUnSubClassClick(Sender: TObject);
begin
  SetWindowLong(SubClassedHandle,gwl_WndProc,LongInt(OldWndProc));
  BtnUnSubClass.Enabled := False;
  BtnSubClass.Enabled := True;
end;

procedure TForm1.Outline1Click(Sender: TObject);
begin
  BtnSubClass.Enabled := Outline1.SelectedItem > 1
end;

procedure TForm1.BtnTrackMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var
  WinPos: TPoint;
  Node: TOutlineNode;

begin
  if GetCapture = (TButton(Sender).Handle) then
    begin
      GetCursorPos(WinPos);
      Outline1.SelectedItem := OutlineIndexOfWindow(WindowFromPoint(WinPos));
      Node := Outline1.Items[Outline1.SelectedItem];
      Node.Expand;
    end;
end;

procedure TForm1.BtnTrackMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  if (Sender is TButton) and (Button = mbLeft) then
    if TButton(Sender).Name = 'BtnTrack' then
      begin
        SetCapture(TButton(Sender).Handle);
        FTracking := True;
        Screen.Cursor := crCross;
      end;
end;

procedure TForm1.BtnTrackMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
var
  Node: TOutlineNode;
  S: String;

begin
  if (Sender is TButton) and (Button = mbLeft) and (FTracking = True) then
    if TButton(Sender).Name = 'BtnTrack' then
      begin
        FTracking := False;
        ReleaseCapture;
        Screen.Cursor := crDefault;
        BtnEnum.SetFocus; { Avoids problems with button down on Find Window }
        ShowMessage('Selected item indicates last window clicked on.');
      end;
end;

function TForm1.MyWndProc(Handle: hWnd; Msg: Word; wParam: word; lParam: LongInt): LongInt;

  procedure UpdateLog(const s: string);
  begin
    MessageLog.Items.BeginUpdate;
    While MessageLog.Items.Count > 100 do
      MessageLog.Items.Delete(0);
    MessageLog.Items.Add(s);
    MessageLog.TopIndex :=
      MessageLog.Items.Count - 1;
    MessageLog.Items.EndUpdate;
  end;

begin
  if CBBeep.Checked then
    MessageBeep($FFFF);
  if (Msg <> wm_SetCursor) and (Msg <> wm_NCHitTest) and (Msg <> wm_MouseMove) then
    UpdateLog(Format('MsgId: %d, wParam: %d, lParam: %d',[Msg,wParam,lParam]));
  if Msg = WM_GETMINMAXINFO then
    begin
      With PMinMaxInfo(lParam)^ do
        begin
          ptMinTrackSize.X := SEMinX.Value;
          ptMinTrackSize.Y := SEMinY.Value;
          ptMaxTrackSize.X := SEMaxX.Value;
          ptMaxTrackSize.Y := SEMaxY.Value;
        end;
      UpdateLog('Processed WM_GETMINMAXINFO message');
      Result := 0;
    end
  else
    Result := CallWindowProc(OldWndProc,Handle,Msg,wParam,lParam);
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  Outline1.Height := (Panel2.Height div 3) * 2;
end;

procedure TForm1.WMGetMinMaxInfo(var Msg: TWMGetMinMaxInfo);
begin
  With Msg.MinMaxInfo^ do
    begin
      ptMinTrackSize.X := 260;
      ptMinTrackSize.Y := 240;
    end;
  Msg.Result := 0;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  if BtnUnSubClass.Enabled then
    SetWindowLong(SubClassedHandle,gwl_WndProc,LongInt(OldWndProc));
  FreeProcInstance(AsmWndProcInstance);
end;

{---------------------------------------------------------------------}
{ AsmWndProc - This is the routine specified in MakeProcInstance. It  }
{              is called when a window has been subclassed.           }
{              It serves as an interface between Windows and the      }
{              class method. It is not normally possibkle for a       }
{              callback to reside in an EXE when 'Smart' callbacks    }
{              are used due to the compiler generated code at the     }
{              at the entry to a function.                            }
{                                                                     }
{              To get round this restriction, this function is        }
{              declared as Far rather than Exported. This removes the }
{              compiler generated code which overwrites the AX        }
{              register (AX is set to the data segment by the         }
{              code generated by MakeProcInstance).                   }
{                                                                     }
{              As some of the standard Entry/Exit code is not         }
{              generated for us, this has been added by hand, below.  }
{                                                                     }
{              See p176-178 of the Object Pascal language guide.      }
{                                                                     }
{---------------------------------------------------------------------}

{$S-}  { <=== DO NOT REMOVE !!!! }

{ The above compiler directive is mandatory for this function. If     }
{ stack checking is activated for this function the code generated    }
{ change the AX register and it will stop working!!!                  }

function AsmWndProc(Handle: hWnd; Msg: Word; wParam: word; lParam: LongInt): LongInt; far; assembler;
asm
  { Entry code to complete functionality of exported function         }

  PUSH DS               { Save callers data segment                   }
  MOV  DS,AX            { Load local data - from MakeProcInstance     }
  PUSH SI               { Save callers source index                   }
  PUSH DI               { Save callers destination index              }

  { Code to call the method                                           }

  PUSH Handle
  PUSH Msg
  PUSH wParam
  PUSH lParam.Word[2]
  PUSH lParam.Word[0]

  PUSH MyWndProcAddr.Word[6]     { Puts the Self 'hidden' paramter    }
  PUSH MyWndProcAddr.Word[4]     { on the stack                       }

  CALL [TFarProc(MyWndProcAddr)] { Call the class method              }

  { Exit code to complete standard functionality of exported function }

  POP  DI               { Restore callers destination index }
  POP  SI               { Restore callers source index }
  POP  DS               { Restore callers data segment }

end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  AsmWndProcInstance := MakeProcInstance(@AsmWndProc,hInstance);
end;

end.

