unit RunWaitU;

interface

uses
  Classes, Windows, SysUtils;

type
  TWaitThread = class(TThread)
  private
    FProcess: THandle;
  public
    constructor Create(HProcess: THandle);
    procedure Execute; override;
  end;

  EExecAppError = class(Exception);

function SysErrorMessageParam(ErrorCode: Integer; Param: String): String;
function ExecApp(AppName, Params: String): THandle;
function ExecApp2(AppName, Params: String): THandle;
procedure WaitForApp(HProcess: THandle; Event: TNotifyEvent);
procedure WaitForApp2(HProcess: THandle);

implementation

uses
  ShellAPI, Forms;

constructor TWaitThread.Create(HProcess: THandle);
begin
  FProcess := HProcess;
  inherited Create(False)
end;

procedure TWaitThread.Execute;
begin
  WaitForSingleObject(FProcess, Infinite)
end;

function SysErrorMessageParam(ErrorCode: Integer; Param: String): String;
var
  Len: Integer;
  Buffer: array[0..255] of Char;
  ArgArray: array[1..1] of PChar;
begin
  ArgArray[1] := PChar(Param);
  Len := FormatMessage(Format_Message_From_System or
    Format_Message_Argument_Array, nil, ErrorCode, 0, Buffer,
    SizeOf(Buffer), @ArgArray);
  while (Len > 0) and (Buffer[Len - 1] in [#0..#32, '.']) do Dec(Len);
  SetString(Result, Buffer, Len);
end;

function ExecApp(AppName, Params: String): THandle;
var
  SI: TStartupInfo;
  PI: TProcessInformation;
begin
  FillChar(SI, SizeOf(SI), 0);
  with SI do
  begin
    cb := SizeOf(TStartupInfo);
    dwFlags := StartF_UseShowWindow;
    wShowWindow := sw_ShowNormal;
  end;
  if not CreateProcess(nil, PChar(AppName + ' ' + Params), nil,
           nil, False, 0, nil, nil, SI, PI) then
    raise EExecAppError.Create(SysErrorMessageParam(GetLastError, AppName));
  Result := PI.HProcess;
end;

function ExecApp2(AppName, Params: String): THandle;
var
  SEI: TShellExecuteInfo;
begin
  FillChar(SEI, SizeOf(SEI), 0);
  with SEI do
  begin
    cbSize := SizeOf(SEI);
    fMask := see_Mask_NoCloseProcess;
    Wnd := Application.Handle;
    lpFile := PChar(AppName);
    lpParameters := PChar(Params);
    nShow := sw_ShowNormal;
    if not ShellExecuteEx(@SEI) then
      raise EExecAppError.Create(SysErrorMessageParam(GetLastError, AppName));
    Result := SEI.HProcess;
  end;
end;

procedure WaitForApp(HProcess: THandle; Event: TNotifyEvent);
begin
  with TWaitThread.Create(HProcess) do
  begin
    FreeOnTerminate := True;
    OnTerminate := Event
  end;
end;

procedure WaitForApp2(HProcess: THandle);
begin
  while WaitForSingleObject(HProcess, 100) = Wait_TimeOut do
  begin
    Application.ProcessMessages;
    if Application.Terminated then
      Break
  end
end;

end.
