unit FACTS;
interface
uses
  DB, DBTables, SysUtils;

Type
  TName32 = String[32];

  TValue = (UnKnown, Yes, No);

var
  ValueStr: Array[TValue] of String[7] = ('UnKnown','Yes','No');

type
  TFact = class(TObject)
  private
    FFact: Integer;
    FGoal: Boolean;
    FName: TName32;
    FValue: TValue;
    FQuestion: ShortString;
  public
    constructor Create(Table: TTable); virtual;
  published
    property Fact: Integer read FFact;
    property Goal: Boolean read FGoal;
    property Name: TName32 read FName;
    property Value: TValue read FValue write FValue;
    property Question: ShortString read FQuestion;
  end {TFact};

const
  MaxFact = 512;
var
  NumFact: Integer = 0;
  _Fact: Array[1..MaxFact] of TFact; { optimisation: fact # in position # }

implementation

  constructor TFact.Create(Table: TTable);
  begin
    inherited Create;
    with Table do
    begin
      FFact := FieldByName('Fact').AsInteger;
      FGoal := FieldByName('Goal').AsBoolean;
      FName := FieldByName('Name').AsString;
      FValue := UnKnown;
      FQuestion := FieldByName('Question').AsString
    end
  end {Create};

const
  FactTableName = 'FACTS.DB';

  procedure CreateFACTS;
  begin
    with TTable.Create(nil) do
    try
      Active := False;
      TableType := ttParadox;
      TableName := FactTableName;
      with FieldDefs do
      begin
        Clear;
        Add('Fact', ftInteger, 0, TRUE);
        Add('Goal', ftBoolean, 0, TRUE);
        Add('Name', ftString, 32, TRUE);
        Add('Question', ftString, 255, FALSE)
      end;
      with IndexDefs do
      begin
        Clear;
        Add('index', 'Fact', [ixPrimary,ixUnique])
      end;
      CreateTable
    finally
      Free
    end
  end {CreateFACTS};

var
  FactTable: TTable = nil;

initialization
  if not FileExists(FactTableName) then CreateFACTS;
  FactTable := TTable.Create(nil);
  FactTable.TableName := FactTableName;
  FactTable.Open;
  while not FactTable.Eof do
  begin
    Inc(NumFact);
    if FactTable.FieldByName('Fact').AsInteger <> NumFact then
      raise Exception.Create('Error: facts are not sorted...');
    _Fact[NumFact] := TFact.Create(FactTable);
    FactTable.Next
  end;
finalization
  FactTable.Close;
  FactTable.Free
end.
