unit RULES;
interface
uses
  DB, DBTables, SysUtils, FACTS;

type
  TRule = class(TObject)
  private
    FRule: Integer;
    FCF:  SmallInt;
    FFact: Integer;
    FValue: TValue;
    FComments: ShortString;
  protected
    FFired: Boolean;
  public
    constructor Create(Table: TTable); virtual;
  published
    property Rule: Integer read FRule;
    property CF:  SmallInt read FCF;
    property Fact: Integer read FFact;
    property Value: TValue read FValue;
    property Fired: Boolean read FFired write FFired;
    property Comments: ShortString read FComments;
  end {TRule};

const
  MaxRule = 1024;
var
  NumRule: Integer = 0;
  RuleMax: Integer = 0;
  _Rule: Array[1..MaxRule] of TRule;

  function TestRule(RuleNr: Integer): Boolean;
  procedure FireRule(RuleNr: Integer);
  function Conclude(RuleNr, FactNr: Integer): Boolean;

implementation

  constructor TRule.Create(Table: TTable);
  begin
    inherited Create;
    with Table do
    begin
      FRule := FieldByName('Rule').AsInteger;
      FCF := FieldByName('CF').AsInteger;
      FFact := FieldByName('Fact').AsInteger;
      FValue := TValue(FieldByName('Value').AsInteger); // 0,1,2
      FComments := FieldByName('Comments').AsString
    end
  end {Create};

  function TestRule(RuleNr: Integer): Boolean;
  var i: Integer;
  begin
    Result := True;
    for i:=1 to NumRule do
      if (_Rule[i].Rule = RuleNr) and (_Rule[i].CF = 0) then { check }
        Result := Result AND (_Fact[_Rule[i].Fact].Value = _Rule[i].Value)
  end {Hypothesis};

  procedure FireRule(RuleNr: Integer);
  var i: Integer;
  begin
    for i:=1 to NumRule do
      if (_Rule[i].Rule = RuleNr) and
         (_Rule[i].CF > 0) and not _Rule[i].Fired then { fire }
      begin
        _Fact[_Rule[i].Fact].Value := _Rule[i].Value;
        _Rule[i].FFired := True
      end
  end {FireRule};

  function Conclude(RuleNr, FactNr: Integer): Boolean;
  var i: Integer;
  begin
    Result := False;
    for i:=1 to NumRule do
      if (_Rule[i].Rule = RuleNr) and
         (_Rule[i].Fact = FactNr) and
         (_Rule[i].CF > 0) then Result := True { rule can be used }
  end {Conclude};

const
  RuleTableName = 'RULES.DB';

  procedure CreateRULES;
  begin
    with TTable.Create(nil) do
    try
      Active := False;
      TableType := ttParadox;
      TableName := RuleTableName;
      with FieldDefs do
      begin
        Clear;
        Add('Rule', ftInteger, 0, TRUE);
        Add('CF',  ftSmallInt, 0, TRUE);
        Add('Fact', ftInteger, 0, TRUE);
        Add('Value', ftInteger, 0, FALSE);
        Add('Comments', ftString, 255, FALSE)
      end;
      with IndexDefs do
      begin
        Clear;
        Add('index', 'Rule;CF;Fact', [ixPrimary,ixUnique])
      end;
      CreateTable
    finally
      Free
    end
  end {CreateRULES};

var
  RuleTable: TTable = nil;

initialization
  if not FileExists(RuleTableName) then CreateRULES;
  RuleTable := TTable.Create(nil);
  RuleTable.TableName := RuleTableName;
  RuleTable.Open;
  while not RuleTable.Eof do
  begin
    Inc(NumRule);
    _Rule[NumRule] := TRule.Create(RuleTable);
    if _Rule[NumRule].Rule > RuleMax then RuleMax := _Rule[NumRule].Rule;
    RuleTable.Next
  end;
finalization
  RuleTable.Close;
  RuleTable.Free
end.
