unit NewHintU;

interface

uses
  Controls, WinTypes;

type
  TMyHintWindowClass = class(THintWindow)
  protected
    procedure Paint; override;
    procedure CreateParams(var Params: TCreateParams); override;
  public
    procedure ActivateHint(Rect: TRect; const AHint: string); override;
  end;

const
  InflateX: Byte = 16;
  InflateY: Byte = 8;

implementation

uses
  Forms, Graphics, WinProcs, SysUtils;

procedure TMyHintWindowClass.CreateParams(var Params: TCreateParams);
begin
  { Make window transparent }
  Brush.Style := bsClear;
  { Set up normal attributes }
  inherited CreateParams(Params);
  { Remove border }
  Params.Style := Params.Style and not ws_Border;
end;

procedure TMyHintWindowClass.ActivateHint(Rect: TRect; const AHint: string);
begin
  { Make window bigger to accommodate new non-rectangular shape }
  Inc(Rect.Right, InflateX);
  Inc(Rect.Bottom, InflateY);
  { Set modified window size & move it on-screen }
  inherited ActivateHint(Rect, AHint);
end;

procedure TMyHintWindowClass.Paint;
var
  R: TRect;
  CCaption: array[0..255] of Char;
begin
  Canvas.Brush.Color := Color;
  R := ClientRect;
  Canvas.Ellipse(R.Left, R.Top, R.Right,
    R.Bottom);
  OffsetRect(R, InflateX div 2, InflateY div 2);
  Inc(R.Left);
  Canvas.Brush.Style := bsClear;
  StrPCopy(CCaption, Caption);
  DrawText(Canvas.Handle, CCaption, -1, R,
    dt_Left or dt_NoPrefix or dt_WordBreak);
  Canvas.Brush.Style := bsSolid;
end;

initialization
  Application.ShowHint := False;
  HintWindowClass := TMyHintWindowClass;
  Application.ShowHint := True;
end.
