unit Main;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, Sprites, Spritebx, IniFiles, Waveplay;

type
  TSaverForm = class(TForm)
    SpriteBox1: TSpriteBox;
    Sprite1: TSprite;
    WavePlayer1: TWavePlayer;
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure Sprite1Move(Sender: TObject; var Bounds: TRect);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    ptMpos: TPoint;     { The mouse position }
    procedure Trigger(Sender: TObject; var Done: Boolean);    { On Idle Trigger }
    procedure SafeExit;
  public
    { Public declarations }
  end;

var
  SaverForm: TSaverForm;
  ScreenWd: Integer;                                       { Screen width }
  ScreenHt: Integer;                                       { Screen Height }

implementation

{$R *.DFM}

procedure CursorOff;                                       { Turn the Cursor Off }
var
  Cstate : Integer;                                        { Current cursor State }
begin
  Cstate := ShowCursor(True);                              { Get State }
  while Cstate >= 0 do Cstate := ShowCursor(False);        { While ON turn Off }
end;

procedure CursorOn;                                        { Turn Cursor On }
var
  Cstate : Integer;                                        { Current cursor State }
begin
  Cstate := ShowCursor(True);                              { Get current State }
  while Cstate < 0 do Cstate := ShowCursor(True);          { While off turn on }
end;

procedure TSaverForm.Trigger(Sender: TObject; var Done: Boolean);    { Called when Delphi gets bored }
begin
  SpriteBox1.DrawSprite;
  PostMessage(Handle, WM_USER + 1,0,0);    { Move it, and trigger again }
end;

procedure TSaverForm.FormCreate(Sender: TObject);
var
  Ini: TIniFile;
begin
  Top := 0; Left := 0;
  Width := Screen.Width;
  Height := Screen.Height;
  BorderStyle := bsNone;
  { Create the Ini file }
  Ini := TIniFile.Create('CONTROL.INI');
  SpriteBox1.Color := TColor(Ini.ReadInteger(Application.Title,'Color',clBlack));
  if (Ini.ReadString(Application.Title,'FileName','') <> '') and
    FileExists(Ini.ReadString(Application.Title,'FileName','')) then
      SpriteBox1.Image.LoadFromFile(Ini.ReadString(Application.Title,'FileName',''));
  SpriteBox1.Stretch := Ini.ReadBool(Application.Title,'Stretch',false);
  SpriteBox1.Center := Ini.ReadBool(Application.Title,'Center',true);
  SpriteBox1.Gradient := Ini.ReadBool(Application.Title,'Gradient',false);
  WavePlayer1.Enabled := Ini.ReadBool(Application.Title,'Sound',false);
  Ini.Free;
end;

procedure TSaverForm.FormActivate(Sender: TObject);
begin
  CursorOff;
  ptMpos.X := -1;                  { Set Mouse X }
  ptMpos.Y := -1;                  { Set Mouse Y }
  Application.OnIdle := Trigger;   { Set the OnIdle Proc }
  {set this form as the capture control}
  SetCaptureControl(Self);
end;

procedure TSaverForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  WavePlayer1.Abort;
  Application.OnIdle := nil;       { Lose the trigger }
  CursorOn;                        { Set the Cursor On }
end;

procedure TSaverForm.SafeExit;
begin
  {switch off the capture to the form}
  SetCaptureControl(NIL);
  Close;
end;

procedure TSaverForm.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  SafeExit;
end;

procedure TSaverForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  SafeExit;
end;

procedure TSaverForm.FormMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if (ptMpos.X = -1) and (ptMpos.Y = -1) then
  begin
    ptMpos.X := X;                  { Grab the X }
    ptMpos.Y := Y;                  { Grab the Y }
  end else
  if (Abs(X - ptMpos.X) > 2) and (Abs(Y - ptmPos.Y) > 2) then
    SafeExit;                       { Get the Password }
end;

procedure TSaverForm.Sprite1Move(Sender: TObject; var Bounds: TRect);
begin
  Bounds.Left := Bounds.Left+(Sender as TSprite).VX;
  if Bounds.Left > SpriteBox1.Width then
  begin
    Bounds.Left := 0-(Sender as TSprite).Width;
    Bounds.Top := (Random(5)+1)*(SpriteBox1.Height div 6);
    if Bounds.Top < SpriteBox1.Height div 4 then
      Bounds.Top := SpriteBox1.Height div 4;
    if Bounds.Top > SpriteBox1.Height then
      Bounds.Top := SpriteBox1.Height-SpriteBox1.Height div 6;
  end;
  if Bounds.Left mod 4 = 0 then
    Bounds.Top := Bounds.Top-(Sender as TSprite).VY;
end;

procedure TSaverForm.FormShow(Sender: TObject);
begin
  WavePlayer1.Play;
end;

begin
  randomize;
end.
