unit Setup;
{$N-}

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, StdCtrls,
     Buttons, IniFiles, SysUtils, ExtCtrls, ColorGrd, Dialogs, Spritebx;

type
  TSetupForm = class(TForm)
    Ok: TBitBtn;
    Cancel: TBitBtn;
    GroupBox1: TGroupBox;
    BitBtn1: TBitBtn;
    ColorGrid1: TColorGrid;
    Label1: TLabel;
    OpenDialog1: TOpenDialog;
    Label3: TLabel;
    Panel1: TPanel;
    SpriteBox1: TSpriteBox;
    Label2: TLabel;
    BitBtn2: TBitBtn;
    CheckBox1: TCheckBox;
    CheckBox2: TCheckBox;
    CheckBox3: TCheckBox;
    CheckBox4: TCheckBox;
    procedure CancelClick(Sender: TObject);
    procedure OkClick(Sender: TObject);
    procedure Image1Click(Sender: TObject);
    procedure ColorGrid1Change(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
    procedure CheckBox2Click(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure CheckBox3Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private declarations }
    procedure CreateProportionalImage(FileName: string);
  public

    { Public declarations }
  end;

var
  SetupForm : TSetupForm;

implementation

{$R *.DFM}

procedure TSetupForm.CancelClick(Sender: TObject);
begin
  Close;                                                   { Bye Bye }
end;

procedure TSetupForm.OkClick(Sender: TObject);
var
  Ini: TIniFile;
begin
  Ini := TIniFile.Create('CONTROL.INI'); { Create the Ini file }
  Ini.WriteInteger(Application.Title,'Color',LongInt(ColorGrid1.ForegroundColor));
  Ini.WriteInteger(Application.Title,'ColorSel',ColorGrid1.ForegroundIndex);
  Ini.WriteString(Application.Title,'FileName',OpenDialog1.FileName);
  Ini.WriteBool(Application.Title,'Stretch',SpriteBox1.Stretch);
  Ini.WriteBool(Application.Title,'Center',SpriteBox1.Center);
  Ini.WriteBool(Application.Title,'Gradient',SpriteBox1.Gradient);
  Ini.WriteBool(Application.Title,'Sound',CheckBox4.Checked);
  Ini.Free;
  Close;
end;

procedure TSetupForm.Image1Click(Sender: TObject);
begin
  if OpenDialog1.Execute then
  begin
    Label3.Caption := ExtractFileName(OpenDialog1.FileName);
    CreateProportionalImage(OpenDialog1.FileName);
  end;
end;

procedure TSetupForm.CreateProportionalImage(FileName: string);
var
  TmpBMP, PropBMP: TBitMap;
  Dest: TRect;
begin
  Label3.Caption := ExtractFileName(FileName);
  TmpBMP := TBitMap.Create;
  PropBMP := TBitMap.Create;
  Dest := ClientRect;
  try
    TmpBMP.LoadFromFile(FileName);
    Dest.Right := Round((TmpBMP.Width/Screen.Width)*SpriteBox1.Width);
    Dest.Bottom := Round((TmpBMP.Height/Screen.Height)*SpriteBox1.Height);
    PropBMP.Width := Dest.Right;
    PropBMP.Height := Dest.Bottom;
    PropBMP.Canvas.StretchDraw(Dest,TmpBMP);
    SpriteBox1.Image.Assign(PropBMP);
  finally
    TmpBMP.Free;
    PropBMP.Free;
  end;
end;

procedure TSetupForm.ColorGrid1Change(Sender: TObject);
begin
  SpriteBox1.Color := (Sender as TColorGrid).ForegroundColor;
end;

procedure TSetupForm.BitBtn2Click(Sender: TObject);
begin
  SpriteBox1.Image.Assign(nil);
  Label3.Caption := '';
  OpenDialog1.FileName := '';
  ColorGrid1.ForeGroundIndex := 0;
  CheckBox1.Checked := false;
  CheckBox2.Checked := true;
  CheckBox3.Checked := false;
  CheckBox4.Checked := false;
end;

procedure TSetupForm.CheckBox1Click(Sender: TObject);
begin
  SpriteBox1.Stretch := not SpriteBox1.Stretch;
end;

procedure TSetupForm.CheckBox2Click(Sender: TObject);
begin
  SpriteBox1.Center := not SpriteBox1.Center;
end;

procedure TSetupForm.CheckBox3Click(Sender: TObject);
begin
  SpriteBox1.Gradient := not SpriteBox1.Gradient;
end;

procedure TSetupForm.BitBtn1Click(Sender: TObject);
begin
  MessageDlg('Delphi Screen Saver, by HomeGrown Software Development.'#13+
             'Display logos on a colored background, with or without'#13+
             'a background image.', mtInformation, [mbOk], 0);
end;

procedure TSetupForm.FormShow(Sender: TObject);
var
  Ini: TIniFile;
begin
  { Create the Ini file }
  Ini := TIniFile.Create('CONTROL.INI');
  ColorGrid1.ForeGroundIndex := Ini.ReadInteger(Application.Title,'ColorSel',0);
  if (Ini.ReadString(Application.Title,'FileName','') <> '') and
    FileExists(Ini.ReadString(Application.Title,'FileName','')) then
      CreateProportionalImage(Ini.ReadString(Application.Title,'FileName',''));
  OpenDialog1.FileName := Ini.ReadString(Application.Title,'FileName','');
  CheckBox1.Checked := Ini.ReadBool(Application.Title,'Stretch',false);
  CheckBox2.Checked := Ini.ReadBool(Application.Title,'Center',false);
  CheckBox3.Checked := Ini.ReadBool(Application.Title,'Gradient',false);
  CheckBox4.Checked := Ini.ReadBool(Application.Title,'Sound',false);
  Ini.Free;
end;

end.
