{ *****************************************************
                       SSaver Expert

  A project expert to create a 16 bit screen saver application.

                  Paul Warren
         HomeGrown Software Development
       (c) 1996 Langley British Columbia.
                (604) 530-9097
         e-mail:  hg_soft@uniserve.com
    Home page: http://haven.uniserve.com/~hg_soft
  ***************************************************** }

unit Ssexpt16;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Buttons, StdCtrls, ExtCtrls, Proxies, ExptIntf, ToolIntf,
  Tranimag, Browse, Sprites, Spritebx;

type
  TSSExptForm = class(TForm)
    Edit2: TEdit;
    Label1: TLabel;
    RadioGroup1: TRadioGroup;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    Edit1: TEdit;
    Label2: TLabel;
    BitBtn4: TBitBtn;
    Edit3: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    Timer1: TTimer;
    Panel1: TPanel;
    SpriteBox1: TSpriteBox;
    Sprite1: TSprite;
    Bevel1: TBevel;
    procedure BitBtn3Click(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
    procedure BitBtn4Click(Sender: TObject);
    procedure Edit2KeyPress(Sender: TObject; var Key: Char);
    procedure Edit3KeyPress(Sender: TObject; var Key: Char);
    procedure Edit3Enter(Sender: TObject);
    procedure Sprite1Move(Sender: TObject; var Bounds: TRect);
    procedure Timer1Timer(Sender: TObject);
  private
    { Private declarations }
    SourceBuffer: PChar;
    procedure FmtWrite(Stream: TStream; Fmt: PChar; const Args: array of const);
    function DoFormCreation: TForm;
    function CreateProjectSource: TFileName;
    function CreateFormSource: TMemoryStream;
    function CreateAboutSource: TMemoryStream;
    function CreateSetupSource: TMemoryStream;
    function CreateForm(const FormIdent: string): TMemoryStream;
    function CreateAboutForm(const FormIdent: string): TMemoryStream;
    function CreateSetupForm(const FormIdent: string): TMemoryStream;
  end;

type
  TSSaverProjectExpert = class(TIExpert)
  public
    function GetStyle: TExpertStyle; override;
    function GetIDString: string; override;
    function GetName: string; override;
    function GetComment: string; override;
    function GetGlyph: HBITMAP; override;
    function GetState: TExpertState; override;
    function GetMenuText: string; override;
    procedure Execute; override;
  end;

procedure Register;

var
  SSExptForm: TSSExptForm;

implementation

uses VirtIntf, IStreams;

{$R *.DFM}

const
  SourceBufferSize = 1024;

procedure Register;
begin
  RegisterLibraryExpert(TSSaverProjectExpert.Create);
end;

{ TSSaverProjectExpert code }

{$R SSEXPT16.RES}
const
  SSEXPTBITMAP = 777; { Bitmap ID }

function TSSaverProjectExpert.GetStyle: TExpertStyle;
begin
  Result := esProject;
end;

function TSSaverProjectExpert.GetIDString: String;
begin
  Result := 'hg_soft.SSaverProjectExpert';
end;

function TSSaverProjectExpert.GetName: String;
begin
  Result := 'Screen Saver Generator'
end;

function TSSaverProjectExpert.GetComment: String;
begin
  Result := 'This expert generates a Screen Saver application framework. '+
    'Screen Saver Expert (c) 1996 by Paul Warren. Generated by '+
    'HomeGrown''s ''Expert'' expert.';
end;

function TSSaverProjectExpert.GetGlyph: HBITMAP;
begin
  Result := LoadBitMap(HInstance, MakeIntResource(SSEXPTBITMAP))
end;

function TSSaverProjectExpert.GetState: TExpertState;
begin
  Result := [esEnabled];
end;

function TSSaverProjectExpert.GetMenuText: String;
begin
  Result := ''
end;

procedure TSSaverProjectExpert.Execute;
begin
  if not Assigned(SSExptForm) then
    SSExptForm := TSSExptForm.Create(Application);
  SSExptForm.Show;
  SSExptForm.SetFocus
end;

{ TForm1 code }

procedure TSSExptForm.BitBtn3Click(Sender: TObject);
begin
  Application.MessageBox('HomeGrown''s Screen Saver project expert.', 'HomeGrown Expert',  MB_OK or MB_ICONINFORMATION );
end;

procedure TSSExptForm.BitBtn2Click(Sender: TObject);
begin
  Close;
end;

procedure TSSExptForm.FmtWrite(Stream: TStream; Fmt: PChar;
  const Args: array of const);
begin
  if (Stream <> nil) and (SourceBuffer <> nil) then
  begin
    StrLFmt(SourceBuffer, SourceBufferSize, Fmt, Args);
    Stream.Write(SourceBuffer[0], StrLen(SourceBuffer));
  end;
end;

function TSSExptForm.CreateProjectSource: TFileName;
const
  CRLF = #13#10;
var
  ProjectFile: TFileStream;
begin
  Result := Edit1.Text;
  if (Result > '') and not (Result[Length(Result)] in [':', '\']) then
    Result := Result + '\';
  Result := Result + Edit2.Text + '.DPR';
  ProjectFile := TFileStream.Create(Result, fmCreate);
  try
    FmtWrite(ProjectFile,
    'program %s;'+CRLF+CRLF+
    '{$D SCRNSAVE : %s}'+CRLF+CRLF+
    'uses'+CRLF+
    '  Forms,'+CRLF+
    '  SysUtils;'+CRLF+CRLF, [Edit2.Text,Edit3.Text]);

    FmtWrite(ProjectFile,
    '{$R *.RES}'+CRLF+CRLF+
    'var'+CRLF+
    '  CmdLine: String;'+CRLF+
    'begin'+CRLF+
    '  CmdLine := UpperCase(ParamStr(1));'+CRLF+
    '  if (CmdLine  = ''/C'') or (CmdLine = ''-C'') or (CmdLine = ''C'') then'+CRLF+
    '  begin'+CRLF, [nil]);

    case RadioGroup1.ItemIndex of
      0: begin
        FmtWrite(ProjectFile,
        '    AboutBox := TAboutBox.Create(Application);'+CRLF+
        '    AboutBox.ShowModal;'+CRLF+
        '    AboutBox.Free;'+CRLF, [nil]);
      end;
      1: begin
        FmtWrite(ProjectFile,
        '    SetupForm := TSetupForm.Create(Application);'+CRLF+
        '    SetupForm.ShowModal;'+CRLF+
        '    SetupForm.Free;'+CRLF, [nil]);
      end;
    end;

    FmtWrite(ProjectFile,
    '  end else begin'+CRLF+
    '    Application.Title := ''%s'';'+CRLF+
    '    Application.Run;'+CRLF+
    '  end;'+CRLF+
    'end.' , [Edit3.Text]);
  finally
    ProjectFile.Free;
  end;
end;

{ CreateSource method - write the synchronous source to SourceBuffer. }
function TSSExptForm.CreateFormSource: TMemoryStream;
const
  CRLF = #13#10;
begin
  Result := TMemoryStream.Create;
  try
    FmtWrite(Result,
      'unit Main;'+CRLF+CRLF+
      'interface'+CRLF+CRLF+
      'uses'+CRLF+
      '  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,'+CRLF+
      '  Forms, Dialogs;'+CRLF+CRLF+
      'type'+CRLF+
      '  TSaverForm = class(TForm)'+CRLF+
      '    procedure FormCreate(Sender: TObject);'+CRLF, [nil]);

    FmtWrite(Result,
      '    procedure FormActivate(Sender: TObject);'+CRLF+
      '    procedure FormClose(Sender: TObject; var Action: TCloseAction);'+CRLF+
      '    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;'+CRLF+
      '      Shift: TShiftState; X, Y: Integer);'+CRLF, [nil]);

    FmtWrite(Result,
      '    procedure FormKeyDown(Sender: TObject; var Key: Word;'+CRLF+
      '      Shift: TShiftState);'+CRLF+
      '    procedure FormMouseMove(Sender: TObject; Shift: TShiftState; X,'+CRLF+
      '      Y: Integer);'+CRLF+
      '  private'+CRLF, [nil]);

    FmtWrite(Result,
      '    { Private declarations }'+CRLF+
      '    ptMpos: TPoint;'+CRLF+
      '    procedure Trigger(Sender: TObject; var Done: Boolean);'+CRLF+
      '    procedure SafeExit;'+CRLF+
      '  public'+CRLF+
      '    { Public declarations }'+CRLF, [nil]);

    FmtWrite(Result,
      '  end;'+CRLF+CRLF+
      'var'+CRLF+
      '  SaverForm: TSaverForm;'+CRLF+
      '  ScreenWd: Integer;'+CRLF+
      '  ScreenHt: Integer;'+CRLF+CRLF, [nil]);

    FmtWrite(Result,
      'implementation'+CRLF+CRLF+
      '{$R *.DFM}'+CRLF+CRLF+
      'procedure CursorOff;'+CRLF+
      'var'+CRLF+
      '  Cstate : Integer;'+CRLF+
      'begin'+CRLF+
      '  Cstate := ShowCursor(True);'+CRLF+
      '  while Cstate >= 0 do Cstate := ShowCursor(False);'+CRLF+
      'end;'+CRLF+CRLF, [nil]);

     FmtWrite(Result,
     'procedure CursorOn;'+CRLF+
     'var'+CRLF+
     '  Cstate : Integer;'+CRLF+
     'begin'+CRLF+
     '  Cstate := ShowCursor(True);'+CRLF+
     '  while Cstate < 0 do Cstate := ShowCursor(True);'+CRLF+
     'end;'+CRLF+CRLF, [nil]);

     FmtWrite(Result,
     '{ TSaverForm code }'+CRLF+CRLF+
     'procedure TSaverForm.FormCreate(Sender: TObject);'+CRLF+
     'begin'+CRLF+
     '  Top := 0; Left := 0;'+CRLF+
     '  Width := Screen.Width;'+CRLF+
     '  Height := Screen.Height;'+CRLF+
     '  BorderStyle := bsNone;'+CRLF+
     'end;'+CRLF+CRLF, [nil]);

     FmtWrite(Result,
     'procedure TSaverForm.FormActivate(Sender: TObject);'+CRLF+
     'begin'+CRLF+
     '  CursorOff;'+CRLF+
     '  ptMpos.X := -1;'+CRLF+
     '  ptMpos.Y := -1;'+CRLF+
     '  Application.OnIdle := Trigger; { Set the OnIdle Proc }'+CRLF+
     '  {set this form as the capture control}'+CRLF+
     '  SetCaptureControl(Self);'+CRLF+
     'end;'+CRLF+CRLF, [nil]);

     FmtWrite(Result,
     'procedure TSaverForm.FormClose(Sender: TObject; var Action: TCloseAction);'+CRLF+
     'begin'+CRLF+
     '  Application.OnIdle := nil;'+CRLF+
     '  CursorOn;'+CRLF+
     'end;'+CRLF+CRLF, [nil]);

     FmtWrite(Result,
     'procedure TSaverForm.Trigger(Sender: TObject; var Done: Boolean);'+CRLF+
     'begin'+CRLF+
     '  PostMessage(Handle, WM_USER + 1,0,0);'+CRLF+
     'end;'+CRLF+CRLF, [nil]);

     FmtWrite(Result,
     'procedure TSaverForm.SafeExit;'+CRLF+
     'begin'+CRLF+
     '  {switch off the capture to the form}'+CRLF+
     '  SetCaptureControl(NIL);'+CRLF+
     '  Close;'+CRLF+
     'end;'+CRLF+CRLF, [nil]);

     FmtWrite(Result,
     'procedure TSaverForm.FormMouseDown(Sender: TObject; Button: TMouseButton;'+CRLF+
     '  Shift: TShiftState; X, Y: Integer);'+CRLF+
     'begin'+CRLF+
     '  SafeExit;'+CRLF+
     'end;'+CRLF+CRLF, [nil]);

     FmtWrite(Result,
     'procedure TSaverForm.FormKeyDown(Sender: TObject; var Key: Word;'+CRLF+
     '  Shift: TShiftState);'+CRLF+
     'begin'+CRLF+
     '  SafeExit;'+CRLF+
     'end;'+CRLF+CRLF, [nil]);

     FmtWrite(Result,
     'procedure TSaverForm.FormMouseMove(Sender: TObject; Shift: TShiftState; X,'+CRLF+
     '  Y: Integer);'+CRLF+
     'begin'+CRLF+
     '  if (ptMpos.X = -1) and (ptMpos.Y = -1) then'+CRLF, [nil]);

     FmtWrite(Result,
     '  begin'+CRLF+
     '    ptMpos.X := X;'+CRLF+
     '    ptMpos.Y := Y;'+CRLF+
     '  end else'+CRLF+
     '  if (Abs(X - ptMpos.X) > 2) and (Abs(Y - ptmPos.Y) > 2) then'+CRLF+
     '    SafeExit;'+CRLF+
     'end;'+CRLF+CRLF+
     'end.', [nil]);

    Result.Position := 0;
  except
    Result.Free;
    raise;
  end;
end;

{ CreateAboutSource method - write the synchronous source to SourceBuffer. }
function TSSExptForm.CreateAboutSource: TMemoryStream;
const
  CRLF = #13#10;
begin
  Result := TMemoryStream.Create;
  try
    FmtWrite(Result,
    'unit About;'+CRLF+CRLF+
    'interface'+CRLF+CRLF+
    'uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, StdCtrls,'+CRLF+
    '  Buttons, ExtCtrls;'+CRLF+CRLF+
    'type'+CRLF+
    '  TAboutBox = class(TForm)'+CRLF, [nil]);

    FmtWrite(Result,
    '    Panel1: TPanel;'+CRLF+
    '    OKButton: TBitBtn;'+CRLF+
    '    ProgramIcon: TImage;'+CRLF+
    '    ProductName: TLabel;'+CRLF+
    '    Version: TLabel;'+CRLF+
    '    Copyright: TLabel;'+CRLF, [nil]);

    FmtWrite(Result,
    '    Comments: TLabel;'+CRLF+
    '  private'+CRLF+
    '    { Private declarations }'+CRLF+
    '  public'+CRLF+
    '    { Public declarations }'+CRLF+
    '  end;'+CRLF+CRLF, [nil]);

    FmtWrite(Result,
    'var'+CRLF+
    '  AboutBox: TAboutBox;'+CRLF+CRLF+
    'implementation'+CRLF+CRLF+
    '{$R *.DFM}'+CRLF+CRLF+
    'end.', [nil]);

    Result.Position := 0;
  except
    Result.Free;
    raise;
  end;
end;

{ CreateSetupSource method - write the synchronous source to SourceBuffer. }
function TSSExptForm.CreateSetupSource: TMemoryStream;
const
  CRLF = #13#10;
begin
  Result := TMemoryStream.Create;
  try
    FmtWrite(Result,
    'unit Setup;'+CRLF+
    '{$N-}'+CRLF+CRLF+
    'interface'+CRLF+CRLF+
    'uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, StdCtrls,'+CRLF+
    '  Buttons, IniFiles, SysUtils, ExtCtrls, ColorGrd, Dialogs, Spritebx;'+CRLF+CRLF+
    'type'+CRLF+
    '  TSetupForm = class(TForm)'+CRLF+
    '    Ok: TBitBtn;'+CRLF, [nil]);

    FmtWrite(Result,
    '    Cancel: TBitBtn;'+CRLF+
    '    GroupBox1: TGroupBox;'+CRLF+
    '    BitBtn1: TBitBtn;'+CRLF+
    '    procedure CancelClick(Sender: TObject);'+CRLF+
    '    procedure OkClick(Sender: TObject);'+CRLF+
    '    procedure FormShow(Sender: TObject);'+CRLF+
    '  private'+CRLF, [nil]);

    FmtWrite(Result,
    '    { Private declarations }'+CRLF+
    '  public'+CRLF+
    '    { Public declarations }'+CRLF+
    '  end;'+CRLF+CRLF+
    'var'+CRLF+
    '  SetupForm : TSetupForm;'+CRLF+CRLF+
    'implementation'+CRLF+CRLF+
    '{$R *.DFM}'+CRLF+CRLF, [nil]);

    FmtWrite(Result,
    'procedure TSetupForm.CancelClick(Sender: TObject);'+CRLF+
    'begin'+CRLF+
    '  Close;'+CRLF+
    'end;'+CRLF+CRLF+
    'procedure TSetupForm.OkClick(Sender: TObject);'+CRLF+
    'var'+CRLF+
    '  Ini: TIniFile;'+CRLF+
    'begin'+CRLF+
    '  Ini := TIniFile.Create(''CONTROL.INI'');'+CRLF+
    '  Ini.Free;'+CRLF+
    '  Close;'+CRLF+
    'end;'+CRLF+CRLF, [nil]);

    FmtWrite(Result,
    'procedure TSetupForm.FormShow(Sender: TObject);'+CRLF+
    'var'+CRLF+
    '  Ini: TIniFile;'+CRLF+
    'begin'+CRLF+
    '  { Create the Ini file }'+CRLF+
    '  Ini := TIniFile.Create(''CONTROL.INI'');'+CRLF+
    '  Ini.Free;'+CRLF+
    'end;'+CRLF+CRLF+
    'end.', [nil]);

    Result.Position := 0;
  except
    Result.Free;
    raise;
  end;
end;

{ DoFormCreation method - Create the dialog defined by the user }
function TSSExptForm.DoFormCreation: TForm;
var
  Method: TMethod;
begin
  Result := TProxyForm.CreateAs('TSaverForm');
  with Result do
  begin
    BorderStyle := bsNone;
    Top := 50; Left := 100;
    Width := 300;
    Height := 200;
    Name := 'SaverForm';
  end;

  { create the Form's OnCreate method }
  Method.Code := TProxyForm(Result).CreateMethod('FormCreate');
  Method.Data := Result;
  Result.OnCreate := TNotifyEvent(Method);

  { create the Form's OnActivate method }
  Method.Code := TProxyForm(Result).CreateMethod('FormActivate');
  Method.Data := Result;
  Result.OnActivate := TNotifyEvent(Method);

  { create the Form's OnClose method }
  Method.Code := TProxyForm(Result).CreateMethod('FormClose');
  Method.Data := Result;
  Result.OnClose := TCloseEvent(Method);

  { create the Form's OnMouseDown method }
  Method.Code := TProxyForm(Result).CreateMethod('FormMouseDown');
  Method.Data := Result;
  Result.OnMouseDown := TMouseEvent(Method);

  { create the Form's OnKeyDown method }
  Method.Code := TProxyForm(Result).CreateMethod('FormKeyDown');
  Method.Data := Result;
  Result.OnKeyDown := TKeyEvent(Method);

  { create the Form's OnMouseMove method }
  Method.Code := TProxyForm(Result).CreateMethod('FormMouseMove');
  Method.Data := Result;
  Result.OnMouseMove := TMouseMoveEvent(Method);
end;

{ CreateForm method - create the form, write it out to disk }
function TSSExptForm.CreateForm(const FormIdent: string): TMemoryStream;
var
  NewForm: TForm;
begin
  Result := nil;
  NewForm := DoFormCreation;
  try
    Result := TMemoryStream.Create;
    Result.WriteComponentRes(FormIdent+'.DFM', NewForm);
    Result.Position := 0;
  finally
    NewForm.Free;
  end;
end;

{ CreateAboutForm method - create the form, write it out to disk }
function TSSExptForm.CreateAboutForm(const FormIdent: string): TMemoryStream;
var
  NewForm: TForm;
begin
  Result := nil;
  NewForm := TProxyForm.CreateAs('TAboutBox');
  try
    Result := TMemoryStream.Create;
    ReadComponentResFile('\WINDOWS\SYSTEM\HGA.DFM', NewForm);
    Result.WriteComponentRes(FormIdent+'.DFM', NewForm);
    Result.Position := 0;
  finally
    NewForm.Free;
  end;
end;

{ CreateSetupForm method - create the form, write it out to disk }
function TSSExptForm.CreateSetupForm(const FormIdent: string): TMemoryStream;
var
  NewForm: TForm;
begin
  Result := nil;
  NewForm := TProxyForm.CreateAs('TSetupForm');
  try
    Result := TMemoryStream.Create;
    ReadComponentResFile('\WINDOWS\SYSTEM\HGS.DFM', NewForm);
    Result.WriteComponentRes(FormIdent+'.DFM', NewForm);
    Result.Position := 0;
  finally
    NewForm.Free;
  end;
end;

procedure TSSExptForm.BitBtn1Click(Sender: TObject);
var
  ProjectName: TFileName;
  ISourceStream, IFormStream,
  IAboutSourceStream, IAboutFormStream,
  ISetupSourceStream, ISetupFormStream: TIMemoryStream;
begin
  SourceBuffer := StrAlloc(SourceBufferSize);
  try
    if ToolServices <> nil then { I'm an expert!! }
    begin
      try
        IFormStream := TIMemoryStream.Create(CreateForm('Main'));
        ISourceStream := TIMemoryStream.Create(CreateFormSource);
        case RadioGroup1.ItemIndex of
          0: begin
            IAboutFormStream := TIMemoryStream.Create(CreateAboutForm('About'));
            IAboutSourceStream := TIMemoryStream.Create(CreateAboutSource);
          end;
          1: begin
            ISetupFormStream := TIMemoryStream.Create(CreateSetupForm('Setup'));
            ISetupSourceStream := TIMemoryStream.Create(CreateSetupSource);
          end;
        end;
        { open the new project }
        if ToolServices.CloseProject then ToolServices.OpenProject(CreateProjectSource);
        { create the new modules }
        ToolServices.CreateModule(Edit1.Text+'\'+'Main.pas', ISourceStream, IFormStream,
          [cmAddToProject, cmShowSource, cmShowForm, cmMainForm, cmMarkModified]);
         case RadioGroup1.ItemIndex of
           0: ToolServices.CreateModule(Edit1.Text+'\'+'About.pas', IAboutSourceStream, IAboutFormStream,
                [cmAddToProject, cmShowSource, cmShowForm, cmMarkModified]);
           1: ToolServices.CreateModule(Edit1.Text+'\'+'Setup.pas', ISetupSourceStream, ISetupFormStream,
                [cmAddToProject, cmShowSource, cmShowForm, cmMarkModified]);
         end;
      finally
        ISourceStream.OwnStream := true;
        ISourceStream.Free;
        IFormStream.OwnStream := true;
        IFormStream.Free;
        case RadioGroup1.ItemIndex of
          0: begin
            IAboutSourceStream.OwnStream := true;
            IAboutSourceStream.Free;
            IAboutFormStream.OwnStream := true;
            IAboutFormStream.Free;
          end;
          1: begin
            ISetupSourceStream.OwnStream := true;
            ISetupSourceStream.Free;
            ISetupFormStream.OwnStream := true;
            ISetupFormStream.Free;
          end;
        end;
      end;
    end;
  finally
    StrDispose(SourceBuffer);
    Close;
  end;
end;

procedure TSSExptForm.BitBtn4Click(Sender: TObject);
begin
  BrowseDlg := TBrowseDlg.Create(nil);
  BrowseDlg.ShowModal;
  if BrowseDlg.ModalResult <> mrCancel then
    Edit1.Text := BrowseDlg.DirectoryListBox1.Directory;
  BrowseDlg.Free;
end;

procedure TSSExptForm.Edit2KeyPress(Sender: TObject; var Key: Char);
begin
  case Key of
    'a'..'z', 'A'..'Z', '0'..'9', '-', '_',
    #0..#31: ;
  else
    Key := #0;
  end;
end;

procedure TSSExptForm.Edit3KeyPress(Sender: TObject; var Key: Char);
begin
  case Key of
    'a'..'z', 'A'..'Z', '0'..'9', '-', '_',
    #0..#33, #35..#38, #40..#43, #46: ;
  else
    Key := #0;
  end;
end;

procedure TSSExptForm.Edit3Enter(Sender: TObject);
begin
  (Sender as TEdit).SelStart := 13;
end;

procedure TSSExptForm.Sprite1Move(Sender: TObject; var Bounds: TRect);
begin
  if ((Bounds.Left <= 0) or (Bounds.Right >= SpriteBox1.Width)) then
    (Sender as TSprite).VX := -(Sender as TSprite).VX;
  if ((Bounds.Top <= 0) or (Bounds.Bottom >= SpriteBox1.Height)) then
    (Sender as TSprite).VY := -(Sender as TSprite).VY;
  Bounds.Left := Bounds.Left+(Sender as TSprite).VX;
  Bounds.Top := Bounds.Top+(Sender as TSprite).VY;
end;

procedure TSSExptForm.Timer1Timer(Sender: TObject);
begin
  SpriteBox1.DrawSprite;
end;

end.
