unit DMLogin;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DB, DBTables, Login;

const
  evtLoginSuccessful  = 100;
  evtLoginFail        = 101;
  evtLogout           = 102;
  evtChangePassword   = 103;

type
  TLoginDM = class(TDataModule)
    qryGetUserValues: TQuery;
    qryPostAuditTrail: TQuery;
    qrySetLastLoggedInDate: TQuery;
  protected
    FLogin: TLoginManager;
  public
    constructor Create(AOwner: TComponent); override;
    procedure GetUserValues(var UserID: LongInt;
                            var FirstName: String;
                            var LastName: String;
                            var DateLastLogin: TDateTime;
                            var PasswordExpired: Boolean);
    procedure PostAuditTrail(EventID: Integer; EventMsg: String);
    procedure PostUserLoginDate;
  end;

var
  LoginDM: TLoginDM;

implementation

{$R *.DFM}

constructor TLoginDM.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  if not (AOwner is TLoginManager) then
    raise Exception.Create('TLoginDM cannot be created independently of TLoginManager.');

  FLogin := AOwner as TLoginManager;
end;

procedure TLoginDM.GetUserValues(var UserID: LongInt;
                                 var FirstName: String;
                                 var LastName: String;
                                 var DateLastLogin: TDateTime;
                                 var PasswordExpired: Boolean);
{ Returns key information about the current user }
begin
  with qryGetUserValues do
  begin
    Close;
    ParamByName('Username').AsString := FLogin.Username;
    Open;
    try
      UserID          := FieldByName('UserID').AsInteger;
      FirstName       := FieldByName('FirstName').AsString;
      LastName        := FieldByName('LastName').AsString;
      DateLastLogin   := FieldByName('DateLastLogin').AsDateTime;
      PasswordExpired := False; { we'll expand this next month }
    finally
      Close;
    end;
  end;
end;

procedure TLoginDM.PostAuditTrail(EventID: Integer; EventMsg: String);
{ Writes an entry in the audit trail }
begin
  with qryPostAuditTrail do
  begin
    ParamByName('ApplicationID').AsInteger := FLogin.ApplicationID;
    if FLogin.UserID = cNoUserID then  { This would happen when posting a bad login event }
      ParamByName('UserID').Clear     { We want a null userID in this case }
    else
      ParamByName('UserID').AsInteger := FLogin.UserID;

    ParamByName('EventID').AsInteger := EventID;
    ParamByName('Description').AsString := EventMsg;
    ExecSQL;
  end;
end;

procedure TLoginDM.PostUserLoginDate;
{ Writes the current date as the user's "date last logged in" }
begin
  with qrySetLastLoggedInDate do
  begin
    ParamByName('UserID').AsInteger := FLogin.UserID;
    ExecSQL;
  end;
end;

end.
