unit xFileDnD;
  { accept many files from windows explorer and convert it to
    a Delphi Drop event }

interface

uses
  Windows, Messages, Classes, Controls, Forms;

type
  TxDropFile = class(TComponent)
  private
    FDrop        : THandle;
    FOldWndProc  : TFarProc;
    FNextDestroy : TNotifyEvent;
    FOnDrop      : TNotifyEvent;
    FFiles       : TStrings;
    procedure wmDropFiles(var Msg: TMessage);
    procedure FormWndProc(var Message: TMessage);
    procedure FormDestroy(Sender:TObject);
    procedure DefaultProc(var Message: TMessage);
    function  GetFileCount: Integer;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    property  Files: TStrings read FFiles;
    property  FileCount: Integer read GetFileCount;
  published
    property  OnDrop: TNotifyEvent read FOnDrop write FOnDrop;
  end;

procedure Register;

implementation

uses
  ShellAPI, SysUtils;

{ TxDropFile }

constructor TxDropFile.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  if not (csDesigning in ComponentState) then
     if Owner is TForm then
        with TForm(Owner) do
        begin
          FOldWndProc:=TFarProc(SetWindowLong(Handle,GWL_WNDPROC,
            Longint(MakeObjectInstance(FormWndProc))));
          FNextDestroy:=OnDestroy;
          OnDestroy:=FormDestroy;
          DragAcceptFiles(Handle, True);
        end;

  FDrop:=0;
  FFiles:=TStringList.Create;
end;

destructor TxDropFile.Destroy;
begin
  FFiles.Free;
  if FDrop<>0 then DragFinish(FDrop);
  inherited Destroy;
end;

function TxDropFile.GetFileCount: Integer;
begin
  Result:=FFiles.Count;
end;

procedure TxDropFile.wmDropFiles(var Msg: TMessage);
var
  i,
  aFiles,
  aLen      : Integer;
  aFilename : array[0..150] of char;
  aPoint    : TPoint;
  aControl  : TControl;
begin
  Screen.Cursor := crHourglass;
  try
    FDrop:=Msg.wParam;
    DragQueryPoint(FDrop,aPoint);
    aPoint:=TForm(Owner).ClientToScreen(aPoint);

    aFiles:=DragQueryFile(FDrop, $FFFFFFFF, Nil, 0);
    FFiles.Clear;
    for i:=0 to aFiles-1 do
    begin
      aLen:=DragQueryFile(Msg.wParam, i,nil,0);
      DragQueryFile(Msg.wParam, i, aFilename, aLen+1);
      FFiles.Add(StrPas(aFileName));
    end;

    if Assigned(FOnDrop) then FOnDrop(Self);

    aControl:=FindDragTarget(aPoint,False);
    if Assigned(aControl) then
    begin
      aPoint:=aControl.ScreenToClient(aPoint);
      aControl.DragDrop(Self,aPoint.X,aPoint.Y);
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TxDropFile.DefaultProc(var Message:TMessage);
begin
  with Message do
    Result:=CallWindowProc(FOldWndProc,TForm(Owner).Handle,Msg,wParam,lParam);
end;

procedure TxDropFile.FormWndProc(var Message:TMessage);
begin
  if Message.Msg = WM_DROPFILES then
     wmDropFiles(Message)
  else
     DefaultProc(Message);
end;

procedure TxDropFile.FormDestroy(Sender:TObject);
begin
  with TForm(Owner) do
    FreeObjectInstance(Pointer(SetWindowLong(Handle,GWL_WNDPROC,
       Longint(FOldWndProc))));
  if Assigned(FNextDestroy) then FNextDestroy(Sender);;
end;

procedure Register;
begin
  RegisterComponents('xTool', [TxDropFile]);
end;

end.
