unit Main1;

interface

uses
  Windows, SysUtils, Classes, TDMComm;

type
  TQueryDocument = class
  protected
    DataPath: String;
    Authors: TStringList;
    Keywords: TStringList;
    DBUpdateDate: TDateTime;
    LastIssue: Integer;
    LastIssueMonthYear: ShortString;
  public
    constructor Create;
    destructor Destroy; override;
    procedure GetDatabaseInfo;
    procedure SendOutputToClient;
  end;

var
  QueryDocument: TQueryDocument;

implementation

uses
  DB, DBTables;

constructor TQueryDocument.Create;
begin
  inherited Create;
  Authors := TStringList.Create;
  Authors.Sorted := True;
  Keywords := TStringList.Create;
  Keywords.Sorted := True;

  DataPath := ExtractFilePath(ParamStr(0)) + DataDirectory;

  GetDatabaseInfo;
  SendOutputToClient;
end;

destructor TQueryDocument.Destroy;
begin
  Authors.Free;
  Keywords.Free;
  inherited Destroy;
end;

procedure TQueryDocument.GetDatabaseInfo;
begin
  with TTable.Create(nil) do
  begin
    DatabaseName := DataPath;

    { Fill the authors list }
    TableName := 'AUTHOR.DB';
    ReadOnly := True;
    try
      Active := True;
      First;
      while not Eof do
      begin
        Authors.AddObject(FieldByName('NAME').AsString, Pointer(FieldByName('AUTHOR ID').AsInteger));
        Next
      end
    finally
      Active := False
    end;
  end;

  with TQuery.Create(nil) do
  begin
    try
      DatabaseName := DataPath;

      { Fill the keyword list }
      SQL.SetText('SELECT DISTINCT Keyword FROM KEYWORD');
      Open;
      try
        while not Eof do
        begin
          Keywords.Add(FieldByName('KEYWORD').AsString);
          Next
        end
      finally
        Close;
      end;

      { Find the last article indexed }
      SQL.SetText('SELECT MAX(ARTICLE."Issue #") FROM ARTICLE');
      Open;
      try
        LastIssue := Fields[0].Value;
        LastIssueMonthYear := IssueNumToMonthYear(LastIssue);
      finally
        Close;
      end;
    finally
      Free;
    end;
  end;

  { Find the date the data was last updated }
  DBUpdateDate := FileDateToDateTime(FileGetDate(FileOpen(DataPath + 'AUTHOR.DB', fmOpenRead)));
end;

procedure TQueryDocument.SendOutputToClient;
var
  OutputFCB: TextFile;
  I: Integer;
begin
  AssignFile(OutputFCB, '');
  Rewrite(OutputFCB);
  try
    WriteLn(OutputFCB, '<HTML><HEAD>');
    WriteLn(OutputFCB, '<TITLE>The Delphi Magazine Article Index Database - Search Form</TITLE>');
    WriteLn(OutputFCB, '</HEAD><BODY>');
    WriteLn(OutputFCB, '<BR><A HREF="' + DelphiMagURL + 'index.htm">Click to return to The Delphi Magazine main page...</A>');
    WriteLn(OutputFCB, '<P><IMG SRC="' + DelphiMagURL + 'dmlogo2a.gif" ALT="The Delphi Magazine" ALIGN=CENTER>');
    WriteLn(OutputFCB, '<FONT SIZE=+3><STRONG>Article Index Database</STRONG></FONT></P>');

    WriteLn(OutputFCB, '<HR>');
    WriteLn(OutputFCB, '<TABLE BORDER=0 WIDTH="100%" HEIGHT="10">');
    WriteLn(OutputFCB, '<TR><TD>');
    WriteLn(OutputFCB, Format('<H3>Database Last Updated: %s</H3>', [FormatDateTime('d mmm yy', DBUpdateDate)]));
    WriteLn(OutputFCB, '</TD>');
    WriteLn(OutputFCB, '<TD ALIGN=RIGHT>');
    WriteLn(OutputFCB, Format('<H3>Current Through Issue #%d  (%s)</H3>', [LastIssue, LastIssueMonthYear]));
    WriteLn(OutputFCB, '</TD></TR>');
    WriteLn(OutputFCB, '</TABLE>');
    WriteLn(OutputFCB, '<HR>');

    WriteLn(OutputFCB, '<STRONG>Enter your search criteria:</STRONG>');
    WriteLn(OutputFCB, '<BR>');
    WriteLn(OutputFCB, '<FORM METHOD="POST" ACTION="tdmaid2.exe">');

    WriteLn(OutputFCB, '<BR>');

    WriteLn(OutputFCB, '<TABLE BORDER=0 WIDTH="300" HEIGHT="60">');
    WriteLn(OutputFCB, '<TR><TD WIDTH="35%">Article Type</TD>');
    WriteLn(OutputFCB, '<TD><SELECT NAME="ArticleType" SIZE=1>');
    WriteLn(OutputFCB, '  <OPTION>&ltAny&gt</OPTION>');
    WriteLn(OutputFCB, '  <OPTION>Article</OPTION>');
    WriteLn(OutputFCB, '  <OPTION>Review</OPTION>');
    WriteLn(OutputFCB, '  <OPTION>Tip</OPTION>');
    WriteLn(OutputFCB, '  <OPTION>Clinic</OPTION>');
    WriteLn(OutputFCB, '  <OPTION>Disk</OPTION>');
    WriteLn(OutputFCB, '  <OPTION>Misc</OPTION>');
    WriteLn(OutputFCB, '</SELECT>');
    WriteLn(OutputFCB, '</TD></TR>');

    WriteLn(OutputFCB, '<TR><TD WIDTH="35%">Issue #</TD>');
    WriteLn(OutputFCB, '<TD><INPUT TYPE="TEXT" NAME="IssueNo" SIZE=11></TD></TR>');

    WriteLn(OutputFCB, '<TR><TD WIDTH="35%">Author</TD>');
    WriteLn(OutputFCB, '<TD><SELECT NAME="Author" SIZE=1>');
    WriteLn(OutputFCB, '  <OPTION>&ltAny&gt</OPTION>');
    for I := 0 to Authors.Count - 1 do
      WriteLn(OutputFCB, '  <OPTION VALUE="' + IntToStr(LongInt(Authors.Objects[I])) + '">' + Authors[I] + '</OPTION>');
    WriteLn(OutputFCB, '</SELECT>');
    WriteLn(OutputFCB, '</TD></TR>');

    WriteLn(OutputFCB, '<TR><TD WIDTH="35%">Keyword Select</TD>');
    WriteLn(OutputFCB, '<TD><SELECT NAME="KeywordSelect" SIZE=1>');
    WriteLn(OutputFCB, '  <OPTION>&ltAny&gt</OPTION>');
    for I := 0 to Keywords.Count - 1 do
      WriteLn(OutputFCB, '  <OPTION>' + Keywords[I] + '</OPTION>');
    WriteLn(OutputFCB, '</SELECT><BR>');
    WriteLn(OutputFCB, '</TD></TR>');

    WriteLn(OutputFCB, '<TR><TD WIDTH="35%">Keyword Search</TD>');
    WriteLn(OutputFCB, '<TD><INPUT TYPE="TEXT" NAME="KeywordSearch" SIZE=29></TD></TR>');

    WriteLn(OutputFCB, '</TABLE>');
    WriteLn(OutputFCB, '<BR>');

    WriteLn(OutputFCB, '<INPUT TYPE="SUBMIT" NAME="SubmitBtn" VALUE="Search">');
    WriteLn(OutputFCB, '<INPUT TYPE="RESET" NAME="ResetBtn" VALUE="Reset">');
    WriteLn(OutputFCB, '</FORM>');

    WriteLn(OutputFCB, '<BR>');
    WriteLn(OutputFCB, '<A HREF="' + BackIssuesURL + '">Click here to order back issues</A>.');

    WriteLn(OutputFCB, '<HR>');
    WriteLn(OutputFCB, 'The Article Index Query Page was developed by Steve Troxell ');
    WriteLn(OutputFCB, 'and Roy Salisbury of TurboPower Software''s Custom Group, based ');
    WriteLn(OutputFCB, 'on original work by Bob Swart, and is kindly hosted by ');
    WriteLn(OutputFCB, '<A HREF="' + TurboPowerURL + '">TurboPower Software</A>.');
    WriteLn(OutputFCB, '<BR>');
    WriteLn(OutputFCB, '<BR>');
    WriteLn(OutputFCB, 'TurboPower''s Custom Group develops database applications ');
    WriteLn(OutputFCB, 'for parent company Casino Data Systems to aid in casino management. ');
    WriteLn(OutputFCB, '');
    WriteLn(OutputFCB, '');
    WriteLn(OutputFCB, '</BODY></HTML>');
  finally
    CloseFile(OutputFCB);
  end;
end;

initialization
  QueryDocument := TQueryDocument.Create;
finalization
  QueryDocument.Free;
end.
