unit Main2;

interface

procedure Process;

implementation

uses
  CGIAPI, TDMComm, AidData, BldSQL,
  Windows, SysUtils, Classes, DB, DBTables;

procedure WriteBLOB(BlobField: TBlobField);
{ This procedure writes a BLOb datafield to the CGI output stream. }
var
  Buffer: PChar;
  Size: Integer;
  Stream: TBlobStream;
begin
  Stream := TBlobStream.Create(BlobField, bmRead);
  try
    Size := Stream.Size;
    Inc(Size);
    Buffer := AllocMem(Size);
    try
      Stream.Read(Buffer^, Size);
      CGI.Write(StrPas(Buffer));
    finally
      FreeMem(Buffer, Size);
    end;
  finally
    Stream.Free;
  end;
end;

procedure Process;
var
  A, I: Integer;
  FieldName: string;
  ArticleType: string;
  AuthorName: string;
  IssueNo: string;
  KeywordSelect: string;
  KeywordSearch: string;
  NoneFound: Boolean;
begin

  { Extract form variables and hand off to query builder }
  with CGI.FormItems do
  begin
    ArticleType := Values['ArticleType'];
    if ArticleType = '<Any>' then ArticleType := '';

    AuthorName := Values['Author'];
    if AuthorName = '<Any>' then AuthorName := '';

    IssueNo := Values['IssueNo'];

    KeywordSelect := Values['KeywordSelect'];
    if KeywordSelect = '<Any>' then KeywordSelect := '';

    KeywordSearch := Values['KeywordSearch'];
  end;

  { Construct the SQL query }
  BuildSQL(ArticleType, AuthorName, IssueNo, KeywordSelect, KeywordSearch,
    AIDDataModule.ArticleQuery.SQL);

  with CGI do
  begin
    WriteLn('<HTML><HEAD>');
    WriteLn('<TITLE>TDMAid - Query Results</TITLE>');
    WriteLn('</HEAD><BODY>');
    WriteLn('<BR><A HREF="' + DelphiMagURL + 'index.htm">Click to return to The Delphi Magazine main page...</A><BR>');
    WriteLn('<A HREF="' + PagesURL + 'tdmaid.htm">Return to Search Form...</A><BR>');
    try
      with AIDDataModule.ArticleQuery do
      begin
        NoneFound := True;
        Open;
        try
          NoneFound := Eof;

          WriteLn('<CENTER>');
          WriteLn('<H1>The Delphi Magazine - Article Index Database</H1>');
          if NoneFound then
            WriteLn('<H3>Query Results - No entries matched your selection</H3>')
          else
            WriteLn('<H3>Query Results - ' + IntToStr(RecordCount) + ' entries matched your selection</H3>');
          WriteLn('</CENTER>');
          if not NoneFound then WriteLn('<HR><BR>');

          while not Eof do
          begin
            { Show Article Information... }
            WriteLn('<H2>' + FieldByName('Title').AsString + '</H2><BR>');
            Write('<EM>Type:</EM> ' + FieldByName('Article Type').AsString + ', ');
            Write('<EM>Issue #</EM>' + IntToStr(FieldByName('Issue #').AsInteger));
            Write(' (' + FieldByName('MonthYear').AsString + '), ');
            Write('<EM>Page #</EM>' + IntToStr(FieldByName('Page Number').AsInteger) + '<BR>');
            WriteLn('<BR>');
            Write('<EM>Summary:</EM>  ');
            WriteBLOB(TBlobField(FieldByName('Summary')));
            WriteLn('<BR><BR>');

            { Show Keywords... }
            with AIDDataModule.KeywordQuery do
            begin
              ParamByName('ArticleID').Value := AIDDataModule.ArticleQuery.FieldByName('Article ID').Value;
              Open;
              try
                Write('<EM>Keywords:</EM>  ');
                Write(AIDDataModule.KeywordQueryKeyword.Value);
                Next;
                while not EOF do
                begin
                  Write(', ' + AIDDataModule.KeywordQueryKeyword.Value);
                  Next;
                end;
                WriteLn('<BR><BR>');
              finally
                Close;
              end;
            end;

            { Show Author Information... }
            for A := 1 to 3 do
            begin
              FieldName := 'Author ' + IntToStr(A);
              if not FieldByName(FieldName).IsNull then
              begin
                with AIDDataModule.AuthorQuery do
                begin
                  ParamByName('AuthorID').AsInteger := AIDDataModule.ArticleQuery.FieldByName(FieldName).AsInteger;
                  Open;
                  WriteLn('<EM>Author:</EM> <STRONG>' + FieldByName('Name').AsString + '</STRONG><BR>');
                  WriteBlob(TBlobField(FieldByName('Bio & E-mail address')));
                  WriteLn('<BR><BR>');
                  Close;
                end;
              end;
            end;

            WriteLn('<BR>');
            WriteLn('<A HREF="' + BackIssuesURL + '">Click here to order back issues...</A>');
            WriteLn('<HR><BR>');
            Next;
          end;
        finally
          Close;
        end;
      end;
    finally
      if not NoneFound then
      begin
        WriteLn('<H3>End of list.</H3><BR><BR>');
        WriteLn('<A HREF="' + PagesURL + 'tdmaid.htm">Return to Search Form...</A>');
      end;
      WriteLn('</BODY></HTML>');
    end;
  end;
end;

end.

