unit NQDEMU16;

interface

uses
  WinProcs, WinTypes, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls,
  Enque16, ExtCtrls;

type
  TFrmNQDemo = class(TForm)
    ListBox1: TListBox;
    BtnGetReadLock: TButton;
    BtnGetUpdateLock: TButton;
    BtnCleanClose: TButton;
    BtnDirtyClose: TButton;
    BtnReleaseReadLock: TButton;
    BtnReleaseUpdateLock: TButton;
    Panel1: TPanel;
    BtnGetNumReaders: TButton;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure BtnCleanCloseClick(Sender: TObject);
    procedure BtnGetReadLockClick(Sender: TObject);
    procedure BtnGetUpdateLockClick(Sender: TObject);
    procedure BtnReleaseReadLockClick(Sender: TObject);
    procedure BtnReleaseUpdateLockClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BtnGetNumReadersClick(Sender: TObject);
    procedure BtnDirtyCloseClick(Sender: TObject);
  private
    { Private declarations }
    DirtyClose: Boolean;
    procedure LogInfo(const Text: string);
    procedure LogLockCounts;
    procedure DisableButtons;
    procedure EnableButtons;
  public
    { Public declarations }
  end;

var
  FrmNQDemo: TFrmNQDemo;

implementation

var
  Enque: TEnque;

{$R *.DFM}

procedure TFrmNQDemo.LogInfo(const Text: string);
begin
  While ListBox1.Items.Count > 100 do
    ListBox1.Items.Delete(0);
  listBox1.ItemIndex := ListBox1.Items.Add(Text);
end;

procedure TFrmNQDemo.FormCreate(Sender: TObject);
begin
  Enque := TEnque.Create('TestSharedMemory');
end;

procedure TFrmNQDemo.BtnCleanCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TFrmNQDemo.BtnGetReadLockClick(Sender: TObject);
var                  
  StartTicks: LongInt;
  ReadAccessOK: Boolean;

begin
  Label1.Caption := '';
  DisableButtons;
  StartTicks := GetTickCount;
  ReadAccessOK := False;
  LogInfo('Waiting to get read lock...');
  try
    ReadAccessOK := Enque.GetReadAccess;
  finally
    EnableButtons;
    if ReadAccessOK then
      begin
        LogInfo(Format('Got read lock after %d millisecs.',[GetTickCount - StartTicks]));
        LogLockCounts;
      end
    else
      LogInfo('Failed to get read access.');
  end;
end;

procedure TFrmNQDemo.LogLockCounts;
begin
  LogInfo(Format('Have %d read, %d update locks.',[Enque.ReadLocks,Enque.UpdateLocks]));
end;

procedure TFrmNQDemo.BtnGetUpdateLockClick(Sender: TObject);
var
  StartTicks: LongInt;
  UpdateAccessOK: Boolean;

begin
  Label1.Caption := '';
  DisableButtons;
  StartTicks := GetTickCount;
  UpdateAccessOK := False;
  LogInfo('Waiting to get update lock...');
  try
    UpdateAccessOK := Enque.GetUpdateAccess;
  finally
    EnableButtons;
    if UpdateAccessOK then
      begin
        LogInfo(Format('Got update lock after %d millisecs.',[GetTickCount - StartTicks]));
      end
    else
      LogInfo('Failed to get update lock.');
    LogLockCounts;
  end;
end;

procedure TFrmNQDemo.DisableButtons;
begin
  BtnGetReadLock.Enabled := False;
  BtnGetUpdateLock.Enabled := False;
  BtnReleaseReadLock.Enabled := False;
  BtnReleaseUpdateLock.Enabled := False;
  BtnCleanClose.Enabled := False;
  BtnDirtyClose.Enabled := False;
  { Do this as the WaitFor..Object(s) can stop paint and button becomes invisible }
  BtnGetReadLock.Refresh;
  BtnGetUpdateLock.Refresh;
  BtnReleaseReadLock.Refresh;
  BtnReleaseUpdateLock.Refresh;
  BtnCleanClose.Refresh;
  BtnDirtyClose.Refresh;
end;

procedure TFrmNQDemo.EnableButtons;
begin
  BtnGetReadLock.Enabled := True;
  BtnGetUpdateLock.Enabled := True;
  BtnReleaseReadLock.Enabled := True;
  BtnreleaseUpdateLock.Enabled := True;
  BtnCleanClose.Enabled := True;
  BtnDirtyClose.Enabled := True;
  { Do this as the WaitFor..Object(s) can stop paint and button becomes invisible }
end;

procedure TFrmNQDemo.BtnReleaseReadLockClick(Sender: TObject);
var
  ReleaseReadAccessOK: Boolean;

begin
  Label1.Caption := '';
  DisableButtons;
  ReleaseReadAccessOK := False;
  try
    ReleaseReadAccessOK := Enque.ReleaseReadAccess;
  finally
    EnableButtons;
    if not ReleaseReadAccessOK then
      LogInfo('Failed to release read lock.');
    LogLockCounts;
  end;
end;

procedure TFrmNQDemo.BtnReleaseUpdateLockClick(Sender: TObject);
var
  ReleaseUpdateAccessOK: Boolean;

begin
  Label1.Caption := '';
  DisableButtons;
  ReleaseUpdateAccessOK := False;
  try
    ReleaseUpdateAccessOK := Enque.ReleaseUpdateAccess;
  finally
    EnableButtons;
    if not ReleaseUpdateAccessOK then
      LogInfo('Failed to release update lock.');
    LogLockCounts;
  end;
end;

procedure TFrmNQDemo.FormDestroy(Sender: TObject);
begin
  if not DirtyClose then
    Enque.Free;
end;

procedure TFrmNQDemo.BtnGetNumReadersClick(Sender: TObject);
begin
  Label1.Caption := Format('%d locks',[Enque.NumReaders]);
end;

procedure TFrmNQDemo.BtnDirtyCloseClick(Sender: TObject);
begin
  DirtyClose := True;
  Close;
end;

end.
