{$A+,B-,D+,F-,G+,I+,K+,L+,N+,P+,Q-,R-,S+,T-,V+,W-,X+,Y+}
{$M 16384,8192}
unit Sharem16;

interface

uses Wintypes, Classes, Messages, Forms,
     Enque16;

{$I SMCommon.INC}

type
  TSharedMemory = class
  private
    FCtrlFileMapHandle: LongInt;
    FCtrlAddress32: LongInt;
    FCtrlAddress: PSMCtrlDetails;
    FCtrlSelector: Word;
    FEnque: TEnque;
    FUserFileMapHandle: LongInt;
    FUserAddress: LongInt;
    FWindowHandle16: HWnd;
    FWindowHandle: LongInt;
    FOnChange: TSharedMemoryChangeEvent;
    { Internal procedures/functions }
    procedure AddHWnd;
    procedure AllocHwnd;
    function  FindWnd(Wnd: LongInt): Integer;
    procedure FreeHwnd;
    procedure RemoveHWnd;
    { Property access routines }
    function  GetCapacity: LongInt;
    function  GetCount: LongInt;
    function  GetElement_Size: LongInt;
    function  GetReadWithoutIntegrity: Boolean;
    procedure SetOnChange(Value: TSharedMemoryChangeEvent);
    procedure SetReadWithoutIntegrity(Value: Boolean);
  protected
    procedure NotifyDataChanged; virtual;
    procedure WndProc(var Msg: TMessage); virtual;
  public
    { Constructor/ Destructors }
    constructor Create(Name: string; Element_Size,Capacity: LongInt);
    destructor Destroy; override;
    { Methods }
    function  Add(PBuff: Pointer): LongInt;
    procedure Get(Idx: LongInt; Buffer: Pointer);
    function  GetReadAccess: Boolean;
    function  GetUpdateAccess: Boolean;
    procedure Put(Idx: LongInt; Buffer: Pointer);
    procedure ReleaseReadAccess;
    procedure ReleaseUpdateAccess;
    procedure Reset;

    { Properties }
    property Capacity: LongInt read GetCapacity;
    property Count: LongInt read GetCount;
    property Element_Size: LongInt read GetElement_Size;
    property OnChange: TSharedMemoryChangeEvent read FOnChange write SetOnChange;
    property ReadWithoutIntegrity: Boolean read GetReadWithoutIntegrity write SetReadWithoutIntegrity;
  end;

implementation

Uses SysUtils, WinProcs, Controls, Thunk_32, Win32Def, Call32NT;

function Convert32BitAddrTo16(var Selector: Word; StartAddr,Length: LongInt): Pointer;
begin
  Selector := AllocSelector(DSeg);
  if Selector <> 0 then
    begin
      SetSelectorBase(Selector,StartAddr);
      SetSelectorLimit(Selector,Length);
      Result := Ptr(Selector,0);
    end
  else
    Raise Exception.Create('Failed to convert 32bit address to 16 bit address');
end;

{===============================================================================}

constructor TSharedMemory.Create(Name: string; Element_Size,Capacity: LongInt);
var
  WrkName: array[0..255] of char;
  FirstInstance: Boolean;
  MName: string;

begin

  { Truncate name to allow for required extensions }

  MName := Copy(Name,1,249);

  { Create the TEnque object to control access to the user and control data }

  StrPCopy(WrkName,MName);

  FEnque := TEnque.Create(StrPas(WrkName));

  if not Assigned(FEnque) then
    Raise Exception.CreateFmt('Failed to create TEnque object for %s',[MName]);

  { Lock out all processes until we initialise }

  FEnque.GetUpdateAccess;

  try

    { Create the 'Ctrl' memory mapped area which is private to this class and     }
    { maintained exclusively by this class.                                       }

    StrPCopy(WrkName,MName + 'Ctrl');

    FCtrlFileMapHandle := CreateFileMapping(MEMORY_MAPPED_FILE,nil,PAGE_READWRITE,0,
                            SizeOf(TSMCtrlDetails),@WrkName,idCreateFileMapping);

    if FCtrlFileMapHandle <> 0 then
      begin
        FirstInstance := GetLastError(idGetLastError) <> ERROR_ALREADY_EXISTS;

        FCtrlAddress32 := MapViewOfFile(FCtrlFileMapHandle,FILE_MAP_WRITE,0,0,0,idMapViewOfFile);
        if FCtrlAddress32 = 0 then
          Raise Exception.CreateFmt('Failed to create view of the control file ' +
                                     'mapping object %s',[StrPas(WrkName)]);

        FCtrlAddress := Convert32BitAddrTo16(FCtrlSelector,FCtrlAddress32,sizeof(TSMCtrlDetails));

        if FirstInstance then
          begin
            if Capacity < 1 then
              Raise Exception.CreateFmt('Capacity invalid for new TSharedMemory ' +
                                        'object (%d)',[Capacity]);
            if Element_Size < 1 then
              Raise Exception.CreateFmt('Element_Size invalid for new TSharedMemory ' +
                                        'object (%d)',[Element_Size]);
            Fillchar(FCtrlAddress^,sizeof(TSMCtrlDetails),0);
            with FCtrlAddress^ do
              begin
                Ctrl_Element_Size := Element_Size;
                Ctrl_Capacity := Capacity;
              end;
          end
        else
          begin
            if FCtrlAddress^.Ctrl_Capacity <> Capacity then
              Raise Exception.CreateFmt('Capacity invalid for TSharedMemory object. ' +
                    'Was %d, should be %d',[Capacity,FCtrlAddress^.Ctrl_Capacity]);
            if FCtrlAddress^.Ctrl_Element_Size <> Element_Size then
              Raise Exception.CreateFmt('Element size invalid for TSharedMemory object. ' +
                    'Was %d, should be %d',[Element_Size,FCtrlAddress^.Ctrl_Element_Size]);
          end;
      end
    else
      Raise Exception.CreateFmt('Failed to create control file mapping object %s',
               [StrPas(WrkName)]);

    { Create the 'User' memory mapped area which can be accessed by any user      }
    { application.                                                                }

    StrPCopy(WrkName,MName + 'User');

    FUserFileMapHandle := CreateFileMapping(MEMORY_MAPPED_FILE,nil,PAGE_READWRITE,0,
                             Element_Size * Capacity,@WrkName,idCreateFileMapping);

    if FUserFileMapHandle <> 0 then
      begin
        FUserAddress := MapViewOfFile(FUserFileMapHandle,FILE_MAP_WRITE,0,0,0,idMapViewOfFile);
        if FUserAddress = 0 then
          Raise Exception.CreateFmt('Failed to create view of the file ' +
            'mapping object %s',[MName]);
      end
    else
      Raise Exception.CreateFmt('Failed to create file mapping object %s',[MName]);

  finally
    FEnque.ReleaseUpdateAccess;
  end;

end;

{===============================================================================}

destructor TSharedMemory.Destroy;
begin
  if FUserAddress = 0 then
    UnMapViewOfFile(FUserAddress,idUnMapViewOfFile);
  if FUserFileMapHandle <> 0 then
    CloseHandle(FUserFileMapHandle,idCloseHandle);
  if FCtrlAddress32 = 0 then
    UnMapViewOfFile(FCtrlAddress32,idUnMapViewOfFile);
  if FCtrlFileMapHandle <> 0 then
    CloseHandle(FCtrlFileMapHandle,idCloseHandle);
  if FCtrlSelector <> 0 then
    FreeSelector(FCtrlSelector);
  FEnque.Free;
  FreeHWnd;
end;

{===============================================================================}

function TSharedMemory.Add(PBuff: Pointer): LongInt;
var
  GotExplicitUpdateAccess: Boolean;

begin
  Result := -1;
  GotExplicitUpdateAccess := False;
  if not FEnque.HaveUpdateAccess then
    GotExplicitUpdateAccess := GetUpdateAccess;
  try
    if Count < Capacity then
      begin
        MoveMemoryIn(FUserAddress + (Count * Element_Size),PBuff,Element_Size,idMoveMemoryIn);
        Result := Count;
        Inc(FCtrlAddress^.Ctrl_Count);
        FCtrlAddress^.Ctrl_Data_Updated := True;
      end;
  finally
    if GotExplicitUpdateAccess then
      ReleaseUpdateAccess;
  end;
end;

{===============================================================================}

procedure TSharedMemory.AddHWnd;
var
  WndIndex: Integer;

begin
  AllocHWnd;
  GetUpdateAccess;
  try
    WndIndex := FindWnd(FWindowHandle);
    if WndIndex = -1 then
      With FCtrlAddress^ do
        begin
          if Ctrl_Num_Windows = Max_OnChange_Notifications then
            Raise Exception.Create('Max number of shared objects notifications reached');
          Inc(Ctrl_Num_Windows);
          Ctrl_WinHandles[Ctrl_Num_Windows] := FWindowHandle;
          Ctrl_NotificationBeenPosted[Ctrl_Num_Windows] := False;
        end;
  finally
    ReleaseUpdateAccess;
  end;
end;

{===============================================================================}
{ Allocate a window handle if we haven't done so already                        }

procedure TSharedMemory.AllocHwnd;
begin
  if FWindowHandle = 0 then
    begin
      FWindowHandle16 := AllocateHwnd(WndProc);
      FWindowHandle := FWindowHandle16;
      XLatHWnd(FWindowHandle);
      if FWindowHandle = 0 then
        Raise EOutOfResources.Create('Failed to create window handle for ' +
          'TSharedMemory object.');
    end;
end;

{===============================================================================}

function TSharedMemory.FindWnd(Wnd: LongInt): Integer;
var
  I: Integer;

begin
  Result := -1;
  With FCtrlAddress^ do
    for I := 1 to Ctrl_Num_Windows do
      if Ctrl_WinHandles[I] = Wnd then
        begin
          Result := I;
          Break;
        end;
end;

{===============================================================================}
{ If we have allocated a window, free it                                        }

procedure TSharedMemory.FreeHwnd;
begin
  if FWindowHandle <> 0 then
    begin
      DeallocateHwnd(FWindowHandle16);
      FWindowHandle16 := 0;
      FWindowHandle := 0;
    end;
end;

{===============================================================================}

procedure TSharedMemory.Get(Idx: LongInt; Buffer: Pointer);
var
  GotExplicitReadAccess: Boolean;

begin
  GotExplicitReadAccess := False;
  With FEnque do
    if not HaveUpdateAccess and not HaveReadAccess and not ReadWithoutIntegrity then
     GotExplicitReadAccess := GetReadAccess;
  try
    if (Idx >= 0) and (Idx < Count) then
      MoveMemoryOut(Buffer,FUserAddress + (Idx * Element_Size),Element_size,idMoveMemoryOut)
    else
      Raise Exception.CreateFmt('Index [%d] out of bounds. Current count is %d',[Idx,Count]);
  finally
    if GotExplicitReadAccess then
      ReleaseReadAccess;
  end;
end;

{===============================================================================}

function TSharedMemory.GetCapacity: LongInt;
begin
  Result := FCtrlAddress^.Ctrl_Capacity;
end;

{===============================================================================}

function TSharedMemory.GetCount: LongInt;
begin
  Result := FCtrlAddress^.Ctrl_Count;
end;

{===============================================================================}

function TSharedMemory.GetElement_Size: LongInt;
begin
  Result := FCtrlAddress^.Ctrl_Element_size;
end;

{===============================================================================}

function TSharedMemory.GetReadAccess: Boolean;
begin
  Result := FEnque.GetReadAccess;
end;

{===============================================================================}

function TSharedMemory.GetReadWithoutIntegrity: Boolean;
begin
  Result := FEnque.ReadWithoutIntegrity;
end;

{===============================================================================}

function TSharedMemory.GetUpdateAccess: Boolean;
begin
  Result := FEnque.GetUpdateAccess;
  if FEnque.UpdateLocks = 1 then
    FCtrlAddress^.Ctrl_Data_Updated := False;
end;

{===============================================================================}

procedure TSharedMemory.NotifyDataChanged;
var
  I: Integer;

begin
  With FCtrlAddress^ do
    begin
      Inc(Ctrl_Update_Count);
      for I := 1 to Ctrl_Num_Windows do
        if not Ctrl_NotificationBeenPosted[I] then
          begin
            PostMessage32(Ctrl_WinHandles[I],WM_USER_SHAREMEM_CHANGED,0,Ctrl_Update_Count,idPostMessage32);
            Ctrl_NotificationBeenPosted[I] := True;
          end;
    end;
end;

{===============================================================================}

procedure TSharedMemory.Put(Idx: LongInt; Buffer: Pointer);
var
  GotExplicitUpdateAccess: Boolean;

begin
  GotExplicitUpdateAccess := False;
  if not FEnque.HaveUpdateAccess then
    GotExplicitUpdateAccess := GetUpdateAccess;
  try
    if (Idx > 0) and (Idx < Count) then
      begin
        MoveMemoryIn(FUserAddress + (Idx * Element_Size),Buffer,Element_Size,idMoveMemoryIn);
        FCtrlAddress^.Ctrl_Data_Updated := True;
      end
    else
      Raise Exception.CreateFmt('Index [%d] out of bounds. Current count is %d',[Idx,Count]);
  finally
    if GotExplicitUpdateAccess then
      ReleaseUpdateAccess;
  end;
end;

{===============================================================================}

procedure TSharedMemory.ReleaseReadAccess;
begin
  FEnque.ReleaseReadAccess;
end;

{===============================================================================}

procedure TSharedMemory.ReleaseUpdateAccess;
begin
  try
    if (FEnque.UpdateLocks = 1) and (FCtrlAddress^.Ctrl_Data_Updated) then
      NotifyDataChanged;
  finally
    FEnque.ReleaseUpdateAccess;
  end;
end;

{===============================================================================}

procedure TSharedMemory.Reset;
var
  GotExplicitUpdateAccess: Boolean;

begin
  GotExplicitUpdateAccess := False;
  if not FEnque.HaveUpdateAccess then
    GotExplicitUpdateAccess := GetUpdateAccess;
  try
    FCtrlAddress^.Ctrl_Count := 0;
    FCtrlAddress^.Ctrl_Data_Updated := True;
  finally
    if GotExplicitUpdateAccess then
      ReleaseUpdateAccess;
  end;
end;

{===============================================================================}

procedure TSharedMemory.RemoveHWnd;
var
  WndIndex: Integer;
  I: Integer;

begin
  WndIndex := FindWnd(FWindowHandle);
  if WndIndex <> -1 then
    With FCtrlAddress^ do
      begin
        for I := WndIndex + 1 to Ctrl_Num_Windows - 1 do
          begin
            Ctrl_WinHandles[I-1] := Ctrl_WinHandles[I];
            Ctrl_NotificationBeenPosted[I-1] := Ctrl_NotificationBeenPosted[I];
          end;
        Dec(Ctrl_Num_Windows);
      end;
    FreeHWnd;
end;

{===============================================================================}

procedure TSharedMemory.SetOnChange(Value: TSharedMemoryChangeEvent);
begin
  if Assigned(Value) then
    begin
      if not Assigned(FOnChange) then
        AddHWnd;
      FOnChange := Value;
    end
  else
    begin
      if not Assigned(Value) then
        RemoveHWnd;
      FOnChange := Value;
    end;
end;

{===============================================================================}

procedure TSharedMemory.SetReadWithoutIntegrity(Value: Boolean);
begin
  FEnque.ReadWithoutIntegrity := Value;
end;

{===============================================================================}

procedure TSharedMemory.WndProc(var Msg: TMessage);

  procedure ResetPostNotifyFlag;
  var
    WIndex: Integer;

  begin
    WIndex := FindWnd(FWindowHandle);
    if WIndex <> -1 then
      FCtrlAddress^.Ctrl_NotificationBeenPosted[WIndex] := False;
  end;

begin
  With Msg do
    if Msg = WM_USER_SHAREMEM_CHANGED then
      { Trap any exception in the user written OnChange event procedure }
      try
        { May have been reset after the message was posted }
        if Assigned(FOnChange) then
          begin
            { If we are currently updating don't call the OnChange, just reset }
            { the flag so we are posted again.                                 }
            if FEnque.UpdateLocks = 0 then
              begin
                FOnChange(Self);
                { If a further update has been made since we were posted post  }
                { ourself another message with the current count - this update }
                { could have occurred as we left the FOnChanged event proc.    }
                if lParam = FCtrlAddress^.Ctrl_Update_Count then
                  ResetPostNotifyFlag
                else
                  PostMessage32(FWindowHandle,WM_USER_SHAREMEM_CHANGED,0,FCtrlAddress^.Ctrl_Update_Count,idPostMessage32);
              end
            else
              ResetPostNotifyFlag;
          end;
      except
        Application.HandleException(Self);
      end
    else
      Result := DefWindowProc(FWindowHandle,Msg,wParam,lParam);
end;

end.

