unit Thunk_32;

{ This unit declares all required variables for 32bit API's required  }

interface

var
  { Identifiers returned from Declare32 }

  idCloseHandle: LongInt;
  idCreateFileMapping: LongInt;
  idCreateEvent: LongInt;
  idCreateMutex: LongInt;
  idCreateSemaphore: LongInt;
  idGetCurrentProcessId: LongInt;
  idGetCurrentThreadId: LongInt;
  idGetLastError: LongInt;
  idGetSystemInfo: LongInt;
  idMapViewOfFile: LongInt;
  idMoveMemoryIn: LongInt;
  idMoveMemoryOut: LongInt;
  idPostMessage32: LongInt;
  idReleaseMutex: LongInt;
  idReleaseSemaphore: LongInt;
  idResetEvent: LongInt;
  idSetEvent: LongInt;
  idSleep: LongInt;
  idUnmapViewOfFile: LongInt;
  idVirtualAlloc: LongInt;
  idVirtualFree: LongInt;
  idWaitForMultipleObjects: LongInt;
  idWaitForSingleObject: LongInt;

  { Function prototypes for Win32 functions with required ID for Call32 }

  CloseHandle:function(hFile: LongInt; id:LongInt): LongInt;
  CreateFileMapping:function(hFile: LongInt; lpsa: Pointer;
               fdwProtect,dwMaxSizeHigh,dwMaxSizeLow: LongInt;
               lpszMapName: Pointer; id: LongInt): LongInt;
  CreateEvent:function(lpsa: Pointer; fManualreset,fInitialState: LongBool; lpszEventName: Pointer; id: LongInt): LongInt;
  CreateMutex:function(lpsa: Pointer; fInitialOwner: LongBool; lpszMutexName: Pointer; id: LongInt): LongInt;
  CreateSemaphore:function(lpsa: Pointer; cSemInitial,cSemMax: LongInt; SemName: Pointer;
               id:LongInt): LongInt;
  GetCurrentProcessId:function(id:LongInt): LongInt;
  GetCurrentThreadId:function(id:LongInt): LongInt;
  GetLastError:function(id:LongInt): LongInt;
  GetSystemInfo:procedure(SysInfo: Pointer; id:LongInt);
  MapViewOfFile:function(hMapObject,fdwAccess,dwOffSetHigh,dwOffSetLow,cbMap: LongInt;
               id: LongInt): LongInt;
  MoveMemoryIn:procedure(Destination: LongInt; Source: Pointer; Length: LongInt;
               id:LongInt);
  MoveMemoryOut:procedure(Destination: Pointer; Source,Length: LongInt; id:LongInt);
  PostMessage32:function(hWnd,Msg,wParam,lParam,id: LongInt): LongInt;
  ReleaseMutex:function(hMutex: LongInt; id:LongInt): LongInt;
  ReleaseSemaphore:function(hSem,cReleaseCount: LongInt; lpPrevCount: Pointer;
               id:LongInt): LongInt;
  ResetEvent:function(hEvent: LongInt; id:LongInt): LongBool;
  SetEvent:function(hEvent: LongInt; id:LongInt): LongInt;
  Sleep:procedure(Delay: LongInt; id:LongInt);
  UnmapViewOfFile:function(lpBaseAddress: LongInt; id:LongInt): LongInt;
  VirtualAlloc:function(lpvAddress,cbSize,fdwAllocationType,fdwProtect: LongInt;
               id: LongInt): LongInt;
  VirtualFree:function(lpvAddress,cbSize,fdwFreeType: LongInt; id: LongInt): longBool;
  WaitForMultipleObjects:function(cObjects: LongInt; lphObjects: Pointer; fWaitAll: LongBool;
               dwTimeOut: LongInt; id: LongInt): LongInt;
  WaitForSingleObject:function(hObject,dwTimeOut: LongInt; id: LongInt): LongInt;

implementation

uses Call32NT, WinProcs, WinTypes;

{ Unit initialisation routine }

{ Note the differences for MoveMemoryIn/MoveMemoryOut.                          }
{ The both refer to the Win32 API MoveMemory which accepts two 32bit flat       }
{ addresses and a length parameter. The difference between these declarations   }
{ is the way there are defined in Declare32. MoveMemoryIn requires Call32 to    }
{ convert the 2nd paramter only - this is a 16bit pointer to the users buffer.  }
{ the first paramter is the 32bit style address of the user memory mapped file  }
{ and requires no conversion (in fact we never convert that address). The       }
{ definition for MoveMemoryIn specifies that the first parameter be changed and }
{ leaves the second alone.                                                      }

begin
  @CloseHandle := @Call32;
  @CreateFileMapping := @Call32;
  @CreateEvent := @Call32;
  @CreateMutex := @Call32;
  @CreateSemaphore := @Call32;
  @GetCurrentProcessId := @Call32;
  @GetCurrentThreadId := @Call32;
  @GetLastError := @Call32;
  @GetSystemInfo := @Call32;
  @MapViewOfFile := @Call32;
  @MoveMemoryIn := @Call32;
  @MoveMemoryOut := @Call32;
  @PostMessage32 := @Call32;
  @ReleaseMutex := @Call32;
  @ReleaseSemaphore := @Call32;
  @ResetEvent := @Call32;
  @SetEvent := @Call32;
  @Sleep := @Call32;
  @UnmapViewOfFile := @Call32;
  @VirtualAlloc := @Call32;
  @VirtualFree := @Call32;
  @WaitForMultipleObjects := @Call32;
  @WaitForSingleObject := @Call32;

  idCloseHandle := Declare32('CloseHandle','Kernel32','i');
  idCreateFileMapping := Declare32('CreateFileMapping','Kernel32','iiiiip');
  idCreateEvent := Declare32('CreateEvent','Kernel32','piip');
  idCreateMutex := Declare32('CreateMutex','Kernel32','pip');
  idCreateSemaphore := Declare32('CreateSemaphore','Kernel32','iiip');
  idGetCurrentProcessId := Declare32('GetCurrentProcessId','Kernel32','');
  idGetCurrentThreadId := Declare32('GetCurrentThreadId','Kernel32','');
  idGetLastError := Declare32('GetLastError','Kernel32','');
  idGetSystemInfo := Declare32('GetSystemInfo','Kernel32','p');
  idMapViewOfFile := Declare32('MapViewOfFile','Kernel32','iiiii');
  idMoveMemoryIn := Declare32('RtlMoveMemory','Kernel32','ipi');
  idMoveMemoryOut := Declare32('RtlMoveMemory','Kernel32','pii');
  idPostMessage32 := Declare32('PostMessage','User32','iiii');
  idReleaseMutex := Declare32('ReleaseMutex','Kernel32','i');
  idReleaseSemaphore := Declare32('ReleaseSemaphore','Kernel32','iip');
  idResetEvent := Declare32('ResetEvent','Kernel32','i');
  idSetEvent := Declare32('SetEvent','Kernel32','i');
  idSleep := Declare32('Sleep','Kernel32','i');
  idUnmapViewOfFile := Declare32('UnmapViewOfFile','Kernel32','i');
  idVirtualAlloc := Declare32('VirtualAlloc','Kernel32','iiii');
  idVirtualFree :=  Declare32('VirtualFree','Kernel32','iii');
  idWaitForMultipleObjects := Declare32('WaitForMultipleObjects','Kernel32','ipii');
  idWaitForSingleObject := Declare32('WaitForSingleObject','Kernel32','ii');

  if Call32NTError then
    MessageBox(0,'Error initialising Win32 functions in unit Thunk_32','Error',mb_ok);

end.
