unit testthrd;
{
  Author : Neil McClements
  Date   : 2nd September 1996
  C/right: (c) 1996 N. McClements
  Purpose: Simple application to illustrate multi-threaded queries and callbacks
}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, dbthread, DB, Grids, DBGrids, DBTables;

type
  TTestThreads = class(TForm)
    grpfirstQuery: TGroupBox;
    FirstQueryGrid: TDBGrid;
    grpSecondQuery: TGroupBox;
    SecondQueryGrid: TDBGrid;
    GroupBox3: TGroupBox;
    btnConnect: TBitBtn;
    btnRunQuery: TBitBtn;
    lbConnected: TLabel;
    lbFirstQuery: TLabel;
    lbSecondQuery: TLabel;
    lbFirstStarted: TLabel;
    lbFirstFinished: TLabel;
    lbSecondStarted: TLabel;
    lbSecondFinished: TLabel;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    procedure btnConnectClick(Sender: TObject);
    procedure btnRunQueryClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    connected:boolean;
  public
    procedure ShowFirstQueryResults(qry:TQuery);
    procedure ShowSecondQueryResults(qry:TQuery);
  end;

var
  TestThreads: TTestThreads;
  MyDB:TThreadDatabase;

implementation

{$R *.DFM}

procedure TTestThreads.btnConnectClick(Sender: TObject);
var
   disconnected:boolean;
begin
// Attempt to connect to the database...
     if not(connected) then
        begin
           btnConnect.caption:='Connecting...';
           connected:=MyDB.Connect;
           if not(connected) then
             lbConnected.caption:='Unable to open the database!'
           else
              begin
                lbConnected.caption:='Connected successfully';
                btnConnect.caption:='Disconnect';
              end;
        end
     else
        begin
           disconnected:=MyDB.Disconnect;
           if disconnected then
               lbConnected.caption:='Disconnected successfully'
           else
             lbConnected.caption:='Unable to disconnect';
        end;

end;

procedure TTestThreads.btnRunQueryClick(Sender: TObject);
var
   OnQuery1Return, OnQuery2Return:TCallbackQuery;
   handle1, handle2:THandle;
   sql1, sql2:string;
begin
// Nominate a procedure to run when the first query returns its resultset
     OnQuery1Return:=ShowFirstQueryResults;

// Nominate another procedure to handle the second query resultset
     OnQuery2Return:=ShowSecondQueryResults;

// Set a few buttons on front-end...
   btnConnect.enabled:=false;
   btnRunQuery.enabled:=false;

// Define the SQL string for query 1...
     sql1:='select * from employee';
     lbFirstQuery.caption:='SQL : ' + sql1;
     lbFirstStarted.caption:='Started : ' + DateTimeToStr(now);
// Start the first query running...
     handle1:=MyDB.RunQuery(sql1, OnQuery1Return);
     grpFirstQuery.caption:='First query - thread id : ' + IntToStr(handle1);

// Define the second SQL string for query 2...
     sql2:='select * from employee';
     lbSecondQuery.caption:='SQL : ' + sql2;
     lbSecondStarted.caption:='Started : ' + DateTimeToStr(now);
// Start the second query running...
     handle2:=MyDB.RunQuery(sql2, OnQuery2Return);
     grpSecondQuery.caption:='Second query  - thread id : ' + IntToStr(handle2);
end;

procedure TTestThreads.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
     Action:=caFree;
end;

procedure TTestThreads.ShowFirstQueryResults(qry:TQuery);
var
   dSource:TDatasource;
begin
// Log the return
   lbFirstFinished.caption:='Finished : ' + DateTimeToStr(now);

// Create a datasource programmatically to connectthe TQuery to the TDBGrid...
   dSource:=TDatasource.Create(Self);
   dSource.dataset:=qry;

// Connect ...
   FirstQueryGrid.datasource:=dSource;

// Reset the interface
   lbFirstQuery.caption:='SQL : ' + qry.sql.strings[0] + ' returning ' + IntToStr(qry.recordcount) + ' rows.';
   btnConnect.enabled:=true;
   btnRunQuery.enabled:=true;
end;

procedure TTestThreads.ShowSecondQueryResults(qry:TQuery);
var
   dSource:TDatasource;
begin
// Log the return
   lbSecondFinished.caption:='Finished : ' + DateTimeToStr(now);

// Create a datasource programmatically to connectthe TQuery to the TDBGrid...
   dSource:=TDatasource.Create(Self);
   dSource.dataset:=qry;

// Connect ...
   SecondQueryGrid.datasource:=dSource;

// Reset the interface
   lbSecondQuery.caption:='SQL : ' + qry.sql.strings[0] + ' returning ' + IntToStr(qry.recordcount) + ' rows.';
   btnConnect.enabled:=true;
   btnRunQuery.enabled:=true;
end;


end.


