unit WatchEx;

interface

uses
   ExptIntf, WinProcs, WinTypes, ExtCtrls;

type
  TWatchWindowExpert = class(TIExpert)
  private
    FOnTop: Boolean;
    FTimer: TTimer;
    procedure DoTimer(Sender: TObject);
  public
    constructor Create;
    destructor Destroy; override;
    function GetStyle: TExpertStyle; override;
    function GetIDString:    string; override;
    function GetName:        string; override;
{$ifdef Win32}
    function GetAuthor:      string; override;
    function GetPage:        string; override;
{$endif}
    function GetGlyph:       HICON;  override;
    function GetComment:     string; override;
    function GetMenuText:    string; override;
    function GetState: TExpertState; override;
    procedure Execute;               override;
  end;

procedure Register;

implementation

uses
  SysUtils;

constructor TWatchWindowExpert.Create;
begin
  inherited Create;
  { Using a timer to ensure the watch stays as we want it }
  { and Delphi doesn't put things back how they were intended }
  FTimer := TTimer.Create(nil);
  FTimer.Interval := 100;
  FTimer.OnTimer := DoTimer;
end;

destructor TWatchWindowExpert.Destroy;
begin
  FTimer.Free;
  inherited Destroy
end;

function TWatchWindowExpert.GetStyle: TExpertStyle;
begin
  Result := esStandard;
end;

function TWatchWindowExpert.GetIDString: string;
begin
  Result := 'Watch Window Enforcer';
end;

function TWatchWindowExpert.GetName: string;
begin
  Result := 'Watch Window Expert';
end;

{$ifdef Win32}
function TWatchWindowExpert.GetAuthor: string;
begin
  Result := 'Roy Nelson & Brian Long';
end ;

function TWatchWindowExpert.GetPage: string;
begin
  Result := '';
end;
{$endif}

function TWatchWindowExpert.GetGlyph: HICON;
begin
  Result := 0;
end;

function TWatchWindowExpert.GetComment: string;
begin
  Result := '';
end ;

function TWatchWindowExpert.GetMenuText: string;
begin
  Result := '&Put Watches On Top'
end ;

function TWatchWindowExpert.GetState: TExpertState;
begin
  Result := [esEnabled];
  if FOnTop then
    Include(Result, esChecked);
end;

procedure WatchesOnTop(DoIt: Boolean);
var
  Wnd: HWnd;
  PID: Integer;
const
  Flags: array[Boolean] of Longint = (HWnd_NoTopMost, HWnd_TopMost);
begin
  Wnd := FindWindow('TWatchWindow', 'Watch List');
  { Bearing in mind some will run Delphi 1 and 2 }
  { simultaneously, or multiple copies of Delphi 2 }
  { we should really check the watch window comes }
  { from this copy of Delphi, but hey... }

  { If another task is active, then don't do stay on top }
{$ifdef Win32}
  GetWindowThreadProcessId(GetForegroundWindow, @PID);
  if DoIt and (PID <> GetCurrentProcessID) then
{$else}
  if DoIt and (GetWindowTask(GetActiveWindow) <> GetCurrentTask) then
{$endif}
    DoIt := False;
  { Check whether watch window is currently }
  { in appropriate state. If not, then change it }
  if (Wnd <> 0) and IsWindow(Wnd) and
     ((GetWindowLong(Wnd, gwl_ExStyle) and
      ws_Ex_TopMost <> 0) <> DoIt) then
    SetWindowPos(Wnd, Flags[DoIt], 0, 0, 0, 0,
      swp_NoMove or swp_NoSize or swp_NoActivate);
end;

procedure TWatchWindowExpert.Execute;
begin
  FOnTop := not FOnTop
end;

procedure TWatchWindowExpert.DoTimer(Sender: TObject);
begin
  WatchesOnTop(FOnTop);
end;

procedure Register;
begin
  RegisterLibraryExpert(TWatchWindowExpert.Create)
end;

end.
