unit win32box;

interface

Uses
	SysUtils, Windows, Graphics;

type
	 TSysFont = (sfCaptionFont, sfSmCaptionFont, sfMenuFont,
			sfStatusBarFont, sfMessageFont, sfIconFont);


Function GetSysFont(FontParam: TSysFont; var ResFont: TFont): Boolean;
procedure TFontFromLogFont(const LogFont: TLogFont; var NewFont: TFont);
function GetWinVersion: string;

implementation

{ Gets the System font setting of the passed TSysFont parameter }
Function GetSysFont(FontParam: TSysFont; var ResFont: TFont): Boolean;
var
	TempNonClientMetrics: TNonClientMetrics;
	TempLogFont: TLogFont;
	ret: boolean;
begin
	Result := false;
	If FontParam = sfIconFont then
		{ The IconFont is retreived with a different method
			than all the other system fonts }
		ret := SystemParametersInfo(SPI_GETICONTITLELOGFONT,
				SizeOf(TempLogFont), @TempLogFont, 0)
	else
		begin
			TempNonClientMetrics.cbSize := SizeOf(TempNonClientMetrics);
			ret := SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0,
					@TempNonClientMetrics, 0);
		end;

	{ If call to SystemParametersInfo() not successful then exit}
	If not ret then exit;

	{ Load appropriate TempNonClientMetrics field into TempLogFont }
	case FontParam of
		sfCaptionFont:
			TempLogFont := TempNonClientMetrics.lfCaptionFont;
		sfSmCaptionFont:
			TempLogFont := TempNonClientMetrics.lfSmCaptionFont;
		sfMenuFont:
			TempLogFont := TempNonClientMetrics.lfMenuFont;
		sfStatusBarFont:
			TempLogFont := TempNonClientMetrics.lfStatusFont;
		sfMessageFont:
			TempLogFont := TempNonClientMetrics.lfMessageFont;
	end;

	{ Convert TempLogFont into a TFont }
	TFontFromLogFont(TempLogFont, ResFont);
	Result := true;
end;

{ Converts a LogFont structure to a Delphi TFont }
procedure TFontFromLogFont(const LogFont: TLogFont; var NewFont: TFont);
var
	Style: TFontStyles;
begin
	with LogFont do
	begin
		NewFont.Name := lfFaceName;
		NewFont.Height := lfHeight;
		Style := [];

		if lfWeight > FW_REGULAR then Include(Style, fsBold);
		if lfItalic <> 0 then Include(Style, fsItalic);
		if lfUnderline <> 0 then Include(Style, fsUnderline);
		if lfStrikeOut <> 0 then Include(Style, fsStrikeOut);

		NewFont.Style := Style;
	end;
end;

{ Returns the Windows version as a string }
function GetWinVersion: string;
var
	TempOSVersionInfo: TOSVersionInfo;
	strVersion: String;
	ret: boolean;
begin
	strVersion := '';
	TempOSVersionInfo.dwOSVersionInfoSize := sizeof(TempOSVersionInfo);
	ret := GetVersionEx(TempOSVersionInfo);
	If ret then
		strVersion := IntToStr(TempOSVersionInfo.dwMajorVersion) + '.' +
			IntToStr(TempOSVersionInfo.dwMinorVersion) + '.' +
			IntToStr(loWord(TempOSVersionInfo.dwBuildNumber));
	Result := strVersion;
end;

end.
