{*********************************************************}
{                                                         }
{       Improved Delphi MessageDlg Functions              }
{                                                         }
{       Copyright (c) 1996 Steven J. Colagiovanni         }
{                                                         }
{*********************************************************}

unit Msgdlgs;

{$S-,W-,R-}
{$C PRELOAD}

interface

uses WinTypes, WinProcs, SysUtils, Classes, Graphics, Consts, Controls,
   StdCtrls, ExtCtrls, Forms;

type
   TMsgBoxType = (mtWarning, mtError, mtInformation, mtConfirmation,
      mtCustom);
   TMsgBoxBtn = (mbYes, mbNo, mbOK, mbAbort, mbRetry, mbIgnore,
      mbAll, mbCancel, mbHelp);
   TMsgBoxButtons = set of TMsgBoxBtn;
   TDefaultBtn = (dfFirst, dfSecond, dfThird);      { Default Button }

const
   mbYesNo = [mbYes, mbNo];
   mbYesNoCancel = [mbYes, mbNo, mbCancel];
   mbOKCancel = [mbOK, mbCancel];
   mbRetryCancel = [mbRetry, mbCancel];
   mbAbortRetryIgnore = [mbAbort, mbRetry, mbIgnore];

function MsgDlg(const AMsg: string; const ACaption: string;
   AType: TMsgBoxType; AButtons: TMsgBoxButtons;
   ADefaultButton: TDefaultBtn): Word;
function MsgDlgPos(const AMsg: string; const ACaption: string;
   AType: TMsgBoxType; AButtons: TMsgBoxButtons;
   ADefaultButton: TDefaultBtn; HelpCtx: Longint; X, Y: Integer): Word;

procedure ShowMsg(const AMsg: string);
procedure ShowMsgPos(const AMsg: string; X, Y: Integer);

function FormCenteredMsgDlg(const AMsg: string;
   const ACaption: string; AType: TMsgBoxType;
   AButtons: TMsgBoxButtons; ADefaultButton: TDefaultBtn;
   HelpCtx: Longint; AForm: TForm): Word;

function CreateMsgBox(const AMsg: string; const ACaption: string;
   AType: TMsgBoxType; AButtons: TMsgBoxButtons;
   ADefaultButton: TDefaultBtn): TForm;


implementation

const
   { MessageBox Icons stored in video driver file ie: vga.drv }
   ResIDs: array[TMsgBoxType] of PChar = (IDI_EXCLAMATION, IDI_HAND,
      IDI_ASTERISK,    IDI_QUESTION, nil);

   { Button.Caption - strings stored in Windows, load on call }
   BtnCaptions: array[TMsgBoxBtn] of Word = (SMsgdlgYes, SMsgdlgNo,
      SMsgdlgOK, SMsgDlgAbort, SMsgDlgRetry, SMsgDlgIgnore,
      SMsgDlgAll, SMsgdlgCancel, SMsgdlgHelp);

   { Button.Name }
   BtnNames: array[TMsgBoxBtn] of PChar = ('btnYes', 'btnNo', 'btnOK',
      'btnAbort', 'btnRetry', 'btnIgnore', 'btnAll', 'btnCancel',
      'btnHelp');

   {Button.Result}
   BtnResult: array[TMsgBoxBtn] of TModalResult = (mrYes, mrNo, mrOK,
      mrAbort, mrRetry, mrIgnore, mrAll, mrCancel, mrNone);

   MsgBeep: array[TMsgBoxType] of word = (MB_ICONEXCLAMATION,
      MB_ICONHAND, MB_ICONASTERISK, MB_ICONQUESTION, MB_OK);


function CreateMsgBox(const AMsg: string; const ACaption: string;
   AType: TMsgBoxType; AButtons: TMsgBoxButtons;
   ADefaultButton: TDefaultBtn): TForm;

const
   MsgDlgMinWidth = 150;
   MsgDlgMinHeight = 55;
   MsgDlgBtnSize: TPoint = (X: 77; Y: 28);

   { mgTextMargin = Top, left and right margins around text and glyph }
   mgTextMargin = 10;
   mgGlyphSpacing = 15;      { Spacing between glyph and text }
   mgBtnLRMargin = 15;
   mgBtnTopMargin = 20;
   mgBtnBottomMargin = 8;
   mgButtonSpacing = 8;

var
   MsgLabel: TLabel;
   Glyph: TImage;
   FIcon: TIcon;
   Buttons: array[TMsgBoxBtn] of TButton;
   Btn: TMsgBoxBtn;
   ButtonCount: Integer;
   ButtonSize: TPoint;
   InfoSize: TPoint;
   TextRect: TRect;
   C: array[0..255] of Char;
   ButtonX: Integer;
   ButtonTop: Integer;


   function Max(v1, v2: Integer): Integer;
   begin
      if v2 > v1 then Result := v2
      else Result := v1;
   end;


begin
   Result := TForm.CreateNew(Application);
   with Result do
   begin
      PixelsPerInch := 96;
      BorderStyle := bsDialog;
      BorderIcons := [biSystemMenu];
      Ctl3D := True;
      Font.Name := 'MS Sans Serif';
      Font.Height := -13;
      Font.Style := [fsBold];

      TextRect := Rect(0, 0, Screen.Width div 2, 0);
      DrawText(Canvas.Handle, StrPCopy(C, AMsg), -1, TextRect, DT_CALCRECT
         or DT_WORDBREAK or DT_NOPREFIX);

      { create the text }
      MsgLabel := TLabel.Create(Result);
      MsgLabel.Name := 'Message';
      MsgLabel.Parent := Result;
      MsgLabel.WordWrap := True;
      MsgLabel.ShowAccelChar := False;
      MsgLabel.Caption := AMsg;
      MsgLabel.BoundsRect := TextRect;

      if ResIDs[AType] <> nil then
      begin
         Glyph := TImage.Create(Result);
         Glyph.Name := 'Image';
         Glyph.Parent := Result;

         FIcon := TIcon.Create;
         try
            FIcon.Handle := LoadIcon(0, ResIDs[AType]);
            Glyph.Picture.Graphic := FIcon;
            Glyph.BoundsRect := Bounds(mgTextMargin, mgTextMargin,
               FIcon.Width, FIcon.Height);
         finally
            FIcon.Free;
         end;
      end
      else Glyph := nil;

      { sum up the size of the informational items }
      InfoSize.X := (TextRect.Right - TextRect.Left) + (mgTextMargin * 2);
      if Glyph <> nil then
         Inc(InfoSize.X, Glyph.Picture.Graphic.Width + mgGlyphSpacing);

      if Glyph <> nil then
         InfoSize.Y := Max(Glyph.Picture.Graphic.Height, TextRect.Bottom -
            TextRect.Top) + mgTextMargin
      else
         InfoSize.Y := (TextRect.Bottom - TextRect.Top) + mgTextMargin;

      { create the buttons }
      ButtonCount := 0;
      for Btn := Low(TMsgBoxBtn) to High(TMsgBoxBtn) do
      begin
         if Btn in AButtons then
         begin
            Inc(ButtonCount);
            Buttons[Btn] := TButton.Create(Result);
            with Buttons[Btn] do
            begin
               Parent := Result;
               SetBounds(0, 0, MsgDlgBtnSize.X, MsgDlgBtnSize.Y);
               Caption := LoadStr(BtnCaptions[Btn]);
               Name := StrPas(BtnNames[Btn]);
               ModalResult := Btnresult[Btn];

               { Need to set Default and Cancel properties for button }
               If (Btn = mbNo) or (Btn = mbCancel) then Cancel := True;
               If (Btn = mbYes) or (Btn = mbOK) then Default := True;

               { Determine if button is to be the default }
               If ButtonCount = Ord(ADefaultButton) + 1 Then
                  ActiveControl := TButton(Buttons[Btn]);
            end;
         end
         else Buttons[Btn] := nil;

      end;

      { If both a No and a Cancel button exist, then turn off the Cancel
         style of the NO button }
      if (mbNo in AButtons) and (mbCancel in AButtons) then
         Buttons[mbNo].Cancel := False;

      { If both a Yes and an OK button exist, then turn off the Default
         style of the Yes button }
      if (mbYes in AButtons) and (mbOK in AButtons) then
         Buttons[mbYes].Default := False;

      { if only an OK button exists, mark it as Cancel also }
      if (mbOK in AButtons) and (ButtonCount = 1) then
         Buttons[mbOK].Cancel := True;

      ButtonSize.X := (ButtonCount * MsgDlgBtnSize.X) +
         (mgButtonSpacing * (ButtonCount - 1)) + (mgBtnLRMargin * 2);
      ButtonSize.Y := MsgDlgBtnSize.Y + mgBtnTopMargin + mgBtnBottomMargin;

      { set the caption }
      if ACaption <> EmptyStr then Caption := ACaption
         else Caption := Application.Title;

      { Set minimum width and height for TForm necessary to display
         complete caption, icon, message and buttons. }
      ClientWidth := Max(Max(Canvas.TextWidth(Caption) + 50,
         MsgDlgMinWidth), Max(InfoSize.X, ButtonSize.X));
      ClientHeight := Max(MsgDlgMinHeight, InfoSize.Y + ButtonSize.Y);

      { layout the text and glyph }
      if (Glyph <> nil) and (Glyph.Height >
         (TextRect.Bottom - TextRect.Top)) then
         begin
            Glyph.Top := mgTextMargin;
            MsgLabel.Top := Glyph.Top + (Glyph.Picture.Graphic.Height div
               2) - ((TextRect.Bottom - TextRect.Top) div 2);
            ButtonTop := Glyph.Top + Glyph.Height;
         end
      else
         begin
            MsgLabel.Top := mgTextMargin;
            if Glyph <> nil then Glyph.Top := MsgLabel.Top +
               (((TextRect.Bottom - TextRect.Top) div 2) -
               (Glyph.Height div 2));
            ButtonTop := MsgLabel.Top + MsgLabel.Height;
         end;

      if Glyph <> nil then
         MsgLabel.Left := Glyph.Left + Glyph.Width + mgGlyphSpacing
      else
         MsgLabel.Left := mgTextMargin;

      { layout the buttons }
      ButtonX := (Result.ClientWidth div 2) - (ButtonSize.X div 2) +
         mgBtnLRMargin;
      for Btn := Low(TMsgBoxBtn) to High(TMsgBoxBtn) do
         if Buttons[Btn] <> nil then
         begin
            Buttons[Btn].Left := ButtonX;
            Buttons[Btn].Top := mgBtnTopMargin + ButtonTop;
            Inc(ButtonX, Buttons[Btn].Width + mgButtonSpacing);
         end;

      { Removes CLOSE option from system menu. }
      { GetSystemMenu(Result.Handle, False) Obtains Handle to System Menu }
      RemoveMenu(GetSystemMenu(Result.Handle, False), 1, MF_BYPOSITION);
   end;

   { Center form on the screen }
   Result.Left := (Screen.Width div 2) - (Result.Width div 2);
   Result.Top := (Screen.Height div 2) - (Result.Height div 2);
end;


function MsgDlgPos(const AMsg: string; const ACaption: string;
   AType: TMsgBoxType; AButtons: TMsgBoxButtons;
   ADefaultButton: TDefaultBtn; HelpCtx: Longint; X, Y: Integer): Word;

var
   W: TForm;
   TempCursor: TCursor ;

begin
   Result := 0;

   { Store the current Screen.Cursor }
   TempCursor := Screen.Cursor;

   { Use the following line if the message box is to be System Modal }
   { If SysModal Then AType := mtError; }

   W := CreateMsgBox(AMsg, ACaption, AType, AButtons, ADefaultButton);
   try
      W.HelpContext := HelpCtx;
      if X > -1 then W.Left := X;
      if Y > -1 then W.Top := Y;
      W.ScaleBy(Screen.PixelsPerInch, 96);

      { Set the Screen.Cursor to Default }
      Screen.Cursor := crDefault;

      { Make some noise }
      MessageBeep(MsgBeep[AType]);

      { Use the following line if the message box is to be System Modal }
      { If SysModal Then SetSysModalWindow(W.Handle); }

      Result := W.ShowModal;
   finally
      { Restore the original Screen.Cursor }
      Screen.Cursor := TempCursor;
      W.Free;
   end;
end;


function MsgDlg(const AMsg: string; const ACaption: string;
   AType: TMsgBoxType; AButtons: TMsgBoxButtons;
   ADefaultButton: TDefaultBtn): Word;

begin
   Result := MsgDlgPos(AMsg, ACaption, AType, AButtons,
      ADefaultButton, 0, -1, -1);
end;


procedure ShowMsgPos(const AMsg: string; X, Y: Integer);
begin
   MsgDlgPos(AMsg, '', mtCustom, [mbOK], dfFirst, 0, X, Y);
end;


procedure ShowMsg(const AMsg: string);
begin
   MsgDlgPos(AMsg, '', mtCustom, [mbOK], dfFirst, 0, -1, -1);
end;


function FormCenteredMsgDlg(const AMsg: string;
   const ACaption: string; AType: TMsgBoxType;
   AButtons: TMsgBoxButtons; ADefaultButton: TDefaultBtn;
   HelpCtx: Longint; AForm: TForm): Word;

var
   W: TForm;
   TempCursor: TCursor ;

begin
   Result := 0;

   { Store the current Screen.Cursor }
   TempCursor := Screen.Cursor;

   { Use the following line if the message box is to be System Modal }
   { If SysModal Then AType := mtError; }

   W := CreateMsgBox(AMsg, ACaption, AType, AButtons, ADefaultButton);
   try
      W.HelpContext := HelpCtx;

      W.Left := (AForm.Width div 2) - (W.Width div 2) + AForm.Left;
      W.Top := (AForm.Height div 2) - (W.Height div 2) + AForm.Top;

      W.ScaleBy(Screen.PixelsPerInch, 96);

      { Set the Screen.Cursor to Default }
      Screen.Cursor := crDefault;

      { Make some noise }
      MessageBeep(MsgBeep[AType]);

      { Use the following line if the message box is to be System Modal }
      { If SysModal Then SetSysModalWindow(W.Handle); }

      Result := W.ShowModal;
   finally
      { Restore the original Screen.Cursor }
      Screen.Cursor := TempCursor;
      W.Free;
   end;
end;

end.

