unit CoolBar;

interface

{$R *.RES }

uses Messages, Windows, Classes, Controls, Forms, SysUtils, CommCtrl;

function AddCoolBar (Wnd: hWnd): hWnd;
procedure AlignCoolBar (CoolBarWnd: hWnd);
procedure AddBand (CoolBarWnd: hWnd; Comp: TWinControl);

implementation

const
    { Flags for dwICC bitmask in TICCEx record }

    ICC_ListView_Classes	=     $00000001;
    ICC_TreeView_Classes        =     $00000002;
    ICC_Bar_Classes             =     $00000004;
    ICC_TAB_Classes		=     $00000008;
    ICC_UpDown_Class		=     $00000010;
    ICC_Progress_Class		=     $00000020;
    ICC_HotKey_CLASS		=     $00000040;
    ICC_Animate_CLASS		=     $00000080;
    ICC_Win95_Classes		=     $000000FF;
    ICC_Date_Classes		=     $00000100;
    ICC_UserEx_Classes		=     $00000200;
    ICC_Cool_Classes		=     $00000400;

    // Common Control Styles

    CCS_Vert                	=     $00000080;
    CCS_Left                	=     (CCS_VERT or CCS_TOP);
    CCS_Right               	=     (CCS_VERT or CCS_BOTTOM);
    CCS_NoMoveEx             	=     (CCS_VERT or CCS_NOMOVEY);

    RBIM_Style			=     $00000001;
    RBIM_ImageList		=     $00000002;
    RBIM_Background		=     $00000004;
    RBS_ToolTips		=     $00000100;
    RBS_VarHeight		=     $00000200;
    RBS_BandBorder		=     $00000400;
    RBS_FixedOrder		=     $00000800;

    RBBS_Break	    		=     $00000001;
    RBBS_FixedSize		=     $00000002;
    RBBS_KeepHeight		=     $00000004;
    RBBS_ChildEdge		=     $00000004;
    RBBS_Hidden	    		=     $00000008;
    RBBS_NoVert	    		=     $00000010;
    RBBS_FixedBmp		=     $00000020;

    RBBIM_Style			=     $00000001;
    RBBIM_Colors		=     $00000002;
    RBBIM_Text			=     $00000004;
    RBBIM_Image			=     $00000008;
    RBBIM_Child			=     $00000010;
    RBBIM_ChildSize		=     $00000020;
    RBBIM_Size			=     $00000040;
    RBBIM_Background		=     $00000080;
    RBBIM_Id			=     $00000100;

    RB_InsertBandA		=     wm_User +1;
    RB_DeleteBand		=     wm_User +2;
    RB_GetBarInfo		=     wm_User +3;
    RB_SetBarInfo		=     wm_User +4;
    RB_GetBandInfo		=     wm_User +5;
    RB_SetBandInfoA		=     wm_User +6;
    RB_SetParent		=     wm_User +7;
    RB_EraseDark		=     wm_User +8;
    RB_Animate			=     wm_User +9;
    RB_InsertBandW		=     wm_User +10;
    RB_SetBandInfoW		=     wm_User +11;
    RB_GetBandCount		=     wm_User +12;
    RB_GetRowCount		=     wm_User +13;
    RB_GetRowHeight		=     wm_User +14;
    RB_InsertBand		=     RB_InsertBandA;
    RB_SetBandInfo		=     RB_SetBandInfoA;
    RBN_HeightChange		=     -831;

    ReBarClassName              =     'ReBarWindow32';

type
    TICCEx = record
                 dwSize: DWord;
                 dwFlags: DWord;
             end;

    TRebarInfo = record
                     cbSize: UInt;
                     fMask: UInt;
                     fStyle: UInt;
                     himl: HImageList;
                     hbmBack: HBitmap;
                 end;

    TRebarBandInfo = record
                         cbSize: UInt;
                         fMask: UInt;
                         fStyle: UInt;
                         clrFore: TColorRef;
                         clrBack: TColorRef;
                         lpText: PChar;
                         cch: UInt;
                         iImage: Integer;
                         hWndChild: hWnd;
                         cxMinChild: UInt;
                         cyMinChild: UInt;
                         cx: UInt;
                         hbmBack: hBitmap;
                         wID: UInt;
                     end;

function InitCommonControlsEx (var ICCRec: TICCEx): Boolean; stdcall; external 'COMCTL32.DLL';

procedure CoolBarInit;
var
    ICCRec: TICCEx;
begin
    ICCRec.dwSize := sizeof (ICCRec);
    ICCRec.dwFlags := ICC_Cool_Classes;
    InitCommonControlsEx (ICCRec);
end;

procedure AlignCoolBar (CoolBarWnd: hWnd);
var
    rcForm: TRect;
begin
    GetClientRect (GetParent (CoolBarWnd), rcForm);
    MoveWindow (CoolBarWnd, 0, 0, rcForm.right, rcForm.Bottom, True);
end;

procedure AddBand (CoolBarWnd: hWnd; Comp: TWinControl);
var
    rc: TRect;
    rbbi: TRebarBandInfo;
    szBuff: array [0..100] of Char;
begin
    FillChar (rbbi, sizeof (rbbi), 0);
    GetWindowRect (Comp.Handle, rc);

    with rbbi do
    begin
        cbSize := sizeof (rbbi);
        fMask := RBBIM_Child or RBBIM_ChildSize or RBBIM_Style or RBBIM_Colors;
        cxMinChild := 50; { Maybe pass this as a parameter ? }
        cyMinChild := rc.bottom - rc.top;
        clrFore := GetSysColor (Color_BtnText);
        clrBack := GetSysColor (Color_BtnFace);
        fStyle := RBBS_ChildEdge or RBBS_FixedBmp;
        hwndChild := Comp.Handle;
        iImage := 0;
    end;

    SendMessage (CoolBarWnd, RB_InsertBand, -1, LongInt (@rbbi));
    AlignCoolBar (CoolBarWnd);
end;

function AddCoolBar (Wnd: hWnd): hWnd;
var
    style: DWord;
begin
    style := ws_Visible or ws_Child or ws_Border or ws_ClipChildren or ws_ClipSiblings;
    style := style or RBS_VarHeight or rbs_BandBorder;
    style := style or CCS_NoDivider or CCS_NoParentAlign;

    Result := CreateWindowEx (ws_ex_ToolWindow, ReBarClassName,
                              Nil, style, 0, 0, 200, 100, Wnd,
                              0, hInstance, Nil);
end;

{ Initialisation code for the unit }

begin
    CoolBarInit;
end.

