unit DataMod;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  DB, DBTables;

type
  TSortOrder = (soByUserID, soByUsername, soByFirstName, soByLastName);

  TDM = class(TDataModule)
    dbMain: TDatabase;
    qryUserList: TQuery;
    dsUserList: TDataSource;
    qryUserListUsername: TStringField;
    qryUserListUserID: TIntegerField;
    qryUserListFirstName: TStringField;
    qryUserListLastName: TStringField;
    qryUserListDateLastLogin: TDateTimeField;
    qryUserListDateLastPasswordChange: TDateTimeField;
    qryUserListPasswordLifespan: TSmallintField;
    qryUserListFullName: TStringField;
    qryGetDefaults: TQuery;
    qryGetDefaultsPasswordLifespan: TSmallintField;
    qryGetDefaultsMustChangeNewPassword: TStringField;
    procedure qryUserListCalcFields(DataSet: TDataSet);
    procedure dsUserListDataChange(Sender: TObject; Field: TField);
  protected
    FMustChangePwd: Boolean;
    FPwdLifespan: Integer;
    FNoUsers: Boolean;

    function GetPwdNeverExpires: Boolean;
  public
    procedure GetConfigSettings;
    procedure PostUserAdd(
                Username,
                Password,
                FirstName,
                LastName: string;
                MustChangePassword: Boolean;
                PasswordLifespan: Integer);
    procedure PostUserEdit(
                Username,
                Password,
                FirstName,
                LastName: string;
                MustChangePassword: Boolean;
                PasswordLifespan: Integer);
    procedure Refresh;
    procedure SetSortOrder(SortOrder: TSortOrder);
    procedure UpdateUserDetails;

    property MustChangePwd: Boolean
      read FMustChangePwd;
    property PwdNeverExpires: Boolean
      read GetPwdNeverExpires;
    property PwdLifespan: Integer
      read FPwdLifespan;
  end;

var
  DM: TDM;

implementation

{$R *.DFM}

uses
  FmMain, UserAPI;

procedure TDM.qryUserListCalcFields(DataSet: TDataSet);
begin
  qryUserListFullName.AsString :=
    qryUserListFirstName.AsString + ' ' +
    qryUserListLastName.AsString;
end;

procedure TDM.GetConfigSettings;
begin
  with qryGetDefaults do begin
    Open;
    try
      FMustChangePwd :=
        qryGetDefaultsMustChangeNewPassword.AsString = 'Y';
      FPwdLifespan :=
        qryGetDefaultsPasswordLifespan.AsInteger;
    finally
      Close;
    end;
  end;
end;

function TDM.GetPwdNeverExpires: Boolean;
begin
  Result := (FPwdLifespan = 0);
end;

procedure TDM.PostUserAdd(
                Username,
                Password,
                FirstName,
                LastName: string;
                MustChangePassword: Boolean;
                PasswordLifespan: Integer);
begin
  with qryUserList do begin
    DisableControls;
    try
      AddDBUser(dbMain,
                Username,
                Password,
                FirstName,
                LastName,
                MustChangePassword,
                PasswordLifespan);
      Close;
      Open;
    finally
      EnableControls;
    end;
  end;
end;

procedure TDM.PostUserEdit(
                Username,
                Password,
                FirstName,
                LastName: string;
                MustChangePassword: Boolean;
                PasswordLifespan: Integer);
begin
  with qryUserList do begin
    DisableControls;
    try
      EditDBUser(dbMain,
                 Username,
                 Password,
                 FirstName,
                 LastName,
                 MustChangePassword,
                 PasswordLifespan);
      Close;
      Open;
    finally
      EnableControls;
    end;
  end;
end;

procedure TDM.Refresh;
begin
  Screen.Cursor := crHourglass;
  try
    GetConfigSettings;
    with qryUserList do begin
      DisableControls;
      Close;
      Open;
      EnableControls;
    end;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TDM.SetSortOrder(SortOrder: TSortOrder);
begin
  with qryUserList do begin
    DisableControls;
    try
      Close;
      case SortOrder of
        soByUserID:
          SQL.SetText('SELECT * FROM Users ORDER BY UserID');
        soByUsername:
          SQL.SetText('SELECT * FROM Users ORDER BY Username');
        soByFirstName:
          SQL.SetText('SELECT * FROM Users ORDER BY FirstName');
        soByLastName:
          SQL.SetText('SELECT * FROM Users ORDER BY LastName');
      end;
      Open;
    finally
      EnableControls;
    end;
  end;
end;

procedure TDM.UpdateUserDetails;
begin
  with frmMain do begin
    edtUsername.Text := qryUserListUsername.AsString;
    edtFirstName.Text := qryUserLIstFirstName.AsString;
    edtLastName.Text := qryUserListLastName.AsString;
    edtPassword.Text := '                    ';
    edtConfirmPassword.Text := edtPassword.Text;
    lblConfirmPassword.Enabled := False;
    edtConfirmPassword.Enabled := False;

    chkPwdMustChange.Checked := False;
    { if DatLastPasswordChange is null, then password change forced }
    if qryUserListDateLastPasswordChange.IsNull then
      chkPwdMustChange.Checked := True
    else
      { if PasswordLifespan is null, then password never expires }
      if not qryUserListPasswordLifespan.IsNull then
        if Date - qryUserListDateLastPasswordChange.AsDateTime >=
          qryUserListPasswordLifespan.AsInteger then
          chkPwdMustChange.Checked := True;

    radPwdNeverExpires.Checked := qryUserListPasswordLifespan.IsNull;
    radPwdExpires.Checked := not radPwdNeverExpires.Checked;
    if radPwdExpires.Checked then
      edtPwdExpiresDays.Value := qryUserListPasswordLifespan.AsInteger
    else
      edtPwdExpiresDays.Value := 0;

    FNoUsers := qryUserList.RecordCount = 0;
  end;
end;

procedure TDM.dsUserListDataChange(Sender: TObject; Field: TField);
begin
  UpdateUserDetails;
end;

end.
