unit FmMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Grids, DBGrids, ExtCtrls, Menus, StdCtrls, ComCtrls, Mask, DBCtrls, Spin;

type
  TSystemMode = (smBrowsing, smAdding);

  TfrmMain = class(TForm)
    mnuMain: TMainMenu;
    mnuUser: TMenuItem;
    mnuOptions: TMenuItem;
    pnlUserList: TPanel;
    grdUserList: TDBGrid;
    mnuUserNew: TMenuItem;
    mnuUserDelete: TMenuItem;
    N1: TMenuItem;
    mnuUserExit: TMenuItem;
    pnlDetail: TPanel;
    edtUsername: TEdit;
    edtFirstName: TEdit;
    edtPassword: TEdit;
    edtConfirmPassword: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label4: TLabel;
    lblConfirmPassword: TLabel;
    Label3: TLabel;
    edtLastName: TEdit;
    lblDetailHeader: TLabel;
    chkPwdMustChange: TCheckBox;
    grpPasswordLifespan: TGroupBox;
    radPwdNeverExpires: TRadioButton;
    radPwdExpires: TRadioButton;
    edtPwdExpiresDays: TSpinEdit;
    Label5: TLabel;
    btnNewUser: TButton;
    btnExit: TButton;
    btnSave: TButton;
    btnUndo: TButton;
    mnuOptionsSort: TMenuItem;
    mnuOptionsSortUsername: TMenuItem;
    mnuOptionsSortFirst: TMenuItem;
    mnuOptionsSortLast: TMenuItem;
    mnuOptionsSortUserID: TMenuItem;
    procedure btnSaveClick(Sender: TObject);
    procedure mnuUserNewClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure mnuUserExitClick(Sender: TObject);
    procedure btnUndoClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure edtConfirmPasswordExit(Sender: TObject);
    procedure mnuOptionsSortUserIDClick(Sender: TObject);
    procedure mnuOptionsSortUsernameClick(Sender: TObject);
    procedure mnuOptionsSortFirstClick(Sender: TObject);
    procedure mnuOptionsSortLastClick(Sender: TObject);
    procedure edtPasswordChange(Sender: TObject);
  private
  protected
    Mode: TSystemMode;
  public
    procedure LockScreen(Switch: Boolean);
    procedure SetSortOrder(Sender: TMenuItem);
  end;

var
  frmMain: TfrmMain;

implementation

uses
  DataMod;

{$R *.DFM}

procedure TfrmMain.LockScreen(Switch: Boolean);
begin
  btnNewUser.Enabled := not Switch;
  grdUserList.Enabled := not Switch;
  edtUsername.ReadOnly := not Switch;

  if Switch then begin
    grdUserList.ParentColor := True;
    edtUsername.Color := clWindow;
  end
  else begin
    grdUserList.Color := clWindow;
    edtUsername.ParentColor := True;
  end;
end;

procedure TfrmMain.SetSortOrder(Sender: TMenuItem);
var
  I: Integer;
begin
  with mnuOptionsSort do
    for I := 0 to Count - 1 do
      Items[I].Checked := False;

  Sender.Checked := True;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Mode := smBrowsing;
end;

procedure TfrmMain.FormShow(Sender: TObject);
begin
  DM.GetConfigSettings;
  mnuOptionsSortUsernameClick(mnuOptionsSortUsername);
end;

procedure TfrmMain.mnuUserNewClick(Sender: TObject);
begin
  Mode := smAdding;

  LockScreen(True);

  edtUsername.Clear;
  edtFirstName.Clear;
  edtLastName.Clear;

  edtPassword.Text := '';
  lblConfirmPassword.Enabled := True;
  edtConfirmPassword.Enabled := True;
  edtConfirmPassword.Text := '';

  chkPwdMustChange.Checked := DM.MustChangePwd;

  radPwdNeverExpires.Checked := DM.PwdNeverExpires;
  radPwdExpires.Checked := not radPwdNeverExpires.Checked;
  edtPwdExpiresDays.Value := 0;

  edtUsername.SetFocus;
end;

procedure TfrmMain.mnuUserExitClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.btnSaveClick(Sender: TObject);
var
  PasswordLifespan: Integer;
begin
  if edtPassword.Text = '' then begin
    edtPassword.SetFocus;
    raise Exception.Create('Password required');
  end;

  if edtConfirmPassword.Text <> edtPassword.Text then begin
    edtConfirmPassword.SetFocus;
    raise Exception.Create('Password was not correctly verified.  Please re-enter password in ''Confirm Password''');
  end;

  if radPwdNeverExpires.Checked then
    PasswordLifespan := 0
  else
    PasswordLifespan := edtPwdExpiresDays.Value;

  if Mode = smAdding then
    DM.PostUserAdd(edtUsername.Text,
                   edtPassword.Text,
                   edtFirstName.Text,
                   edtLastName.Text,
                   chkPwdMustChange.Checked,
                   PasswordLifespan)
  else
    DM.PostUserEdit(edtUsername.Text,
                    edtPassword.Text,
                    edtFirstName.Text,
                    edtLastName.Text,
                    chkPwdMustChange.Checked,
                    PasswordLifespan);
  DM.Refresh;

  LockScreen(False);
end;

procedure TfrmMain.btnUndoClick(Sender: TObject);
begin
  lblConfirmPassword.Enabled := False;
  edtConfirmPassword.Enabled := False;
  if Mode = smAdding then begin
    DM.UpdateUserDetails;
    Mode := smBrowsing;
  end;
  LockScreen(False);
end;

procedure TfrmMain.edtConfirmPasswordExit(Sender: TObject);
begin
  if edtConfirmPassword.Text <> edtPassword.Text then begin
    edtConfirmPassword.SetFocus;
    raise Exception.Create('Password was not correctly validated.  Please re-enter passsword in ''Confirm Password''');
  end;
end;

procedure TfrmMain.mnuOptionsSortUserIDClick(Sender: TObject);
begin
  if not TMenuItem(Sender).Checked then begin
    SetSortOrder(TMenuItem(Sender));
    DM.SetSortOrder(soByUserID);
  end;
end;

procedure TfrmMain.mnuOptionsSortUsernameClick(Sender: TObject);
begin
  if not TMenuItem(Sender).Checked then begin
    SetSortOrder(TMenuItem(Sender));
    DM.SetSortOrder(soByUsername);
  end;
end;

procedure TfrmMain.mnuOptionsSortFirstClick(Sender: TObject);
begin
  if not TMenuItem(Sender).Checked then begin
    SetSortOrder(TMenuItem(Sender));
    DM.SetSortOrder(soByFirstName);
  end;
end;

procedure TfrmMain.mnuOptionsSortLastClick(Sender: TObject);
begin
  if not TMenuItem(Sender).Checked then begin
    SetSortOrder(TMenuItem(Sender));
    DM.SetSortOrder(soByLastName);
  end;
end;

procedure TfrmMain.edtPasswordChange(Sender: TObject);
begin
  lblConfirmPassword.Enabled := True;
  edtConfirmPassword.Enabled := True;
end;

end.
