unit MManager;

interface
var
  HeapSt: THeapStatus;
procedure SetDebugManager(FName: string; AStep: integer);
procedure ClearDebugManager;
procedure WriteDebug(const S: string);

implementation
uses
  SysUtils;
var
  FileName : string; // the log Filename
  OldMM : TMemoryManager; // the default memory manager
  F: System.Text; // the log file
  OldAllocated : integer;// save the last value of allocated memory
  Step : integer; // the treshold after which we write on the log file


{ Memory manager routines }

function DebugGetMem(Size: integer): pointer;
begin
  Result:=OldMM.GetMem(Size);
  HeapSt:= GetHeapStatus;
  if (FileName <>'') and ((HeapSt.TotalAllocated - OldAllocated) >= Step) then
    Writeln(F,'GetMem : ',HeapSt.TotalAllocated);
  OldAllocated := HeapSt.TotalAllocated;
end;

function DebugFreeMem(P: Pointer): integer;
begin
  Result := OldMM.FreeMem(P);
  HeapSt:= GetHeapStatus;
  if (FileName <>'') and ((OldAllocated -HeapSt.TotalAllocated) >= Step) then
    Writeln(F,'FreeMem : ',HeapSt.TotalAllocated);
  OldAllocated := HeapSt.TotalAllocated;
end;

function DebugReallocMem(P: pointer; Size: integer): Pointer;
begin
  Result := OldMM.ReallocMem(P,Size);
  HeapSt:= GetHeapStatus;
  if (FileName <>'') and ((HeapSt.TotalAllocated - OldAllocated) >= Step) then
    Writeln(F,'ReallocMem : ',HeapSt.TotalAllocated);
  OldAllocated := HeapSt.TotalAllocated;
end;

const
{ the new memory manager }
  DebugMM: TMemoryManager =(
    GetMem : DebugGetMem;
    FreeMem : DebugFreeMem;
    ReallocMem : DebugReallocMem);


{ exported routines }

procedure SetDebugManager(FName: string; AStep: integer);
begin
  FileName := FName;
  Step:= AStep;
  OldAllocated := 0;
  if FileName <> '' then begin
    AssignFile(F,FileName);
    Rewrite(F);
  end;

  GetMemoryManager(OldMM);
  SetMemoryManager(DebugMM);
  HeapSt:= GetHeapStatus;
  if FileName <> '' then
    writeln(F,'Initially allocated Memory : ',HeapSt.TotalAllocated);
  OldAllocated := HeapSt.TotalAllocated;
end;

procedure ClearDebugManager;
begin
  if FileName <> '' then begin
    writeln(F,'Finally allocated Memory : ',HeapSt.TotalAllocated);
    CloseFile(F);
  end;
  SetMemoryManager(OldMM);
end;

procedure WriteDebug(const S: string);
begin
  if FileName <> '' then
    writeln(F,S);
end;


end.
