unit TFooU;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DsgnIntf;

const
  irFirst = 1;
  irSecond = 2;
  irThird = 3;

type
  TEnumProp = (epFirst, epSecond, epThird);
  TFooIntRange = 1..6;

  TFoo = class(TComponent)
  private
    FEnumProp: TEnumProp;
    FIntProp: TFooIntRange;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property EnumProp: TEnumProp read FEnumProp write FEnumProp;
    property IntProp: TFooIntRange read FIntProp write FIntProp default 1;
  end;

  TFooIntProperty = class(TIntegerProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
    procedure GetValues(Proc: TGetStrProc); override;
    procedure SetValue(const Value: string); override;
  end;

procedure Register;

implementation

constructor TFoo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FIntProp := 1;
end;

const
  IntProps: array[irFirst..irThird] of string[8] = (
    'irFirst',
    'irSecond',
    'irThird');

function TFooIntProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paMultiSelect, paValueList];
end;

function TFooIntProperty.GetValue: string;
var
  CurValue: Longint;
begin
  CurValue := GetOrdValue;
  case CurValue of
    Low(IntProps)..High(IntProps): Result := IntProps[CurValue];
  else
    Result := IntToStr(CurValue);
  end;
end;

procedure TFooIntProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
begin
  for I := Low(IntProps) to High(IntProps) do
    Proc(IntProps[I]);
end;

procedure TFooIntProperty.SetValue(const Value: string);
var
  I: Integer;
begin
  if Value = '' then
  begin
    SetOrdValue(Low(TFooIntRange));
    Exit;
  end;
  for I := Low(IntProps) to High(IntProps) do
    if CompareText(IntProps[I], Value) = 0 then
    begin
      SetOrdValue(I);
      Exit;
    end;
  inherited SetValue(Value);
end;

procedure Register;
begin
  RegisterComponents('Clinic', [TFoo]);
  RegisterPropertyEditor(TypeInfo(TFooIntRange), TFoo, 'IntProp', TFooIntProperty);
end;

end.