unit CustomU;

interface

uses
  WinProcs, WinTypes, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons;

type
  TForm1 = class(TForm)
    BtnDoIt: TBitBtn;
    BtnFindIt: TSpeedButton;
    EdtDir: TEdit;
    OpenDialog1: TOpenDialog;
    LstFixed: TListBox;
    CmbFixed: TComboBox;
    LstFixed2: TListBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BtnDoItClick(Sender: TObject);
    procedure BtnFindItClick(Sender: TObject);
    procedure LstFixedDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure CmbFixedDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
  public
    Bmps: TStrings;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure EmptyBmpList(S: TStrings);
begin
  with S do
    while Count > 0 do
    begin
      Objects[0].Free;
      Delete(0);
    end;
end;

procedure LoadBitmaps(const Path: String; S: TStrings);
var
  Bmp: TBitmap;
  SearchRec: TSearchRec;
begin
  EmptyBmpList(S);
  with S do
  begin
    BeginUpdate;
    if FindFirst(Path + '*.bmp', faAnyFile, SearchRec) = 0 then
      try
        repeat
          Bmp := TBitmap.Create;
          Bmp.LoadFromFile(Path + SearchRec.Name);
          AddObject(SearchRec.Name, Bmp);
        until FindNext(SearchRec) <> 0;
      finally
        FindClose(SearchRec);
      end;
    EndUpdate;
  end;
end;

function ScaleBmpToItemSize(const R: TRect; Bmp: TBitmap): TRect;
begin
  Result := R;
  with Result, Bmp do
    Right := Trunc(Left + (Bottom - Top) * Width / Height)
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Bmps := TStringList.Create
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  EmptyBmpList(Bmps);
  Bmps.Free;
end;

procedure TForm1.BtnDoItClick(Sender: TObject);
begin
  if Length(EdtDir.Text) = 0 then
    raise Exception.Create('Type a path in the edit control');
  if EdtDir.Text[Length(EdtDir.Text)] <> '\' then
    EdtDir.Text := EdtDir.Text + '\';
  try
    LoadBitmaps(EdtDir.Text, Bmps);
  finally
    Screen.Cursor := crDefault;
  end;
  TStringList(Bmps).Sort;
  LstFixed.Items := Bmps;
  LstFixed2.Items := Bmps;
  CmbFixed.Items := Bmps;
end;

procedure TForm1.BtnFindItClick(Sender: TObject);
begin
  with OpenDialog1 do
  begin
    InitialDir := EdtDir.Text;
    if Execute then
    begin
      EdtDir.Text := ExtractFilePath(FileName);
      BtnDoIt.Click
    end
  end
end;

procedure TForm1.LstFixedDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
  BmpRect, TextRect: TRect;
begin
  with (Control as TListBox), Items, Canvas, Rect do
  begin
    { Since we are only drawing over part of the item, make sure }
    { we blank out any old garbage with a foreground rectangle }
    FillRect(Rect);
    { Draw the bitmap, scaled to fit in the fixed height item }
    BmpRect := ScaleBmpToItemSize(Rect, TBitmap(Objects[Index]));
    StretchDraw(BmpRect, TBitmap(Objects[Index]));
    { Invert the item colours if it is selected }
    if odSelected in State then
      InvertRect(Handle, BmpRect);
    { Write the filename after the bitmap, vertically centred }
    with BmpRect do
      TextOut(Right + 4, (Top + Bottom + Font.Height - 1) div 2, Items[Index]);
  end
end;

procedure TForm1.CmbFixedDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
  BmpRect, TextRect: TRect;
begin
  with CmbFixed, Items, Canvas, Rect do
  begin
    { Since we are only drawing over part of the item, make sure }
    { we blank out any old garbage with a foreground rectangle }
    FillRect(Rect);
    { Draw the bitmap, scaled to fit in the fixed height item }
    BmpRect := ScaleBmpToItemSize(Rect, TBitmap(Objects[Index]));
    StretchDraw(BmpRect, TBitmap(Objects[Index]));
    { Invert the item colours if it is selected }
    if odSelected in State then
      InvertRect(Handle, BmpRect);
    { Write the filename after the bitmap, vertically centred }
    with BmpRect do
      TextOut(Right + 4, (Top + Bottom + Font.Height - 1) div 2, Items[Index]);
  end
end;

end.
