library OurUDFs;

uses
  Windows,
  Dialogs,
  SysUtils;

type
  TIBDateTime = record
    NumDays: LongInt;   { Number of days since 11/17/1858 }
    DayFrac: LongInt;   { Number of ???seconds since midnight (milliseconds * 10) }
  end;
  PIBDateTime = ^TIBDateTime;

  TGetSegmentProc = function(BlobHandle: Pointer;
                             Buffer: Pointer;
                             BufLen: Smallint;
                             var BytesWritten: SmallInt): WordBool; cdecl;

  TPutSegmentProc = procedure(BlobHandle: Pointer;
                              Buffer: Pointer;
                              BufLen: Smallint);

  PBlobControlStruct = ^TBlobControlStruct;
  TBLOBControlStruct = record
    BlobGetSegment: TGetSegmentProc;
    BlobHandle: Pointer;
    NumberOfSegments: LongInt;
    MaxSegmentLength: LongInt;
    TotalSize: LongInt;
    BlobPutSegment: TPutSegmentProc;
  end;

var
  Buffer: array[0..255] of Char;
  ReturnDateTime: TIBDateTime;

function udf_Abs(var X: Double): Double; cdecl;
{ Returns the absolute value of the argument. }
begin
  Result := Abs(X);
end;

function udf_StrLen(S: PChar): SmallInt; cdecl;
{ Returns the length of a character string. }
begin
  Result := StrLen(S);
end;

function udf_Substring(S: PChar; var Start, Len: SmallInt): PChar; cdecl;
{ Returns the substring of the given string started at <start> and continuing
  for at most <len> characters. }
begin
  Buffer[0] := #0;
  if (Start > 0) and (Len >= 0) then
    StrLCopy(Buffer, S + Start - 1, Len);
  StrPCopy(Buffer, IntToStr(Start) + ' ' + IntToStr(Len) + ' "' + StrPas(S) + '"');
  Result := S;
end;

function udf_DayOrd(var Date: TIBDateTime): LongInt; cdecl;
{ Returns the ordinal value of the date }
begin
  Result := Date.NumDays;
end;

function udf_DaysDiff(Date1, Date2: PIBDateTime): SmallInt; cdecl;
{ Returns the number of days between two dates }
begin
  Result := Abs(Date2^.NumDays - Date1^.NumDays);
end;

function udf_CalcDate(Date1: PIBDateTime; var Days: SmallInt): PIBDateTime; cdecl;
{ Returns the date that is +/- a given number of days from a given date }
begin
  ReturnDateTime := Date1^;
  Inc(ReturnDateTime.NumDays, Days);
  Result := @ReturnDateTime;
end;

function udf_BlobStrCount(SubStr: PChar; var Blob: TBLOBControlStruct): SmallInt; cdecl;
{ Returns the number of occurrences of <substr> within the BLOB }
var
  Buffer: PChar;
  BufLen: SmallInt;
  S: PChar;
  BytesBack: SmallInt;
  L: Integer;
begin
  Result := 0;
  L := StrLen(Substr);
  with Blob do begin
    { Check for a null BLOB }
    if BlobHandle = nil then Exit;
    BufLen := MaxSegmentLength + 1;
    GetMem(Buffer, BufLen);
    try
      { Get each BLOB segment }
      while BlobGetSegment(BlobHandle, Buffer, BufLen, BytesBack) do begin
        { Count the occurrences of the substring }
        S := StrPos(Buffer, Substr);
        while S <> nil do begin
          Inc(Result);
          S := StrPos(S + L, Substr);
        end;
      end;
    finally
      FreeMem(Buffer, BufLen);
    end;
  end;
end;


exports
  udf_Abs,
  udf_BlobStrCount,
  udf_CalcDate,
  udf_DayOrd,
  udf_DaysDiff,
  udf_StrLen,
  udf_Substring;

begin
end.

