unit Mainform;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ResFile, StdCtrls;

type
  TForm1 = class(TForm)
    Types: TListBox;
    TypeCount: TLabel;
    ResList: TListBox;
    ResCount: TLabel;
    ResSize: TLabel;
    ResOffset: TLabel;
    ResFlags: TLabel;
    Button1: TButton;
    OpenDialog: TOpenDialog;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TypesClick(Sender: TObject);
    procedure ResListClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure ResListDblClick(Sender: TObject);
  private
    { Private declarations }
    rf: TResFile;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    rf.Free;
end;

procedure TForm1.TypesClick(Sender: TObject);
var
    Count, Idx: Integer;
    ResType: String;
begin
    if (rf <> Nil) and (Types.Items.Count > 0) then
    begin
        ResList.Clear;
        ResType := Types.Items [Types.ItemIndex];
        Count := rf.GetResourceCount (ResType);
        ResCount.Caption := IntToStr (Count) + ' Resources of Selected Type:';
        for Idx := 0 to Count - 1 do
            ResList.Items.Add (rf.GetResourceName (ResType, Idx));
        ResList.ItemIndex := 0;
        ResListClick (Self);
    end;
end;

procedure TForm1.ResListClick(Sender: TObject);
var
    Info: TResInfo;
begin
    if rf <> Nil then
    begin
        rf.GetResourceInfo (Types.Items [Types.ItemIndex], ResList.ItemIndex, Info);
        ResSize.Caption := 'Size of Resource = $' + IntToHex (Info.RLength, 4) + ' bytes.';
        ResOffset.Caption := 'File Offset of Resource = $' + IntToHex (Info.ROffset, 8) + ' bytes.';
        ResFlags.Caption := 'Resource Flags = $' + IntToHex (Info.RFlags, 4) + '.';
    end;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
    Count, Idx: Integer;
begin
    ResList.Clear;
    Types.Clear;
    if rf <> Nil then rf.Free;

    if OpenDialog.Execute then
    begin
        try
            rf := TResFile.Create (OpenDialog.FileName);
            rf.ResMapNames := True;
            Count := rf.ResTypeCount;
            TypeCount.Caption := IntToStr (Count) + ' Resource Types:';
            for Idx := 0 to Count - 1 do Types.Items.Add (rf.ResTypes [Idx]);
            Types.ItemIndex := 0;
            TypesClick (Self);
        except
            Application.HandleException (Self);
        end;
    end;
end;

procedure TForm1.ResListDblClick(Sender: TObject);
var
    Info: TResInfo;
    ResData: Pointer;
    ResIndex: Integer;
    fs: TFileStream;
    TypName, ResName, fName: String;
begin
    with rf do
    begin
        ResIndex := ResList.ItemIndex;
        TypName := Types.Items [Types.ItemIndex];
        ResName := ResList.Items [ResIndex];

        if MessageDlg (Format ('Extract resource ''%s'' (type = ''%s'') ?', [ResName, TypName]),
                       mtConfirmation, [mbYes, mbNo], 0) = idYes then
        begin
            GetResourceInfo (TypName, ResIndex, Info);
            GetMem (ResData, Info.rLength);
            try
                fs := TFileStream.Create (OpenDialog.FileName, fmOpenRead);
                try
                    fs.Position := Info.rOffset;
                    fs.Read (ResData^, Info.rLength);
                finally
                    fs.Free;
                end;

                fName := ExtractFilePath (Application.ExeName) + ResName + '.BIN';
                fs := TFileStream.Create (fName, fmCreate);
                try
                    fs.Write (ResData^, Info.rLength);
                    MessageDlg ('Resource has been written to ' + fName, mtInformation, [mbok], 0);
                finally
                    fs.Free;
                end;

            finally
                FreeMem (ResData, Info.rLength);
            end;
        end;
    end;
end;

end.
