unit BtnRept;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls;

const
  InitRepeatPause = 400;  { pause before repeat timer starts (ms) }
  OtherRepeatPause = 200;  { pause before successsive hits (ms)}

type
  TRepeatBtn = class(TButton)
  private
    FRepeatTimer: TTimer;
    FInitialPause,      { pause before repeat timer starts (ms) }
    FRepeatPause: Word; { pause before successive hits (ms)}
  protected
    procedure TimerExpired(Sender: TObject);
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y:
      Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y:
      Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property InitialPause: Word
      read FInitialPause write FInitialPause default InitRepeatPause;
    property RepeatPause: Word
      read FRepeatPause write FRepeatPause default OtherRepeatPause;
  end;

procedure Register;

implementation

constructor TRepeatBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FInitialPause := InitRepeatPause;
  FRepeatPause := OtherRepeatPause;
  { Don't need to free it - it will happen automatically since }
  { we passed a non-nil value as the constructor parameter }
  FRepeatTimer := TTimer.Create(Self);
  FRepeatTimer.Enabled := False;
  FRepeatTimer.OnTimer := TimerExpired;
end;

procedure TRepeatBtn.TimerExpired(Sender: TObject);
begin
  FRepeatTimer.Interval := RepeatPause;
  try
    Click;
    { If button has been disabled as a result of what }
    { happens in its OnClick method, shutdown timer }
    FRepeatTimer.Enabled := Enabled;
  except
    FRepeatTimer.Enabled := False;
    raise;
  end;
end;

procedure TRepeatBtn.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y:
  Integer);
begin
  inherited MouseDown(Button, Shift, X, Y);
  FRepeatTimer.Interval := InitRepeatPause;
  FRepeatTimer.Enabled := True;
end;

procedure TRepeatBtn.MouseMove(Shift: TShiftState; X, Y: Integer);
var
  Pt: TPoint;
begin
  { Even if mouse goes off button, mouse move events }
  { will still go to the button because it was }
  { created with the csCaptureMouse control style }
  inherited MouseMove(Shift, X, Y);
  Pt.X := X;
  Pt.Y := Y;
  if FRepeatTimer.Enabled then
    FRepeatTimer.Enabled := PtInRect(Rect(0, 0, Width, Height), Pt)
end;

procedure TRepeatBtn.MouseUp(Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  inherited MouseUp(Button, Shift, X, Y);
  FRepeatTimer.Enabled := False;
end;

procedure Register;
begin
  RegisterComponents('Clinic', [TRepeatBtn]);
end;

end.
