unit WizardU;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ActiveX, AxCtrls, WizardX_TLB, ExtCtrls, StdCtrls, Buttons, ComCtrls;

type
  TWizardActiveForm = class(TActiveForm, IWizardActiveForm)
    DrBobWizardTemplatePanel: TPanel;
    Bevel: TBevel;
    Notebook: TNotebook;
    ButtonCancel: TButton;
    ButtonNext: TButton;
    ButtonBack: TButton;
    Panel: TPanel;
    Image: TImage;
    Animate1: TAnimate;
    OpenDialog1: TOpenDialog;
    SaveDialog1: TSaveDialog;
    StaticText1: TStaticText;
    StaticText2: TStaticText;
    EditInputFile: TEdit;
    EditOutputFile: TEdit;
    BtnOutputFile: TSpeedButton;
    BtnInputFile: TSpeedButton;
    Label1: TLabel;
    BtnAction: TBitBtn;
    procedure ButtonStepClick(Sender: TObject);
    procedure BtnInputFileClick(Sender: TObject);
    procedure BtnOutputFileClick(Sender: TObject);
    procedure EditInputFileChange(Sender: TObject);
    procedure BtnActionClick(Sender: TObject);
  private
    { Private declarations }
    FEvents: IWizardActiveFormEvents;
    procedure ActivateEvent(Sender: TObject);
    procedure ClickEvent(Sender: TObject);
    procedure CreateEvent(Sender: TObject);
    procedure DblClickEvent(Sender: TObject);
    procedure DeactivateEvent(Sender: TObject);
    procedure DestroyEvent(Sender: TObject);
    procedure KeyPressEvent(Sender: TObject; var Key: Char);
    procedure PaintEvent(Sender: TObject);
  protected
    { Protected declarations }
    procedure EventSinkChanged(const EventSink: IUnknown); override;
    procedure Initialize; override;
    function Get_Active: WordBool; safecall;
    function Get_AutoScroll: WordBool; safecall;
    function Get_AxBorderStyle: TxActiveFormBorderStyle; safecall;
    function Get_Caption: WideString; safecall;
    function Get_Color: TColor; safecall;
    function Get_Cursor: Smallint; safecall;
    function Get_DropTarget: WordBool; safecall;
    function Get_Enabled: WordBool; safecall;
    function Get_Font: Font; safecall;
    function Get_HelpFile: WideString; safecall;
    function Get_KeyPreview: WordBool; safecall;
    function Get_PixelsPerInch: Integer; safecall;
    function Get_PrintScale: TxPrintScale; safecall;
    function Get_Scaled: WordBool; safecall;
    function Get_Visible: WordBool; safecall;
    function Get_WindowState: TxWindowState; safecall;
    procedure AboutBox; safecall;
    procedure Set_AutoScroll(Value: WordBool); safecall;
    procedure Set_AxBorderStyle(Value: TxActiveFormBorderStyle); safecall;
    procedure Set_Caption(const Value: WideString); safecall;
    procedure Set_Color(Value: TColor); safecall;
    procedure Set_Cursor(Value: Smallint); safecall;
    procedure Set_DropTarget(Value: WordBool); safecall;
    procedure Set_Enabled(Value: WordBool); safecall;
    procedure Set_Font(const Value: Font); safecall;
    procedure Set_HelpFile(const Value: WideString); safecall;
    procedure Set_KeyPreview(Value: WordBool); safecall;
    procedure Set_PixelsPerInch(Value: Integer); safecall;
    procedure Set_PrintScale(Value: TxPrintScale); safecall;
    procedure Set_Scaled(Value: WordBool); safecall;
    procedure Set_Visible(Value: WordBool); safecall;
    procedure Set_WindowState(Value: TxWindowState); safecall;
  public
    { Public declarations }
  end;

implementation

uses ComServ, About1;

{$R *.DFM}

{ TWizardActiveForm }

procedure TWizardActiveForm.EventSinkChanged(const EventSink: IUnknown);
begin
  FEvents := EventSink as IWizardActiveFormEvents;
end;

procedure TWizardActiveForm.Initialize;
begin
  OnActivate := ActivateEvent;
  OnClick := ClickEvent;
  OnCreate := CreateEvent;
  OnDblClick := DblClickEvent;
  OnDeactivate := DeactivateEvent;
  OnDestroy := DestroyEvent;
  OnKeyPress := KeyPressEvent;
  OnPaint := PaintEvent;
end;

function TWizardActiveForm.Get_Active: WordBool;
begin
  Result := Active;
end;

function TWizardActiveForm.Get_AutoScroll: WordBool;
begin
  Result := AutoScroll;
end;

function TWizardActiveForm.Get_AxBorderStyle: TxActiveFormBorderStyle;
begin
  Result := Ord(AxBorderStyle);
end;

function TWizardActiveForm.Get_Caption: WideString;
begin
  Result := WideString(Caption);
end;

function TWizardActiveForm.Get_Color: TColor;
begin
  Result := Color;
end;

function TWizardActiveForm.Get_Cursor: Smallint;
begin
  Result := Smallint(Cursor);
end;

function TWizardActiveForm.Get_DropTarget: WordBool;
begin
  Result := DropTarget;
end;

function TWizardActiveForm.Get_Enabled: WordBool;
begin
  Result := Enabled;
end;

function TWizardActiveForm.Get_Font: Font;
begin
  GetOleFont(Font, Result);
end;

function TWizardActiveForm.Get_HelpFile: WideString;
begin
  Result := WideString(HelpFile);
end;

function TWizardActiveForm.Get_KeyPreview: WordBool;
begin
  Result := KeyPreview;
end;

function TWizardActiveForm.Get_PixelsPerInch: Integer;
begin
  Result := PixelsPerInch;
end;

function TWizardActiveForm.Get_PrintScale: TxPrintScale;
begin
  Result := Ord(PrintScale);
end;

function TWizardActiveForm.Get_Scaled: WordBool;
begin
  Result := Scaled;
end;

function TWizardActiveForm.Get_Visible: WordBool;
begin
  Result := Visible;
end;

function TWizardActiveForm.Get_WindowState: TxWindowState;
begin
  Result := Ord(WindowState);
end;

procedure TWizardActiveForm.AboutBox;
begin
  ShowWizardActiveFormAbout;
end;

procedure TWizardActiveForm.Set_AutoScroll(Value: WordBool);
begin
  AutoScroll := Value;
end;

procedure TWizardActiveForm.Set_AxBorderStyle(
  Value: TxActiveFormBorderStyle);
begin
  AxBorderStyle := TActiveFormBorderStyle(Value);
end;

procedure TWizardActiveForm.Set_Caption(const Value: WideString);
begin
  Caption := TCaption(Value);
end;

procedure TWizardActiveForm.Set_Color(Value: TColor);
begin
  Color := Value;
end;

procedure TWizardActiveForm.Set_Cursor(Value: Smallint);
begin
  Cursor := TCursor(Value);
end;

procedure TWizardActiveForm.Set_DropTarget(Value: WordBool);
begin
  DropTarget := Value;
end;

procedure TWizardActiveForm.Set_Enabled(Value: WordBool);
begin
  Enabled := Value;
end;

procedure TWizardActiveForm.Set_Font(const Value: Font);
begin
  SetOleFont(Font, Value);
end;

procedure TWizardActiveForm.Set_HelpFile(const Value: WideString);
begin
  HelpFile := String(Value);
end;

procedure TWizardActiveForm.Set_KeyPreview(Value: WordBool);
begin
  KeyPreview := Value;
end;

procedure TWizardActiveForm.Set_PixelsPerInch(Value: Integer);
begin
  PixelsPerInch := Value;
end;

procedure TWizardActiveForm.Set_PrintScale(Value: TxPrintScale);
begin
  PrintScale := TPrintScale(Value);
end;

procedure TWizardActiveForm.Set_Scaled(Value: WordBool);
begin
  Scaled := Value;
end;

procedure TWizardActiveForm.Set_Visible(Value: WordBool);
begin
  Visible := Value;
end;

procedure TWizardActiveForm.Set_WindowState(Value: TxWindowState);
begin
  WindowState := TWindowState(Value);
end;

procedure TWizardActiveForm.ActivateEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnActivate;
end;

procedure TWizardActiveForm.ClickEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnClick;
end;

procedure TWizardActiveForm.CreateEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnCreate;
end;

procedure TWizardActiveForm.DblClickEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnDblClick;
end;

procedure TWizardActiveForm.DeactivateEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnDeactivate;
end;

procedure TWizardActiveForm.DestroyEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnDestroy;
end;

procedure TWizardActiveForm.KeyPressEvent(Sender: TObject; var Key: Char);
var
  TempKey: Smallint;
begin
  TempKey := Smallint(Key);
  if FEvents <> nil then FEvents.OnKeyPress(TempKey);
  Key := Char(TempKey);
end;

procedure TWizardActiveForm.PaintEvent(Sender: TObject);
begin
  if FEvents <> nil then FEvents.OnPaint;
end;

procedure TWizardActiveForm.ButtonStepClick(Sender: TObject);
begin
  if Sender IS TButton then
    NoteBook.PageIndex := NoteBook.PageIndex + (Sender AS TButton).Tag
  else NoteBook.PageIndex := 0;
  ButtonBack.Enabled := NoteBook.PageIndex > 0; { first }
  if NoteBook.PageIndex < Pred(NoteBook.Pages.Count) then
  begin
    ButtonNext.Caption := '&Next >';
    ButtonNext.ModalResult := mrNone
  end
  else { Finish }
  begin
    ButtonNext.Caption := '&Finish';
    ButtonNext.ModalResult := mrOk
  end
end;


procedure TWizardActiveForm.BtnInputFileClick(Sender: TObject);
begin
  OpenDialog1.FileName := EditInputFile.Text;
  if OpenDialog1.Execute then
    EditInputFile.Text := OpenDialog1.FileName;
end;

procedure TWizardActiveForm.BtnOutputFileClick(Sender: TObject);
begin
  SaveDialog1.FileName := EditOutputFile.Text;
  if SaveDialog1.Execute then
    EditOutputFile.Text := SaveDialog1.FileName;
end;

procedure TWizardActiveForm.EditInputFileChange(Sender: TObject);
begin
  BtnAction.Enabled := (EditInputFile.Text <> '') and (EditOutputFile.Text <> '')
end;

procedure TWizardActiveForm.BtnActionClick(Sender: TObject);
begin
  { copy EditInputFile.Text to EditOutputFile.Text }
  Animate1.Active := not Animate1.Active;
end;

initialization
  TActiveFormFactory.Create(
    ComServer,
    TActiveFormControl,
    TWizardActiveForm,
    Class_WizardActiveForm,
    1,
    '',
    OLEMISC_SIMPLEFRAME or OLEMISC_ACTSLIKELABEL);
end.
