unit Mainform;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ResFile, StdCtrls;

type
  TForm1 = class(TForm)
    Types: TListBox;
    TypeCount: TLabel;
    ResList: TListBox;
    ResCount: TLabel;
    ResSize: TLabel;
    ResOffset: TLabel;
    ResFlags: TLabel;
    Button1: TButton;
    OpenDialog: TOpenDialog;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TypesClick(Sender: TObject);
    procedure ResListClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure ResListDblClick(Sender: TObject);
    procedure WriteResourceFile (const ResName, TypName: String; Info: TResInfo; ResData: Pointer);
  private
      { Private declarations }
      rf: TResFile;
      procedure WriteMultiIconFile (fs: TFileStream; ImageData: PByte; Len: Word);
      procedure WriteIndividualIcon (fs: TFileStream; IconID, ImageSize: Integer);
      procedure WriteSingleIconFile (fs: TFileStream; ImageData: PByte; Len: Word);
      procedure WriteBitmapFile (fs: TFileStream; ImageData: PByte; Len: Word);
  public
    { Public declarations }
  end;

var
    Form1: TForm1;

implementation

type
    TIconHeader = record
        AlwaysZero: Integer;                   { Always zero }
        AlwaysOne: Integer;                    { Always One }
        NumIcons: Integer;                     { Number of icons in file }
    end;

    TIconDirEntry = record
        Width, Height, Colors: Byte;
        Reserved: Byte;
        dwReserved: LongInt;
        dwBytesInRes: LongInt;
        dwImageOffset: LongInt;
    end;

    TGroupDirEntry = record
        Width, Height, Colors: Byte;
        Reserved: Byte;
        Planes: Integer;
        BitCount: Integer;
        ImageSize: LongInt;
        IconID: Word;
    end;

{$R *.DFM}

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    rf.Free;
end;

procedure TForm1.TypesClick(Sender: TObject);
var
    Count, Idx: Integer;
    ResType: String;
begin
    if (rf <> Nil) and (Types.Items.Count > 0) then
    begin
        ResList.Clear;
        ResType := Types.Items [Types.ItemIndex];
        Count := rf.GetResourceCount (ResType);
        ResCount.Caption := IntToStr (Count) + ' Resources of Selected Type:';
        for Idx := 0 to Count - 1 do
            ResList.Items.Add (rf.GetResourceName (ResType, Idx));
        ResList.ItemIndex := 0;
        ResListClick (Self);
    end;
end;

procedure TForm1.ResListClick(Sender: TObject);
var
    Info: TResInfo;
begin
    if rf <> Nil then
    begin
        rf.GetResourceInfo (Types.Items [Types.ItemIndex], ResList.ItemIndex, Info);
        ResSize.Caption := 'Size of Resource = $' + IntToHex (Info.RLength, 4) + ' bytes.';
        ResOffset.Caption := 'File Offset of Resource = $' + IntToHex (Info.ROffset, 8) + ' bytes.';
        ResFlags.Caption := 'Resource Flags = $' + IntToHex (Info.RFlags, 4) + '.';
    end;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
    Count, Idx: Integer;
begin
    if OpenDialog.Execute then
    begin
        ResList.Clear;
        Types.Clear;
        rf.Free;

        try
            rf := TResFile.Create (OpenDialog.FileName);
            rf.ResMapNames := True;
            Count := rf.ResTypeCount;
            TypeCount.Caption := IntToStr (Count) + ' Resource Types:';
            for Idx := 0 to Count - 1 do Types.Items.Add (rf.ResTypes [Idx]);
            Types.ItemIndex := 0;
            TypesClick (Self);
        except
            Application.HandleException (Self);
        end;
    end;
end;

function GetIconImageSize (ImageData: PByte): LongInt;
var
    bm: TBitmapInfoHeader;
begin
    Result := 0;
    if ImageData^ = sizeof (bm) then
    begin
        bm := PBitmapInfoHeader (ImageData)^;
        bm.biHeight := bm.biHeight div 2;
        Result := sizeof (bm);
        Inc (Result, sizeof (TRGBQuad) * LongInt (1 shl bm.biBitCount));
        { Allocate another 1 bit/pixel for mask }
        Inc (bm.biBitCount);
        { Hack: Older 16*16, 16-color icons use 2 bits/pixel for mask! }
        if (bm.biHeight = 16) and (bm.biWidth = 16) and (bm.biBitCount = 5) then
            Inc (bm.biBitCount);
        Inc (Result, ((bm.biWidth * bm.biHeight + 7) div 8) * bm.biBitCount);
    end;
end;

procedure TForm1.WriteBitmapFile (fs: TFileStream; ImageData: PByte; Len: Word);
var
    Hdr: TBitmapFileHeader;
    pbm: PBitmapInfoHeader absolute ImageData;
begin
    { Initialise and write the file header }
    Hdr.bfType := $4D42;
    Hdr.bfSize := sizeof (Hdr) + Len;
    Hdr.bfReserved1 := 0;
    Hdr.bfReserved2 := 0;
    Hdr.bfOffbits := sizeof (Hdr) + sizeof (TBitmapInfoHeader) + (sizeof (TRGBQuad) * LongInt (1 shl pbm^.biBitCount));
    fs.Write (Hdr, sizeof (Hdr));
    { Then write the data }
    pbm^.biClrImportant := LongInt (1 shl pbm^.biBitCount);
    fs.Write (ImageData^, Len);
end;

procedure TForm1.WriteIndividualIcon (fs: TFileStream; IconID, ImageSize: Integer);
var
    Info: TResInfo;
    ResData: PByte;
    IconList: TStringList;
    IconStream: TFileStream;
begin
    IconList := rf.GetResList ('ICON');
    IconID := IconList.IndexOf ('#' + IntToStr (IconID));
    rf.GetResourceInfo ('ICON', IconID, Info);
    GetMem (ResData, Info.rLength);
    try
        IconStream := TFileStream.Create (OpenDialog.FileName, fmOpenRead);
        try
            IconStream.Position := Info.rOffset;
            IconStream.Read (ResData^, Info.rLength);
        finally
            IconStream.Free;
        end;

        fs.Write (ResData^, ImageSize);
    finally
        FreeMem (ResData, Info.rLength);
    end;
end;

procedure TForm1.WriteMultiIconFile (fs: TFileStream; ImageData: PByte; Len: Word);
var
    Idx: Integer;
    DirStart: PByte;
    Hdr: TIconHeader;
    Offset: LongInt;
    Dir: TIconDirEntry;
    GroupDir: TGroupDirEntry;
begin
    { Get the file header and write it now }
    Move (ImageData^, Hdr, sizeof (Hdr));
    fs.Write (Hdr, sizeof (Hdr));
    Inc (ImageData, sizeof (Hdr));
    { Save current position and calculate initial image offset }
    DirStart := ImageData;
    Offset := sizeof (Hdr) + (Hdr.NumIcons * sizeof (TIconDirEntry));
    { Write all the TIconDirEntry entries to disk }
    for Idx := 0 to Hdr.NumIcons - 1 do
    begin
        Move (ImageData^, GroupDir, sizeof (GroupDir));
        { Build a TIconDirEntry for this icon }
        Dir.Width := GroupDir.Width;
        Dir.Height := GroupDir.Height;
        Dir.Colors := GroupDir.Colors;
        Dir.Reserved := 0;
        Dir.dwReserved := 0;
        Dir.dwBytesInRes := GroupDir.ImageSize;
        Dir.dwImageOffset := Offset;
        fs.Write (Dir, sizeof (Dir));
        Inc (ImageData, sizeof (GroupDir));
        Inc (Offset, GroupDir.ImageSize);
    end;

    { Now loop again, writing the image data for each icon }
    ImageData := DirStart;
    for Idx := 0 to Hdr.NumIcons - 1 do
    begin
        Move (ImageData^, GroupDir, sizeof (GroupDir));
        WriteIndividualIcon (fs, GroupDir.IconID, GroupDir.ImageSize);
        Inc (ImageData, sizeof (GroupDir));
    end;
end;

procedure TForm1.WriteSingleIconFile (fs: TFileStream; ImageData: PByte; Len: Word);
var
    ImageSize: LongInt;
    bm: TBitmapInfoHeader;
    Hdr: TIconHeader;
    Dir: TIconDirEntry;
begin
    bm := PBitmapInfoHeader (ImageData)^;
    ImageSize := GetIconImageSize (ImageData);

    { Firstly, write file header }
    with Hdr do
    begin
        AlwaysZero := 0;
        Hdr.AlwaysOne := 1;
        Hdr.NumIcons := 1;
        fs.Write (Hdr, sizeof (Hdr));
    end;

    { Next, write icon directory entry }
    with Dir do
    begin
        Width := bm.biWidth;
        Height := bm.biHeight div 2;
        Colors := 1 shl bm.biBitCount;
        Reserved := 0;
        dwReserved := 0;
        dwBytesInRes := ImageSize;
        dwImageOffset := sizeof (Hdr) + sizeof (Dir);
        fs.Write (Dir, sizeof (Dir));
    end;

    { Finally, write the data itself }
    fs.Write (ImageData^, ImageSize);
end;

procedure TForm1.WriteResourceFile (const ResName, TypName: String; Info: TResInfo; ResData: Pointer);
var
    fName: String;
    Ext: String [5];
    NumIcons: Integer;
    fs: TFileStream;
begin
    fName := ExtractFilePath (Application.ExeName) + ResName;
    if (TypName = 'ICON') or (TypName = 'GROUPICON') then Ext := '.ICO'
    else if (TypName = 'BITMAP') then Ext := '.BMP' else Ext := '.BIN';
    fName := fName + Ext;

    fs := TFileStream.Create (fName, fmCreate);
    try
        if TypName = 'ICON' then
           WriteSingleIconFile (fs, ResData, Info.rLength)
        else if TypName = 'GROUPICON' then
           WriteMultiIconFile (fs, ResData, Info.rLength)
        else if TypName = 'BITMAP' then
           WriteBitmapFile (fs, ResData, Info.rLength)
        else
           fs.Write (ResData^, Info.rLength);

        MessageDlg ('Resource has been written to ' + fName, mtInformation, [mbok], 0);
    finally
        fs.Free;
    end;
end;

procedure TForm1.ResListDblClick(Sender: TObject);
var
    Info: TResInfo;
    ResData: Pointer;
    ResIndex: Integer;
    fs: TFileStream;
    TypName, ResName: String;
begin
    with rf do
    begin
        ResIndex := ResList.ItemIndex;
        TypName := Types.Items [Types.ItemIndex];
        ResName := ResList.Items [ResIndex];

        if MessageDlg (Format ('Extract resource ''%s'' (type = ''%s'') ?', [ResName, TypName]),
                       mtConfirmation, [mbYes, mbNo], 0) = idYes then
        begin
            GetResourceInfo (TypName, ResIndex, Info);
            GetMem (ResData, Info.rLength);
            try
                fs := TFileStream.Create (OpenDialog.FileName, fmOpenRead);
                try
                    fs.Position := Info.rOffset;
                    fs.Read (ResData^, Info.rLength);
                finally
                    fs.Free;
                end;

                WriteResourceFile (ResName, TypName, Info, ResData);
            finally
                FreeMem (ResData, Info.rLength);
            end;
        end;
    end;
end;

end.
