unit ExpReg;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  DsgnIntf, TypInfo, Dialogs, Forms, ExpComp, ExpEditr;

type
  { TOperandProperty definition }

  TOperandProperty = class(TFloatProperty)
  public
    destructor Destroy; override;
    function   GetName: string; override;
    function   GetValue: string; override;
  end;

  { TExpressionProperty definition  }

  TExpressionProperty = class(TStringProperty)
  protected
    procedure  ProcessOperand(Components: TStringList; Operand: TOperand);
  public
    function   GetAttributes: TPropertyAttributes; override;
    procedure  GetProperties(Proc: TGetPropEditProc); override;
  end;

  { TResultProperty definition  }

  TResultProperty = class(TFloatProperty)
  public
    function   GetAttributes: TPropertyAttributes; override;
  end;

  { TROEnumProperty definition  }

  TROEnumProperty = class(TEnumProperty)
  public
    function   GetAttributes: TPropertyAttributes; override;
    procedure  SetValue(const Value: string); override;
  end;

procedure Register;

var
  bSubPropListExpanded: Boolean;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TExpression]);

  RegisterPropertyEditor(TypeInfo(Extended),
                         TOperand,
                         '',
                         TOperandProperty);

  RegisterPropertyEditor(TypeInfo(string),
                         TExpression,
                         'Expression',
                         TExpressionProperty);

  ExpEditr.Register;    { NamedExpressions property editor }

  RegisterPropertyEditor(TypeInfo(Extended),
                         TExpression,
                         'Result',
                         TResultProperty);

  RegisterPropertyEditor(TypeInfo(Boolean),
                         TExpression,
                         'ValidExpression',
                         TROEnumProperty);
end;

{  TOperandProperty declarations  }

destructor TOperandProperty.Destroy;
begin
  if (bSubPropListExpanded) then
      begin
        bSubPropListExpanded := False;

        Modified;
      end;

  inherited Destroy;
end;

function TOperandProperty.GetName: string;
begin
  Result := TOperand(GetComponent(0)).Name;
end;

function TOperandProperty.GetValue: string;
begin
  Result := inherited GetValue;

  if (Result = '-999999999') then
      Result := '';
end;

{  TExpressionProperty declarations  }

function TExpressionProperty.GetAttributes: TPropertyAttributes;
begin
  Result := inherited GetAttributes + [paSubProperties];

  if bSubPropListExpanded then
     Result := Result + [paReadOnly];
end;

procedure TExpressionProperty.ProcessOperand(Components: TStringList; Operand: TOperand);
var
  CompIdx: Integer;
begin
 {  If Operand is not a System-Generated operand (Sys_Gen_Sym_), }
 {  then generate an editor for its "Value" Property.            }

  if (Operand = nil) or
     (Pos('SYS_GEN_SYM_', UpperCase(Operand.Name)) <> 0) then
      exit;

  CompIdx := Components.IndexOf(UpperCase(Operand.Name));

  if (CompIdx = -1) then
      begin
        Components.AddObject(UpperCase(Operand.Name), TComponentList.Create);

        CompIdx := Components.Count - 1;
      end;

  TComponentList(Components.Objects[CompIdx]).Add(Operand);
end;

procedure TExpressionProperty.GetProperties(Proc: TGetPropEditProc);
var
  OpIdx:       Word;
  SelectIdx:   Word;
  CompIdx:     Word;
  OpCount:     Word;
  Operand:     TOperand;
  Expression:  TExpression;
  Components:  TStringList;
begin
  Components := TStringList.Create;

  { For each selected Expression (TExpression), extract all     }
  { Operands within Expression and generate an appropriate      }
  { Property Editor.                                            }

  try
    for SelectIdx := 0 to PropCount - 1 do
      begin
        Expression := TExpression(GetComponent(SelectIdx));

        OpCount := Expression.OperandCount;

        if (OpCount = 0) then
            continue;

        for OpIdx := 0 to OpCount - 1 do
            ProcessOperand(Components, Expression.GetOperandAt(OpIdx));
      end;

    if (Components.Count = 0) then
        begin
          Components.Free;
          Components := nil;
          exit;
        end;

    Components.Sorted := True;

    for CompIdx := 0 to Components.Count - 1 do
      begin
        GetComponentProperties(TComponentList(Components.Objects[CompIdx]),
                               [tkFloat],
                               Designer,
                               Proc);

        TComponentList(Components.Objects[CompIdx]).Free;
      end;

    bSubPropListExpanded := True;

    Modified;
  finally
    Components.Free;
    Components := nil;
  end;

end;

{  TResultProperty declarations  }

function TResultProperty.GetAttributes: TPropertyAttributes;
begin
  Result := (inherited GetAttributes - [paMultiSelect]) + [paReadOnly];
end;

{  TROEnumProperty declarations  }

function TROEnumProperty.GetAttributes: TPropertyAttributes;
begin
  Result := (inherited GetAttributes - [paValueList]) + [paReadOnly];
end;

procedure TROEnumProperty.SetValue(const Value: string);
begin
  { paReadOnly keeps data from being keyed.  However, a double-click
    rotates through availale values.  Overriding this method ensures
    that the property IS NOT EDITABLE.                                }
end;

initialization

  bSubPropListExpanded := False;

end.
