{TJPSplitter Component Version 1.1
 =================================
Copyright 1996 John Pullen.
This component is completely free to use for personal or professional use.
The component is supplied with no guarantees on performance or stabilibty
and must be used at your own risk.

If you make enhancements or changes to this component, or have any ideas on how
to improve it then please could you let me know.
Enjoy.

John Pullen
zeus@dircon.co.uk
}

unit JPSplitr;

interface

uses
{$IFDEF WIN32}
    Windows, Messages, SysUtils, Classes, Graphics, Controls, ExtCtrls, Forms;
{$ELSE}
    WinProcs, WinTypes, Messages, SysUtils, Classes, Graphics, Controls, ExtCtrls, Forms;
{$ENDIF}


const
     DefaultSplitterSize = 4;

type
  TOrientation = (soVertical, soHorizontal);

  {TJPSplitter Class}
  TJPSplitter = class(TCustomPanel)
  private
    { Private declarations }
    {property variables}
    FControl1: TWinControl;
    FControl2: TWinControl;
    FLeftTopMargin: Integer;
    FRightBottomMargin: Integer;
    FOrientation: TOrientation;
    FOnBeginSplit: TNotifyEvent;
    FonEndSplit: TNotifyEvent;
    {General Variables}
    FAlignRect: TRect;
    ParentWidth, ParentHeight: Integer;
    SplitX, SaveX: Integer;
    SplitY, SaveY: Integer;
    IsSplitting: Boolean;
    procedure SetControl1(Value: TWinControl);
    procedure SetControl2(Value: TWinControl);
    procedure SetOrientation(Value: TOrientation);
    procedure AlignControls;
  protected
    { Protected declarations }
    procedure Paint; override;
    procedure Loaded; override;
    procedure SetParent(AParent: TWinControl); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure SetAlignRect;
    procedure DrawVerticalSplitter(X: Integer);
    procedure DrawHorizontalSplitter(Y: Integer);
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    function  CheckLimits(Value: Integer): Integer;
 public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure Center;
  published
    { Published declarations }
    property Orientation: TOrientation read FOrientation write SetOrientation;
    property LeftTopControl: TWinControl read FControl1 write SetControl1;
    property RightBottomControl: TWinControl read FControl2 write SetControl2;
    property LeftTopMargin: Integer read FLeftTopMargin write FLeftTopMargin;
    property RightBottomMargin: Integer read FRightBottomMargin write FRightBottomMargin;
    property BevelInner;
    property BevelOuter;
    property Color;
    property BorderStyle;
    property OnBeginSplit:  TNotifyEvent read FOnBeginSplit write FOnBeginSplit;
    property OnEndSplit: TNotifyEvent read FOnEndSplit write FOnEndSplit;
  end;

procedure Register;

implementation

{TJPSplitter Implementation}
constructor TJPSplitter.Create(AOwner: TComponent);
begin
     inherited Create(AOwner);
     Width := DefaultSplitterSize;
     BevelInner := bvNone;
     BevelOuter := bvNone;
     Cursor := crHSplit;
     Orientation := soVertical;
     IsSplitting := False;
     Caption := '';
     FLeftTopMargin := 20;
     FRightBottomMargin := 20;
end;

procedure TJPSplitter.SetParent(AParent: TWinControl);
begin
     inherited SetParent(AParent);
     if (Parent <> nil) and (csDesigning in ComponentState) and
        not (csLoading in ComponentState) then
     begin
          Caption := '';
          SetAlignRect;
          SetBounds(FAlignRect.Left + ((ParentWidth div 2) - (Width div 2)),
          FAlignRect.Top, Width, ParentHeight);
          Top := 0;
          Height := ParentHeight;
     end;
end;

{Draw Outline if in design mode just so the Splitter can be seen}
procedure TJPSplitter.Paint;
begin
     inherited Paint;
     if (BevelInner = bvNone) and (BevelOuter = bvNone) and
        (csDesigning in ComponentState) then
     begin
          with Canvas do
          begin
               Pen.Color := clBlack;
               Pen.Style := psDot;
               MoveTo(0, 0);
               LineTo(Width - 1, 0);
               LineTo(Width - 1, Height - 1);
               LineTo(0, Height - 1);
               LineTo(0, 0);
               Pen.Style := psSolid;
          end;
     end;
end;

procedure TJPSplitter.Loaded;
begin
     inherited Loaded;
     Caption := '';
     {This is required as the Align property is not published and not saved}
     if FOrientation = soVertical then
        Align := alLeft
     else
         Align := alTop;
end;

procedure TJPSplitter.SetControl1(Value: TWinControl);
begin
     if (Value <> FControl1) and (Value <> FControl2) and (Value <> Self) then
     begin
          {If Control already set then switch off align}
          if FControl1 <> nil then
             FControl1.Align := alNone;
          FControl1 := Value;
          {$IFDEF WIN32}
            FControl1.FreeNotification(Self);
          {$ENDIF}
          if (csDesigning in ComponentState) and not
             (csLoading in ComponentState) then
             AlignControls;
     end;
end;

procedure TJPSplitter.SetControl2(Value: TWinControl);
begin
     if (Value <> FControl2) and (Value <> FControl1) and (Value <> Self) then
     begin
          {If Control already set then switch off align}
          if FControl2 <> nil then
             FControl2.Align := alNone;
          FControl2 := Value;
          {$IFDEF WIN32}
              FControl2.FreeNotification(Self);
          {$ENDIF}
          if (csDesigning in ComponentState) and not
             (csLoading in ComponentState) then
             AlignControls;
     end;
end;

procedure TJPSplitter.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  if (Operation = opRemove) and not (csDestroying in ComponentState) then
  begin
       {Conponent 1 destroying?}
       if FControl1 = AComponent then
       begin
          FControl1 := nil;
          if FControl2 <> nil then
          begin
               FControl2.Align := alNone;
               Align := alNone;
          end;
       end;

       {Conponent 2 destroying?}
       if FControl2 = AComponent then
       begin
          FControl2 := nil;
          if FControl1 <> nil then
          begin
               FControl1.Align := alNone;
               Align := alNone;
          end;
       end;
  end;
end;

{This function is only called at design time to initially
position the splitter bar.}
procedure TJPSplitter.SetAlignRect;
var
   n: Integer;
begin
     {Default to entire ClientRect}
     FAlignRect := Parent.ClientRect;
     {Reduce depending on aligned controls}
     for n := 0 to Parent.ControlCount - 1 do
     begin
          if (Parent.Controls[n] <> self) and (Parent.Controls[n].Align <> alNone) and
             (Parent.Controls[n] <> FControl1) and (Parent.Controls[n] <> FControl2) then
          begin
               with Parent.Controls[n] do
               begin
                    case Align of
                         alLeft: if Left + Width > FAlignRect.Left then
                                    FAlignRect.Left := Left + Width;
                         alRight: if Left < FAlignRect.Right then
                                     FAlignRect.Right := Left - 1;
                         alTop: if Top + Height > FAlignRect.Top then
                                   FAlignRect.Top := Top + Height;
                         alBottom: if Top < FAlignRect.Bottom then
                                      FAlignRect.Bottom := Top - 1;
                    end;
               end;
          end;
     end;
     ParentWidth := FAlignRect.Right - FAlignRect.Left;
     ParentHeight := FAlignRect.Bottom - FAlignRect.Top;
end;

{Orientation can only be set at design time}
procedure TJPSplitter.SetOrientation(Value: TOrientation);
begin
     if Value <> FOrientation then
     begin
          FOrientation := Value;
          if FOrientation = soVertical then
          begin
               Cursor := crHSplit;
          end
          else
          begin
               Cursor := crVSplit;
          end;

          if (csDesigning in ComponentState) and not
             (csLoading in ComponentState) then
             AlignControls;
     end;
end;

procedure TJPSplitter.AlignControls;
begin
     if (FControl1 <> nil) and (FControl2 <> nil) then
     begin
          FControl1.Align := alNone;
          FControl2.Align := alNone;
          Align := alNone;
          SetAlignRect;
          if FOrientation = soVertical then
          begin
               Width := DefaultSplitterSize;
               FControl1.Align := alLeft;
               FControl1.Width :=  (ParentWidth div 2) - (Width div 2);
               Left := FControl1.Width;
               Align := alLeft;
               FControl2.Align := alClient;
          end
          else
          begin
               Height := DefaultSplitterSize;
               FControl1.Align := alTop;
               FControl1.Height :=  (ParentHeight div 2) - (Height div 2);
               Top := FControl1.Height;
               Align := alTop;
               FControl2.Align := alClient;
          end;
     end
     else
     begin
          if FOrientation = soVertical then
          begin
               Width := DefaultSplitterSize;
               SetBounds(FAlignRect.Left + ((ParentWidth div 2) - (Width div 2)),
                   FAlignRect.Top, Width, ParentHeight)
          end
          else
          begin
               Height := DefaultSplitterSize;
               SetBounds(FAlignRect.Left, FAlignRect.Top + ((ParentHeight div 2) - (Height div 2)),
                   ParentWidth, Height);
          end;
     end;
end;

procedure TJPSplitter.DrawVerticalSplitter(X: Integer);
var
   DC: HDC;
   r: TRect;
   OldPen: HPen;
   n: Integer;
   p: TPoint;
begin
     p := GetClientOrigin;
     DC := GetDC(0); {Get Screen DC }
     Canvas.Pen.Color := clBlack;
     r.Left := p.x + (x - Left);
     r.Top := p.y + 2;
     r.Right := r.Left + Width;
     r.Bottom := r.Top + Height - 4;
     OldPen := SelectObject(DC, Canvas.Pen.Handle);
     SetROP2(DC, R2_NOTXORPEN);
     for n := r.Left to r.Right - 1 do
     begin
          MoveToEx(DC, n, r.Top, nil);
          LineTo(DC, n, r.Bottom);
     end;
     SelectObject(DC, OldPen);
     ReleaseDC(0, DC);
end;

procedure TJPSplitter.DrawHorizontalSplitter(Y: Integer);
var
   DC: HDC;
   r: TRect;
   OldPen: HPen;
   n: Integer;
   p: TPoint;
begin
     p := GetClientOrigin;
     DC := GetDC(0); {Get Screen DC}
     Canvas.Pen.Color := clBlack;
     r.Left := p.x + 2;
     r.Top := p.y + (Y - Top);
     r.Right := r.Left + Width - 4;
     r.Bottom := r.Top + Height;
     OldPen := SelectObject(DC, Canvas.Pen.Handle);
     SetROP2(DC, R2_NOTXORPEN);
     for n := r.Top to r.Bottom - 1 do
     begin
          MoveToEx(DC, r.Left, n, nil);
          LineTo(DC, r.Right, n);
     end;
     SelectObject(DC, OldPen);
     ReleaseDC(0, DC);
end;

procedure TJPSplitter.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
     inherited MouseDown(Button, Shift, X, Y);
     if Assigned(FOnBeginSplit) then
        FOnBeginSplit(Self);
     SetAlignRect;
     IsSplitting := True;
     if FOrientation = soVertical then
     begin
          SplitX := X;
          SaveX := Left - SplitX + X;
          DrawVerticalSplitter(SaveX);
     end
     else
     begin
          SplitY := Y;
          SaveY := Top - SplitY + Y;
          DrawHorizontalSplitter(SaveY);
     end;
end;

procedure TJPSplitter.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
   nx, ny: Integer;
begin
     inherited MouseUp(Button, Shift, X, Y);
     if not IsSplitting then exit;
     if FOrientation = soVertical then
     begin
          DrawVerticalSplitter(SaveX);
          nx := Left - SplitX + X;
          nx := CheckLimits(nx);
          FControl1.Width := nx - FControl1.Left;
     end
     else
     begin
          DrawHorizontalSplitter(SaveY);
          ny := Top - SplitY + Y;
          ny := CheckLimits(ny);
          FControl1.Height := ny - FControl1.Top;
     end;
     if Assigned(FOnEndSplit) then
        FOnEndSplit(Self);
     IsSplitting := False;
end;

procedure TJPSplitter.MouseMove(Shift: TShiftState; X, Y: Integer);
var
   nx, ny: Integer;
begin
     inherited MouseMove(Shift, X, Y);
     if not IsSplitting then exit;
     if FOrientation = soVertical then
     begin
          nx := Left - SplitX + X;
          nx := CheckLimits(nx);
          DrawVerticalSplitter(SaveX);
          DrawVerticalSplitter(nx);
          SaveX := nx;
     end
     else
     begin
          ny := Top - SplitY + Y;
          ny := CheckLimits(ny);
          DrawHorizontalSplitter(SaveY);
          DrawHorizontalSplitter(ny);
          SaveY := ny;
     end;
end;

procedure TJPSPlitter.Center;
begin
     SetAlignRect;
     if FControl1 <> nil then
     begin
          if FOrientation = soVertical then
             FControl1.Width := (ParentWidth div 2) - (Width div 2)
          else
             FControl1.Height := (ParentHeight div 2) - (Height div 2);
     end;
end;

function TJPSplitter.CheckLimits(Value: Integer): Integer;
begin
     Result := Value; {Default to Good Value}
     if FOrientation = soVertical then
     begin
          if Value <= FAlignRect.Left + FLeftTopMargin then
             Result := FAlignRect.Left + FLeftTopMargin
          else
              if Value > (FControl2.Left + FControl2.Width) - Width - FRightBottomMargin then
                 Result := (FControl2.Left + FControl2.Width) - Width - FRightBottomMargin;
     end
     else
     begin
          if Value < FAlignRect.Top + FLeftTopMargin then
             Result := FAlignRect.Top + FLeftTopMargin
          else
              if Value > (FControl2.Top + FControl2.Height) - Height - FRightBottomMargin then
                 Result := (FControl2.Top + FControl2.Height) - Height - FRightBottomMargin;
     end;
end;

procedure Register;
begin
{$IFDEF WIN32}
   RegisterComponents('Win95', [TJPSplitter]);
{$ELSE}
   RegisterComponents('Samples', [TJPSplitter]);
{$ENDIF}

end;

end.
