unit Mainform;
interface
uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  StdCtrls, Forms, DBCtrls, DB, DBTables, Mask, ExtCtrls, Tabs, Buttons,
  Menus;

Const
  DataPath: String = '.';

Type
  TFormMain = class(TForm)
    ScrollBox: TScrollBox;
    LabelTitle: TLabel;
    EditTitle: TDBEdit;
    LabelType: TLabel;
    EditArticleType: TDBEdit;
    LabelIssue: TLabel;
    EditIssue: TDBEdit;
    LabelMonthYear: TLabel;
    EditPageNumber: TDBEdit;
    LabelSummary: TLabel;
    DBNavigator: TDBNavigator;
    ToolBar: TPanel;
    DataSourceArticle: TDataSource;
    PanelArticle: TPanel;
    EditIssueMonthYear: TDBEdit;
    TabSetAuthor: TTabSet;
    BevelAuthor: TBevel;
    DataSourceAuthor: TDataSource;
    ImageAuthor: TDBImage;
    EditAuthorName: TDBEdit;
    EditAuthorBioAndEmailAddress: TDBMemo;
    EditSummary: TDBMemo;
    BtnClose: TSpeedButton;
    LabelAuthor: TLabel;
    BtnAuthorName: TSpeedButton;
    QAuthor: TQuery;
    QAuthorName: TStringField;
    QAuthorPhoto: TGraphicField;
    QAuthorAuthorID: TIntegerField;
    QAuthorBioEmailaddress: TMemoField;
    QArticle: TQuery;
    QArticleArticleID: TIntegerField;
    QArticleArticleType: TStringField;
    QArticleTitle: TStringField;
    QArticleAuthor1: TIntegerField;
    QArticleAuthor2: TIntegerField;
    QArticleAuthor3: TIntegerField;
    QArticleIssue: TIntegerField;
    QArticlePageNumber: TIntegerField;
    QArticleSummary: TMemoField;
    QArticleIssueMonthYear: TStringField;
    BtnQuery: TSpeedButton;
    PopupMenu1: TPopupMenu;
    About1: TMenuItem;
    TheDelphiMagazine1: TMenuItem;
    N1: TMenuItem;
    DrBob1: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure ArticleCalcFields(DataSet: TDataset);
    procedure TabSetAuthorClick(Sender: TObject);
    procedure DBNavigatorClick(Sender: TObject; Button: TNavigateBtn);
    procedure BtnCloseClick(Sender: TObject);
    procedure BtnAuthorNameClick(Sender: TObject);
    procedure BtnQueryClick(Sender: TObject);
    procedure About1Click(Sender: TObject);
  end;

var
  FormMain: TFormMain;

implementation
uses Dialogs, DbiTypes, DbiProcs, DBConsts, Query, SplashFm;
{$R *.DFM}

  function CurrentRecordNumber(DataSet: TDataSet): LongInt;
  { Gives the current recordnumber as result for dBase/Paradox tables only }
  var CursorProps: CurProps;
      RecordProps: RECProps;
  begin
    Result := 0;
    with DataSet do
    try
      if State = dsInactive then DBError(SDataSetClosed);
      Check(DbiGetCursorProps(Handle, Cursorprops));
      UpdateCursorPos;
      Check(DbiGetRecord(Handle, dbiNOLOCK, nil, @RecordProps));
      case CursorProps.iSeqNums of
        0: Result := RecordProps.iPhyRecNum; {dBase}
        1: Result := RecordProps.iSeqNum   {Paradox}
      end
    except
      { skip errors - return 0 }
    end
  end;

procedure TFormMain.FormCreate(Sender: TObject);
begin
  QAuthor.DatabaseName := DataPath;
  QArticle.DatabaseName := DataPath;
  try
    QArticle.Open;
    DBNavigatorClick(Self,nbRefresh)
  except
    on E: Exception do
    begin
      MessageDlg(E.ClassName+': '+E.Message,mtError,[mbOK],0);
      Position := poDesigned;
      Top := -1000; { move form out of the way to prevent flicker }
      Application.Terminate
    end
  end
end;

procedure TFormMain.ArticleCalcFields(DataSet: TDataset);
begin
  case QArticleIssue.Value of
    1: QArticleIssueMonthYear.Value := 'April 1995';
    2: QArticleIssueMonthYear.Value := 'July 1995';
    3: QArticleIssueMonthYear.Value := 'September 1995';
    4: QArticleIssueMonthYear.Value := 'November 1995';
  else
    if QArticleIssue.Value >= 5 then { monthly from now on }
      QArticleIssueMonthYear.Value := Format('%s %d',[
            LongMonthNames[((QArticleIssue.Value-5) mod 12)+1],
            1996 + (Pred(QArticleIssue.Value-4) div 12)])
    else
      QArticleIssueMonthYear.Value := 'unknown'
  end
end;

procedure TFormMain.BtnAuthorNameClick(Sender: TObject);
begin
  if BtnAuthorName.Caption = 'A' then
  begin
    BtnAuthorName.Caption := '1';
    BtnAuthorName.Hint := 'Author Name (slower)'
  end
  else
  begin
    BtnAuthorName.Caption := 'A';
    BtnAuthorName.Hint := 'Number of Author (faster)'
  end;
  DBNavigatorClick(Self,nbRefresh)
end;

procedure TFormMain.DBNavigatorClick(Sender: TObject; Button: TNavigateBtn);
begin
  ToolBar.Caption := Format('Article %d/%d',
                            [CurrentRecordNumber(QArticle),QArticle.RecordCount]);
  TabSetAuthor.Tabs.Clear;
  if BtnAuthorName.Caption = '1' then
  begin
    if QArticleAuthor1.Value > 0 then TabSetAuthor.Tabs.Add('first author');
    if QArticleAuthor2.Value > 0 then TabSetAuthor.Tabs.Add('second author');
    if QArticleAuthor3.Value > 0 then TabSetAuthor.Tabs.Add('third author')
  end
  else { names of authors }
  begin
    Screen.Cursor := crSQLWait;
    if QArticleAuthor1.Value > 0 then
    begin
      QAuthor.Active := False;
      QAuthor.ParamByName('AUTHOR').AsInteger := QArticleAuthor1.Value;
      try
        QAuthor.Active := True;
        TabSetAuthor.Tabs.Add(QAuthorName.Value)
      except
        TabSetAuthor.Tabs.Add('first author')
      end
    end;
    if QArticleAuthor2.Value > 0 then
    begin
      QAuthor.Active := False;
      QAuthor.ParamByName('AUTHOR').AsInteger := QArticleAuthor2.Value;
      try
        QAuthor.Active := True;
        TabSetAuthor.Tabs.Add(QAuthorName.Value)
      except
        TabSetAuthor.Tabs.Add('second author')
      end
    end;
    if QArticleAuthor3.Value > 0 then
    begin
      QAuthor.Active := False;
      QAuthor.ParamByName('AUTHOR').AsInteger := QArticleAuthor3.Value;
      try
        QAuthor.Active := True;
        TabSetAuthor.Tabs.Add(QAuthorName.Value)
      except
        TabSetAuthor.Tabs.Add('third author')
      end
    end;
    Screen.Cursor := crDefault
  end;
  if TabSetAuthor.Tabs.Count > 0 then
    TabSetAuthor.TabIndex := 0
end;

procedure TFormMain.TabSetAuthorClick(Sender: TObject);
begin
  QAuthor.Active := False { reset last query };
  case (Sender AS TTabSet).TabIndex of
    0: QAuthor.ParamByName('AUTHOR').AsInteger := QArticleAuthor1.Value;
    1: QAuthor.ParamByName('AUTHOR').AsInteger := QArticleAuthor2.Value;
    2: QAuthor.ParamByName('AUTHOR').AsInteger := QArticleAuthor3.Value
  end;
  QAuthor.Active := True { execute! }
end;

procedure TFormMain.BtnQueryClick(Sender: TObject);
begin
  if QueryDlg.ShowModal = mrOk then
  begin
    QArticle.Active := False;
    QArticle.SQL := QueryDlg.Memo1.Lines;
    try
      QArticle.Active := True;
      DBNavigatorClick(Self,nbRefresh)
    except
      on E: Exception do
      begin
        ShowMessage(E.ClassName+': '+E.Message);
        QArticle.Active := False;
        Screen.Cursor := crDefault
      end
    end
  end
end;

procedure TFormMain.About1Click(Sender: TObject);
begin
  with TSplash.Create(nil) do
  begin
    if Sender = DrBob1 then
    begin
      Height := 300;
      Label2.Visible := True;
      Label5.Visible := True;
    end
    else
    begin
      if Sender = TheDelphiMagazine1 then
      begin
        Height := 260;
        Label3.Top := Label2.Top;
        Label4.Top := Label5.Top + 1
      end
      else Height := 208
    end;
    Show { will Free itself }
  end
end;

procedure TFormMain.BtnCloseClick(Sender: TObject);
begin
  Close
end;

initialization
  DataPath := ExtractFilePath(ParamStr(0));
end.
