unit MLBData;

interface

uses
  SysUtils, Classes;

type

	TMLBDivision = class;

	TMLBTeam = class(TObject)
  private
  	FDivision	 : TMLBDivision;
		FBitmap		 : Integer;
  	FName 		 : String;
    FWon			 : Integer;
    FLost			 : Integer;
    FGamesBack : Single;
    FHomeWon	 : Integer;
    FHomeLost  : Integer;
    FRoadWon	 : Integer;
    FRoadLost	 : Integer;
    FStreak		 : Integer;
  protected
  	function GetPercent: Single;
  public
  	property Division: TMLBDivision read FDivision write FDivision;
		property BitmapIndex: Integer read FBitmap write FBitmap;
  	property Name: String read FName write FName;
    property Won: Integer read FWon write FWon;
    property Lost: Integer read FLost write FLost;
    property GamesBack: Single read FGamesBack write FGamesBack;
    property Percent: Single read GetPercent;
    property HomeWon: Integer read FHomeWon write FHomeWon;
    property HomeLost: Integer read FHomeLost write FHomeLost;
    property RoadWon: Integer read FRoadWon write FRoadWon;
    property RoadLost: Integer read FRoadLost write FRoadLost;
    property Streak: Integer read FStreak write FStreak;
  end;

  TMLBSortOrder = (soTeam,soWon,soLost,soPercent,soGamesBack);

  TMLBDivision = class(TObject)
  private
  	FName			 : String;
  	FItems		 : TList;
    FSortOrder : TMLBSortOrder;
  protected
    function  GetCount: Integer;
  	function	GetTeam(Index: Integer): TMLBTeam;
  public
  	constructor Create;
    destructor Destroy; override;
    procedure Add(Team: TMLBTeam);
  	procedure Sort(SortOrder: TMLBSortOrder; SortAscending: Boolean);
    property Count: Integer read GetCount;
  	property Name: String read FName write FName;
    property Items[Index: Integer]: TMLBTeam read GetTeam; default;
    property SortOrder: TMLBSortOrder read FSortOrder write FSortOrder;
  end;

  function IntegerCompare(Int1, Int2: Integer): Integer;
  function ExtendedCompare(Ext1, Ext2: Extended): Integer;
  function CompareTeam(Item1, Item2: Pointer): Integer;
  function CompareWon(Item1, Item2: Pointer): Integer;
  function CompareLost(Item1, Item2: Pointer): Integer;
  function ComparePercent(Item1, Item2: Pointer): Integer;
  function CompareGamesBack(Item1, Item2: Pointer): Integer;

var
	ALEast : TMLBDivision;
  ALCentral : TMLBDivision;
  ALWest : TMLBDivision;
  American : TMLBDivision;

implementation

var
	Team: TMLBTeam;
  Ascending: Boolean;

{-}
{ TMLBTeam Implementation }
{-}

function TMLBTeam.GetPercent: Single;
begin
	Result := Won / (Won + Lost);
end;

{-}
{ Comparison functions }
{-}

function IntegerCompare(Int1, Int2: Integer): Integer;
begin
	if (Int1 = Int2) then
  	Result := 0
  else
  	if (Int1 > Int2) then
    	Result := 1
    else
    	Result := -1;
  if not Ascending then Result := -Result;
end;

function ExtendedCompare(Ext1, Ext2: Extended): Integer;
begin
	if (Ext1 = Ext2) then
  	Result := 0
  else
  	if (Ext1 > Ext2) then
    	Result := 1
    else
    	Result := -1;
  if not Ascending then Result := -Result;
end;

function CompareTeam(Item1, Item2: Pointer): Integer;
begin
	Result := AnsiCompareStr(TMLBTeam(Item1).Name,TMLBTeam(Item2).Name);
  if not Ascending then Result := -Result;
end;

function CompareWon(Item1, Item2: Pointer): Integer;
begin
	Result := IntegerCompare(TMLBTeam(Item1).Won,TMLBTeam(Item2).Won);
end;

function CompareLost(Item1, Item2: Pointer): Integer;
begin
	Result := IntegerCompare(TMLBTeam(Item1).Lost,TMLBTeam(Item2).Lost);
end;

function ComparePercent(Item1, Item2: Pointer): Integer;
begin
	Result := ExtendedCompare(TMLBTeam(Item1).Percent,TMLBTeam(Item2).Percent);
end;

function CompareGamesBack(Item1, Item2: Pointer): Integer;
begin
	Result := ExtendedCompare(TMLBTeam(Item1).GamesBack,TMLBTeam(Item2).GamesBack);
end;

{-}
{ TMLBDivision Implementation }
{-}

constructor TMLBDivision.Create;
begin
	inherited Create;
  FItems := TList.Create;
	FSortOrder := soTeam;
end;

destructor TMLBDivision.Destroy;
var
	I : Integer;
begin
	for I := 0 to (FItems.Count - 1) do TObject(FItems[I]).Free;
  FItems.Free;
  inherited Destroy;
end;

function TMLBDivision.GetCount: Integer;
begin
	Result := FItems.Count;
end;

function TMLBDivision.GetTeam(Index: Integer): TMLBTeam;
begin
	Result := TMLBTeam(FItems.Items[Index]);
end;

procedure TMLBDivision.Add(Team: TMLBTeam);
begin
	FItems.Add(Team);
end;

procedure TMLBDivision.Sort(SortOrder: TMLBSortOrder; SortAscending: Boolean);
begin
	Ascending := SortAscending;
	case SortOrder of
  soTeam			: FItems.Sort(CompareTeam);
  soWon				: FItems.Sort(CompareWon);
  soLost			: FItems.Sort(CompareLost);
  soPercent		: FItems.Sort(ComparePercent);
  soGamesBack	: FItems.Sort(CompareGamesBack);
  end;
end;

initialization

	{ Create AL East Division }
  ALEast := TMLBDivision.Create;
  ALEast.Name := 'AL East';

  { Create AL East Teams }

  Team := TMLBTeam.Create;
  try
  	with Team do
    begin
    	BitmapIndex := 0;
  		Name := 'Baltimore Orioles';
    	Won := 7;
    	Lost := 1;
    	GamesBack := 0;
    	HomeWon := 5;
    	HomeLost := 0;
    	RoadWon := 2;
      RoadLost := 1;
      Streak := 3;
    end;
    ALEast.Add(Team)
  except
  	Team.Free;
  end;

  Team := TMLBTeam.Create;
  try
  	with Team do
    begin
    	BitmapIndex := 1;
  		Name := 'Toronto Blue Jays';
    	Won := 5;
    	Lost := 3;
    	GamesBack := 2;
    	HomeWon := 2;
    	HomeLost := 1;
    	RoadWon := 3;
      RoadLost := 2;
      Streak := 1;
    end;
    ALEast.Add(Team)
  except
  	Team.Free;
  end;

  Team := TMLBTeam.Create;
  try
  	with Team do
    begin
    	BitmapIndex := 2;
  		Name := 'New York Yankees';
    	Won := 4;
    	Lost := 3;
    	GamesBack := 2.5;
    	HomeWon := 2;
    	HomeLost := 0;
    	RoadWon := 2;
      RoadLost := 3;
      Streak := 2;
    end;
    ALEast.Add(Team)
  except
  	Team.Free;
  end;

  Team := TMLBTeam.Create;
  try
  	with Team do
    begin
    	BitmapIndex := 3;
  		Name := 'Detroit Tigers';
    	Won := 5;
    	Lost := 5;
    	GamesBack := 3;
    	HomeWon := 2;
    	HomeLost := 1;
    	RoadWon := 3;
      RoadLost := 4;
      Streak := -1;
    end;
    ALEast.Add(Team)
  except
  	Team.Free;
  end;

  Team := TMLBTeam.Create;
  try
  	with Team do
    begin
    	BitmapIndex := 4;
  		Name := 'Boston Red Sox';
    	Won := 2;
    	Lost := 6;
    	GamesBack := 5;
    	HomeWon := 1;
    	HomeLost := 1;
    	RoadWon := 1;
      RoadLost := 5;
      Streak := -1;
    end;
    ALEast.Add(Team)
  except
  	Team.Free;
  end;

	{ Create AL East Division }
  ALCentral := TMLBDivision.Create;
  ALCentral.Name := 'AL Central';

  { Create AL Central Teams }

  Team := TMLBTeam.Create;
  try
  	with Team do
    begin
    	BitmapIndex := 5;
  		Name := 'Minnesota Twins';
    	Won := 4;
    	Lost := 4;
    	GamesBack := 0;
    	HomeWon := 3;
    	HomeLost := 3;
    	RoadWon := 1;
      RoadLost := 1;
      Streak := 1;
    end;
    ALCentral.Add(Team)
  except
  	Team.Free;
  end;

  Team := TMLBTeam.Create;
  try
  	with Team do
    begin
    	BitmapIndex := 6;
  		Name := 'Milwaukee Brewers';
    	Won := 3;
    	Lost := 4;
    	GamesBack := 0.5;
    	HomeWon := 1;
    	HomeLost := 1;
    	RoadWon := 2;
      RoadLost := 3;
      Streak := -1;
    end;
    ALCentral.Add(Team)
  except
  	Team.Free;
  end;

  Team := TMLBTeam.Create;
  try
  	with Team do
    begin
    	BitmapIndex := 7;
  		Name := 'Chicago White Sox';
    	Won := 3;
    	Lost := 5;
    	GamesBack := 1;
    	HomeWon := 1;
    	HomeLost := 1;
    	RoadWon := 2;
      RoadLost := 4;
      Streak := 1;
    end;
    ALCentral.Add(Team)
  except
  	Team.Free;
  end;

  Team := TMLBTeam.Create;
  try
  	with Team do
    begin
    	BitmapIndex := 8;
  		Name := 'Cleveland Indians';
    	Won := 2;
    	Lost := 5;
    	GamesBack := 1.5;
    	HomeWon := 2;
    	HomeLost := 3;
    	RoadWon := 0;
      RoadLost := 2;
      Streak := -2;
    end;
    ALCentral.Add(Team)
  except
  	Team.Free;
  end;

  Team := TMLBTeam.Create;
  try
  	with Team do
    begin
    	BitmapIndex := 9;
  		Name := 'Kansas City Royals';
    	Won := 2;
    	Lost := 6;
    	GamesBack := 2;
    	HomeWon := 2;
    	HomeLost := 1;
    	RoadWon := 0;
      RoadLost := 5;
      Streak := -3;
    end;
    ALCentral.Add(Team)
  except
  	Team.Free;
  end;

	{ Create AL West Division }
  ALWest := TMLBDivision.Create;
  ALWest.Name := 'AL West';

  { Create AL West Teams }

  Team := TMLBTeam.Create;
  try
  	with Team do
    begin
    	BitmapIndex := 10;
  		Name := 'Texas Rangers';
    	Won := 7;
    	Lost := 1;
    	GamesBack := 0;
    	HomeWon := 6;
    	HomeLost := 0;
    	RoadWon := 1;
      RoadLost := 1;
      Streak := -1;
    end;
    ALWest.Add(Team)
  except
  	Team.Free;
  end;

  Team := TMLBTeam.Create;
  try
  	with Team do
    begin
    	BitmapIndex := 11;
  		Name := 'Seattle Mariners';
    	Won := 5;
    	Lost := 4;
    	GamesBack := 2.5;
    	HomeWon := 4;
    	HomeLost := 2;
    	RoadWon := 1;
      RoadLost := 2;
      Streak := 1;
    end;
    ALWest.Add(Team)
  except
  	Team.Free;
  end;

  Team := TMLBTeam.Create;
  try
  	with Team do
    begin
    	BitmapIndex := 12;
  		Name := 'California Angels';
    	Won := 4;
    	Lost := 4;
    	GamesBack := 3;
    	HomeWon := 3;
    	HomeLost := 2;
    	RoadWon := 1;
      RoadLost := 2;
      Streak := -1;
    end;
    ALWest.Add(Team)
  except
  	Team.Free;
  end;

  Team := TMLBTeam.Create;
  try
  	with Team do
    begin
    	BitmapIndex := 13;
  		Name := 'Oakland Athletics';
    	Won := 3;
    	Lost := 5;
    	GamesBack := 4;
    	HomeWon := 2;
    	HomeLost := 4;
    	RoadWon := 1;
      RoadLost := 1;
      Streak := 1;
    end;
    ALWest.Add(Team)
  except
  	Team.Free;
  end;


finalization
begin
	ALEast.Free;
  ALCentral.Free;
  ALWest.Free;
end;

end.