{******************************************************************}
{                                                                  }
{       Improved Delphi MessageDlg and InputDialog Functions       }
{                                                                  }
{       Copyright (c) 1996-1997 Steven J. Colagiovanni             }
{                                                                  }
{******************************************************************}

unit Msgdlg32;

interface

uses
	 WinTypes, WinProcs, Commdlg, SysUtils, Classes, Graphics, Consts, Controls,
		 StdCtrls, ExtCtrls, Forms;

type
	 TMsgBoxType = (mtWarning, mtError, mtInformation, mtConfirmation,
			mtCustom);
	 TMsgBoxBtn = (mbYes, mbAll, mbNo, mbOK, mbAbort, mbRetry,
                        mbIgnore, mbCancel, mbHelp);
	 TMsgBoxButtons = set of TMsgBoxBtn;
	 TDefaultBtn = (dfFirst, dfSecond, dfThird, dfFourth);      { Default Button }

	TMessageBoxStyle = (msScreenCentered, msFormCentered, msCustomPosition);

	TMessageDlg = class(TComponent)
	private
		FCaption: String;
		FMessageText: String;
		FMessageBoxType: TMsgBoxType;
		FButtons: TMsgBoxButtons;
		FDefaultButton: TDefaultBtn;
		FHelpContext: LongInt;
		FPositionLeft: Integer;
		FPositionTop: Integer;
		FCenterOnForm: TForm;
		FMessageBoxStyle: TMessageBoxStyle;
		procedure SetCaption(newValue: String);
		procedure SetMessageText(newValue: String);
		procedure SetMsgBoxType(newValue: TMsgBoxType);
		procedure SetButtons(newValue: TMsgBoxButtons);
		procedure SetDefaultButton(newValue: TDefaultBtn);
		procedure SetCenterOnForm(newValue: TForm);
		procedure SetMsgBoxStyle(newValue: TMessageBoxStyle);
	protected
	public
		constructor Create(AOwner: TComponent); override;
		destructor Destroy; override;
		function Execute: Word;
		property CenterOnForm: TForm read FCenterOnForm write SetCenterOnForm;
	published
		property Caption: String read FCaption write SetCaption;
		property MessageText: String read FMessageText write SetMessageText;
		property MsgDlgType: TMsgBoxType read FMessageBoxType
			write SetMsgBoxType;
		property Buttons: TMsgBoxButtons read FButtons write SetButtons;
		property DefaultButton: TDefaultBtn read FDefaultButton
			write SetDefaultButton;
		property HelpContext: LongInt read FHelpContext write FHelpContext
			default 0;
		property PositionLeft: Integer read FPositionLeft write FPositionLeft
			default -1;
		property PositionTop: Integer read FPositionTop write FPositionTop
			default -1;
		property MsgDlgStyle: TMessageBoxStyle read FMessageBoxStyle
			write SetMsgBoxStyle default msScreenCentered;
	end;

const
	 mbYesNo = [mbYes, mbNo];
	 mbYesNoCancel = [mbYes, mbNo, mbCancel];
	 mbOKCancel = [mbOK, mbCancel];
	 mbRetryCancel = [mbRetry, mbCancel];
	 mbAbortRetryIgnore = [mbAbort, mbRetry, mbIgnore];

        SMsgboxYes    = '&Yes';
        SMsgboxNo     = '&No';
        SMsgboxOK     = 'OK';
        SMsgboxAbort  = '&Abort';
        SMsgboxRetry  = '&Retry';
        SMsgboxIgnore = '&Ignore';
        SMsgboxAll    = 'Yes to &All';
        SMsgboxCancel = 'Cancel';
        SMsgboxHelp   = '&Help';

function MsgDlg(const AMsg: string; const ACaption: string;
	 AType: TMsgBoxType; AButtons: TMsgBoxButtons;
	 ADefaultButton: TDefaultBtn): Word;
function MsgDlgPos(const AMsg: string; const ACaption: string;
	 AType: TMsgBoxType; AButtons: TMsgBoxButtons;
	 ADefaultButton: TDefaultBtn; HelpCtx: Longint; X, Y: Integer): Word;

function FormCenteredMsgDlg(const AMsg: string;
	 const ACaption: string; AType: TMsgBoxType;
	 AButtons: TMsgBoxButtons; ADefaultButton: TDefaultBtn;
	 HelpCtx: Longint; AForm: TForm): Word;

procedure ShowMsg(const AMsg: string);
procedure ShowMsgPos(const AMsg: string; X, Y: Integer);

function InputDlg(const ACaption, APrompt, ADefault: string): string;
function InputQry(const ACaption, APrompt: string;
	var Value: string): Boolean;

function CreateMsgBox(const AMsg: string; const ACaption: string;
	 AType: TMsgBoxType; AButtons: TMsgBoxButtons;
	 ADefaultButton: TDefaultBtn): TForm;

procedure Register;

implementation

{$IFDEF WIN32}
	Uses Win32box;
{$ENDIF}

const
	 MsgBeep: array[TMsgBoxType] of word = (MB_ICONEXCLAMATION,
			MB_ICONHAND, MB_ICONASTERISK, MB_ICONQUESTION, MB_OK);


type
	TMessageForm = class(TForm)
	private
		procedure HelpButtonClick(Sender: TObject);
	end;

procedure TMessageForm.HelpButtonClick(Sender: TObject);
begin
	Application.HelpContext(HelpContext);
end;

function GetAveCharSize(Canvas: TCanvas): TPoint;
var
	I: Integer;
	Buffer: array[0..51] of Char;
begin
	for I := 0 to 25 do Buffer[I] := Chr(I + Ord('A'));
	for I := 0 to 25 do Buffer[I + 26] := Chr(I + Ord('a'));
	GetTextExtentPoint(Canvas.Handle, Buffer, 52, TSize(Result));
	Result.X := Result.X div 52;
end;


function CreateMsgBox(const AMsg: string; const ACaption: string;
	 AType: TMsgBoxType; AButtons: TMsgBoxButtons;
	 ADefaultButton: TDefaultBtn): TForm;

const
	 mcDlgMinWidth = 75;
	 mcDlgMinHeight = 25;

	 mcHorzMargin = 8;        { Left and right margins around text and glyph }
	 mcVertMargin = 5;  { 8;        { Top margin above text and glyph. Also bottom margin below buttons }
	 mcHorzSpacing = 10;      { Left and right margins around button area }
	 mcVertSpacing = 10;      { Vertical spacing between buttons and text }
	 mcButtonWidth = 50;
	 mcButtonHeight = 14;
	 mcButtonSpacing = 4;     { Horizontal spacing between buttons }

const
	 { MessageBox Icons stored in video driver file ie: vga.drv }
	 ResIDs: array[TMsgBoxType] of PChar = (IDI_EXCLAMATION, IDI_HAND,
			IDI_ASTERISK, IDI_QUESTION, nil);

	 { Button.Caption - string constants }
	 BtnCaptions: array[TMsgBoxBtn] of PChar = (SMsgboxYes, SMsgboxAll,
                        SMsgboxNo, SMsgboxOK, SMsgboxAbort, SMsgboxRetry,
                        SMsgboxIgnore, SMsgboxCancel, SMsgboxHelp);

	 { Button.Name }
	 BtnNames: array[TMsgBoxBtn] of PChar = ('btnYes', 'btnAll',
                        'btnNo', 'btnOK', 'btnAbort', 'btnRetry',
                        'btnIgnore', 'btnCancel', 'btnHelp');

	 {Button.Result}
	 BtnResult: array[TMsgBoxBtn] of TModalResult = (mrYes, mrAll, mrNo,
                        mrOK, mrAbort, mrRetry, mrIgnore, mrCancel, mrNone);

var
	 DialogUnits: TPoint;
	 MsgLabel: TLabel;
	 Glyph: TImage;
	 CurrentButton: TButton;
	 Btn, DefaultButton, CancelButton: TMsgBoxBtn;
	 ButtonSize: TPoint;  { Total button area width and height}
	 InfoSize: TPoint;    { Icon and Text width and height}
	 TextRect: TRect;
	 C: array[0..255] of Char;

	 {$IFDEF WIN32}
			MsgFont: TFont;
	 {$ENDIF}

	 DlgMinWidth, DlgMinHeight, HorzMargin, VertMargin, HorzSpacing, VertSpacing,
		 ButtonWidth, ButtonHeight, ButtonSpacing, ButtonX, ButtonTop, ButtonCount: Integer;

	 function Max(v1, v2: Integer): Integer;
	 begin
			if v2 > v1 then Result := v2
			else Result := v1;
	 end;

begin
   Result := TMessageForm.CreateNew(Application);
	 with Result do
	 begin
			BorderStyle := bsDialog;
			BorderIcons := [biSystemMenu];
			Ctl3D := True;

			Font.Name := 'MS Sans Serif';
			Font.Height := -13;
			Font.Style := [fsBold];

			{$IFDEF WIN32}
				{ MessageBox font can only be changed in Windows 95 and Windows NT 4.0 }
				If GetWinVersion > '3.9' Then
				begin
					MsgFont := TFont.Create;
					try
						GetSysFont(sfMessageFont, MsgFont);
						Result.Font := MsgFont;
					finally
						MsgFont.Free;
					end;
				end;
			{$ENDIF}

			{ Removes CLOSE option from system menu. }
			{ GetSystemMenu(Result.Handle, False) Obtains Handle to System Menu }
			RemoveMenu(GetSystemMenu(Result.Handle, False), 1, MF_BYPOSITION);

			DialogUnits := GetAveCharSize(Canvas);
			DlgMinWidth := MulDiv(mcDlgMinWidth, DialogUnits.X, 4);
			DlgMinHeight := MulDiv(mcDlgMinHeight, DialogUnits.Y, 8);

			HorzMargin := MulDiv(mcHorzMargin, DialogUnits.X, 4);
			VertMargin := MulDiv(mcVertMargin, DialogUnits.Y, 8);
			HorzSpacing := MulDiv(mcHorzSpacing, DialogUnits.X, 4);
			VertSpacing := MulDiv(mcVertSpacing, DialogUnits.Y, 8);
			ButtonWidth := MulDiv(mcButtonWidth, DialogUnits.X, 4);
			ButtonHeight := MulDiv(mcButtonHeight, DialogUnits.Y, 8);
			ButtonSpacing := MulDiv(mcButtonSpacing, DialogUnits.X, 4);

			SetRect(TextRect, 0, 0, Screen.Width div 2, 0);
			DrawText(Canvas.Handle, StrPCopy(C, AMsg), -1, TextRect, DT_CALCRECT
				 or DT_WORDBREAK or DT_NOPREFIX or DT_EXPANDTABS);

			{ create the text }
			MsgLabel := TLabel.Create(Result);
			With MsgLabel do
			begin
				Name := 'Message';
				Parent := Result;
				WordWrap := True;
				ShowAccelChar := False;
				Caption := AMsg;
				BoundsRect := TextRect;
			end;

			if ResIDs[AType] <> nil then
			begin
				 Glyph := TImage.Create(Result);
				 with Glyph do
				 begin
					 Name := 'Image';
					 Parent := Result;

					 Picture.Icon.Handle := LoadIcon(0, ResIDs[AType]);
					 BoundsRect := Bounds(HorzMargin, VertMargin,
							 Picture.Icon.Width, Picture.Icon.Height);
				 end;
			end
			else Glyph := nil;

			{ sum up the size of the informational items }
			InfoSize.X := (TextRect.Right - TextRect.Left) + (HorzMargin * 2);
			if Glyph <> nil then
				 InfoSize.X := InfoSize.X + Glyph.Width + HorzMargin;

			if Glyph <> nil then
				 InfoSize.Y := Max(Glyph.Height, TextRect.Bottom -
						TextRect.Top) + VertMargin
			else
				 InfoSize.Y := (TextRect.Bottom - TextRect.Top) + VertMargin;

			ButtonCount := 0;
			{ if only an OK button exists, mark it as Cancel also }
			if (mbOK in AButtons) then CancelButton := mbOK;
      if (mbRetry in AButtons) then DefaultButton := mbRetry;

			for Btn := Low(TMsgBoxBtn) to High(TMsgBoxBtn) do
				if Btn in AButtons then
				begin
					Inc(ButtonCount);

					{ Need to set Default and Cancel properties for button }
					If (Btn = mbNo) or (Btn = mbCancel) then CancelButton := Btn;
					If (Btn = mbYes) or (Btn = mbOK) then DefaultButton := Btn;
			end;

			ButtonSize.X := (ButtonCount * ButtonWidth) +
				 (ButtonSpacing * (ButtonCount - 1)) + (HorzSpacing * 2);
			ButtonSize.Y := ButtonHeight + VertSpacing + VertMargin;

			{ set the caption }
			if ACaption <> EmptyStr then Caption := ACaption
				 else Caption := Application.Title;

			{ Set minimum width and height for TForm necessary to display
				 complete caption, icon, message and buttons. }
			ClientWidth := Max(Max(Canvas.TextWidth(Caption) + 50,
				 DlgMinWidth), Max(InfoSize.X, ButtonSize.X));

			{$IFDEF WIN32}
				{ MessageBox font can only be changed in Windows 95 and Windows NT 4.0 }
				If GetWinVersion > '3.9' Then
				begin
					MsgFont := TFont.Create;
					try
						{ create the text }
            With TLabel.Create(Result) do
			      begin
    				Parent := Result;
    				ShowAccelChar := False;
				    GetSysFont(sfCaptionFont, MsgFont);
						Font := MsgFont;
            Caption := Result.Caption;
			      Result.ClientWidth := Max(Max(Width + 50,
               DlgMinWidth), Max(InfoSize.X, ButtonSize.X));
			      Free
            end;

					finally
						MsgFont.Free;
					end;
				end;
			{$ENDIF}

			ClientHeight := Max(DlgMinHeight, InfoSize.Y + ButtonSize.Y);

			{ Center form on the screen }
			Left := (Screen.Width div 2) - (Width div 2);
			Top := (Screen.Height div 2) - (Height div 2);

			{ layout the text and glyph }
			MsgLabel.Top := VertMargin;
			if Glyph <> nil then
				If (Glyph.Height > (TextRect.Bottom - TextRect.Top)) then
					begin
						Glyph.Top := VertMargin;
						MsgLabel.Top := Glyph.Top + (Glyph.Picture.Graphic.Height div
							2) - ((TextRect.Bottom - TextRect.Top) div 2);
					end
				else
					Glyph.Top := MsgLabel.Top +
						(((TextRect.Bottom - TextRect.Top) div 2) - (Glyph.Height div 2));

			if Glyph <> nil then
				 begin
					 Glyph.Left := HorzMargin;
					 MsgLabel.Left := Glyph.Left + Glyph.Width + HorzMargin;
				 end
			else
				 MsgLabel.Left := HorzMargin;

			{ create the buttons }
			ButtonCount := 0;
			ButtonX := (ClientWidth div 2) - (ButtonSize.X div 2) +
					HorzSpacing;
			ButtonTop := ClientHeight - ButtonHeight - VertMargin;

			for Btn := Low(TMsgBoxBtn) to High(TMsgBoxBtn) do
				 if Btn in AButtons then
				 begin
						Inc(ButtonCount);
						CurrentButton := TButton.Create(Result);
						with CurrentButton do
						begin
							 Parent := Result;
							 SetBounds(ButtonX, ButtonTop, ButtonWidth, ButtonHeight);
							 Inc(ButtonX, ButtonWidth + ButtonSpacing);

							 Caption := StrPas(BtnCaptions[Btn]);
							 Name := StrPas(BtnNames[Btn]);
							 ModalResult := Btnresult[Btn];

							 { Need to set Default and Cancel properties for button }
							 If (Btn = CancelButton) then Cancel := True;
							 If (Btn = DefaultButton) then Default := True;

							 { Determine if button is to be the default }
							 If ButtonCount = Ord(ADefaultButton) + 1 Then
									ActiveControl := CurrentButton;

							 if (Btn = mbHelp) then
								 OnClick := TMessageForm(Result).HelpButtonClick;
						end;

				 end;   { if Btn in AButtons then }

	 end;     { Result := TForm.CreateNew(Application); }

end;


function MsgDlgPos(const AMsg: string; const ACaption: string;
	 AType: TMsgBoxType; AButtons: TMsgBoxButtons;
	 ADefaultButton: TDefaultBtn; HelpCtx: Longint; X, Y: Integer): Word;

var
	 TempCursor: TCursor ;

begin
	 { Store the current Screen.Cursor }
	 TempCursor := Screen.Cursor;

	 { Use the following line if the message box is to be System Modal }
	 { If SysModal Then AType := mtError; }

	 With CreateMsgBox(AMsg, ACaption, AType, AButtons, ADefaultButton) do
	 try
			HelpContext := HelpCtx;
			if X > -1 then Left := X;
			if Y > -1 then Top := Y;

			{ Set the Screen.Cursor to Default }
			Screen.Cursor := crDefault;

			{ Make some noise }
			MessageBeep(MsgBeep[AType]);

			{ Use the following line if the message box is to be System Modal }
			{ If SysModal Then SetSysModalWindow(W.Handle); }

			Result := ShowModal;
	 finally
			{ Restore the original Screen.Cursor }
			Screen.Cursor := TempCursor;
			Free;
	 end;
end;


function MsgDlg(const AMsg: string; const ACaption: string;
	 AType: TMsgBoxType; AButtons: TMsgBoxButtons;
	 ADefaultButton: TDefaultBtn): Word;

begin
	 Result := MsgDlgPos(AMsg, ACaption, AType, AButtons,
			ADefaultButton, 0, -1, -1);
end;


procedure ShowMsgPos(const AMsg: string; X, Y: Integer);
begin
	 MsgDlgPos(AMsg, '', mtCustom, [mbOK], dfFirst, 0, X, Y);
end;


procedure ShowMsg(const AMsg: string);
begin
	 MsgDlgPos(AMsg, '', mtCustom, [mbOK], dfFirst, 0, -1, -1);
end;


function FormCenteredMsgDlg(const AMsg: string;
	 const ACaption: string; AType: TMsgBoxType;
	 AButtons: TMsgBoxButtons; ADefaultButton: TDefaultBtn;
	 HelpCtx: Longint; AForm: TForm): Word;

var
	 TempCursor: TCursor ;

begin
	 { Store the current Screen.Cursor }
	 TempCursor := Screen.Cursor;

	 { Use the following line if the message box is to be System Modal }
	 { If SysModal Then AType := mtError; }

	 With CreateMsgBox(AMsg, ACaption, AType, AButtons, ADefaultButton) do
	 try
			HelpContext := HelpCtx;
			Left := (AForm.Width div 2) - (Width div 2) + AForm.Left;
			Top := (AForm.Height div 2) - (Height div 2) + AForm.Top;

			{ Set the Screen.Cursor to Default }
			Screen.Cursor := crDefault;

			{ Make some noise }
			MessageBeep(MsgBeep[AType]);

			{ Use the following line if the message box is to be System Modal }
			{ If SysModal Then SetSysModalWindow(W.Handle); }

			Result := ShowModal;
	 finally
			{ Restore the original Screen.Cursor }
			Screen.Cursor := TempCursor;
			Free;
	 end;
end;


{ ********** Improved InputBox and InputQuery functions ********** }

function InputQry(const ACaption, APrompt: string;
	var Value: string): Boolean;
var
	W: TForm;
	Edit: TEdit;
	Prompt: TLabel;
	DialogUnits: TPoint;
	ButtonTop, ButtonWidth, ButtonHeight: Integer;
	OKButton: TButton;
	CancelButton: TButton;
	TempCursor: TCursor;

begin
	Result := False;

	{ Store the current Screen.Cursor then set to Default }
	TempCursor := Screen.Cursor;

	W := TForm.Create(Application);
	with W do
		try
			DialogUnits := GetAveCharSize(Canvas);
			BorderStyle := bsDialog;
			Ctl3D := True;
			ClientWidth := MulDiv(200, DialogUnits.X, 4);
			ClientHeight := MulDiv(160, DialogUnits.Y, 8);

			{ Safety incase ACaption = '' }
			if ACaption <> EmptyStr then Caption := ACaption
				else Caption := Application.Title;
			Position := poScreenCenter;

			Font.Name := 'MS Sans Serif';
			Font.Height := -13;

			{$IFDEF WIN32}
				Font.Style := [];
			{$ELSE}
				Font.Style := [fsBold];
			{$ENDIF}

			Prompt := TLabel.Create(W);
			with Prompt do
			begin
				Parent := W;
				AutoSize := True;
				Left := MulDiv(8, DialogUnits.X, 4);
				Top := MulDiv(8, DialogUnits.Y, 8);
				Caption := APrompt;

				{ Set maximum label width = form width - (left margin * 2) }
				Width := W.ClientWidth - (Left * 2);
				{ Wrap Prompt incase length > label.width }
				WordWrap := True;
			end;

			Edit := TEdit.Create(W);
			with Edit do
			begin
				Parent := W;
				Left := Prompt.Left;

				{ Increase spacing between label and TEdit}
				Top := Prompt.Top + Prompt.Height + MulDiv(15, DialogUnits.Y, 8);;
				Width := W.ClientWidth - (Left * 2);
				MaxLength := 255;
				Text := Value;
				SelectAll;
			end;
			Prompt.FocusControl := Edit;

			ButtonTop := Edit.Top + Edit.Height + MulDiv(10, DialogUnits.Y, 8);
			ButtonWidth := MulDiv(50, DialogUnits.X, 4);
			ButtonHeight := MulDiv(14, DialogUnits.Y, 8);

			OKButton := TButton.Create(W);
			with OKButton do
			begin
				Parent := W;

				{ Need to set Caption, ModalResult and Default properties for TButton }
				Caption := SMsgBoxOK;
				ModalResult := mrOK;
				Default := True;
				Top := ButtonTop;
				Width := ButtonWidth;
				Height := ButtonHeight;
				Left := (W.ClientWidth div 2) -
					(((OKButton.Width * 2) + Prompt.Left) div 2)
			end;

			CancelButton := TButton.Create(W);
			with CancelButton do
			begin
				Parent := W;

				{ Need to set Caption, ModalResult and Cancel properties for TButton }
				Caption := SMsgBoxCancel;
				ModalResult := mrCancel;
				Cancel := True;
				Top := ButtonTop;
				Width := ButtonWidth;
				Height := ButtonHeight;
				Left := OKButton.Left + OKButton.Width + Prompt.Left;
			end;

			ClientHeight := ButtonTop + ButtonHeight + Prompt.Top;

			{ Store the current Screen.Cursor then set to Default }
			TempCursor := Screen.Cursor;
			Screen.Cursor := crDefault;

			if W.ShowModal = mrOK then
			begin
				Value := Edit.Text;
				Result := True;
			end;

		finally
			{ Restore the original Screen.Cursor }
			Screen.Cursor := TempCursor;
			W.Free;
		end;
end;


function InputDlg(const ACaption, APrompt, ADefault: string): string;
var
	S: string;

begin
	S := ADefault;
	if InputQry(ACaption, APrompt, S) then Result := S
	else Result := ADefault;
end;


{+---------------------------------------------------------------+
 | Component: TMessageBox                                        |
 | Created: 3/2/97 11:16:25 AM                                   |
 | Author: Steven J. Colagiovanni                                |
 | Copyright 1997, All rights reserved.                          |
 | Description: Message Box Component                            |
 | Version: 1.00.00                                              |
 +---------------------------------------------------------------+}
procedure TMessageDlg.SetCaption(newValue: String);
begin
	if (FCaption <> newValue) then
	begin
		FCaption := newValue;
	end;  { if }
end;  { SetCaption }

procedure TMessageDlg.SetMessageText(newValue: String);
begin
	if (FMessageText <> newValue) then
	begin
		FMessageText := newValue;
	end;  { if }
end;  { SetMessageText }

procedure TMessageDlg.SetMsgBoxType(newValue: TMsgBoxType);
begin
	if (FMessageBoxType <> newValue) then
	begin
		FMessageBoxType := newValue;
	end;  { if }
end;  { SetMessageBoxType }

procedure TMessageDlg.SetButtons(newValue: TMsgBoxButtons);
begin
	if (FButtons <> newValue) then
	begin
		FButtons := newValue;
	end;  { if }
end;  { SetButtons }

procedure TMessageDlg.SetDefaultButton(newValue: TDefaultBtn);
begin
	if (FDefaultButton <> newValue) then
	begin
		FDefaultButton := newValue;
	end;  { if }
end;  { SetDefaultButton }

procedure TMessageDlg.SetCenterOnForm(newValue: TForm);
begin
	if (FCenterOnForm <> newValue) then
	begin
		FCenterOnForm := newValue;
	end;  { if }
end;  { SetCenterOnForm }

procedure TMessageDlg.SetMsgBoxStyle(newValue: TMessageBoxStyle);
begin
	if (FMessageBoxStyle <> newValue) then
	begin
		FMessageBoxStyle := newValue;
	end;  { if }
end;  { SetMessageBoxStyle }

function TMessageDlg.Execute: Word;
var
	TempForm: TForm;
begin
	Result := 0;

  If FCenterOnForm = nil then
		TempForm := TForm(Owner)
	else
		TempForm := FCenterOnForm;

	case FMessageBoxStyle of
		msScreenCentered:
			result := MsgDlg(FMessageText, FCaption, FMessageBoxType, FButtons,
				FDefaultButton);

		msFormCentered:
			result := FormCenteredMsgDlg(FMessageText, FCaption, FMessageBoxType,
				FButtons, FDefaultButton, FHelpContext, TempForm);

		msCustomPosition:
			result := MsgDlgPos(FMessageText, FCaption, FMessageBoxType, FButtons,
				FDefaultButton, FHelpContext, FPositionLeft, FPositionTop);
	end;

end;  { Execute }

destructor TMessageDlg.Destroy;
begin
	inherited Destroy;
end;  { Destroy }

constructor TMessageDlg.Create(AOwner: TComponent);
begin
	inherited Create(AOwner);
	FCaption := '';
	FMessageText := '';
	FMessageBoxType := mtWarning;
	FButtons := [mbOK];
	FDefaultButton := dfFirst;
	FHelpContext := 0;
	FPositionLeft := -1;
	FPositionTop := -1;
	FCenterOnForm := TForm(AOwner);
	FMessageBoxStyle := msScreenCentered;
	end;  { Create }

procedure Register;
begin
	RegisterComponents('Delphi Magazine', [TMessageDlg]);
end;  { Register }

end.

