unit Query;
interface
uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
     StdCtrls, Dialogs, Spin;

Type
  TQueryDlg = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    HelpBtn: TBitBtn;
    Memo1: TMemo;
    CheckArticleType: TCheckBox;
    CheckIssueNumber: TCheckBox;
    CheckAuthorName: TCheckBox;
    CheckKeyword: TCheckBox;
    ArticleType: TComboBox;
    IssueNumber: TSpinEdit;
    AuthorName: TComboBox;
    Keyword: TComboBox;
    procedure CheckArticleTypeClick(Sender: TObject);
    procedure CheckIssueNumberClick(Sender: TObject);
    procedure CheckAuthorNameClick(Sender: TObject);
    procedure CheckKeywordClick(Sender: TObject);
    procedure ArticleTypeChange(Sender: TObject);
    procedure IssueNumberChange(Sender: TObject);
    procedure AuthorNameChange(Sender: TObject);
    procedure KeywordChange(Sender: TObject);
    procedure BtnSQLClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  end;

var
  QueryDlg: TQueryDlg;

implementation
uses MainForm, DBTables, SysUtils;
{$R *.DFM}

procedure TQueryDlg.FormCreate(Sender: TObject);
begin
  Height := (LongInt(190) * PixelsPerInch) div 96;
  with TTable.Create(nil) do
  begin
    DatabaseName := DataPath;
    TableName := 'AUTHOR.DB';
    ReadOnly := True;
    try
      Active := True;
      First;
      while not Eof do
      begin
        AuthorName.Items.Add(Format('%-64s%d',
                  [FieldByName('NAME').AsString,FieldByName('AUTHOR ID').AsInteger]));
        Next
      end
    finally
      Active := False
    end { authors };
    TableName := 'KEYWORD.DB';
    try
      Active := True;
      First;
      while not Eof do
      begin
        Keyword.Items.Add(FieldByName('KEYWORD').AsString);
        Next
      end
    finally
      Active := False;
      Free
    end { keywords }
  end
end;


procedure TQueryDlg.CheckArticleTypeClick(Sender: TObject);
begin
  if not CheckArticleType.Checked then
    ArticleType.ItemIndex := -1
  else
    if (ArticleType.ItemIndex = -1) then
      if (ArticleType.Items.Count > 0) then
        ArticleType.ItemIndex := 0
      else
        CheckArticleType.Checked := False
end;

procedure TQueryDlg.CheckIssueNumberClick(Sender: TObject);
begin
  if not CheckIssueNumber.Checked then
    IssueNumber.Value := 0
  else
    if IssueNumber.Value = 0 then
      IssueNumber.Value := 1
end;

procedure TQueryDlg.CheckAuthorNameClick(Sender: TObject);
begin
  if not CheckAuthorName.Checked then
    AuthorName.ItemIndex := -1
  else
    if (AuthorName.ItemIndex = -1) then
      if (AuthorName.Items.Count > 0) then
        AuthorName.ItemIndex := 0
      else
        CheckAuthorName.Checked := False
end;

procedure TQueryDlg.CheckKeywordClick(Sender: TObject);
begin
  if not CheckKeyword.Checked then
    Keyword.ItemIndex := -1
  else
    if (Keyword.ItemIndex = -1) then
      if (Keyword.Items.Count > 0) then
        Keyword.ItemIndex := 0
      else
        CheckKeyword.Checked := False
end;


procedure TQueryDlg.ArticleTypeChange(Sender: TObject);
begin
  CheckArticleType.Checked := ArticleType.ItemIndex >= 0
end;

procedure TQueryDlg.IssueNumberChange(Sender: TObject);
begin
  CheckIssueNumber.Checked := IssueNumber.Value > 0
end;

procedure TQueryDlg.AuthorNameChange(Sender: TObject);
begin
  CheckAuthorName.Checked := AuthorName.ItemIndex >= 0
end;

procedure TQueryDlg.KeywordChange(Sender: TObject);
begin
  CheckKeyword.Checked := Keyword.ItemIndex >= 0
end;


procedure TQueryDlg.BtnSQLClick(Sender: TObject);
var QueryStr: String;
    Where: Boolean;
    i: Integer;
begin
  if (Sender AS TBitBtn).Caption = '&SQL' then
    Height := (LongInt(362) * PixelsPerInch) div 96;
  Memo1.Lines.Clear;
  Memo1.Lines.Add('SELECT *');
  if CheckKeyword.Checked then Memo1.Lines.Add('FROM ARTICLE, KEYWORD')
                          else Memo1.Lines.Add('FROM ARTICLE');

  if CheckAuthorName.Checked then { look for three authors }
  begin
    for i:=1 to 3 do { "Author i" }
    begin
      if i = 1 then Memo1.Lines.Add('WHERE (')
               else Memo1.Lines.Add(') OR (');
      Memo1.Lines.Add(' (ARTICLE."Author '+IntToStr(i)+'" = '+
                  Copy(AuthorName.Items[AuthorName.ItemIndex],65,10) + ')');

      if CheckArticleType.Checked then
      begin
        Memo1.Lines.Add('  AND');
        Memo1.Lines.Add(' (ARTICLE."Article Type" = "' +
                           ArticleType.Items[ArticleType.ItemIndex] + '")');
      end;
      if CheckIssueNumber.Checked then
      begin
        Memo1.Lines.Add('  AND');
        Memo1.Lines.Add(' (ARTICLE."Issue #" = ' + IssueNumber.Text + ')');
      end;
      if CheckKeyword.Checked then
      begin
        Memo1.Lines.Add('  AND');
        Memo1.Lines.Add(' (ARTICLE."Article ID" = KEYWORD."Article ID")');
        Memo1.Lines.Add('  AND');
        Memo1.Lines.Add(' (KEYWORD."Keyword" = "' + Keyword.Items[Keyword.ItemIndex]+'")')
      end
    end;
    Memo1.Lines.Add(')')
  end
  else { don't look for authors }
  begin
    Where := False;
    if CheckKeyword.Checked then
    begin
      Memo1.Lines.Add('WHERE (ARTICLE."Article ID" =');
      Memo1.Lines.Add('       KEYWORD."Article ID") ');
      Memo1.Lines.Add('  AND (KEYWORD."Keyword" = "' +
                              Keyword.Items[Keyword.ItemIndex]+'")');
      Where := True;
    end;
    QueryStr := '';
    if CheckArticleType.Checked then
    begin
      if not Where then QueryStr := QueryStr + 'WHERE '
                   else QueryStr := QueryStr + '  AND ';
      Where := True;
      QueryStr := QueryStr +
                  '(ARTICLE."Article Type" = "' +
                    ArticleType.Items[ArticleType.ItemIndex] + '")'#13#10
    end;
    if CheckIssueNumber.Checked then
    begin
      if not Where then QueryStr := QueryStr + 'WHERE '
                   else QueryStr := QueryStr + '  AND ';
      Where := True;
      QueryStr := QueryStr +
                  '(ARTICLE."Issue #" = ' +
                    IssueNumber.Text + ')'#13#10
    end;
    Memo1.Lines.Add(QueryStr)
  end;
  Memo1.Lines.Add('ORDER BY ARTICLE."Issue #", ARTICLE."Page Number"')
end;

end.
