{

Demonstration program for the TSolver component

Shows:
  - ParseAndSolve method
  - how to accept values and display results
  - user-defined functions
  - how to display progress
  - how to display error messages

Try the following:
  Lb=La*2
  Lc=200
  Area=2000

  La=Lc-100
  B=0.5
  Area=4000

}

unit triangl1;

interface

uses                     
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, solver, StdCtrls, ExtCtrls, Gauges;

type
  TForm1 = class(TForm)
    Solver1: TSolver;
    pnlError: TPanel;
    edLa: TEdit;
    Label1: TLabel;
    edLb: TEdit;
    Label2: TLabel;
    edA: TEdit;
    Label3: TLabel;
    Label4: TLabel;
    edLc: TEdit;
    edC: TEdit;
    Label5: TLabel;
    Label6: TLabel;
    edB: TEdit;
    edArea: TEdit;
    Label7: TLabel;
    Gauge1: TGauge;
    btnSolve: TButton;
    procedure btnSolveClick(Sender: TObject);
    procedure Solver1ChangeMethod(Sender: TObject;
      NewMethod: TSolveMethod);
    procedure Solver1CountTries(Sender: TObject; NumTries: Word);
    procedure Solver1FunctionCall(Sender: TObject; index: Integer;
      params: PRealArray; NumParams: Integer; var OK: Boolean;
      var result: Double);
    procedure edChange(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    { Private declarations }
    La,Lb,Lc,a,b,c,Area: Double;

    procedure DisplayError;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

const
  equs = 'Area=(abs(lb*lb*sin(a)*cos(a))+abs(la*la*sin(b)*cos(b)))/2'#13#10+
         'la*sin(b)=lb*sin(a)'#13#10+
         'la*sin(c)=lc*sin(a)'#13#10+
         'la*la=lb*lb+lc*lc-2*lb*lc*cos(a)';

{$R *.DFM}

function LineToOffset(memo: TMemo; line: integer): integer;
var i: integer;
begin
  result:=0;
  if line >= memo.lines.count then
    result:=memo.GetTextLen else
  for i:=1 to line do
    inc(result,length(memo.lines.strings[i-1])+2);
end;

procedure TForm1.btnSolveClick(Sender: TObject);
  procedure AddText(const s: string; ed: TEdit);
  begin
    if (ed.Text <> '') and (ed.Font.Style <> []) then
      Solver1.Equations.Add(s+ed.Text) else
      ed.Text:='';
  end;
  procedure ShowAnswer(const s: string; ed: TEdit; i: integer; var x: Double);
  begin
    with Solver1 do
    if (Name[i] = s) and Known[i] then
    begin
      if (ed.Text = '') or (ed.Font.Style = []) then
      begin
        ed.Text:=Format('%1.4g',[Value[i]]);
        ed.Font.Style:=[];
      end;
      x:=Value[i];
    end;
  end;
var i: integer;
    HaveAnswers: boolean;
begin
  pnlError.Caption:='';
  with Solver1 do
  begin
    Equations.SetText(equs);
    AddText('la=',edLa);
    AddText('lb=',edLb);
    AddText('lc=',edLc);
    AddText('a=',eda);
    AddText('b=',edb);
    AddText('c=',edc);
    AddText('Area=',edArea);

    ParseAndSolve;
    if ErrorString = '' then
    begin
      HaveAnswers:=true;
      for i:=0 to NumVariables-1 do
      begin
        ShowAnswer('LA',edLa,i,La);
        ShowAnswer('LB',edLb,i,Lb);
        ShowAnswer('LC',edLc,i,Lc);
        ShowAnswer('A',eda,i,a);
        ShowAnswer('B',edb,i,b);
        ShowAnswer('C',edc,i,c);
        ShowAnswer('AREA',edArea,i,Area);

        HaveAnswers:=HaveAnswers and Known[i];
      end;
      if not HaveAnswers then Area:=0;
    end else
      DisplayError;
    Refresh;
  end;
end;

procedure TForm1.DisplayError;
begin
  pnlError.Caption:=Solver1.ErrorString;
  Messagebeep($FFFF);
end;

procedure TForm1.Solver1ChangeMethod(Sender: TObject;
  NewMethod: TSolveMethod);
begin
  case NewMethod of
    smDirect: pnlError.Caption:='Direct';
    smSecant: pnlError.Caption:='Secant';
    smChop:   pnlError.Caption:='Chop';
    smSimult: pnlError.Caption:='Simult';
  end;
end;

procedure TForm1.Solver1CountTries(Sender: TObject; NumTries: Word);
begin
  Gauge1.MaxValue:=Solver1.Iterations;
  Gauge1.Progress:=NumTries;
end;

procedure TForm1.Solver1FunctionCall(Sender: TObject; index: Integer;
  params: PRealArray; NumParams: Integer; var OK: Boolean;
  var result: Double);
begin
  result:=abs(params^[1]);
end;

procedure TForm1.edChange(Sender: TObject);
begin
  with TEdit(Sender) do
    Font.Style:=[fsBold];
end;

procedure TForm1.FormPaint(Sender: TObject);
var x1,y1,x2,y2,x3,y3: integer;
begin
  if (abs(la) > maxint) or (abs(lb) > maxint) or (abs(lc) > maxint) or
     (abs(a) > maxint) or (abs(b) > maxint) or (abs(c) > maxint) or
     (Area < 1) then exit;

  x1:=20;
  y1:=20;
  x2:=x1+abs(Trunc(lc));
  y2:=y1;
  x3:=x1+abs(Trunc(lb*cos(a)));
  y3:=y1+abs(Trunc(lb*sin(a)));

  SetBkMode(Canvas.Handle,TRANSPARENT);
  Canvas.TextOut(x1,y1,'A');
  Canvas.TextOut(x2,y2,'B');
  Canvas.TextOut(x3,y3,'C');
  Canvas.TextOut((x2+x3) div 2,(y2+y3) div 2,'La');
  Canvas.TextOut((x1+x3) div 2,(y1+y3) div 2,'Lb');
  Canvas.TextOut((x2+x1) div 2,(y2+y1) div 2,'Lc');
  Canvas.Moveto(x1,y1);
  Canvas.Lineto(x2,y2);
  Canvas.Lineto(x3,y3);
  Canvas.Lineto(x1,y1);
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  btnSolve.Left:=Gauge1.Left-btnSolve.Width;
end;

end.


