unit AutoLed;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls;

type
  TAutoLed = class(TGraphicControl)
  private
    { Private declarations }
  protected
     { Protected declarations }
     fLedBitmap    : Array[0..1] of TBITMAP;
     fLedTimer     : TTimer;
     fTrueColor    : TColor;
     fFalseColor   : TColor;
     fBlink        : Boolean;
     fInterval     : longint;
     fValue        : Boolean;
     ColorTemp     : Boolean;
     fOnTimer      : TNotifyEvent;
     fOnMouseEnter : TNotifyEvent;
     fOnMouseLeave : TNotifyEvent;
     procedure Paint;override;
     procedure OnLedTimer(Sender : TObject);
     procedure CMMouseEnter(var Message:TMessage);message CM_MOUSEENTER;
     procedure CMMouseLeave(var Message:TMessage);message CM_MOUSELEAVE;
  public
    { Public declarations }
    constructor Create(AOwner:TComponent);override;
    destructor  Destroy;override;
    procedure CreateLedBitmap;
    procedure FreeLedBitmap;
    procedure ChangeValue(V : Boolean);
    procedure ChangeBlink(V : Boolean);
    procedure SetTrueColor(V : TColor);
    procedure SetFalseColor(V : TColor);
    procedure SetInterval(V : longint);
    procedure SetToTrueColor;
    procedure SetToFalseColor;
    procedure SetLedTimer;
    procedure ResetLedTimer;
  published
    { Published declarations }
    property TrueColor  : TColor  read fTrueColor write SetTrueColor default clLime;
    property FalseColor : TColor  read fFalseColor write SetFalseColor default clSilver;
    property Blink      : Boolean read fBlink write ChangeBlink default true;
    property Value      : Boolean read fValue write ChangeValue default false;
    property Interval   : longint read fInterval write SetInterval default 1000;
    property OnTimer    : TNotifyEvent read fOnTimer write fOnTimer;
    property OnMouseEnter : TNotifyEvent read fOnMouseEnter write fOnMouseEnter;
    property OnMouseLeave : TNotifyEvent read fOnMouseLeave write fOnMouseLeave;
    property OnClick;
  end;

procedure Register;

implementation
{$R Led.res}

constructor TAutoLed.Create(AOwner:TComponent);
begin
   inherited Create(AOwner);
   Height      := 22;
   Width       := 22;
   fTrueColor  := clLime;
   fFalseColor := clSilver;
   fBlink      := true;
   fValue      := false;
   fInterval   := 1000;
   fLedTimer   := nil;
   fLedBitmap[0] := nil;
   fLedBitmap[1] := nil;
   ColorTemp   := true;
   CreateLedBitmap;
end;

destructor TAutoLed.Destroy;
begin
   ResetLedTimer;
   FreeLedBitmap;
   inherited;
end;

procedure TAutoLed.CreateLedBitmap;
begin
   FreeLedBitmap;
   fLedBitmap[0] := TBitmap.Create;
   fLedBitmap[1] := TBitmap.Create;
   fLedBitmap[0].Handle := LoadImage(HINSTANCE,'LLED',IMAGE_BITMAP,0,0,0);
   fLedBitmap[1].Handle := LoadImage(HINSTANCE,'LLED',IMAGE_BITMAP,0,0,0);
   fLedBitmap[0].Canvas.Brush.Color := fTrueColor;
   fLedBitmap[0].Canvas.FloodFill(11, 11, clLime, fsSurface);
   fLedBitmap[1].Canvas.Brush.Color := fFalseColor;
   fLedBitmap[1].Canvas.FloodFill(11, 11, clLime, fsSurface);
end;

procedure TAutoLed.FreeLedBitmap;
begin
   if (Assigned(fLedBitmap[0])) then fLedBitmap[0].Destroy;
   if (Assigned(fLedBitmap[1])) then fLedBitmap[1].Destroy;
   fLedBitmap[0] := nil;
   fLedBitmap[1] := nil;
end;

procedure TAutoLed.Paint;
begin
   Canvas.StretchDraw(Rect(0,0,Width,Height),fLedBitmap[integer(ColorTemp)]);
end;

procedure TAutoLed.OnLedTimer(Sender: TObject);
begin
   ColorTemp := not ColorTemp;
   Canvas.StretchDraw(Rect(0,0,Width,Height),fLedBitmap[integer(ColorTemp)]);
   if (Assigned(OnTimer)) then fOnTimer(Sender);
end;

procedure TAutoLed.SetToTrueColor;
begin
   Canvas.StretchDraw(Rect(0,0,Width,Height),fLedBitmap[0]);
   ColorTemp := false;
end;

procedure TAutoLed.SetToFalseColor;
begin
   Canvas.StretchDraw(Rect(0,0,Width,Height),fLedBitmap[1]);
   ColorTemp := true;
end;

procedure TAutoLed.SetLedTimer;
begin
   if (Assigned(fLedTimer)) then Exit;
   if (csDesigning in ComponentState) then Exit;
   ColorTemp := false;
   fLedTimer := TTimer.Create(Self);
   fLedTimer.Interval := fInterval;
   fLedTimer.OnTimer  := OnLedTimer;
end;

procedure TAutoLed.ResetLedTimer;
begin
   if (Assigned(fLedTimer)) then begin
      fLedTimer.Destroy;
      fLedTimer := nil;
   end;
end;


procedure TAutoLed.ChangeValue(V : Boolean);
begin
   if (fValue <> V) then begin
       fValue := V;
       if (fValue) then begin
          SetToTrueColor;
          if (fBlink) then
             SetLedTimer;
       end else begin
          ResetLedTimer;
          SetToFalseColor;
       end
       (* fValue == true or false *)
   end; (*(fValue <> V)*)
end;

procedure TAutoLed.ChangeBlink(V : Boolean);
begin
   if (fBlink <> V) then begin
       if (fValue) then
          SetToTrueColor
       else
          SetToFalseColor;
       fBlink := V;
       if (V and fValue) then
          SetLedTimer
       else
          ResetLedTimer
   end;
end;

procedure TAutoLed.SetTrueColor(V : TColor);
var
   Temp : Boolean;
begin
   if (fTrueColor <> V) then begin
      Temp := fBlink;
      if (fBlink) then Blink := false;
      fTrueColor := V;
      CreateLedBitmap;
      Blink := Temp;
      Repaint;
   end
end;

procedure TAutoLed.SetFalseColor(V : TColor);
var
   Temp : Boolean;
begin
   if (fFalseColor <> V) then begin
      Temp := fBlink;
      if (fBlink) then Blink := false;
      fFalseColor := V;
      CreateLedBitmap;
      Blink := Temp;
      Repaint;
   end
end;

procedure TAutoLed.SetInterval(V : longint);
begin
   fInterval := V;
   if (Assigned(fLedTimer)) then
      fLedTimer.Interval := V;
end;

procedure TAutoLed.CmMouseEnter(var Message:TMessage);
begin
   inherited;
   if (Assigned(fOnMouseEnter)) then fOnMouseEnter(Self);
end;

procedure TAutoLed.CmMouseLeave(var Message:TMessage);
begin
   inherited;
   if (Assigned(fOnMouseLeave)) then fOnMouseLeave(Self);
end;

procedure Register;
begin
  RegisterComponents('Cool [SU]', [TAutoLed]);
end;

end.
